@@ -14,13 +14,10 @@
 #define FC__AIHAND_H
 
 #include "fc_types.h"
-#include "city.h"		/* enum choice_type */
 
 void ai_do_first_activities(struct player *pplayer);
 void ai_do_last_activities(struct player *pplayer);
 
-bool is_unit_choice_type(enum choice_type type);
-
 void ai_best_government(struct player *pplayer);
 
 #endif  /* FC__AIHAND_H */
