@@ -17,25 +17,28 @@
 
 #include <assert.h>
 
+/* utility */
+#include "government.h"
+#include "mem.h"
+#include "shared.h"
+
+/* common */
 #include "city.h"
 #include "combat.h"
 #include "game.h"
-#include "government.h"
 #include "log.h"
 #include "map.h"
-#include "mem.h"
 #include "movement.h"
 #include "packets.h"
 #include "player.h"
-#include "shared.h"
 #include "unit.h"
 #include "unitlist.h"
 
+/* aicore */
 #include "citymap.h"
-#include "path_finding.h"
 #include "pf_tools.h"
 
-#include "airgoto.h"
+/* server */
 #include "barbarian.h"
 #include "citytools.h"
 #include "cityturn.h"
@@ -47,6 +50,7 @@
 #include "unithand.h"
 #include "unittools.h"
 
+/* ai */
 #include "advmilitary.h"
 #include "aicity.h"
 #include "aidata.h"
@@ -59,9 +63,10 @@
 #include "aitools.h"
 
 /**************************************************************************
-  Return a string describing a unit's AI role.
+  Return the (untranslated) rule name of the ai_unit_task.
+  You don't have to free the return pointer.
 **************************************************************************/
-const char *get_ai_role_str(enum ai_unit_task task)
+const char *ai_unit_task_rule_name(const enum ai_unit_task task)
 {
   switch(task) {
    case AIUNIT_NONE:
@@ -83,6 +88,30 @@ const char *get_ai_role_str(enum ai_unit_task task)
    case AIUNIT_HUNTER:
      return "Hunter";
   }
+  /* no default, ensure all types handled somehow */
+  assert(FALSE);
+  return NULL;
+}
+
+/**************************************************************************
+  Return the (untranslated) rule name of the ai_choice.
+  You don't have to free the return pointer.
+**************************************************************************/
+const char *ai_choice_rule_name(const struct ai_choice *choice)
+{
+  switch (choice->type) {
+  case CT_NONE:
+    return "(nothing)";
+  case CT_BUILDING:
+    return improvement_rule_name(choice->value.building);
+  case CT_CIVILIAN:
+  case CT_ATTACKER:
+  case CT_DEFENDER:
+    return utype_rule_name(choice->value.utype);
+  case CT_LAST:
+    return "(unknown)";
+  };
+  /* no default, ensure all types handled somehow */
   assert(FALSE);
   return NULL;
 }
@@ -138,18 +167,18 @@ bool is_player_dangerous(struct player *pplayer, struct player *aplayer)
   }
 
   ai = ai_data_get(pplayer);
-  adip = &(ai->diplomacy.player_intel[aplayer->player_no]);
+  adip = &(ai->diplomacy.player_intel[player_index(aplayer)]);
   
   if (adip->countdown >= 0 || adip->is_allied_with_enemy) {
     /* Don't trust our war target or someone who will declare war on us soon */
     return TRUE;
   }
   
-  if (pplayer->diplstates[aplayer->player_no].has_reason_to_cancel > 0) {
+  if (pplayer->diplstates[player_index(aplayer)].has_reason_to_cancel > 0) {
     return TRUE;
   }
   
-  if (pplayer->ai.love[aplayer->player_no] < MAX_AI_LOVE / 10) {
+  if (pplayer->ai_data.love[player_index(aplayer)] < MAX_AI_LOVE / 10) {
     /* We don't trust players who we don't like. Note that 
      * aplayer's units inside pplayer's borders decreases AI's love */
     return TRUE;
@@ -168,7 +197,7 @@ bool is_player_dangerous(struct player *pplayer, struct player *aplayer)
 *************************************************************************/
 bool ai_unit_execute_path(struct unit *punit, struct pf_path *path)
 {
-  const bool is_ai = unit_owner(punit)->ai.control;
+  const bool is_ai = unit_owner(punit)->ai_data.control;
   int i;
 
   /* We start with i = 1 for i = 0 is our present position */
@@ -233,10 +262,10 @@ static void ai_gothere_bodyguard(struct unit *punit, struct tile *dest_tile)
       danger += unit_att_rating(aunit);
     }
   } unit_list_iterate_end;
-  dcity = tile_get_city(dest_tile);
+  dcity = tile_city(dest_tile);
   if (dcity && HOSTILE_PLAYER(pplayer, ai, city_owner(dcity))) {
     /* Assume enemy will build another defender, add it's attack strength */
-    struct unit_type *d_type = ai_choose_defender_versus(dcity, unit_type(punit));
+    struct unit_type *d_type = ai_choose_defender_versus(dcity, punit);
 
     if (d_type) {
       /* Enemy really can build something */
@@ -342,25 +371,47 @@ bool ai_gothere(struct player *pplayer, struct unit *punit,
   For example, aircraft need these way-points to refuel.
 **************************************************************************/
 struct tile *immediate_destination(struct unit *punit,
-				   struct tile *dest_tile)
+                                   struct tile *dest_tile)
 {
-  if (!same_pos(punit->tile, dest_tile) && is_air_unit(punit)) {
-    struct tile *waypoint_tile = punit->goto_tile;
+  if (!same_pos(punit->tile, dest_tile) && utype_fuel(unit_type(punit))) {
+    struct pf_parameter parameter;
+    struct pf_map *pfm;
+    struct pf_path *path;
+    size_t i;
 
-    if (find_air_first_destination(punit, &waypoint_tile)) {
-      return waypoint_tile;
-    } else {
-      freelog(LOG_VERBOSE, "Did not find an air-route for "
-	      "%s %s[%d] (%d,%d)->(%d,%d)",
-	      nation_rule_name(nation_of_unit(punit)),
-	      unit_rule_name(punit),
-	      punit->id,
-	      TILE_XY(punit->tile),
-	      TILE_XY(dest_tile));
-      /* Prevent take off */
-      return punit->tile;
+    pft_fill_unit_parameter(&parameter, punit);
+    pfm = pf_map_new(&parameter);
+    path = pf_map_get_path(pfm, punit->goto_tile);
+
+    if (path) {
+      for (i = 1; i < path->length; i++) {
+        if (path->positions[i].tile == path->positions[i - 1].tile) {
+          /* The path-finding code advices us to wait there to refuel. */
+          struct tile *ptile = path->positions[i].tile;
+
+          pf_path_destroy(path);
+          pf_map_destroy(pfm);
+          return ptile;
+        }
+      }
+      pf_path_destroy(path);
+      pf_map_destroy(pfm);
+      /* Seems it's the immediate destination */
+      return punit->goto_tile;
     }
+
+    pf_map_destroy(pfm);
+    freelog(LOG_VERBOSE, "Did not find an air-route for "
+            "%s %s[%d] (%d,%d)->(%d,%d)",
+            nation_rule_name(nation_of_unit(punit)),
+            unit_rule_name(punit),
+            punit->id,
+            TILE_XY(punit->tile),
+            TILE_XY(dest_tile));
+    /* Prevent take off */
+    return punit->tile;
   }
+
   /* else does not need way-points */
   return dest_tile;
 }
@@ -381,12 +432,12 @@ bool ai_follow_path(struct unit *punit, struct pf_path *path,
     return TRUE;
   }
   punit->goto_tile = ptile;
-  handle_unit_activity_request(punit, ACTIVITY_GOTO);
+  unit_activity_handling(punit, ACTIVITY_GOTO);
   alive = ai_unit_execute_path(punit, path);
   if (alive) {
-    handle_unit_activity_request(punit, ACTIVITY_IDLE);
-    send_unit_info(NULL, punit);
-    handle_unit_activity_request(punit, activity);
+    unit_activity_handling(punit, ACTIVITY_IDLE);
+    send_unit_info(NULL, punit); /* FIXME: probably duplicate */
+    unit_activity_handling(punit, activity);
     punit->goto_tile = old_tile; /* May be NULL. */
     send_unit_info(NULL, punit);
   }
@@ -399,7 +450,7 @@ bool ai_follow_path(struct unit *punit, struct pf_path *path,
 void ai_log_path(struct unit *punit,
 		 struct pf_path *path, struct pf_parameter *parameter)
 {
-  const struct pf_position *last = pf_last_position(path);
+  const struct pf_position *last = pf_path_get_last_position(path);
   const int cc = PF_TURN_FACTOR * last->total_MC
                  + parameter->move_rate * last->total_EC;
   const int tc = cc / (PF_TURN_FACTOR *parameter->move_rate); 
@@ -422,8 +473,8 @@ bool ai_unit_goto_constrained(struct unit *punit, struct tile *ptile,
 			      struct pf_parameter *parameter)
 {
   bool alive = TRUE;
-  struct pf_map *map = NULL;
-  struct pf_path *path = NULL;
+  struct pf_map *pfm;
+  struct pf_path *path;
 
   UNIT_LOG(LOG_DEBUG, punit, "constrained goto to %d,%d",
 	   ptile->x, ptile->y);
@@ -450,8 +501,8 @@ bool ai_unit_goto_constrained(struct unit *punit, struct tile *ptile,
     return TRUE;
   }
 
-  map = pf_create_map(parameter);
-  path = pf_get_path(map, ptile);
+  pfm = pf_map_new(parameter);
+  path = pf_map_get_path(pfm, ptile);
 
   if (path) {
     ai_log_path(punit, path, parameter);
@@ -461,8 +512,8 @@ bool ai_unit_goto_constrained(struct unit *punit, struct tile *ptile,
     UNIT_LOG(LOG_DEBUG, punit, "no path to destination");
   }
 
-  pf_destroy_path(path);
-  pf_destroy_map(map);
+  pf_path_destroy(path);
+  pf_map_destroy(pfm);
 
   return alive;
 }
@@ -479,7 +530,7 @@ static int stack_value(const struct tile *ptile,
   if (is_stack_vulnerable(ptile)) {
     unit_list_iterate(ptile->units, punit) {
       if (unit_owner(punit) == pplayer) {
-	cost += unit_build_shield_cost(unit_type(punit));
+	cost += unit_build_shield_cost(punit);
       }
     } unit_list_iterate_end;
   }
@@ -510,10 +561,10 @@ static double chance_killed_at(const struct tile *ptile,
    * If we don't do this, the amphibious movement code has too strong a
    * desire to minimise the length of the path,
    * leading to poor choice for landing beaches */
-  double p = is_ocean(ptile->terrain)? 0.05: 0.15;
+  double p = is_ocean_tile(ptile)? 0.05: 0.15;
 
   /* If we are on defensive terrain, we are more likely to survive */
-  db = 10 + ptile->terrain->defense_bonus / 10;
+  db = 10 + tile_terrain(ptile)->defense_bonus / 10;
   if (tile_has_special(ptile, S_RIVER)) {
     db += (db * terrain_control.river_defense_bonus) / 100;
   }
@@ -548,13 +599,6 @@ static int stack_risk(const struct tile *ptile,
   const double p_killed = chance_killed_at(ptile, risk_cost, param);
   double danger = value * p_killed;
 
-  if (terrain_has_flag(ptile->terrain, TER_UNSAFE)) {
-    danger += risk_cost->unsafe_terrain_cost;
-  }
-  if (is_ocean(ptile->terrain) && !is_safe_ocean(ptile)) {
-    danger += risk_cost->ocean_cost;
-  }
-
   /* Adjust for the fact that we might not stop at this tile,
    * and for our fearfulness */
   risk += danger * risk_cost->fearfulness;
@@ -595,7 +639,6 @@ void ai_avoid_risks(struct pf_parameter *parameter,
 		    struct unit *punit,
 		    const double fearfulness)
 {
-  const struct player *pplayer = unit_owner(punit);
   /* If we stay a short time on each tile, the danger of each individual tile
    * is reduced. If we do not do this,
    * we will not favour longer but faster routs. */
@@ -604,18 +647,9 @@ void ai_avoid_risks(struct pf_parameter *parameter,
   parameter->data = risk_cost;
   parameter->get_EC = prefer_short_stacks;
   parameter->turn_mode = TM_WORST_TIME;
-  risk_cost->base_value = unit_build_shield_cost(unit_type(punit));
+  risk_cost->base_value = unit_build_shield_cost(punit);
   risk_cost->fearfulness = fearfulness * linger_fraction;
 
-  if (unit_has_type_flag(punit, F_TRIREME)) {
-    risk_cost->ocean_cost = risk_cost->base_value
-      * (double)base_trireme_loss_pct(pplayer, punit)
-      / 100.0;
-  } else {
-    risk_cost->ocean_cost = 0;
-  }
-  risk_cost->unsafe_terrain_cost = risk_cost->base_value
-    * (double)base_unsafe_terrain_loss_pct(pplayer, punit) / 100.0;
   risk_cost->enemy_zoc_cost = PF_TURN_FACTOR * 20;
 }
 
@@ -646,22 +680,33 @@ void ai_fill_unit_param(struct pf_parameter *parameter,
 			struct ai_risk_cost *risk_cost,
 			struct unit *punit, struct tile *ptile)
 {
-  const bool is_ferry = get_transporter_capacity(punit) > 0
-                        && !unit_has_type_flag(punit, F_MISSILE_CARRIER)
-                        && punit->ai.ai_role != AIUNIT_HUNTER;
-  const bool is_air = is_air_unit(punit)
-                      && punit->ai.ai_role != AIUNIT_ESCORT;
   const bool long_path = LONG_TIME < (map_distance(punit->tile, punit->tile)
 				      * SINGLE_MOVE
 				      / unit_type(punit)->move_rate);
   const bool barbarian = is_barbarian(unit_owner(punit));
-  const bool is_ai = unit_owner(punit)->ai.control;
+  const bool is_ai = unit_owner(punit)->ai_data.control;
+  bool is_ferry = FALSE;
+
+  if (punit->ai.ai_role != AIUNIT_HUNTER
+      && get_transporter_capacity(punit) > 0) {
+    unit_class_iterate(uclass) {
+      /* FIXME: BOTH_MOVING units need ferry only if they use fuel */
+      if (can_unit_type_transport(unit_type(punit), uclass)
+          && (uclass->move_type == LAND_MOVING
+              || (uclass->move_type == BOTH_MOVING
+                  && !uclass_has_flag(uclass, UCF_MISSILE)))) {
+        is_ferry = TRUE;
+        break;
+      }
+    } unit_class_iterate_end;
+  }
 
   if (is_ferry) {
     /* The destination may be a coastal land tile,
      * in which case the ferry should stop on an adjacent tile. */
     pft_fill_unit_overlap_param(parameter, punit);
-  } else if (is_ai && !is_air && is_military_unit(punit)
+  } else if (is_ai && !utype_fuel(unit_type(punit))
+             && is_military_unit(punit)
 	     && (punit->ai.ai_role == AIUNIT_DEFEND_HOME
 		 || punit->ai.ai_role == AIUNIT_ATTACK
 		 || punit->ai.ai_role ==  AIUNIT_ESCORT
@@ -676,10 +721,10 @@ void ai_fill_unit_param(struct pf_parameter *parameter,
   /* Should we use the risk avoidance code?
    * The risk avoidance code uses omniscience, so do not use for
    * human-player units under temporary AI control.
-   * Air units are immune to most risks, especially dangerous terrain.
    * Barbarians bravely/stupidly ignore risks
    */
-  if (is_ai && !is_air && !barbarian) {
+  if (is_ai && !uclass_has_flag(unit_class(punit), UCF_UNREACHABLE)
+      && !barbarian) {
     ai_avoid_risks(parameter, risk_cost, punit, NORMAL_STACKING_FEARFULNESS);
   }
 
@@ -690,7 +735,7 @@ void ai_fill_unit_param(struct pf_parameter *parameter,
    * TODO: This is compatible with old code,
    * but probably ought to be more cautious for non military units
    */
-  if (is_ai && !is_ferry && !is_air) {
+  if (is_ai && !is_ferry && !utype_fuel(unit_type(punit))) {
     parameter->get_moves_left_req = NULL;
   }
 
@@ -738,9 +783,8 @@ void ai_fill_unit_param(struct pf_parameter *parameter,
   } else if (is_ferry) {
     /* Ferries are not warships */
     parameter->get_TB = no_fights;
-  } else if (is_air) {
-    /* Default tile behaviour */
-  } else if (is_heli_unit(punit)) {
+  } else if (is_losing_hp(punit)) {
+    /* Losing hitpoints over time (helicopter in default rules) */
     /* Default tile behaviour */
   } else if (is_military_unit(punit)) {
     switch (punit->ai.ai_role) {
@@ -807,7 +851,8 @@ void ai_unit_new_role(struct unit *punit, enum ai_unit_task task,
   assert(!unit_has_orders(punit) || task == AIUNIT_NONE);
 
   UNIT_LOG(LOG_DEBUG, punit, "changing role from %s to %s",
-           get_ai_role_str(punit->ai.ai_role), get_ai_role_str(task));
+           ai_unit_task_rule_name(punit->ai.ai_role),
+           ai_unit_task_rule_name(task));
 
   /* Free our ferry.  Most likely it has been done already. */
   if (task == AIUNIT_NONE || task == AIUNIT_DEFEND_HOME) {
@@ -816,7 +861,7 @@ void ai_unit_new_role(struct unit *punit, enum ai_unit_task task,
 
   if (punit->activity == ACTIVITY_GOTO) {
     /* It would indicate we're going somewhere otherwise */
-    handle_unit_activity_request(punit, ACTIVITY_IDLE);
+    unit_activity_handling(punit, ACTIVITY_IDLE);
   }
 
   if (punit->ai.ai_role == AIUNIT_BUILD_CITY) {
@@ -837,7 +882,7 @@ void ai_unit_new_role(struct unit *punit, enum ai_unit_task task,
     struct unit *target = game_find_unit_by_number(punit->ai.target);
 
     if (target) {
-      target->ai.hunted &= ~(1 << unit_owner(punit)->player_no);
+      target->ai.hunted &= ~(1 << player_index(unit_owner(punit)));
       UNIT_LOG(LOGLEVEL_HUNT, target, "no longer hunted (new role %d, old %d)",
                task, punit->ai.ai_role);
     }
@@ -845,8 +890,8 @@ void ai_unit_new_role(struct unit *punit, enum ai_unit_task task,
 
   aiguard_clear_charge(punit);
   /* Record the city to defend; our goto may be to transport. */
-  if (task == AIUNIT_DEFEND_HOME && ptile && ptile->city) {
-    aiguard_assign_guard_city(ptile->city, punit);
+  if (task == AIUNIT_DEFEND_HOME && ptile && tile_city(ptile)) {
+    aiguard_assign_guard_city(tile_city(ptile), punit);
   }
 
   punit->ai.ai_role = task;
@@ -860,7 +905,7 @@ void ai_unit_new_role(struct unit *punit, enum ai_unit_task task,
   }
 
   /* Reserve city spot, _unless_ we want to add ourselves to a city. */
-  if (punit->ai.ai_role == AIUNIT_BUILD_CITY && !tile_get_city(ptile)) {
+  if (punit->ai.ai_role == AIUNIT_BUILD_CITY && !tile_city(ptile)) {
     citymap_reserve_city_spot(ptile, punit->id);
   }
   if (punit->ai.ai_role == AIUNIT_HUNTER) {
@@ -868,24 +913,21 @@ void ai_unit_new_role(struct unit *punit, enum ai_unit_task task,
     struct unit *target = game_find_unit_by_number(punit->ai.target);
 
     assert(target != NULL);
-    target->ai.hunted |= (1 << unit_owner(punit)->player_no);
+    target->ai.hunted |= (1 << player_index(unit_owner(punit)));
     UNIT_LOG(LOGLEVEL_HUNT, target, "is being hunted");
 
     /* Grab missiles lying around and bring them along */
-    if (unit_has_type_flag(punit, F_MISSILE_CARRIER)
-        || unit_has_type_flag(punit, F_CARRIER)) {
-      unit_list_iterate(punit->tile->units, missile) {
-        if (missile->ai.ai_role != AIUNIT_ESCORT
-            && missile->transported_by == -1
-            && unit_owner(missile) == unit_owner(punit)
-            && unit_has_type_flag(missile, F_MISSILE)
-            && can_unit_load(missile, punit)) {
-          UNIT_LOG(LOGLEVEL_HUNT, missile, "loaded on hunter");
-          ai_unit_new_role(missile, AIUNIT_ESCORT, target->tile);
-          load_unit_onto_transporter(missile, punit);
-        }
-      } unit_list_iterate_end;
-    }
+    unit_list_iterate(punit->tile->units, missile) {
+      if (missile->ai.ai_role != AIUNIT_ESCORT
+          && missile->transported_by == -1
+          && unit_owner(missile) == unit_owner(punit)
+          && uclass_has_flag(unit_class(missile), UCF_MISSILE)
+          && can_unit_load(missile, punit)) {
+        UNIT_LOG(LOGLEVEL_HUNT, missile, "loaded on hunter");
+        ai_unit_new_role(missile, AIUNIT_ESCORT, target->tile);
+        load_unit_onto_transporter(missile, punit);
+      }
+    } unit_list_iterate_end;
   }
 }
 
@@ -945,7 +987,7 @@ static void ai_unit_bodyguard_move(struct unit *bodyguard, struct tile *ptile)
     return;
   }
 
-  handle_unit_activity_request(bodyguard, ACTIVITY_IDLE);
+  unit_activity_handling(bodyguard, ACTIVITY_IDLE);
   (void) ai_unit_move(bodyguard, ptile);
 }
 
@@ -959,11 +1001,11 @@ bool ai_unit_attack(struct unit *punit, struct tile *ptile)
   bool alive;
 
   CHECK_UNIT(punit);
-  assert(unit_owner(punit)->ai.control);
+  assert(unit_owner(punit)->ai_data.control);
   assert(is_tiles_adjacent(punit->tile, ptile));
 
-  handle_unit_activity_request(punit, ACTIVITY_IDLE);
-  (void) handle_unit_move_request(punit, ptile, FALSE, FALSE);
+  unit_activity_handling(punit, ACTIVITY_IDLE);
+  (void) unit_move_handling(punit, ptile, FALSE, FALSE);
   alive = (game_find_unit_by_number(sanity) != NULL);
 
   if (alive && same_pos(ptile, punit->tile)
@@ -990,10 +1032,14 @@ bool ai_unit_move(struct unit *punit, struct tile *ptile)
   struct unit *bodyguard;
   int sanity = punit->id;
   struct player *pplayer = unit_owner(punit);
-  const bool is_ai = pplayer->ai.control;
+  const bool is_ai = pplayer->ai_data.control;
 
   CHECK_UNIT(punit);
-  assert(is_tiles_adjacent(punit->tile, ptile));
+  RETURN_VAL_IF_FAIL_MSG(is_tiles_adjacent(unit_tile(punit), ptile), FALSE,
+                         "Tiles not adjacent: Unit = %d, "
+                         "from = (%d, %d]) to = (%d, %d).",
+                         punit->id, TILE_XY(unit_tile(punit)),
+                         TILE_XY(ptile));
 
   /* if enemy, stop and give a chance for the ai attack function
    * or the human player to handle this case */
@@ -1019,8 +1065,8 @@ bool ai_unit_move(struct unit *punit, struct tile *ptile)
   }
 
   /* Try not to end move next to an enemy if we can avoid it by waiting */
-  if (punit->moves_left <= map_move_cost(punit, ptile)
-      && unit_move_rate(punit) > map_move_cost(punit, ptile)
+  if (punit->moves_left <= map_move_cost_unit(punit, ptile)
+      && unit_move_rate(punit) > map_move_cost_unit(punit, ptile)
       && enemies_at(punit, ptile)
       && !enemies_at(punit, punit->tile)) {
     UNIT_LOG(LOG_DEBUG, punit, "ending move early to stay out of trouble");
@@ -1028,8 +1074,8 @@ bool ai_unit_move(struct unit *punit, struct tile *ptile)
   }
 
   /* go */
-  handle_unit_activity_request(punit, ACTIVITY_IDLE);
-  (void) handle_unit_move_request(punit, ptile, FALSE, TRUE);
+  unit_activity_handling(punit, ACTIVITY_IDLE);
+  (void) unit_move_handling(punit, ptile, FALSE, TRUE);
 
   /* handle the results */
   if (game_find_unit_by_number(sanity) && same_pos(ptile, punit->tile)) {
@@ -1054,7 +1100,7 @@ struct city *dist_nearest_city(struct player *pplayer, struct tile *ptile,
 { 
   struct city *pc=NULL;
   int best_dist = -1;
-  Continent_id con = tile_get_continent(ptile);
+  Continent_id con = tile_continent(ptile);
 
   players_iterate(pplay) {
     /* If "enemy" is set, only consider cities whose owner we're at
@@ -1070,7 +1116,7 @@ struct city *dist_nearest_city(struct player *pplayer, struct tile *ptile,
        * continent. */
       if ((best_dist == -1 || city_dist < best_dist)
 	  && (everywhere || con == 0
-	      || con == tile_get_continent(pcity->tile))
+	      || con == tile_continent(pcity->tile))
 	  && (!pplayer || map_is_known(pcity->tile, pplayer))) {
 	best_dist = city_dist;
         pc = pcity;
@@ -1093,11 +1139,11 @@ int stack_cost(struct unit *pdef)
   if (is_stack_vulnerable(pdef->tile)) {
     /* lotsa people die */
     unit_list_iterate(pdef->tile->units, aunit) {
-      victim_cost += unit_build_shield_cost(unit_type(aunit));
+      victim_cost += unit_build_shield_cost(aunit);
     } unit_list_iterate_end;
   } else {
     /* Only one unit dies if attack is successful */
-    victim_cost = unit_build_shield_cost(unit_type(pdef));
+    victim_cost = unit_build_shield_cost(pdef);
   }
   
   return victim_cost;
@@ -1112,7 +1158,7 @@ void ai_government_change(struct player *pplayer, struct government *gov)
     return;
   }
 
-  handle_player_change_government(pplayer, gov->index);
+  handle_player_change_government(pplayer, government_number(gov));
 
   city_list_iterate(pplayer->cities, pcity) {
     auto_arrange_workers(pcity); /* update cities */
@@ -1128,7 +1174,7 @@ void ai_government_change(struct player *pplayer, struct government *gov)
 int ai_gold_reserve(struct player *pplayer)
 {
   int i = total_player_citizens(pplayer)*2;
-  return MAX(pplayer->ai.maxbuycost, i);
+  return MAX(pplayer->ai_data.maxbuycost, i);
 }
 
 /**************************************************************************
@@ -1136,7 +1182,7 @@ int ai_gold_reserve(struct player *pplayer)
 **************************************************************************/
 void init_choice(struct ai_choice *choice)
 {
-  choice->choice = A_UNSET;
+  choice->value.utype = NULL;
   choice->want = 0;
   choice->type = CT_NONE;
   choice->need_boat = FALSE;
@@ -1155,17 +1201,28 @@ void adjust_choice(int value, struct ai_choice *choice)
 **************************************************************************/
 void copy_if_better_choice(struct ai_choice *cur, struct ai_choice *best)
 {
-  if (cur->want > best->want) {
-    best->choice =cur->choice;
-    best->want = cur->want;
-    best->type = cur->type;
-    best->need_boat = cur->need_boat;
+  if (best->want < cur->want) {
+    /* This simple minded copy works for now.
+     *
+     * TODO: We should get rid of this function. Choice structures should
+     *       be accessed via pointers, and those pointers should be updated
+     *       instead of copying whole structures.
+     */
+    *best = *cur;
   }
 }
 
+/**************************************************************************
+  ...
+**************************************************************************/
+bool is_unit_choice_type(enum choice_type type)
+{
+   return type == CT_CIVILIAN || type == CT_ATTACKER || type == CT_DEFENDER;
+}
+
 /**************************************************************************
   Calls ai_wants_role_unit to choose the best unit with the given role and 
-  set tech wants.  Sets choice->choice if we can build something.
+  set tech wants.  Sets choice->value.utype when we can build something.
 **************************************************************************/
 bool ai_choose_role_unit(struct player *pplayer, struct city *pcity,
 			 struct ai_choice *choice, enum choice_type type,
@@ -1175,7 +1232,7 @@ bool ai_choose_role_unit(struct player *pplayer, struct city *pcity,
 
   if (iunit != NULL) {
     choice->type = type;
-    choice->choice = iunit->index;
+    choice->value.utype = iunit;
     choice->want = want;
 
     choice->need_boat = need_boat;
@@ -1194,30 +1251,34 @@ bool ai_choose_role_unit(struct player *pplayer, struct city *pcity,
 **************************************************************************/
 void ai_advisor_choose_building(struct city *pcity, struct ai_choice *choice)
 {
-  Impr_type_id id = B_LAST;
+  struct impr_type *chosen = NULL;
   int want = 0;
   struct player *plr = city_owner(pcity);
 
-  impr_type_iterate(i) {
-    if (!plr->ai.control && is_wonder(i)) {
+  improvement_iterate(pimprove) {
+    if (!plr->ai_data.control && is_wonder(pimprove)) {
       continue; /* Humans should not be advised to build wonders or palace */
     }
-    if (pcity->ai.building_want[i] > want
-        && can_build_improvement(pcity, i)) {
-      want = pcity->ai.building_want[i];
-      id = i;
+    if (pcity->ai->building_want[improvement_index(pimprove)] > want
+        && can_city_build_improvement_now(pcity, pimprove)) {
+      want = pcity->ai->building_want[improvement_index(pimprove)];
+      chosen = pimprove;
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 
   choice->want = want;
-  choice->choice = id;
-  choice->type = CT_BUILDING;
+  choice->value.building = chosen;
+
+  if (chosen) {
+    choice->type = CT_BUILDING;
 
-  if (id != B_LAST) {
     CITY_LOG(LOG_DEBUG, pcity, "wants most to build %s at %d",
-             improvement_rule_name(id),
+             improvement_rule_name(chosen),
              want);
+  } else {
+    choice->type = CT_NONE;
   }
+  choice->need_boat = FALSE;
 }
 
 /**********************************************************************
@@ -1229,42 +1290,19 @@ void ai_advisor_choose_building(struct city *pcity, struct ai_choice *choice)
   sure whether it is fully general for all possible parameters/
   combinations." --dwp
 **********************************************************************/
-bool ai_assess_military_unhappiness(struct city *pcity,
-                                    struct government *g)
+bool ai_assess_military_unhappiness(struct city *pcity)
 {
-  int free_happy;
+  int free_unhappy = get_city_bonus(pcity, EFT_MAKE_CONTENT_MIL);
   int unhap = 0;
 
   /* bail out now if happy_cost is 0 */
   if (get_player_bonus(city_owner(pcity), EFT_UNHAPPY_FACTOR) == 0) {
     return FALSE;
   }
-  free_happy = get_city_bonus(pcity, EFT_MAKE_CONTENT_MIL);
 
   unit_list_iterate(pcity->units_supported, punit) {
-    int happy_cost = utype_happy_cost(unit_type(punit), city_owner(pcity));
+    int happy_cost = city_unit_unhappiness(punit, &free_unhappy);
 
-    if (happy_cost <= 0) {
-      continue;
-    }
-
-    /* See discussion/rules in common/city.c:city_support() */
-    if (!unit_being_aggressive(punit)) {
-      if (is_field_unit(punit)) {
-	happy_cost = 1;
-      } else {
-	happy_cost = 0;
-      }
-    }
-    if (happy_cost <= 0) {
-      continue;
-    }
-
-    if (get_city_bonus(pcity, EFT_MAKE_CONTENT_MIL_PER) > 0) {
-      happy_cost--;
-    }
-    adjust_city_free_cost(&free_happy, &happy_cost);
-    
     if (happy_cost > 0) {
       unhap += happy_cost;
     }
