@@ -18,17 +18,20 @@
 #include <assert.h>
 #include <string.h>
 
+/* utility */
 #include "log.h"
 #include "mem.h"
 
+/* common */
 #include "map.h"
+#include "movement.h"
 #include "packets.h"
 #include "pf_tools.h"
 #include "unit.h"
 #include "unitlist.h"
 
-#include "civclient.h"
-#include "clinet.h"
+/* client */
+#include "client_main.h"
 #include "control.h"
 #include "mapview_g.h"
 
@@ -38,21 +41,6 @@
 #define PATH_LOG_LEVEL          LOG_DEBUG
 #define PACKET_LOG_LEVEL        LOG_DEBUG
 
-/* For each tile and each direction we store the number of lines going out 
- * of the tile in this direction.  Since each line is undirected, we only 
- * store the 4 lower-numbered directions for each tile; the 4 upper-numbered
- * directions are stored as reverses from the target tile.
- * Notes: 1. This assumes that 
- * - there are 8 directions
- * - out of every two opposite directions (like NORTH and SOUTH) one and 
- *   only one has number less than 4
- * 2. There _can_ be more than one line drawn between two tiles, because of 
- * the waypoints. */
-struct goto_tiles {
-  unsigned char drawn[4];
-};
-static struct goto_tiles *tiles = NULL;
-
 /*
  * The whole path is separated by waypoints into parts.  Each part has its
  * own starting position and requires its own map.  When the unit is unable
@@ -62,7 +50,7 @@ struct part {
   int start_moves_left, start_fuel_left;
   struct tile *start_tile, *end_tile;
   int end_moves_left, end_fuel_left;
-  int time;
+  int mp; /* scaled by SINGLE_MOVE */
   struct pf_path *path;
   struct pf_map *map;
 };
@@ -71,8 +59,8 @@ struct goto_map {
   struct unit *focus;
   struct part *parts;
   int num_parts;
-  int connect_initial;
-  int connect_speed;
+  int initial_mp;     /* scaled by SINGLE_MOVE */
+  int connect_speed;  /* scaled by ACTIVITY_FACTOR */
   struct pf_parameter template;
 };
 
@@ -94,11 +82,8 @@ struct goto_map {
   } goto_map_list_iterate_end;
 
 static struct goto_map_list *goto_maps = NULL;
+static bool goto_warned = FALSE;
 
-#define DRAWN(ptile, dir) (tiles[(ptile)->index].drawn[dir])
-
-static void increment_drawn(struct tile *src_tile, enum direction8 dir);
-static void decrement_drawn(struct tile *src_tile, enum direction8 dir);
 static void reset_last_part(struct goto_map *goto_map);
 static void remove_last_part(struct goto_map *goto_map);
 
@@ -118,7 +103,7 @@ static struct goto_map *goto_map_new(void)
   goto_map->focus = NULL;
   goto_map->parts = NULL;
   goto_map->num_parts = 0;
-  goto_map->connect_initial = 0;
+  goto_map->initial_mp = 0;
   goto_map->connect_speed = 0;
 
   return goto_map;
@@ -146,8 +131,6 @@ void init_client_goto(void)
   free_client_goto();
 
   goto_maps = goto_map_list_new();
-
-  tiles = fc_calloc(MAP_INDEX_SIZE, sizeof(*tiles));
 }
 
 /********************************************************************** 
@@ -159,15 +142,12 @@ void free_client_goto(void)
     goto_map_list_iterate(goto_maps, goto_map) {
       goto_map_free(goto_map);
     } goto_map_list_iterate_end;
-    goto_map_list_unlink_all(goto_maps);
     goto_map_list_free(goto_maps);
     goto_maps = NULL;
   }
 
-  if (NULL != tiles) {
-    free(tiles);
-    tiles = NULL;
-  }
+  goto_destination = NULL;
+  goto_warned = FALSE;
 }
 
 /**********************************************************************
@@ -193,16 +173,44 @@ static bool update_last_part(struct goto_map *goto_map,
 
   freelog(LOG_DEBUG, "update_last_part(%d,%d) old (%d,%d)-(%d,%d)",
           TILE_XY(ptile), TILE_XY(p->start_tile), TILE_XY(p->end_tile));
-  new_path = pf_get_path(p->map, ptile);
+  new_path = pf_map_get_path(p->map, ptile);
 
   if (!new_path) {
     freelog(PATH_LOG_LEVEL, "  no path found");
+
+    if (p->start_tile == ptile) {
+      /* This mean we cannot reach the start point.  It is probably,
+       * a path-finding bug, but don't make infinite recursion. */
+
+      if (!goto_warned) {
+        freelog(LOG_ERROR, "No path found to reach the start point.");
+        goto_warned = TRUE;
+      }
+
+      if (p->path) {
+        /* Remove the goto lines. */
+        for (i = 0; i < p->path->length - 1; i++) {
+          struct pf_position *a = &p->path->positions[i];
+
+          if (is_valid_dir(a->dir_to_next_pos)) {
+            mapdeco_remove_gotoline(a->tile, a->dir_to_next_pos);
+          } else {
+            assert(i < p->path->length - 1
+                   && a->tile == p->path->positions[i + 1].tile);
+          }
+        }
+        pf_path_destroy(p->path);
+        p->path = NULL;
+      }
+      return FALSE;
+    }
+
     reset_last_part(goto_map);
     return FALSE;
   }
 
   freelog(PATH_LOG_LEVEL, "  path found:");
-  pf_print_path(PATH_LOG_LEVEL, new_path);
+  pf_path_print(new_path, PATH_LOG_LEVEL);
 
   if (p->path) {
     /* We had a path drawn already.  Determine how much of it we can reuse
@@ -223,13 +231,13 @@ static bool update_last_part(struct goto_map *goto_map,
       struct pf_position *a = &p->path->positions[i];
 
       if (is_valid_dir(a->dir_to_next_pos)) {
-	decrement_drawn(a->tile, a->dir_to_next_pos);
+        mapdeco_remove_gotoline(a->tile, a->dir_to_next_pos);
       } else {
 	assert(i < p->path->length - 1
 	       && a->tile == p->path->positions[i + 1].tile);
       }
     }
-    pf_destroy_path(p->path);
+    pf_path_destroy(p->path);
     p->path = NULL;
 
     old_tile = p->end_tile;
@@ -240,7 +248,7 @@ static bool update_last_part(struct goto_map *goto_map,
     struct pf_position *a = &new_path->positions[i];
 
     if (is_valid_dir(a->dir_to_next_pos)) {
-      increment_drawn(a->tile, a->dir_to_next_pos);
+      mapdeco_add_gotoline(a->tile, a->dir_to_next_pos);
     } else {
       assert(i < new_path->length - 1
 	     && a->tile == new_path->positions[i + 1].tile);
@@ -248,22 +256,16 @@ static bool update_last_part(struct goto_map *goto_map,
   }
   p->path = new_path;
   p->end_tile = ptile;
-  p->end_moves_left = pf_last_position(p->path)->moves_left;
-  p->end_fuel_left = pf_last_position(p->path)->fuel_left;
+  p->end_moves_left = pf_path_get_last_position(p->path)->moves_left;
+  p->end_fuel_left = pf_path_get_last_position(p->path)->fuel_left;
 
-  if (hover_state == HOVER_CONNECT) {
-    int move_rate = goto_map->template.move_rate;
-    int moves = pf_last_position(p->path)->total_MC;
-
-    p->time = moves / move_rate;
-    if (goto_map->connect_initial > 0) {
-      p->time += goto_map->connect_initial;
-    }
-    freelog(PATH_LOG_LEVEL, "To (%d,%d) MC: %d, connect_initial: %d",
-	    TILE_XY(ptile), moves, goto_map->connect_initial);
-  } else {
-    p->time = pf_last_position(p->path)->turn;
+  p->mp = pf_path_get_last_position(p->path)->total_MC;
+  if (goto_map->num_parts == 1) {
+    p->mp += goto_map->initial_mp;
   }
+  freelog(PATH_LOG_LEVEL, "To (%d,%d) part %d: total_MC: %d, mp: %d",
+          TILE_XY(ptile), goto_map->num_parts,
+          pf_path_get_last_position(p->path)->total_MC, p->mp);
 
   /* Refresh tiles so turn information is shown. */
   refresh_tile_mapcanvas(old_tile, FALSE, FALSE);
@@ -322,9 +324,9 @@ static void add_part(struct goto_map *goto_map)
   }
   p->path = NULL;
   p->end_tile = p->start_tile;
-  p->time = 0;
+  p->mp = 0;
   parameter.start_tile = p->start_tile;
-  p->map = pf_create_map(&parameter);
+  p->map = pf_map_new(&parameter);
 }
 
 /********************************************************************** 
@@ -339,9 +341,9 @@ static void remove_last_part(struct goto_map *goto_map)
   reset_last_part(goto_map);
   if (p->path) {
     /* We do not always have a path */
-    pf_destroy_path(p->path);
+    pf_path_destroy(p->path);
   }
-  pf_destroy_map(p->map);
+  pf_map_destroy(p->map);
   goto_map->num_parts--;
 }
 
@@ -415,7 +417,7 @@ bool goto_pop_waypoint(void)
   all shortest paths).
 ***********************************************************************/
 static int get_EC(const struct tile *ptile, enum known_type known,
-		  struct pf_parameter *param)
+                  const struct pf_parameter *param)
 {
   return 1;
 }
@@ -425,8 +427,8 @@ static int get_EC(const struct tile *ptile, enum known_type known,
   don't plan our route through enemy city/tile.
 ***********************************************************************/
 static enum tile_behavior get_TB_aggr(const struct tile *ptile,
-				      enum known_type known,
-                                      struct pf_parameter *param)
+                                      enum known_type known,
+                                      const struct pf_parameter *param)
 {
   if (known == TILE_UNKNOWN) {
     if (!goto_into_unknown) {
@@ -445,8 +447,8 @@ static enum tile_behavior get_TB_aggr(const struct tile *ptile,
   don't attack enemy units but enter enemy cities.
 ***********************************************************************/
 static enum tile_behavior get_TB_caravan(const struct tile *ptile,
-					 enum known_type known,
-					 struct pf_parameter *param)
+                                         enum known_type known,
+                                         const struct pf_parameter *param)
 {
   if (known == TILE_UNKNOWN) {
     if (!goto_into_unknown) {
@@ -472,7 +474,7 @@ static enum tile_behavior get_TB_caravan(const struct tile *ptile,
 static int get_activity_time(const struct tile *ptile,
 			     struct player *pplayer)
 {
-  struct terrain *pterrain = ptile->terrain;
+  struct terrain *pterrain = tile_terrain(ptile);
   int activity_mc = 0;
 
   assert(hover_state == HOVER_CONNECT);
@@ -542,14 +544,13 @@ static bool is_non_allied_city_adjacent(struct player *pplayer,
   (2) (the tie-breaker) time to build the path (travel plus activity time).
   In rail-connect the priorities are reversed.
 
-  param->data should contain the result of
-  get_activity_rate(punit) / ACTIVITY_FACTOR.
+  param->data should contain the result of get_activity_rate(punit).
 ****************************************************************************/
 static int get_connect_road(const struct tile *src_tile, enum direction8 dir,
-			    const struct tile *dest_tile,
-			    int src_cost, int src_extra,
-			    int *dest_cost, int *dest_extra,
-			    struct pf_parameter *param)
+                            const struct tile *dest_tile,
+                            int src_cost, int src_extra,
+                            int *dest_cost, int *dest_extra,
+                            const struct pf_parameter *param)
 {
   int activity_time, move_cost, moves_left;
   int total_cost, total_extra;
@@ -592,18 +593,21 @@ static int get_connect_road(const struct tile *src_tile, enum direction8 dir,
   total_cost = src_cost;
   moves_left = param->move_rate - (src_cost % param->move_rate);
   if (moves_left < move_cost) {
-    /* Emulating TM_WORST_TIME */
     total_cost += moves_left;
+    if (param->turn_mode == TM_WORST_TIME) {
+      total_cost += move_cost;
+    } /* else emulate TM_BEST_TIME */
+  } else {
+    total_cost += move_cost;
   }
-  total_cost += move_cost;
 
   /* Now need to include the activity cost.  If we have moves left, they
    * will count as a full turn towards the activity time */
   moves_left = param->move_rate - (total_cost % param->move_rate);
   if (activity_time > 0) {
     int speed = *(int *)param->data;
-    
-    activity_time /= speed;
+    activity_time = ((activity_time * ACTIVITY_FACTOR)
+                     + (speed - 1)) / speed;
     activity_time--;
     total_cost += moves_left;
   }
@@ -624,7 +628,7 @@ static int get_connect_road(const struct tile *src_tile, enum direction8 dir,
     } else {
       /* assert(connect_activity == ACTIVITY_RAILROAD) */
       if (total_cost > *dest_cost 
-	  || (total_cost == *dest_cost && total_cost >= *dest_cost)) {
+	  || (total_cost == *dest_cost && total_extra >= *dest_extra)) {
 	return -1;
       }
     }
@@ -646,11 +650,11 @@ static int get_connect_road(const struct tile *src_tile, enum direction8 dir,
   param->data should contain the result of get_activity_rate(punit) / 10.
 ****************************************************************************/
 static int get_connect_irrig(const struct tile *src_tile,
-			     enum direction8 dir,
-			     const struct tile *dest_tile,
+                             enum direction8 dir,
+                             const struct tile *dest_tile,
                              int src_cost, int src_extra,
                              int *dest_cost, int *dest_extra,
-                             struct pf_parameter *param)
+                             const struct pf_parameter *param)
 {
   int activity_time, move_cost, moves_left, total_cost;
 
@@ -683,18 +687,21 @@ static int get_connect_irrig(const struct tile *src_tile,
   total_cost = src_cost;
   moves_left = param->move_rate - (src_cost % param->move_rate);
   if (moves_left < move_cost) {
-    /* Emulating TM_WORST_TIME */
     total_cost += moves_left;
+    if (param->turn_mode == TM_WORST_TIME) {
+      total_cost += move_cost;
+    } /* else emulate TM_BEST_TIME */
+  } else {
+    total_cost += move_cost;
   }
-  total_cost += move_cost;
 
   /* Now need to include the activity cost.  If we have moves left, they
    * will count as a full turn towards the activity time */
   moves_left = param->move_rate - (total_cost % param->move_rate);
   if (activity_time > 0) {
     int speed = *(int *)param->data;
-    
-    activity_time /= speed;
+    activity_time = ((activity_time * ACTIVITY_FACTOR)
+                     + (speed - 1)) / speed;
     activity_time--;
     total_cost += moves_left;
   }
@@ -716,9 +723,10 @@ static int get_connect_irrig(const struct tile *src_tile,
   PF callback to prohibit going into the unknown (conditionally).  Also
   makes sure we don't plan to attack anyone.
 ***********************************************************************/
-static enum tile_behavior no_fights_or_unknown_goto(const struct tile *ptile,
-						    enum known_type known,
-						    struct pf_parameter *p)
+static enum tile_behavior
+no_fights_or_unknown_goto(const struct tile *ptile,
+                          enum known_type known,
+                          const struct pf_parameter *p)
 {
   if (known == TILE_UNKNOWN && goto_into_unknown) {
     /* Special case allowing goto into the unknown. */
@@ -730,10 +738,12 @@ static enum tile_behavior no_fights_or_unknown_goto(const struct tile *ptile,
 
 /********************************************************************** 
   Fill the PF parameter with the correct client-goto values.
+  The storage behind "connect_speed" must remain valid for the lifetime
+  of the pf_map.
 ***********************************************************************/
 static void fill_client_goto_parameter(struct unit *punit,
 				       struct pf_parameter *parameter,
-				       int *connect_initial,
+				       int *initial_mp,
 				       int *connect_speed)
 {
   pft_fill_unit_parameter(parameter, punit);
@@ -749,29 +759,36 @@ static void fill_client_goto_parameter(struct unit *punit,
     } else {
       parameter->get_costs = get_connect_road;
     }
-    parameter->is_pos_dangerous = NULL;
+    parameter->get_moves_left_req = NULL;
 
-    *connect_speed = get_activity_rate(punit) / ACTIVITY_FACTOR;
+    *connect_speed = get_activity_rate(punit);
     parameter->data = connect_speed;
 
     /* Take into account the activity time at the origin */
-    *connect_initial = get_activity_time(punit->tile, unit_owner(punit))
-                     / *connect_speed;
-    if (*connect_initial > 0) {
-      parameter->moves_left_initially = 0;
-      if (punit->moves_left == 0) {
-	*connect_initial += 1;
+    {
+      int activity_initial = get_activity_time(punit->tile, unit_owner(punit));
+      if (activity_initial > 0) {
+        /* First action is activity */
+        parameter->moves_left_initially = 0;
+        /* Number of turns, rounding up */
+        *initial_mp = ((activity_initial * ACTIVITY_FACTOR)
+                       + (*connect_speed - 1)) / *connect_speed;
+        if (punit->moves_left == 0) {
+          *initial_mp += 1;
+        }
+        *initial_mp *= parameter->move_rate;
+      } else {
+        /* First action is a move */
+        /* moves_left_initially = punit->moves_left (default) */
+        *initial_mp = parameter->move_rate - parameter->moves_left_initially;
       }
-    } else {
-      /* otherwise moves_left_initially = punit->moves_left (default) */
-      assert(*connect_initial == 0);
     }
     break;
   case HOVER_NUKE:
-    parameter->is_pos_dangerous = NULL; /* nuclear safety? pwah! */
+    parameter->get_moves_left_req = NULL; /* nuclear safety? pwah! */
     /* FALLTHRU */
   default:
-    *connect_initial = 0;
+    *initial_mp = parameter->move_rate - parameter->moves_left_initially;
     break;
   };
 
@@ -813,13 +830,14 @@ void enter_goto_state(struct unit_list *punits)
     goto_map->focus = punit;
 
     fill_client_goto_parameter(punit, &goto_map->template,
-                               &goto_map->connect_initial,
+                               &goto_map->initial_mp,
                                &goto_map->connect_speed);
 
     add_part(goto_map);
 
     goto_map_list_append(goto_maps, goto_map);
   } unit_list_iterate_end;
+  goto_warned = FALSE;
 }
 
 /********************************************************************** 
@@ -834,9 +852,10 @@ void exit_goto_state(void)
   goto_map_list_iterate(goto_maps, goto_map) {
     goto_map_free(goto_map);
   } goto_map_list_iterate_end;
-  goto_map_list_unlink_all(goto_maps);
+  goto_map_list_clear(goto_maps);
 
   goto_destination = NULL;
+  goto_warned = FALSE;
 }
 
 /********************************************************************** 
@@ -889,19 +908,25 @@ bool goto_get_turns(int *min, int *max)
   }
 
   goto_map_list_iterate(goto_maps, goto_map) {
-    int i, time = 0;
+    int i, mp = 0, turns;
 
     for (i = 0; i < goto_map->num_parts; i++) {
-      time += goto_map->parts[i].time;
+      mp += goto_map->parts[i].mp;
     }
 
+    mp = MAX(0, mp);
+    /* Round down -- if we can get there this turn with MP left, report 0,
+     * if we get there with 0 MP, report 1 */
+    turns = mp / goto_map->template.move_rate;
+
     if (min) {
-      *min = MIN(*min, time);
+      *min = MIN(*min, turns);
     }
     if (max) {
-      *max = MAX(*max, time);
+      *max = MAX(*max, turns);
     }
   } goto_map_list_iterate_end;
+
   return TRUE;
 }
 
@@ -952,7 +977,7 @@ void request_orders_cleared(struct unit *punit)
   p.length = 0;
   p.dest_x = punit->tile->x;
   p.dest_y = punit->tile->y;
-  send_packet_unit_orders(&aconnection, &p);
+  send_packet_unit_orders(&client.conn, &p);
 }
 
 /**************************************************************************
@@ -1014,7 +1039,7 @@ static void send_path_orders(struct unit *punit, struct pf_path *path,
   p.dest_x = old_tile->x;
   p.dest_y = old_tile->y;
 
-  send_packet_unit_orders(&aconnection, &p);
+  send_packet_unit_orders(&client.conn, &p);
 }
 
 /**************************************************************************
@@ -1034,22 +1059,17 @@ bool send_goto_tile(struct unit *punit, struct tile *ptile)
 {
   int dummy1, dummy2;
   struct pf_parameter parameter;
-  struct pf_map *map;
-  struct pf_path *path = NULL;
+  struct pf_map *pfm;
+  struct pf_path *path;
 
   fill_client_goto_parameter(punit, &parameter, &dummy1, &dummy2);
-  map = pf_create_map(&parameter);
+  pfm = pf_map_new(&parameter);
+  path = pf_map_get_path(pfm, ptile);
+  pf_map_destroy(pfm);
 
-  pf_iterator(map, pos) {
-    if (pos.tile == ptile) {
-      path = pf_next_get_path(map);
-      break;
-    }
-  } pf_iterator_end;
-
-  pf_destroy_map(map);
   if (path) {
     send_goto_path(punit, path, NULL);
+    pf_path_destroy(path);
     return TRUE;
   } else {
     return FALSE;
@@ -1065,7 +1085,7 @@ void send_patrol_route(void)
   assert(goto_is_active());
   goto_map_unit_iterate(goto_maps, goto_map, punit) {
     int i;
-    struct pf_map *map;
+    struct pf_map *pfm;
     struct pf_path *return_path;
     struct pf_path *path = NULL;
     struct pf_parameter parameter = goto_map->template;
@@ -1079,10 +1099,12 @@ void send_patrol_route(void)
     parameter.start_tile = last_part->end_tile;
     parameter.moves_left_initially = last_part->end_moves_left;
     parameter.fuel_left_initially = last_part->end_fuel_left;
-    map = pf_create_map(&parameter);
-    return_path = pf_get_path(map, goto_map->parts[0].start_tile);
+    pfm = pf_map_new(&parameter);
+    return_path = pf_map_get_path(pfm, goto_map->parts[0].start_tile);
     if (!return_path) {
-      die("No return path found!");
+      /* Cannot make a path */
+      pf_map_destroy(pfm);
+      continue;
     }
 
     for (i = 0; i < goto_map->num_parts; i++) {
@@ -1090,12 +1112,12 @@ void send_patrol_route(void)
     }
     path = pft_concat(path, return_path);
 
-    pf_destroy_map(map);
-    pf_destroy_path(return_path);
+    pf_map_destroy(pfm);
+    pf_path_destroy(return_path);
 
     send_path_orders(punit, path, TRUE, TRUE, NULL);
 
-    pf_destroy_path(path);
+    pf_path_destroy(path);
   } goto_map_unit_iterate_end;
 }
 
@@ -1181,7 +1203,7 @@ void send_connect_route(enum unit_activity activity)
     p.dest_x = old_tile->x;
     p.dest_y = old_tile->y;
 
-    send_packet_unit_orders(&aconnection, &p);
+    send_packet_unit_orders(&client.conn, &p);
   } goto_map_unit_iterate_end;
 }
 
@@ -1207,6 +1229,7 @@ void send_goto_route(void)
       path = pft_concat(path, goto_map->parts[i].path);
     }
 
+    clear_unit_orders(punit);
     if (goto_last_order == ORDER_LAST) {
       send_goto_path(punit, path, NULL);
     } else {
@@ -1216,93 +1239,17 @@ void send_goto_route(void)
       order.dir = -1;
       order.activity = ACTIVITY_LAST;
 
+      /* ORDER_MOVE would require real direction,
+       * ORDER_ACTIVITY would require real activity */
+      assert(goto_last_order != ORDER_MOVE
+	     && goto_last_order != ORDER_ACTIVITY);
+
       send_goto_path(punit, path, &order);
     }
-    pf_destroy_path(path);
+    pf_path_destroy(path);
   } goto_map_unit_iterate_end;
 }
 
-/* ================= drawn functions ============================ */
-
-/********************************************************************** 
-  Every line segment has 2 ends; we only keep track of it at one end
-  (the one from which dir i <4). This function returns pointer to the
-  correct char. This function is for internal use only. Use get_drawn
-  when in doubt.
-***********************************************************************/
-static unsigned char *get_drawn_char(struct tile *ptile, enum direction8 dir)
-{
-  struct tile *tile1;
-
-  tile1 = mapstep(ptile, dir);
-
-  if (dir >= 4) {
-    ptile = tile1;
-    dir = DIR_REVERSE(dir);
-  }
-
-  return &DRAWN(ptile, dir);
-}
-
-/**************************************************************************
-  Increments the number of segments at the location, and draws the
-  segment if necessary.
-**************************************************************************/
-static void increment_drawn(struct tile *src_tile, enum direction8 dir)
-{
-  unsigned char *count = get_drawn_char(src_tile, dir);
-
-  freelog(LOG_DEBUG, "increment_drawn(src=(%d,%d) dir=%s)",
-          TILE_XY(src_tile), dir_get_name(dir));
-
-  if (*count < 255) {
-    (*count)++;
-  } else {
-    /* don't overflow unsigned char. */
-    assert(*count < 255);
-  }
-
-  if (*count == 1) {
-    draw_segment(src_tile, dir);
-  }
-}
-
-/**************************************************************************
-  Decrements the number of segments at the location, and clears the
-  segment if necessary.
-**************************************************************************/
-static void decrement_drawn(struct tile *src_tile, enum direction8 dir)
-{
-  unsigned char *count = get_drawn_char(src_tile, dir);
-
-  freelog(LOG_DEBUG, "decrement_drawn(src=(%d,%d) dir=%s)",
-          TILE_XY(src_tile), dir_get_name(dir));
-
-  if (*count > 0) {
-    (*count)--;
-  } else {
-    /* don't underflow unsigned char. */
-    assert(*count > 0);
-  }
-
-  if (*count == 0) {
-    undraw_segment(src_tile, dir);
-  }
-}
-
-/****************************************************************************
-  Return TRUE if there is a line drawn from (x,y) in the given direction.
-  This is used by mapview to determine whether to draw a goto line.
-****************************************************************************/
-bool is_drawn_line(struct tile *ptile, int dir)
-{
-  if (!mapstep(ptile, dir)) {
-    return 0;
-  }
-
-  return (*get_drawn_char(ptile, dir) != 0);
-}
-
 /**************************************************************************
   Find the path to the nearest (fastest to reach) allied city for the
   unit, or NULL if none is reachable.
@@ -1311,33 +1258,25 @@ struct pf_path *path_to_nearest_allied_city(struct unit *punit)
 {
   int dummy1, dummy2;
   struct pf_parameter parameter;
-  struct pf_map *map;
+  struct pf_map *pfm;
   struct pf_path *path = NULL;
-  struct city *pcity = is_allied_city_tile(punit->tile, unit_owner(punit));
 
-  if (pcity) {
+  if (is_allied_city_tile(punit->tile, unit_owner(punit))) {
     /* We're already on a city - don't go anywhere. */
     return NULL;
   }
 
   fill_client_goto_parameter(punit, &parameter, &dummy1, &dummy2);
-  map = pf_create_map(&parameter);
+  pfm = pf_map_new(&parameter);
 
-  while (pf_next(map)) {
-    struct pf_position pos;
-
-    pf_next_get_position(map, &pos);
-
-    if ((pcity = is_allied_city_tile(pos.tile, unit_owner(punit)))) {
+  pf_map_iterate_tiles(pfm, ptile, FALSE) {
+    if (is_allied_city_tile(ptile, unit_owner(punit))) {
+      path = pf_map_get_path(pfm, ptile);
       break;
     }
-  }
+  } pf_map_iterate_tiles_end;
 
-  if (pcity) {
-    path = pf_next_get_path(map);
-  }
-
-  pf_destroy_map(map);
+  pf_map_destroy(pfm);
 
   return path;
 }
