@@ -23,32 +23,40 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "city.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "packets.h"
+#include "log.h"
 #include "shared.h"
 #include "support.h"
+
+/* common */
+#include "city.h"
+#include "game.h"
+#include "packets.h"
 #include "unit.h"
 
-#include "chatline.h"
+/* agents */
+#include "cma_fec.h"
+
+/* client */
 #include "citydlg_common.h"
 #include "cityrepdata.h"
-#include "civclient.h"
-#include "clinet.h"
+#include "client_main.h"
+#include "climisc.h"
+#include "global_worklist.h"
+#include "mapview_common.h"
+#include "options.h"
+
+/* gui-gtk-2.0 */
+#include "chatline.h"
+#include "citydlg.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "mapctrl.h"    /* is_city_hilited() */
-#include "mapview_common.h"
 #include "optiondlg.h"
-#include "options.h"
 #include "repodlgs.h"
-#include "climisc.h"
 
-#include "cma_fec.h"
-
-#include "citydlg.h"
 #include "cityrep.h"
 
 #define NEG_VAL(x)  ((x)<0 ? (x) : (-x))
@@ -57,8 +65,14 @@
 #define CMA_NONE	(10000)
 #define CMA_CUSTOM	(10001)
 
+struct sell_data {
+  int count;                    /* Number of cities. */
+  int gold;                     /* Amount of gold. */
+  struct impr_type *target;     /* The target for selling. */
+};
+
 enum city_operation_type {
-  CO_CHANGE, CO_LAST, CO_NEXT, CO_FIRST, CO_NEXT_TO_LAST, CO_NONE
+  CO_CHANGE, CO_LAST, CO_NEXT, CO_FIRST, CO_NEXT_TO_LAST, CO_SELL, CO_NONE
 };
 
 /******************************************************************/
@@ -81,6 +95,7 @@ static void create_last_menu(GtkWidget *item);
 static void create_first_menu(GtkWidget *item);
 static void create_next_menu(GtkWidget *item);
 static void create_next_to_last_menu(GtkWidget *item);
+static void create_sell_menu(GtkWidget *item);
 
 static struct gui_dialog *city_dialog_shell = NULL;
 
@@ -99,14 +114,16 @@ static void popup_first_menu(GtkMenuShell *menu, gpointer data);
 static void popup_next_menu(GtkMenuShell *menu, gpointer data);
 static void popup_next_to_last_menu(GtkMenuShell *menu, gpointer data);
 
+static void recreate_sell_menu(void);
+
 static GtkWidget *city_center_command;
 static GtkWidget *city_popup_command;
 static GtkWidget *city_buy_command;
 static GtkWidget *city_production_command;
 static GtkWidget *city_governor_command;
+static GtkWidget *city_sell_command;
 static GtkWidget *city_total_buy_cost_label;
 
-
 static GtkWidget *change_improvements_item;
 static GtkWidget *change_units_item;
 static GtkWidget *change_wonders_item;
@@ -211,7 +228,7 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 					     int size)
 {
   GtkWidget *menu;
-  struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+  struct universal targets[MAX_NUM_PRODUCTION_TARGETS];
   struct item items[MAX_NUM_PRODUCTION_TARGETS];
   int i, item, targets_used;
   char *row[4];
@@ -275,7 +292,7 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
   }
   
   for (item = 0; item < targets_used; item++) {
-    struct city_production target = items[item].item;
+    struct universal target = items[item].item;
     GtkWidget *menu_item, *hbox, *label;
     char txt[256];
 
@@ -290,6 +307,10 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 	continue;
       }
 
+      if (city_operation == CO_SELL && i != 0) {
+        continue;
+      }
+
       my_snprintf(txt, ARRAY_SIZE(txt), "<span %s>%s</span>",
 		  markup[i], row[i]);
 
@@ -326,7 +347,7 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 static void impr_or_unit_iterate(GtkTreeModel *model, GtkTreePath *path,
 				 GtkTreeIter *it, gpointer data)
 {
-  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
+  struct universal target = cid_decode(GPOINTER_TO_INT(data));
   gint id;
 
   gtk_tree_model_get(model, it, 1, &id, -1);
@@ -345,7 +366,7 @@ static void worklist_last_impr_or_unit_iterate(GtkTreeModel *model,
 						 GtkTreeIter *it, 
 						 gpointer data)
 {
-  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
+  struct universal target = cid_decode(GPOINTER_TO_INT(data));
   gint id;
   struct city *pcity;  
 
@@ -369,7 +390,7 @@ static void worklist_first_impr_or_unit_iterate(GtkTreeModel *model,
 						 GtkTreeIter *it, 
 						 gpointer data)
 {
-  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
+  struct universal target = cid_decode(GPOINTER_TO_INT(data));
   gint id;
   struct city *pcity;  
 
@@ -393,7 +414,7 @@ static void worklist_next_impr_or_unit_iterate(GtkTreeModel *model,
 {
   struct city *pcity;
   gint id;
-  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
+  struct universal target = cid_decode(GPOINTER_TO_INT(data));
 
   gtk_tree_model_get(model, it, 1, &id, -1);
   pcity = game_find_city_by_number(id);
@@ -412,7 +433,7 @@ static void worklist_next_to_last_impr_or_unit_iterate(GtkTreeModel *model,
                                                        GtkTreeIter *it,
                                                        gpointer data)
 {
-  struct city_production target;
+  struct universal target;
   struct city *pcity;
   gint id, n;
 
@@ -428,12 +449,32 @@ static void worklist_next_to_last_impr_or_unit_iterate(GtkTreeModel *model,
   city_queue_insert(pcity, n, target);
 }
 
+/****************************************************************
+  Iterate the cities going to sell.
+*****************************************************************/
+static void sell_impr_iterate(GtkTreeModel *model, GtkTreePath *path,
+                              GtkTreeIter *it, gpointer data)
+{
+  struct city *pcity;
+  gint id;
+  struct sell_data *sd = (struct sell_data *) data; 
+
+  gtk_tree_model_get(model, it, 1, &id, -1);
+  pcity = game_find_city_by_number(id);
+
+  if (pcity && !pcity->did_sell && city_has_building(pcity, sd->target)) {
+    sd->count++;
+    sd->gold += impr_sell_gold(sd->target);
+    city_sell_improvement(pcity, improvement_number(sd->target));
+  }
+}
+
 /****************************************************************
 ...
 *****************************************************************/
 static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
 {
-  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
+  struct universal target = cid_decode(GPOINTER_TO_INT(data));
   GObject *parent = G_OBJECT(w->parent);
   TestCityFunc test_func = g_object_get_data(parent, "freeciv_test_func");
   enum city_operation_type city_operation = 
@@ -458,7 +499,7 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
     }
   } else {
     GtkTreeSelectionForeachFunc foreach_func;
-    connection_do_buffer(&aconnection);
+    connection_do_buffer(&client.conn);
     switch (city_operation) {
     case CO_LAST:
       gtk_tree_selection_selected_foreach(city_selection,
@@ -485,10 +526,49 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
       gtk_tree_selection_selected_foreach(city_selection, foreach_func,
                                           GINT_TO_POINTER(cid_encode(target)));
       break;
-    default:
-      assert(FALSE); /* should never get here. */
+    case CO_SELL:
+      assert(target.kind == VUT_IMPROVEMENT);
+      {
+        struct impr_type *building = target.value.building;
+        struct sell_data sd = { 0, 0, building };
+        GtkWidget *w;
+        gint res;
+        char buf[128];
+        const char *imprname = improvement_name_translation(building);
+
+        /* Ask confirmation */
+        my_snprintf(buf, sizeof(buf),
+                    _("Are you sure you want to sell those %s?"), imprname);
+        w = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+                                   GTK_MESSAGE_QUESTION,
+                                   GTK_BUTTONS_YES_NO, "%s", buf);
+        res = gtk_dialog_run(GTK_DIALOG(w));    /* Synchron. */
+        gtk_widget_destroy(w);
+        if (res == GTK_RESPONSE_NO) {
+          break;
+        }
+
+        gtk_tree_selection_selected_foreach(city_selection,
+                                            sell_impr_iterate, &sd);
+        if (sd.count > 0) {
+          my_snprintf(buf, sizeof(buf), _("Sold %d %s for %d gold."),
+                      sd.count, imprname, sd.gold);
+        } else {
+          my_snprintf(buf, sizeof(buf), _("No %s could be sold."),
+                      imprname);
+        }
+        w = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+                                   GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+                                   "%s", buf);
+        g_signal_connect(w, "response",
+                         G_CALLBACK(gtk_widget_destroy), NULL);
+        gtk_window_present(GTK_WINDOW(w));      /* Asynchron. */
+      }
+      break;
+    case CO_NONE:
+      break;
     }
-    connection_do_unbuffer(&aconnection);
+    connection_do_unbuffer(&client.conn);
   }
 }
 
@@ -600,6 +680,7 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), w);
     g_signal_connect(w, "activate", G_CALLBACK(select_cma_callback),
 		     GINT_TO_POINTER(CMA_NONE));
+    assert(GPOINTER_TO_INT(GINT_TO_POINTER(CMA_NONE)) == CMA_NONE);
 
     for (i = 0; i < cmafec_preset_num(); i++) {
       w = gtk_menu_item_new_with_label(cmafec_preset_get_descr(i));
@@ -613,7 +694,7 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
     int found;
 
     found = 0;
-    city_list_iterate(game.player_ptr->cities, pcity) {
+    city_list_iterate(client.conn.playing->cities, pcity) {
       if (!cma_is_city_under_agent(pcity, NULL)) {
 	found = 1;
 	break;
@@ -632,7 +713,7 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
      * take a lonnggg time.
      */
     found = 0;
-    city_list_iterate(game.player_ptr->cities, pcity) {
+    city_list_iterate(client.conn.playing->cities, pcity) {
       if (cma_is_city_under_agent(pcity, &parameter) &&
 	  cmafec_preset_get_index_of_parameter(&parameter) == -1) {
 	found = 1;
@@ -652,7 +733,7 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
     /* only fill in presets that are being used. */
     for (i = 0; i < cmafec_preset_num(); i++) {
       found = 0;
-      city_list_iterate(game.player_ptr->cities, pcity) {
+      city_list_iterate(client.conn.playing->cities, pcity) {
 	if (cma_is_city_under_agent(pcity, &parameter) &&
 	    cm_are_parameter_equal(&parameter,
 				   cmafec_preset_get_parameter(i))) {
@@ -683,12 +764,11 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
 static void append_worklist_foreach(GtkTreeModel *model, GtkTreePath *path,
                                     GtkTreeIter *it, gpointer data)
 {
-  struct worklist *pwl;
+  const struct worklist *pwl;
   struct city *pcity;
 
   pwl = data;
   g_return_if_fail(pwl != NULL);
-  g_return_if_fail(pwl->is_valid);
 
   gtk_tree_model_get(model, it, 0, &pcity, -1);
   if (!pcity || !game_find_city_by_number(pcity->id)) {
@@ -704,9 +784,19 @@ static void append_worklist_foreach(GtkTreeModel *model, GtkTreePath *path,
 **************************************************************************/
 static void append_worklist_callback(GtkMenuItem *menuitem, gpointer data)
 {
+  struct global_worklist *pgwl =
+    global_worklist_by_id(GPOINTER_TO_INT(data));
+
   g_return_if_fail(city_selection != NULL);
+
+  if (!pgwl) {
+    /* Maybe removed by an other way, not an error. */
+    return;
+  }
+
   gtk_tree_selection_selected_foreach(city_selection,
-                                      append_worklist_foreach, data);
+                                      append_worklist_foreach,
+                                      (gpointer) global_worklist_get(pgwl));
 }
 
 /**************************************************************************
@@ -716,12 +806,11 @@ static void append_worklist_callback(GtkMenuItem *menuitem, gpointer data)
 static void set_worklist_foreach(GtkTreeModel *model, GtkTreePath *path,
                                  GtkTreeIter *it, gpointer data)
 {
-  struct worklist *pwl;
+  const struct worklist *pwl;
   struct city *pcity;
 
   pwl = data;
   g_return_if_fail(pwl != NULL);
-  g_return_if_fail(pwl->is_valid);
 
   gtk_tree_model_get(model, it, 0, &pcity, -1);
   if (!pcity || !game_find_city_by_number(pcity->id)) {
@@ -736,9 +825,21 @@ static void set_worklist_foreach(GtkTreeModel *model, GtkTreePath *path,
 **************************************************************************/
 static void set_worklist_callback(GtkMenuItem *menuitem, gpointer data)
 {
+  struct global_worklist *pgwl =
+    global_worklist_by_id(GPOINTER_TO_INT(data));
+
   g_return_if_fail(city_selection != NULL);
   gtk_tree_selection_selected_foreach(city_selection, set_worklist_foreach,
-                                      data);
+                                      (gpointer) global_worklist_get(pgwl));
+
+  if (!pgwl) {
+    /* Maybe removed by an other way, not an error. */
+    return;
+  }
+
+  gtk_tree_selection_selected_foreach(city_selection,
+                                      set_worklist_foreach,
+                                      (gpointer) global_worklist_get(pgwl));
 }
 
 /**************************************************************************
@@ -747,12 +848,10 @@ static void set_worklist_callback(GtkMenuItem *menuitem, gpointer data)
 **************************************************************************/
 static void production_menu_shown(GtkWidget *widget, gpointer data)
 {
-  struct worklist *worklists;
-  struct player *pplayer;
   GtkWidget *menu, *item;
   GtkMenuItem *parent_item;
   GCallback callback;
-  int i, count = 0;
+  int count = 0;
 
   parent_item = data;
   g_return_if_fail(parent_item != NULL);
@@ -771,24 +870,20 @@ static void production_menu_shown(GtkWidget *widget, gpointer data)
     gtk_menu_item_set_submenu(parent_item, menu);
   }
 
-  pplayer = game.player_ptr;
-  if (!pplayer || !can_client_issue_orders()) {
+  if (!can_client_issue_orders()) {
     return;
   }
 
   gtk_container_forall(GTK_CONTAINER(menu),
                        (GtkCallback) gtk_widget_destroy, NULL);
 
-  worklists = pplayer->worklists;
-  for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
-    if (!worklists[i].is_valid) {
-      continue;
-    }
-    item = gtk_menu_item_new_with_label(worklists[i].name);
+  global_worklists_iterate(pgwl) {
+    item = gtk_menu_item_new_with_label(global_worklist_name(pgwl));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    g_signal_connect(item, "activate", callback, &worklists[i]);
+    g_signal_connect(item, "activate", callback,
+                     GINT_TO_POINTER(global_worklist_id(pgwl)));
     count++;
-  }
+  } global_worklists_iterate_end;
 
   if (count == 0) {
     item = gtk_menu_item_new_with_label(_("(no worklists defined)"));
@@ -918,6 +1013,11 @@ static GtkWidget *create_city_report_menubar(void)
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
   append_cma_to_menu_item(GTK_MENU_ITEM(item), TRUE);
 
+  item = gtk_menu_item_new_with_mnemonic(_("S_ell"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  city_sell_command = item;
+  create_sell_menu(item);
+
   item = gtk_menu_item_new_with_mnemonic(_("_Select"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
   create_select_menu(item);
@@ -990,6 +1090,14 @@ static gint cityrep_sort_func(GtkTreeModel *model,
   return cityrepfield_compare(buf1, buf2);
 }
 
+/****************************************************************
+  Clean up when city report tree view is destroyed
+*****************************************************************/
+static void city_view_destroyed(GtkObject *object, gpointer column_tooltips)
+{
+  gtk_object_destroy(column_tooltips);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1000,6 +1108,7 @@ static void create_city_report_dialog(bool make_modal)
   struct city_report_spec *spec;
 
   GtkWidget *w, *sw, *menubar;
+  GtkTooltips *column_tooltips = gtk_tooltips_new();
   int i;
 
   gui_dialog_new(&city_dialog_shell, GTK_NOTEBOOK(top_notebook), NULL);
@@ -1028,8 +1137,10 @@ static void create_city_report_dialog(bool make_modal)
   city_buy_command = w;
 
   city_total_buy_cost_label = gtk_label_new(NULL);
+  gtk_label_set_ellipsize(GTK_LABEL(city_total_buy_cost_label),
+                          PANGO_ELLIPSIZE_START);
   gtk_box_pack_end(GTK_BOX(city_dialog_shell->action_area),
-                   city_total_buy_cost_label, FALSE, FALSE, 0);
+                   city_total_buy_cost_label, TRUE, TRUE, 0);
 
   gui_dialog_add_button(city_dialog_shell,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
@@ -1049,13 +1160,17 @@ static void create_city_report_dialog(bool make_modal)
 
   city_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(city_model));
   g_object_unref(city_model);
-  gtk_widget_set_name(city_view, "small font");
+  gtk_widget_set_name(city_view, "small_font");
   g_signal_connect(city_view, "row_activated",
 		   G_CALLBACK(city_activated_callback), NULL);
   city_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(city_view));
   gtk_tree_selection_set_mode(city_selection, GTK_SELECTION_MULTIPLE);
   g_signal_connect(city_selection, "changed",
 	G_CALLBACK(city_selection_changed_callback), NULL);
+
+  /* Ensure column_tooltips gets cleaned up */
+  g_signal_connect(city_view, "destroy",
+                   G_CALLBACK(city_view_destroyed), column_tooltips);
   
   for (i=0; i<NUM_CREPORT_COLS; i++) {
     gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(city_model), i,
@@ -1065,9 +1180,15 @@ static void create_city_report_dialog(bool make_modal)
   for (i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *col;
+    GtkLabel *header;
 
     renderer = gtk_cell_renderer_text_new();
-    col = gtk_tree_view_column_new_with_attributes(titles[i], renderer,NULL);
+    col = gtk_tree_view_column_new_with_attributes(NULL, renderer, NULL);
+    header = GTK_LABEL(gtk_label_new(titles[i]));
+    gtk_tooltips_set_tip(column_tooltips, GTK_WIDGET(header),
+                         spec->explanation, NULL);
+    gtk_widget_show(GTK_WIDGET(header));
+    gtk_tree_view_column_set_widget(col, GTK_WIDGET(header));
     gtk_tree_view_column_set_visible(col, spec->show);
     gtk_tree_view_column_set_sort_column_id(col, i);
     gtk_tree_view_column_set_reorderable(col, TRUE);
@@ -1168,8 +1289,8 @@ static void same_island_iterate(GtkTreeModel *model, GtkTreePath *path,
     itree_get(&it, 0, &res, -1);
     pcity = res;
 
-    if (tile_get_continent(pcity->tile)
-	== tile_get_continent(selectedcity->tile)) {
+    if (tile_continent(pcity->tile)
+	== tile_continent(selectedcity->tile)) {
       itree_select(city_selection, &it);
     }
   }
@@ -1201,11 +1322,13 @@ static void city_select_building_callback(GtkMenuItem *item, gpointer data)
     itree_get(&it, 0, &res, -1);
     pcity = res;
 
-    if ( (which == TYPE_UNIT && pcity->production.is_unit)
-         || (which == TYPE_NORMAL_IMPROVEMENT && !pcity->production.is_unit
-             && !is_wonder(pcity->production.value))
-         || (which == TYPE_WONDER && !pcity->production.is_unit
-             && is_wonder(pcity->production.value)) ) {
+    if ( (which == PCT_UNIT && VUT_UTYPE == pcity->production.kind)
+         || (which == PCT_NORMAL_IMPROVEMENT
+             && VUT_IMPROVEMENT == pcity->production.kind
+             && !is_wonder(pcity->production.value.building))
+         || (which == PCT_WONDER
+             && VUT_IMPROVEMENT == pcity->production.kind
+             && is_wonder(pcity->production.value.building)) ) {
       itree_select(city_selection, &it);
     }
   }
@@ -1327,8 +1450,10 @@ static void update_row(GtkTreeIter *row, struct city *pcity)
 *****************************************************************/
 static void city_model_init(void)
 {
-  if (city_dialog_shell && !is_report_dialogs_frozen() && game.player_ptr) {
-    city_list_iterate(game.player_ptr->cities, pcity) {
+  if (NULL != client.conn.playing
+      && city_dialog_shell
+      && !is_report_dialogs_frozen()) {
+    city_list_iterate(client.conn.playing->cities, pcity) {
       GtkTreeIter it;
 
       gtk_list_store_append(city_model, &it);
@@ -1365,8 +1490,8 @@ void city_report_dialog_update(void)
     /* update. */
     gtk_list_store_clear(city_model);
 
-    if (game.player_ptr) {
-      city_list_iterate(game.player_ptr->cities, pcity) {
+    if (NULL != client.conn.playing) {
+      city_list_iterate(client.conn.playing->cities, pcity) {
 	gtk_list_store_append(city_model, &it);
 	update_row(&it, pcity);
 
@@ -1524,6 +1649,17 @@ static void create_next_to_last_menu(GtkWidget *parent_item)
   next_to_last_wonders_item = item;
 }
 
+/****************************************************************
+  Create the sell menu (empty).
+*****************************************************************/
+static void create_sell_menu(GtkWidget *item)
+{
+  GtkWidget *menu;
+
+  menu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+}
+
 
 /****************************************************************
 ...
@@ -1536,15 +1672,15 @@ static void popup_change_menu(GtkMenuShell *menu, gpointer data)
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(change_improvements_item),
 				  FALSE, FALSE, CO_CHANGE,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(change_units_item),
 				  TRUE, FALSE, CO_CHANGE,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(change_wonders_item),
 				  FALSE, TRUE, CO_CHANGE,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
 }
 
@@ -1559,15 +1695,15 @@ static void popup_last_menu(GtkMenuShell *menu, gpointer data)
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(last_improvements_item),
 				  FALSE, FALSE, CO_LAST,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(last_units_item),
 				  TRUE, FALSE, CO_LAST,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(last_wonders_item),
 				  FALSE, TRUE, CO_LAST,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
 }
 
@@ -1582,15 +1718,15 @@ static void popup_first_menu(GtkMenuShell *menu, gpointer data)
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(first_improvements_item),
 				  FALSE, FALSE, CO_FIRST,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(first_units_item),
 				  TRUE, FALSE, CO_FIRST,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(first_wonders_item),
 				  FALSE, TRUE, CO_FIRST,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
 }
 
@@ -1605,15 +1741,15 @@ static void popup_next_menu(GtkMenuShell *menu, gpointer data)
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(next_improvements_item),
 				  FALSE, FALSE, CO_NEXT,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(next_units_item),
 				  TRUE, FALSE, CO_NEXT,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(next_wonders_item),
 				  FALSE, TRUE, CO_NEXT,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
 }
 
@@ -1634,20 +1770,57 @@ static void popup_next_to_last_menu(GtkMenuShell *menu, gpointer data)
   item = next_to_last_improvements_item;
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(item),
                                    FALSE, FALSE, CO_NEXT_TO_LAST,
-                                   city_can_build_impr_or_unit,
+                                   can_city_build_now,
                                    callback, n);
   item = next_to_last_units_item;
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(item),
                                    TRUE, FALSE, CO_NEXT_TO_LAST,
-                                   city_can_build_impr_or_unit,
+                                   can_city_build_now,
                                    callback, n);
   item = next_to_last_wonders_item;
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(item),
                                    FALSE, TRUE, CO_NEXT_TO_LAST,
-                                   city_can_build_impr_or_unit,
+                                   can_city_build_now,
                                    callback, n);
 }
 
+/****************************************************************
+  Same as can_city_sell_building(), but with universal argument.
+*****************************************************************/
+static bool can_city_sell_universal(const struct city *pcity,
+                                    struct universal target)
+{
+  return (target.kind == VUT_IMPROVEMENT
+          && can_city_sell_building(pcity, target.value.building));
+}
+
+/****************************************************************
+  Update the sell menu.
+*****************************************************************/
+static void recreate_sell_menu(void)
+{
+  int n;
+  GList *children;
+  GtkWidget *menu;
+
+  n = gtk_tree_selection_count_selected_rows(city_selection);
+  menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(city_sell_command));
+  gtk_menu_popdown(GTK_MENU(menu));
+
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(city_sell_command),
+                                   FALSE, FALSE, CO_SELL,
+                                   can_city_sell_universal,
+                                   G_CALLBACK(select_impr_or_unit_callback),
+                                   n);
+
+  menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(city_sell_command));
+  children = gtk_container_get_children(GTK_CONTAINER(menu));
+
+  n = g_list_length(children);
+  gtk_widget_set_sensitive(city_sell_command, n > 0);
+  g_list_free(children);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1683,19 +1856,19 @@ static void create_select_menu(GtkWidget *item)
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "activate",
   		   G_CALLBACK(city_select_building_callback),
-		   GINT_TO_POINTER(TYPE_UNIT));
+		   GINT_TO_POINTER(PCT_UNIT));
 
   item = gtk_menu_item_new_with_label( _("Building Improvements"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "activate",
   		   G_CALLBACK(city_select_building_callback),
-		   GINT_TO_POINTER(TYPE_NORMAL_IMPROVEMENT));
+		   GINT_TO_POINTER(PCT_NORMAL_IMPROVEMENT));
 
   item = gtk_menu_item_new_with_label(_("Building Wonders"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "activate",
   		   G_CALLBACK(city_select_building_callback),
-		   GINT_TO_POINTER(TYPE_WONDER));
+		   GINT_TO_POINTER(PCT_WONDER));
 
 
   item = gtk_separator_menu_item_new();
@@ -1771,11 +1944,9 @@ static void create_select_menu(GtkWidget *item)
 ...
 *****************************************************************/
 static bool city_building_impr_or_unit(const struct city *pcity,
-				       struct city_production target)
+				       struct universal target)
 {
-  struct city_production target2 = pcity->production;
-
-  return target.is_unit == target2.is_unit && target.value == target2.value;
+  return are_universals_equal(&pcity->production, &target);
 }
 
 /****************************************************************
@@ -1823,15 +1994,15 @@ static void popup_select_menu(GtkMenuShell *menu, gpointer data)
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_improvements_item),
 				  FALSE, FALSE, CO_NONE,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_units_item),
 				  TRUE, FALSE, CO_NONE,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_wonders_item),
 				  FALSE, TRUE, CO_NONE,
-				  city_can_build_impr_or_unit,
+				  can_city_build_now,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_cma_to_menu_item(GTK_MENU_ITEM(select_cma_item), FALSE);
 
@@ -1870,7 +2041,7 @@ static void update_total_buy_cost(void)
       gtk_tree_model_get(model, &iter, 0, &res, -1);
       pcity = res;
       if (pcity != NULL) {
-        total += city_buy_cost(pcity);
+        total += city_production_buy_gold_cost(pcity);
       }
     }
     gtk_tree_path_free(path);
@@ -1893,17 +2064,22 @@ static void update_total_buy_cost(void)
 static void city_selection_changed_callback(GtkTreeSelection *selection)
 {
   int n;
+  bool obs_may, plr_may;
 
   n = gtk_tree_selection_count_selected_rows(selection);
+  obs_may = n > 0;
+  plr_may = obs_may && can_client_issue_orders();
 
-  gtk_widget_set_sensitive(city_production_command,
-                           n > 0 && can_client_issue_orders());
-  gtk_widget_set_sensitive(city_governor_command,
-                           n > 0 && can_client_issue_orders());
-  gtk_widget_set_sensitive(city_center_command, n > 0);
-  gtk_widget_set_sensitive(city_popup_command, n > 0);
-  gtk_widget_set_sensitive(city_buy_command,
-                           n > 0 && can_client_issue_orders());
+  gtk_widget_set_sensitive(city_production_command, plr_may);
+  gtk_widget_set_sensitive(city_governor_command, plr_may);
+  gtk_widget_set_sensitive(city_center_command, obs_may);
+  gtk_widget_set_sensitive(city_popup_command, obs_may);
+  gtk_widget_set_sensitive(city_buy_command, plr_may);
+  if (plr_may) {
+    recreate_sell_menu();
+  } else {
+    gtk_widget_set_sensitive(city_sell_command, FALSE);
+  }
 
   update_total_buy_cost();
 }
@@ -1931,7 +2107,7 @@ static void clear_worklist_foreach_func(GtkTreeModel *model,
 **************************************************************************/
 static void city_clear_worklist_callback(GtkMenuItem *item, gpointer data)
 {
-  struct connection *pconn = &aconnection;
+  struct connection *pconn = &client.conn;
 
   g_return_if_fail(city_selection != NULL);
 
