@@ -19,15 +19,20 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include "events.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
+#include "log.h"
 #include "mem.h"
 #include "support.h"
 
+/* common */
+#include "events.h"
+#include "game.h"
+
+/* client */
 #include "chatline_g.h"
 #include "citydlg_g.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "cma_fec.h"
 #include "messagewin_g.h"
 
@@ -65,9 +70,8 @@ static void cma_preset_remove_response(GtkWidget *w, gint response,
 				       gpointer data);
 
 static void cma_add_preset_callback(GtkWidget *w, gpointer data);
-static void cma_preset_add_callback_yes(GtkWidget *w, gpointer data);
-static void cma_preset_add_callback_no(GtkWidget *w, gpointer data);
-static void cma_preset_add_callback_destroy(GtkWidget *w, gpointer data);
+static void cma_preset_add_popup_callback(gpointer data, gint response,
+                                          const char *input);
 
 static void cma_active_callback(GtkWidget *w, gpointer data);
 static void cma_activate_preset_callback(GtkTreeView *view, GtkTreePath *path,
@@ -207,7 +211,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   struct cma_dialog *pdialog;
   struct cm_parameter param;
   GtkWidget *frame, *page, *hbox, *label, *table;
-  GtkWidget *vbox, *sw, *hscale, *button, *align;
+  GtkWidget *vbox, *sw, *hscale, *button, *align, *image;
   GtkListStore *store;
   GtkCellRenderer *rend;
   GtkWidget *view;
@@ -266,7 +270,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   label = g_object_new(GTK_TYPE_LABEL,
                        "use-underline", TRUE,
                        "mnemonic-widget", view,
-                       "label", _("_Presets:"),
+                       "label", _("Prese_ts:"),
                        "xalign", 0.0, "yalign", 0.5, NULL);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
@@ -282,10 +286,13 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_EDGE);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  pdialog->add_preset_command = gtk_button_new_from_stock(GTK_STOCK_NEW);
-  gtk_container_add(GTK_CONTAINER(hbox), pdialog->add_preset_command);
-  g_signal_connect(pdialog->add_preset_command, "clicked",
-		   G_CALLBACK(cma_add_preset_callback), pdialog);
+  button = gtk_button_new_with_mnemonic(_("Ne_w"));
+  image = gtk_image_new_from_stock(GTK_STOCK_NEW, GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_image(GTK_BUTTON(button), image);
+  gtk_container_add(GTK_CONTAINER(hbox), button);
+  g_signal_connect(button, "clicked",
+                   G_CALLBACK(cma_add_preset_callback), pdialog);
+  pdialog->add_preset_command = button;
 
   pdialog->del_preset_command = gtk_button_new_from_stock(GTK_STOCK_DELETE);
   gtk_container_add(GTK_CONTAINER(hbox), pdialog->del_preset_command);
@@ -304,13 +311,13 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
 
   pdialog->result_label =
       gtk_label_new("food\n prod\n trade\n\n people\n grow\n prod\n name");
-  gtk_widget_set_name(pdialog->result_label, "city label");
+  gtk_widget_set_name(pdialog->result_label, "city_label");
   gtk_container_add(GTK_CONTAINER(frame), pdialog->result_label);
   gtk_label_set_justify(GTK_LABEL(pdialog->result_label), GTK_JUSTIFY_LEFT);
 
   /* Minimal Surplus and Factor */
 
-  table = gtk_table_new(O_COUNT + 2, 3, TRUE);
+  table = gtk_table_new(O_LAST + 2, 3, TRUE);
   gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 2);
 
   label = gtk_label_new(_("Minimal Surplus"));
@@ -353,12 +360,12 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
 
   label = gtk_label_new(_("Celebrate"));
   gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1,
-			    O_COUNT + 1, O_COUNT + 2);
+			    O_LAST + 1, O_LAST + 2);
   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2,
-			    O_COUNT + 1, O_COUNT + 2);
+			    O_LAST + 1, O_LAST + 2);
 
   pdialog->happy_button = gtk_check_button_new();
   gtk_box_pack_start(GTK_BOX(hbox), pdialog->happy_button, FALSE, FALSE,
@@ -369,16 +376,16 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   g_signal_connect(pdialog->happy_button, "toggled",
 		   G_CALLBACK(hscale_changed), pdialog);
 
-  pdialog->factor[O_COUNT] =
-      GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 50, 1, 0, 0));
+  pdialog->factor[O_LAST] =
+      GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 50, 1, 1, 0));
 
-  hscale = gtk_hscale_new(GTK_ADJUSTMENT(pdialog->factor[O_COUNT]));
+  hscale = gtk_hscale_new(GTK_ADJUSTMENT(pdialog->factor[O_LAST]));
   gtk_table_attach_defaults(GTK_TABLE(table), hscale, 2, 3,
-			    O_COUNT + 1, O_COUNT + 2);
+			    O_LAST + 1, O_LAST + 2);
   gtk_scale_set_digits(GTK_SCALE(hscale), 0);
   gtk_scale_set_value_pos(GTK_SCALE(hscale), GTK_POS_LEFT);
 
-  g_signal_connect(pdialog->factor[O_COUNT],
+  g_signal_connect(pdialog->factor[O_LAST],
 		   "value_changed",
 		   G_CALLBACK(hscale_changed), pdialog);
 
@@ -406,7 +413,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   gtk_box_pack_start(GTK_BOX(vbox), pdialog->active_image, FALSE, FALSE, 0);
 
   pdialog->active_label = gtk_label_new(NULL);
-  gtk_widget_set_name(pdialog->active_label, "comment label");
+  gtk_widget_set_name(pdialog->active_label, "comment_label");
   gtk_box_pack_end(GTK_BOX(vbox), pdialog->active_label, FALSE, FALSE, 0);
 
   gtk_widget_show_all(pdialog->shell);
@@ -435,7 +442,7 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   cmafec_get_fe_parameter(pcity, &param);
 
   /* fill in result label */
-  cm_copy_result_from_city(pcity, &result);
+  cm_result_from_main_map(&result, pcity, TRUE);
   gtk_label_set_text(GTK_LABEL(pdialog->result_label),
 		     cmafec_get_result_descr(pcity, &result, &param));
 
@@ -537,55 +544,34 @@ static void cma_add_preset_callback(GtkWidget *w, gpointer data)
   }
 
   pdialog->name_shell = input_dialog_create(GTK_WINDOW(parent),
-				    _("Name new preset"),
-				    _("What should we name the preset?"),
-				    default_name,
-				    G_CALLBACK(cma_preset_add_callback_yes),
-				    pdialog,
-				    G_CALLBACK(cma_preset_add_callback_no),
-				    pdialog);
-
-  g_signal_connect(pdialog->name_shell, "destroy",
-		   G_CALLBACK(cma_preset_add_callback_destroy), data);
+                                    _("Name new preset"),
+                                    _("What should we name the preset?"),
+                                    default_name,
+                                    cma_preset_add_popup_callback, pdialog);
 }
 
 /****************************************************************
- callback for the add_preset popup (delete popup)
+  callback for the add_preset popup
 *****************************************************************/
-static void cma_preset_add_callback_destroy(GtkWidget *w, gpointer data)
-{
-  struct cma_dialog *pdialog = (struct cma_dialog *) data;
-
-  pdialog->name_shell = NULL;
-}
-
-/****************************************************************
- callback for the add_preset popup (don't add it)
-*****************************************************************/
-static void cma_preset_add_callback_no(GtkWidget *w, gpointer data)
-{
-  input_dialog_destroy(w);
-}
-
-/****************************************************************
- callback for the add_preset popup (add it)
-*****************************************************************/
-static void cma_preset_add_callback_yes(GtkWidget *w, gpointer data)
+static void cma_preset_add_popup_callback(gpointer data, gint response,
+                                          const char *input)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
 
   if (pdialog) {
-    struct cm_parameter param;
+    if (response == GTK_RESPONSE_OK) {
+      struct cm_parameter param;
 
-    cmafec_get_fe_parameter(pdialog->pcity, &param);
-    cmafec_preset_add(input_dialog_get_input(w), &param);
-    update_cma_preset_list(pdialog);
-    refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_HSCALES);
-    /* if this or other cities have this set as "custom" */
-    city_report_dialog_update();
+      cmafec_get_fe_parameter(pdialog->pcity, &param);
+      cmafec_preset_add(input, &param);
+      update_cma_preset_list(pdialog);
+      refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_HSCALES);
+      /* if this or other cities have this set as "custom" */
+      city_report_dialog_update();
+    } /* else CANCEL or DELETE_EVENT */
+
+    pdialog->name_shell = NULL;
   }
-
-  input_dialog_destroy(w);
 }
 
 /****************************************************************
@@ -712,7 +698,7 @@ static void set_hscales(const struct cm_parameter *const parameter,
   } output_type_iterate_end;
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdialog->happy_button),
 			       parameter->require_happy);
-  gtk_adjustment_set_value(pdialog->factor[O_COUNT],
+  gtk_adjustment_set_value(pdialog->factor[O_LAST],
 			   parameter->happy_factor);
   allow_refreshes = 1;
 }
@@ -736,7 +722,7 @@ static void hscale_changed(GtkAdjustment *get, gpointer data)
   } output_type_iterate_end;
   param.require_happy =
       (GTK_TOGGLE_BUTTON(pdialog->happy_button)->active ? 1 : 0);
-  param.happy_factor = (int) (pdialog->factor[O_COUNT]->value);
+  param.happy_factor = (int) (pdialog->factor[O_LAST]->value);
 
   /* save the change */
   cmafec_set_fe_parameter(pdialog->pcity, &param);
