@@ -17,18 +17,20 @@
 
 #include <gtk/gtk.h>
 
+/* utility */
 #include "support.h"
 
+/* common */
 #include "game.h"
 #include "unit.h"
 #include "unitlist.h"
 
+/* client */
 #include "dialogs_g.h"
 #include "chatline.h"
 #include "choice_dialog.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "connectdlg_common.h"
 #include "control.h"
 #include "gui_main.h"
@@ -87,11 +89,11 @@ void popup_bribe_dialog(struct unit *punit, int cost)
                                  GTK_STOCK_OK, NULL, NULL, NULL);
     gtk_window_present(GTK_WINDOW(shell));
     return;
-  } else if (game.player_ptr->economic.gold >= cost) {
+  } else if (cost <= client.conn.playing->economic.gold) {
     shell = gtk_message_dialog_new(NULL, 0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
       _("Bribe unit for %d gold?\nTreasury contains %d gold."),
-      cost, game.player_ptr->economic.gold);
+      cost, client.conn.playing->economic.gold);
     gtk_window_set_title(GTK_WINDOW(shell), _("Bribe Enemy Unit"));
     setup_dialog(shell, toplevel);
   } else {
@@ -99,7 +101,7 @@ void popup_bribe_dialog(struct unit *punit, int cost)
       0,
       GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
       _("Bribing the unit costs %d gold.\nTreasury contains %d gold."),
-      cost, game.player_ptr->economic.gold);
+      cost, client.conn.playing->economic.gold);
     gtk_window_set_title(GTK_WINDOW(shell), _("Traitors Demand Too Much!"));
     setup_dialog(shell, toplevel);
   }
@@ -226,7 +228,6 @@ static void create_advances_list(struct player *pplayer,
 				 struct player *pvictim)
 {  
   GtkWidget *sw, *label, *vbox, *view;
-  int i;
   GtkListStore *store;
   GtkCellRenderer *rend;
   GtkTreeViewColumn *col;
@@ -287,20 +288,20 @@ static void create_advances_list(struct player *pplayer,
     GtkTreeIter it;
     GValue value = { 0, };
 
-    for(i=A_FIRST; i<game.control.num_tech_types; i++) {
-      if(get_invention(pvictim, i)==TECH_KNOWN && 
-	 (get_invention(pplayer, i)==TECH_UNKNOWN || 
-	  get_invention(pplayer, i)==TECH_REACHABLE)) {
+    advance_index_iterate(A_FIRST, i) {
+      if(player_invention_state(pvictim, i)==TECH_KNOWN && 
+	 (player_invention_state(pplayer, i)==TECH_UNKNOWN || 
+	  player_invention_state(pplayer, i)==TECH_PREREQS_KNOWN)) {
 	gtk_list_store_append(store, &it);
 
 	g_value_init(&value, G_TYPE_STRING);
 	g_value_set_static_string(&value,
-				  advance_name_for_player(game.player_ptr, i));
+				  advance_name_for_player(client.conn.playing, i));
 	gtk_list_store_set_value(store, &it, 0, &value);
 	g_value_unset(&value);
 	gtk_list_store_set(store, &it, 1, i, -1);
       }
-    }
+    } advance_index_iterate_end;
 
     gtk_list_store_append(store, &it);
 
@@ -430,12 +431,15 @@ static void create_improvements_list(struct player *pplayer,
   gtk_list_store_append(store, &it);
   gtk_list_store_set(store, &it, 0, _("City Production"), 1, -1, -1);
 
-  built_impr_iterate(pcity, i) {
-    if (improvement_by_number(i)->sabotage > 0) {
+  city_built_iterate(pcity, pimprove) {
+    if (pimprove->sabotage > 0) {
       gtk_list_store_append(store, &it);
-      gtk_list_store_set(store, &it, 0, get_impr_name_ex(pcity, i), 1, i, -1);
+      gtk_list_store_set(store, &it,
+                         0, city_improvement_name_translation(pcity, pimprove),
+                         1, improvement_number(pimprove),
+                         -1);
     }  
-  } built_impr_iterate_end;
+  } city_built_iterate_end;
 
   gtk_list_store_append(store, &it);
   gtk_list_store_set(store, &it, 0, _("At Spy's Discretion"), 1, B_LAST, -1);
@@ -471,7 +475,7 @@ has happened to the city during latency.  Therefore we must initialize
 pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
   
   if(!spy_tech_shell){
-    create_advances_list(game.player_ptr, pvictim);
+    create_advances_list(client.conn.playing, pvictim);
     gtk_window_present(GTK_WINDOW(spy_tech_shell));
   }
   gtk_widget_destroy(diplomat_dialog);
@@ -498,7 +502,7 @@ static void spy_request_sabotage_list(GtkWidget *w, gpointer data)
 void popup_sabotage_dialog(struct city *pcity)
 {
   if(!spy_sabotage_shell){
-    create_improvements_list(game.player_ptr, pcity);
+    create_improvements_list(client.conn.playing, pcity);
     gtk_window_present(GTK_WINDOW(spy_sabotage_shell));
   }
 }
@@ -543,12 +547,12 @@ void popup_incite_dialog(struct city *pcity, int cost)
       city_name(pcity));
     gtk_window_set_title(GTK_WINDOW(shell), _("City can't be incited!"));
   setup_dialog(shell, toplevel);
-  } else if (game.player_ptr->economic.gold >= cost) {
+  } else if (cost <= client.conn.playing->economic.gold) {
     shell = gtk_message_dialog_new(NULL,
       0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
       _("Incite a revolt for %d gold?\nTreasury contains %d gold."),
-      cost, game.player_ptr->economic.gold);
+      cost, client.conn.playing->economic.gold);
     gtk_window_set_title(GTK_WINDOW(shell), _("Incite a Revolt!"));
     setup_dialog(shell, toplevel);
   } else {
@@ -556,7 +560,7 @@ void popup_incite_dialog(struct city *pcity, int cost)
       0,
       GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
       _("Inciting a revolt costs %d gold.\nTreasury contains %d gold."),
-      cost, game.player_ptr->economic.gold);
+      cost, client.conn.playing->economic.gold);
     gtk_window_set_title(GTK_WINDOW(shell), _("Traitors Demand Too Much!"));
     setup_dialog(shell, toplevel);
   }
@@ -613,7 +617,7 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 
   diplomat_id = punit->id;
 
-  if ((pcity = tile_get_city(dest_tile))) {
+  if ((pcity = tile_city(dest_tile))) {
     /* Spy/Diplomat acting against a city */
 
     diplomat_target_id = pcity->id;
