@@ -19,14 +19,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <math.h> /* sqrt */
 
 #include <gtk/gtk.h>
 #include "gtkpixcomm.h"
 
+/* common & utility */
 #include "city.h"
 #include "fcintl.h"
-#include "game.h"
-#include "genlist.h"
 #include "government.h"
 #include "mem.h"
 #include "shared.h"
@@ -36,6 +36,8 @@
 #include "support.h"
 #include "version.h"
 
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "colors.h"
 #include "graphics.h"
@@ -61,7 +63,7 @@ static GtkWidget *help_frame;
 static GtkTextBuffer *help_text;
 static GtkWidget *help_text_sw;
 static GtkWidget *help_vbox;
-static GtkWidget *unit_tile;
+static GtkWidget *help_tile;
 static GtkWidget *help_box;
 static GtkWidget *help_itable;
 static GtkWidget *help_wtable;
@@ -85,13 +87,13 @@ static GPtrArray *help_history;
 static int	  help_history_pos;
 
 
-static char *help_ilabel_name[6] =
+static const char *help_ilabel_name[6] =
 { N_("Cost:"), NULL, N_("Upkeep:"), NULL, N_("Requirement:"), NULL };
 
-static char *help_wlabel_name[6] =
+static const char *help_wlabel_name[6] =
 { N_("Cost:"), NULL, N_("Requirement:"), NULL, N_("Obsolete by:"), NULL };
 
-static char *help_ulabel_name[5][5] =
+static const char *help_ulabel_name[5][5] =
 {
     { N_("Cost:"),		NULL, NULL, N_("Attack:"),	NULL },
     { N_("Defense:"),		NULL, NULL, N_("Move:")	,	NULL },
@@ -100,14 +102,16 @@ static char *help_ulabel_name[5][5] =
     { N_("Requirement:"),	NULL, NULL, N_("Obsolete by:"),	NULL }
 };
 
-static char *help_tlabel_name[4][5] =
+static const char *help_tlabel_name[4][5] =
 {
     { N_("Move/Defense:"),	NULL, NULL, N_("Food/Res/Trade:"),	NULL },
-    { N_("Sp1 F/R/T:"),		NULL, NULL, N_("Sp2 F/R/T:"),		NULL },
+    { N_("Resources:"),		NULL, NULL, NULL,			NULL },
     { N_("Road Rslt/Time:"),	NULL, NULL, N_("Irrig. Rslt/Time:"),	NULL },
     { N_("Mine Rslt/Time:"),	NULL, NULL, N_("Trans. Rslt/Time:"),	NULL }
 };
 
+#define REQ_LABEL_NONE _("None")
+#define REQ_LABEL_NEVER _("(Never)")
 
 static void create_help_dialog(void);
 static void help_update_dialog(const struct help_item *pitem);
@@ -164,7 +168,7 @@ callers of this function should do so themselves... --dwp
 *****************************************************************/
 void popup_help_dialog_string(const char *item)
 {
-  popup_help_dialog_typed(_(item), HELP_ANY);
+  popup_help_dialog_typed(Q_(item), HELP_ANY);
 }
 
 /**************************************************************************
@@ -184,8 +188,9 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
   GtkTreeIter   l;
   GValue        value = { 0, };
 
-  if (advances[tech].req[0] == A_LAST && advances[tech].req[1] == A_LAST) {
-    bg = COLOR_STD_RED;
+  if (advance_required(tech, AR_ONE) == A_LAST
+   && advance_required(tech, AR_TWO) == A_LAST) {
+    bg = COLOR_REQTREE_UNKNOWN;
 
     gtk_tree_store_append(tstore, &l, parent);
     help_advances[tech] = TRUE;
@@ -198,18 +203,24 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
     gtk_tree_store_set(tstore, &l,
 		       1, -1,
 		       2, tech,
-		       3, colors_standard[bg],
+		       3, &get_color(tileset, bg)->color
 		       -1);
     return;
   }
 
-  switch (get_invention(game.player_ptr, tech)) {
-  case TECH_UNKNOWN:	      bg = COLOR_STD_RED;	      break;
-  case TECH_KNOWN:	      bg = COLOR_STD_GROUND;	      break;
-  case TECH_REACHABLE:        bg = COLOR_STD_YELLOW;	      break;
-  default:		      bg = COLOR_STD_WHITE;	      break;
+  bg = COLOR_REQTREE_BACKGROUND;
+  switch (player_invention_state(client.conn.playing, tech)) {
+  case TECH_UNKNOWN:
+    bg = COLOR_REQTREE_UNKNOWN;
+    break;
+  case TECH_KNOWN:
+    bg = COLOR_REQTREE_KNOWN;
+    break;
+  case TECH_PREREQS_KNOWN:
+    bg = COLOR_REQTREE_PREREQS_KNOWN;
+    break;
   }
-  turns_to_tech = num_unknown_techs_for_goal(game.player_ptr, tech);
+  turns_to_tech = num_unknown_techs_for_goal(client.conn.playing, tech);
 
   /* l is the original in the tree. */
   original = !help_advances[tech];
@@ -218,14 +229,14 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
   help_advances[tech] = TRUE;
 
   g_value_init(&value, G_TYPE_STRING);
-  g_value_set_static_string(&value, advances[tech].name);
+  g_value_set_static_string(&value, advance_name_for_player(client.conn.playing, tech));
   gtk_tree_store_set_value(tstore, &l, 0, &value);
   g_value_unset(&value);
 
   gtk_tree_store_set(tstore, &l,
 		     1, turns_to_tech,
 		     2, tech,
-		     3, colors_standard[bg],
+		     3, &get_color(tileset, bg)->color,
 		     -1);
 
   if (--levels <= 0)
@@ -233,10 +244,10 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
 
   if (original) {
     /* only add children to orginals */
-    if (advances[tech].req[0] != A_NONE)
-      create_tech_tree(advances[tech].req[0], levels, &l);
-    if (advances[tech].req[1] != A_NONE)
-      create_tech_tree(advances[tech].req[1], levels, &l);
+    if (advance_required(tech, AR_ONE) != A_NONE)
+      create_tech_tree(advance_required(tech, AR_ONE), levels, &l);
+    if (advance_required(tech, AR_TWO) != A_NONE)
+      create_tech_tree(advance_required(tech, AR_TWO), levels, &l);
   }
   return;
 }
@@ -254,7 +265,7 @@ static void help_tech_tree_activated_callback(GtkTreeView *view,
 
   gtk_tree_model_get_iter(GTK_TREE_MODEL(tstore), &it, path);
   gtk_tree_model_get(GTK_TREE_MODEL(tstore), &it, 2, &tech, -1);
-  select_help_item_string(advances[tech].name, HELP_TECH);
+  select_help_item_string(advance_name_for_player(client.conn.playing, tech), HELP_TECH);
 }
 
 /**************************************************************************
@@ -284,11 +295,12 @@ static void help_hyperlink_callback(GtkWidget *w)
   s=gtk_label_get_text(GTK_LABEL(w));
   type=GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(w), "page_type"));
 
-  /* May be able to skip, or may need to modify, advances[A_NONE].name
+  /* FIXME: May be able to skip, or may need to modify, advances[A_NONE]
      below, depending on which i18n is done elsewhere.
   */
-  if (strcmp(s, _("(Never)")) != 0 && strcmp(s, _("None")) != 0
-      && strcmp(s, advances[A_NONE].name) != 0)
+  if (strcmp(s, REQ_LABEL_NEVER) != 0
+      && strcmp(s, REQ_LABEL_NONE) != 0
+      && strcmp(s, advance_name_translation(advance_by_number(A_NONE))) != 0)
     select_help_item_string(s, type);
 }
 
@@ -302,6 +314,7 @@ static GtkWidget *help_hyperlink_new(GtkWidget *label, enum help_page_type type)
   button = gtk_button_new();
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+  gtk_widget_set_name(label, "help_link");
   gtk_container_add(GTK_CONTAINER(button), label);
   gtk_widget_show(button);
   g_signal_connect_swapped(button, "clicked",
@@ -313,18 +326,13 @@ static GtkWidget *help_hyperlink_new(GtkWidget *label, enum help_page_type type)
 /**************************************************************************
 ...
 **************************************************************************/
-static GtkWidget *help_slink_new(gchar *txt, enum help_page_type type)
+static GtkWidget *help_slink_new(const gchar *txt, enum help_page_type type)
 {
   GtkWidget *button, *label;
 
-  button = gtk_button_new_with_label(txt);
-  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-  label = GTK_BIN(button)->child;
-  gtk_widget_set_name(label, "help link");
-  gtk_widget_show(button);
-  g_signal_connect_swapped(button, "clicked",
-			   G_CALLBACK(help_hyperlink_callback), label);
-  g_object_set_data(G_OBJECT(label), "page_type", GUINT_TO_POINTER(type));
+  label = gtk_label_new(txt);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+  button = help_hyperlink_new(label, type);
   return button;
 }
 
@@ -335,14 +343,14 @@ static void help_box_hide(void)
 {
   gtk_widget_hide(help_box);
 
-  gtk_widget_hide(unit_tile);
+  gtk_widget_hide(help_tile);
 
   gtk_widget_hide(help_itable);
   gtk_widget_hide(help_wtable);
   gtk_widget_hide(help_utable);
   gtk_widget_hide(help_ttable);
   
-  gtk_widget_hide(unit_tile);
+  gtk_widget_hide(help_tile); /* FIXME: twice? */
 
   gtk_widget_hide(help_vbox);
   gtk_widget_hide(help_text_sw);
@@ -428,12 +436,7 @@ static void create_help_dialog(void)
 						  GTK_STOCK_CLOSE,
 						  GTK_RESPONSE_CLOSE,
 						  NULL);
-  if (dialogs_on_top) {
-    gtk_window_set_transient_for(GTK_WINDOW(help_dialog_shell),
-				 GTK_WINDOW(toplevel));
-  }
-  gtk_window_set_type_hint(GTK_WINDOW(help_dialog_shell),
-			   GDK_WINDOW_TYPE_HINT_NORMAL);
+  setup_dialog(help_dialog_shell, toplevel);
   gtk_dialog_set_default_response(GTK_DIALOG(help_dialog_shell),
 				  GTK_RESPONSE_CLOSE);
   gtk_widget_set_name(help_dialog_shell, "Freeciv");
@@ -494,7 +497,7 @@ static void create_help_dialog(void)
 
   help_view_sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(help_view_sw),
-  			  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  			  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_widget_set_size_request(help_view_sw, 190, -1);
   gtk_container_add(GTK_CONTAINER(help_view_sw), help_view);
   gtk_widget_show(help_view);
@@ -509,8 +512,8 @@ static void create_help_dialog(void)
   help_box = gtk_vbox_new(FALSE, 5);
   gtk_container_add(GTK_CONTAINER(help_frame), help_box);
 
-  unit_tile = gtk_pixcomm_new(UNIT_TILE_WIDTH, UNIT_TILE_HEIGHT);
-  gtk_box_pack_start(GTK_BOX(help_box), unit_tile, FALSE, FALSE, 0);
+  help_tile = gtk_pixcomm_new(tileset_full_tile_width(tileset), tileset_full_tile_height(tileset));
+  gtk_box_pack_start(GTK_BOX(help_box), help_tile, FALSE, FALSE, 0);
 
   help_itable = gtk_table_new(1, 6, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_itable, FALSE, FALSE, 0);
@@ -522,11 +525,10 @@ static void create_help_dialog(void)
     if (i==5) {
       button = help_hyperlink_new(help_ilabel[i], HELP_TECH);
       gtk_table_attach_defaults(GTK_TABLE(help_itable), button, i, i+1, 0, 1);
-      gtk_widget_set_name(help_ilabel[i], "help link");
     } else {
       gtk_table_attach_defaults(GTK_TABLE(help_itable),
 			        help_ilabel[i], i, i+1, 0, 1);
-      gtk_widget_set_name(help_ilabel[i], "help label");
+      gtk_widget_set_name(help_ilabel[i], "help_label");
     }
     gtk_widget_show(help_ilabel[i]);
   }
@@ -541,11 +543,10 @@ static void create_help_dialog(void)
     if (i==3 || i==5) {
       button = help_hyperlink_new(help_wlabel[i], HELP_TECH);
       gtk_table_attach_defaults(GTK_TABLE(help_wtable), button, i, i+1, 0, 1);
-      gtk_widget_set_name(help_wlabel[i], "help link");
     } else {
       gtk_table_attach_defaults(GTK_TABLE(help_wtable),
 			        help_wlabel[i], i, i+1, 0, 1);
-      gtk_widget_set_name(help_wlabel[i], "help label");
+      gtk_widget_set_name(help_wlabel[i], "help_label");
     }
     gtk_widget_show(help_wlabel[i]);
   }
@@ -567,11 +568,10 @@ static void create_help_dialog(void)
 
         gtk_table_attach_defaults(GTK_TABLE(help_utable),
 					    button, i, i+1, j, j+1);
-        gtk_widget_set_name(help_ulabel[j][i], "help link");
       } else {
         gtk_table_attach_defaults(GTK_TABLE(help_utable),
 			          help_ulabel[j][i], i, i+1, j, j+1);
-        gtk_widget_set_name(help_ulabel[j][i], "help label");
+        gtk_widget_set_name(help_ulabel[j][i], "help_label");
       }
       gtk_widget_show(help_ulabel[j][i]);
     }
@@ -580,15 +580,24 @@ static void create_help_dialog(void)
   help_ttable = gtk_table_new(5, 5, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_ttable, FALSE, FALSE, 0);
 
-  for (i=0; i<5; i++) {
-    for (j=0; j<4; j++) {
+  for (j=0; j<4; j++) {
+    for (i=0; i<5; i++) {
       help_tlabel[j][i] =
 	  gtk_label_new(help_tlabel_name[j][i] ? _(help_tlabel_name[j][i]) : "");
-      gtk_widget_set_name(help_tlabel[j][i], "help label");
+      gtk_widget_set_name(help_tlabel[j][i], "help_label");
 
-      gtk_table_attach_defaults(GTK_TABLE(help_ttable),
-					  help_tlabel[j][i], i, i+1, j, j+1);
-      gtk_widget_show(help_tlabel[j][i]);
+      /* Ugly (but these numbers are hardcoded in help_update_terrain() too) */
+      if (j==1 && i==1) {
+          /* Extra wide cell for terrain specials */
+          gtk_table_attach_defaults(GTK_TABLE(help_ttable),
+                                    help_tlabel[j][i], i, i+4, j, j+1);
+          gtk_widget_show(help_tlabel[j][i]);
+          break; /* skip rest of row */
+      } else {
+          gtk_table_attach_defaults(GTK_TABLE(help_ttable),
+                                    help_tlabel[j][i], i, i+1, j, j+1);
+          gtk_widget_show(help_tlabel[j][i]);
+      }
     }
   }
 
@@ -598,10 +607,11 @@ static void create_help_dialog(void)
   gtk_box_pack_start(GTK_BOX(help_box), help_vbox, FALSE, FALSE, 0);
 					     
   text = gtk_text_view_new();
+  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(text), FALSE);
   gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
   gtk_container_set_border_width(GTK_CONTAINER(text), 5);
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
-  gtk_widget_set_name(text, "help text");
+  gtk_widget_set_name(text, "help_text");
   help_text = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
   gtk_widget_show(text);
 
@@ -685,34 +695,42 @@ static void create_help_page(enum help_page_type type)
 ...
 **************************************************************************/
 static void help_update_improvement(const struct help_item *pitem,
-				    char *title, int which)
+				    char *title)
 {
-  char *buf = &long_buffer[0];
-  
+  char buf[8192];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
+
   create_help_page(HELP_IMPROVEMENT);
-  
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d", imp->build_cost);
-    gtk_set_label(help_ilabel[1], buf);
+
+  if (imp  &&  !is_great_wonder(imp)) {
+    const char *req = REQ_LABEL_NONE;
+    char req_buf[512];
+
+    sprintf(buf, "%d", impr_build_shield_cost(imp));
+    gtk_label_set_text(GTK_LABEL(help_ilabel[1]), buf);
     sprintf(buf, "%d", imp->upkeep);
-    gtk_set_label(help_ilabel[3], buf);
-    if (imp->tech_req == A_LAST) {
-      gtk_set_label(help_ilabel[5], _("(Never)"));
-    } else {
-      gtk_set_label(help_ilabel[5], advances[imp->tech_req].name);
-    }
+    gtk_label_set_text(GTK_LABEL(help_ilabel[3]), buf);
+
+    /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+     * Currently it's limited to 1 req but this code is partially prepared
+     * to be extended.  Remember MAX_NUM_REQS is a compile-time
+     * definition. */
+    requirement_vector_iterate(&imp->reqs, preq) {
+      req = universal_name_translation(&preq->source, req_buf, sizeof(req_buf));
+      break;
+    } requirement_vector_iterate_end;
+    gtk_label_set_text(GTK_LABEL(help_ilabel[5]), req);
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
-    gtk_set_label(help_ilabel[1], "0");
-    gtk_set_label(help_ilabel[3], "0");
-    gtk_set_label(help_ilabel[5], _("(Never)"));
-/*    create_tech_tree(help_improvement_tree, 0, game.num_tech_types, 3);*/
+    gtk_label_set_text(GTK_LABEL(help_ilabel[1]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ilabel[3]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ilabel[5]), REQ_LABEL_NEVER);
+/*    create_tech_tree(help_improvement_tree, 0, advance_count(), 3);*/
   }
   gtk_widget_show(help_itable);
 
-  helptext_improvement(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   gtk_text_buffer_set_text(help_text, buf, -1);
   gtk_widget_show(help_text_sw);
 }
@@ -721,38 +739,50 @@ static void help_update_improvement(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_wonder(const struct help_item *pitem,
-			       char *title, int which)
+			       char *title)
 {
-  char *buf = &long_buffer[0];
+  char buf[8192];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
 
   create_help_page(HELP_WONDER);
 
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d", imp->build_cost);
-    gtk_set_label(help_wlabel[1], buf);
-    if (imp->tech_req == A_LAST) {
-      gtk_set_label(help_wlabel[3], _("(Never)"));
+  if (imp  &&  is_great_wonder(imp)) {
+    int i;
+    char req_buf[512];
+
+    sprintf(buf, "%d", impr_build_shield_cost(imp));
+    gtk_label_set_text(GTK_LABEL(help_wlabel[1]), buf);
+
+    /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+     * Currently it's limited to 1 req but this code is partially prepared
+     * to be extended.  Remember MAX_NUM_REQS is a compile-time
+     * definition. */
+    i = 0;
+    requirement_vector_iterate(&imp->reqs, preq) {
+      gtk_label_set_text(GTK_LABEL(help_wlabel[3 + i]),
+			 universal_name_translation(&preq->source,
+					     req_buf, sizeof(req_buf)));
+      i++;
+      break;
+    } requirement_vector_iterate_end;
+    if (valid_advance(imp->obsolete_by)) {
+      gtk_label_set_text(GTK_LABEL(help_wlabel[5]),
+			 advance_name_for_player(client.conn.playing, advance_number(imp->obsolete_by)));
     } else {
-      gtk_set_label(help_wlabel[3], advances[imp->tech_req].name);
-    }
-    if (tech_exists(imp->obsolete_by)) {
-      gtk_set_label(help_wlabel[5], advances[imp->obsolete_by].name);
-    } else {
-      gtk_set_label(help_wlabel[5], _("(Never)"));
+      gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_LABEL_NEVER);
     }
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
     /* can't find wonder */
-    gtk_set_label(help_wlabel[1], "0");
-    gtk_set_label(help_wlabel[3], _("(Never)"));
-    gtk_set_label(help_wlabel[5], _("None"));
-/*    create_tech_tree(help_improvement_tree, 0, game.num_tech_types, 3); */
+    gtk_label_set_text(GTK_LABEL(help_wlabel[1]), "0");
+    gtk_label_set_text(GTK_LABEL(help_wlabel[3]), REQ_LABEL_NEVER);
+    gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_LABEL_NONE);
+/*    create_tech_tree(help_improvement_tree, 0, advance_count(), 3); */
   }
   gtk_widget_show(help_wtable);
 
-  helptext_wonder(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   gtk_text_buffer_set_text(help_text, buf, -1);
   gtk_widget_show(help_text_sw);
 }
@@ -761,63 +791,74 @@ static void help_update_wonder(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_unit_type(const struct help_item *pitem,
-				  char *title, int i)
+				  char *title)
 {
-  char *buf = &long_buffer[0];
+  char buf[8192];
+  struct unit_type *utype = find_unit_type_by_translated_name(title);
 
   create_help_page(HELP_UNIT);
 
-  if (i<game.num_unit_types) {
-    struct unit_type *utype = get_unit_type(i);
-    sprintf(buf, "%d", utype->build_cost);
-    gtk_set_label(help_ulabel[0][1], buf);
+  if (utype) {
+    sprintf(buf, "%d", utype_build_shield_cost(utype));
+    gtk_label_set_text(GTK_LABEL(help_ulabel[0][1]), buf);
     sprintf(buf, "%d", utype->attack_strength);
-    gtk_set_label(help_ulabel[0][4], buf);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[0][4]), buf);
     sprintf(buf, "%d", utype->defense_strength);
-    gtk_set_label(help_ulabel[1][1], buf);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[1][1]), buf);
     sprintf(buf, "%d", utype->move_rate/3);
-    gtk_set_label(help_ulabel[1][4], buf);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[1][4]), buf);
     sprintf(buf, "%d", utype->firepower);
-    gtk_set_label(help_ulabel[2][1], buf);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[2][1]), buf);
     sprintf(buf, "%d", utype->hp);
-    gtk_set_label(help_ulabel[2][4], buf);
-    gtk_set_label(help_ulabel[3][1], helptext_unit_upkeep_str(i));
-    sprintf(buf, "%d", utype->vision_range);
-    gtk_set_label(help_ulabel[3][4], buf);
-    if(utype->tech_requirement==A_LAST) {
-      gtk_set_label(help_ulabel[4][1], _("(Never)"));
+    gtk_label_set_text(GTK_LABEL(help_ulabel[2][4]), buf);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[3][1]),
+		       helptext_unit_upkeep_str(utype));
+    sprintf(buf, "%d", (int)sqrt((double)utype->vision_radius_sq));
+    gtk_label_set_text(GTK_LABEL(help_ulabel[3][4]), buf);
+    if (A_NEVER == utype->require_advance) {
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_LABEL_NEVER);
     } else {
-      gtk_set_label(help_ulabel[4][1], advances[utype->tech_requirement].name);
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]),
+			 advance_name_for_player(client.conn.playing,
+				       advance_number(utype->require_advance)));
     }
-/*    create_tech_tree(help_improvement_tree, 0, utype->tech_requirement, 3);*/
-    if(utype->obsoleted_by==-1) {
-      gtk_set_label(help_ulabel[4][4], _("None"));
+/*    create_tech_tree(help_improvement_tree, 0, advance_number(utype->require_advance), 3);*/
+    if (U_NOT_OBSOLETED == utype->obsoleted_by) {
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_LABEL_NONE);
     } else {
-      gtk_set_label(help_ulabel[4][4], get_unit_type(utype->obsoleted_by)->name);
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]),
+			 utype_name_translation(utype->obsoleted_by));
     }
 
-    helptext_unit(buf, i, pitem->text);
+    helptext_unit(buf, sizeof(buf), client.conn.playing, pitem->text, utype);
 
     gtk_text_buffer_set_text(help_text, buf, -1);
     gtk_widget_show(help_text_sw);
 
-    create_overlay_unit(unit_tile
-    , i);
-    gtk_widget_show(unit_tile);
+    gtk_pixcomm_freeze(GTK_PIXCOMM(help_tile));
+    {
+      struct canvas store;
+
+      store.type = CANVAS_PIXCOMM;
+      store.v.pixcomm = GTK_PIXCOMM(help_tile);
+      create_overlay_unit(&store, utype);
+    }
+    gtk_pixcomm_thaw(GTK_PIXCOMM(help_tile));
+    gtk_widget_show(help_tile);
   }
   else {
-    gtk_set_label(help_ulabel[0][1], "0");
-    gtk_set_label(help_ulabel[0][4], "0");
-    gtk_set_label(help_ulabel[1][1], "0");
-    gtk_set_label(help_ulabel[1][4], "0");
-    gtk_set_label(help_ulabel[2][1], "0");
-    gtk_set_label(help_ulabel[2][4], "0");
-    gtk_set_label(help_ulabel[3][1], "0");
-    gtk_set_label(help_ulabel[3][4], "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[0][1]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[0][4]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[1][1]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[1][4]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[2][1]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[2][4]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[3][1]), "0");
+    gtk_label_set_text(GTK_LABEL(help_ulabel[3][4]), "0");
 
-    gtk_set_label(help_ulabel[4][1], _("(Never)"));
+    gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_LABEL_NEVER);
 /*    create_tech_tree(help_improvement_tree, 0, A_LAST, 3);*/
-    gtk_set_label(help_ulabel[4][4], _("None"));
+    gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_LABEL_NONE);
 
     gtk_text_buffer_set_text(help_text, buf, -1);
     gtk_widget_show(help_text_sw);
@@ -847,15 +888,16 @@ static char *my_chomp(char *str, size_t len)
 /**************************************************************************
 ...
 **************************************************************************/
-static void help_update_tech(const struct help_item *pitem, char *title, int i)
+static void help_update_tech(const struct help_item *pitem, char *title)
 {
-  int j;
+  int i, j;
   GtkWidget *w, *hbox;
-  char *buf = &long_buffer[0];
+  char buf[8192];
+  struct advance *padvance = find_advance_by_translated_name(title);
 
   create_help_page(HELP_TECH);
 
-  if (!is_future_tech(i)) {
+  if (padvance  &&  !is_future_tech(i = advance_number(padvance))) {
     GtkTextBuffer *txt;
     size_t len;
 
@@ -869,19 +911,23 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_widget_show(help_tree_sw);
     gtk_widget_show(help_tree_buttons_hbox);
 
-    helptext_tech(buf, i, pitem->text);
+    helptext_advance(buf, sizeof(buf), client.conn.playing, pitem->text, i);
     len = strlen(buf);
-    if (advances[i].helptext) {
-      if (len) strcat(buf, "\n");
-      sprintf(buf+len, "%s\n", _(advances[i].helptext));
-      len = strlen(buf);
-    }
-
     my_chomp(buf, len);
 
+    if (get_tech_sprite(tileset, i)) {
+      GdkPixbuf *pixbuf = sprite_get_pixbuf(get_tech_sprite(tileset, i));
+
+      w = gtk_image_new_from_pixbuf(pixbuf);
+      gtk_widget_set_name(w, "help_tech_image");
+      gtk_box_pack_start(GTK_BOX(help_vbox), w, FALSE, FALSE, 0);
+      gtk_widget_show(w);
+    }
+
     w = gtk_text_view_new();
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), FALSE);
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
-    gtk_widget_set_name(w, "help text");
+    gtk_widget_set_name(w, "help_text");
     gtk_container_set_border_width(GTK_CONTAINER(w), 5);
     gtk_text_view_set_editable(GTK_TEXT_VIEW(w), FALSE);
     gtk_box_pack_start(GTK_BOX(help_vbox), w, TRUE, TRUE, 0);
@@ -896,83 +942,95 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_box_pack_start(GTK_BOX(help_vbox), w, TRUE, TRUE, 5);
     gtk_widget_show(w);
 
-    impr_type_iterate(j) {
-      if(i==improvement_types[j].tech_req) {
-        hbox = gtk_hbox_new(FALSE, 0);
-        gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
-        w = gtk_label_new(_("Allows"));
-        gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(improvement_types[j].name,
-			  is_wonder(j)?HELP_WONDER:HELP_IMPROVEMENT);
-        gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        gtk_widget_show_all(hbox);
-      }
-      if(i==improvement_types[j].obsolete_by) {
+    improvement_iterate(pimprove) {
+      /* FIXME: need a more general mechanism for this, since this
+       * helptext needs to be shown in all possible req source types. */
+      requirement_vector_iterate(&pimprove->reqs, preq) {
+	if (VUT_ADVANCE == preq->source.kind
+	    && preq->source.value.advance == padvance) {
+	  hbox = gtk_hbox_new(FALSE, 0);
+	  gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
+	  w = gtk_label_new(_("Allows"));
+	  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	  w = help_slink_new(improvement_name_translation(pimprove),
+			     is_great_wonder(pimprove)
+			     ? HELP_WONDER
+			     : HELP_IMPROVEMENT);
+	  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	  gtk_widget_show_all(hbox);
+	}
+      } requirement_vector_iterate_end;
+      if (padvance == pimprove->obsolete_by) {
         hbox = gtk_hbox_new(FALSE, 0);
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
         w = gtk_label_new(_("Obsoletes"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(improvement_types[j].name,
-			  is_wonder(j)?HELP_WONDER:HELP_IMPROVEMENT);
+        w = help_slink_new(improvement_name_translation(pimprove),
+			   is_great_wonder(pimprove)
+			   ? HELP_WONDER
+			   : HELP_IMPROVEMENT);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         gtk_widget_show_all(hbox);
       }
-    } impr_type_iterate_end;
+    } improvement_iterate_end;
 
-    unit_type_iterate(j) {
-      if(i!=get_unit_type(j)->tech_requirement) continue;
+    unit_type_iterate(punittype) {
+      if (padvance != punittype->require_advance) {
+	continue;
+      }
       hbox = gtk_hbox_new(FALSE, 0);
       gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
       w = gtk_label_new(_("Allows"));
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-      w = help_slink_new(get_unit_type(j)->name, HELP_UNIT);
+      w = help_slink_new(utype_name_translation(punittype), HELP_UNIT);
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
       gtk_widget_show_all(hbox);
     } unit_type_iterate_end;
 
-    for (j = 0; j < game.num_tech_types; j++) {
-      if(i==advances[j].req[0]) {
-	if(advances[j].req[1]==A_NONE) {
+    advance_iterate(A_NONE, ptest) {
+      if (padvance == advance_requires(ptest, AR_ONE)) {
+	if (advance_by_number(A_NONE) == advance_requires(ptest, AR_TWO)) {
           hbox = gtk_hbox_new(FALSE, 0);
           gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
           w = gtk_label_new(_("Allows"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(advances[j].name, HELP_TECH);
+          w = help_slink_new(advance_name_translation(ptest), HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           gtk_widget_show_all(hbox);
-	}
-	else {
+	} else {
           hbox = gtk_hbox_new(FALSE, 0);
           gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
           w = gtk_label_new(_("Allows"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(advances[j].name, HELP_TECH);
+          w = help_slink_new(advance_name_translation(ptest), HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = gtk_label_new(_("with"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(advances[advances[j].req[1]].name, HELP_TECH);
+          w = help_slink_new(advance_name_translation(advance_requires(ptest, AR_TWO)),
+                             HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = gtk_label_new(Q_("?techhelp:"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           gtk_widget_show_all(hbox);
 	}
       }
-      if(i==advances[j].req[1]) {
+      if (padvance == advance_requires(ptest, AR_TWO)) {
         hbox = gtk_hbox_new(FALSE, 0);
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
         w = gtk_label_new(_("Allows"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(advances[j].name, HELP_TECH);
+        w = help_slink_new(advance_name_translation(ptest), HELP_TECH);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         w = gtk_label_new(_("with"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(advances[advances[j].req[0]].name, HELP_TECH);
+        w = help_slink_new(advance_name_translation(advance_requires(ptest, AR_ONE)),
+                           HELP_TECH);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         w = gtk_label_new(Q_("?techhelp:"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         gtk_widget_show_all(hbox);
       }
-    }
+    } advance_iterate_end;
     gtk_widget_show(help_vbox);
   }
 }
@@ -981,121 +1039,97 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 ...
 **************************************************************************/
 static void help_update_terrain(const struct help_item *pitem,
-				char *title, int i)
+				char *title)
 {
-  char *buf = &long_buffer[0];
+  char buf[8192];
+  struct terrain *pterrain = find_terrain_by_translated_name(title);
 
   create_help_page(HELP_TERRAIN);
 
-  if (i < T_COUNT)
-    {
-      sprintf (buf, "%d/%d.%d",
-	       tile_types[i].movement_cost,
-	       (int)(tile_types[i].defense_bonus/10), tile_types[i].defense_bonus%10);
-      gtk_set_label (help_tlabel[0][1], buf);
+  if (pterrain) {
+    sprintf(buf, "%d/%d.%d",
+	    pterrain->movement_cost,
+	    (int)((pterrain->defense_bonus + 100) / 100),
+	    (pterrain->defense_bonus + 100) % 100 / 10);
+    gtk_label_set_text(GTK_LABEL(help_tlabel[0][1]), buf);
 
-      sprintf (buf, "%d/%d/%d",
-	       tile_types[i].food,
-	       tile_types[i].shield,
-	       tile_types[i].trade);
-      gtk_set_label (help_tlabel[0][4], buf);
+    sprintf(buf, "%d/%d/%d",
+	    pterrain->output[O_FOOD],
+	    pterrain->output[O_SHIELD],
+	    pterrain->output[O_TRADE]);
+    gtk_label_set_text(GTK_LABEL(help_tlabel[0][4]), buf);
 
-      if (*(tile_types[i].special_1_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_1_name);
-	  gtk_set_label (help_tlabel[1][0], buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_1,
-		   tile_types[i].shield_special_1,
-		   tile_types[i].trade_special_1);
-	  gtk_set_label (help_tlabel[1][1], buf);
-	} else {
-	  gtk_set_label (help_tlabel[1][0], "");
-	  gtk_set_label (help_tlabel[1][1], "");
-	}
+    buf[0] = '\0';
+    if (*(pterrain->resources)) {
+      struct resource **r;
 
-      if (*(tile_types[i].special_2_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_2_name);
-	  gtk_set_label (help_tlabel[1][3], buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_2,
-		   tile_types[i].shield_special_2,
-		   tile_types[i].trade_special_2);
-	  gtk_set_label (help_tlabel[1][4], buf);
-	} else {
-	  gtk_set_label (help_tlabel[1][3], "");
-	  gtk_set_label (help_tlabel[1][4], "");
-	}
-
-      if (tile_types[i].road_trade_incr > 0)
-	{
-	  sprintf (buf, _("+%d Trade / %d"),
-		   tile_types[i].road_trade_incr,
-		   tile_types[i].road_time);
-	}
-      else if (tile_types[i].road_time > 0)
-	{
-	  sprintf (buf, _("no extra / %d"),
-		   tile_types[i].road_time);
-	}
-      else
-	{
-	  strcpy (buf, _("n/a"));
-	}
-      gtk_set_label (help_tlabel[2][1], buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].irrigation_result == i)
-	{
-	  if (tile_types[i].irrigation_food_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Food / %d"),
-		       tile_types[i].irrigation_food_incr,
-		       tile_types[i].irrigation_time);
-	    }
-	}
-      else if (tile_types[i].irrigation_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].irrigation_result].terrain_name,
-		   tile_types[i].irrigation_time);
-	}
-      gtk_set_label (help_tlabel[2][4], buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].mining_result == i)
-	{
-	  if (tile_types[i].mining_shield_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Res. / %d"),
-		       tile_types[i].mining_shield_incr,
-		       tile_types[i].mining_time);
-	    }
-	}
-      else if (tile_types[i].mining_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].mining_result].terrain_name,
-		   tile_types[i].mining_time);
-	}
-      gtk_set_label (help_tlabel[3][1], buf);
-
-      if (tile_types[i].transform_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].transform_result].terrain_name,
-		   tile_types[i].transform_time);
-	} else {
-	  strcpy (buf, "n/a");
-	}
-      gtk_set_label (help_tlabel[3][4], buf);
+      for (r = pterrain->resources; *r; r++) {
+        /* TRANS: " Whales (2/1/2)," */
+        sprintf (buf + strlen (buf), " %s (%d/%d/%d),",
+                 resource_name_translation(*r),
+                 pterrain->output[O_FOOD]   + (*r)->output[O_FOOD],
+                 pterrain->output[O_SHIELD] + (*r)->output[O_SHIELD],
+                 pterrain->output[O_TRADE]  + (*r)->output[O_TRADE]);
+      }
+      buf[strlen (buf) - 1] = '.';
+    } else {
+      /* TRANS: "Resources: (none)" */
+      sprintf (buf + strlen (buf), _("(none)"));
     }
+    gtk_label_set_text(GTK_LABEL(help_tlabel[1][1]), buf);
+
+    if (pterrain->road_trade_incr > 0) {
+      sprintf(buf, _("+%d Trade / %d"),
+	      pterrain->road_trade_incr,
+	      pterrain->road_time);
+    } else if (pterrain->road_time > 0) {
+      sprintf(buf, _("no extra / %d"),
+	      pterrain->road_time);
+    } else {
+      strcpy(buf, _("n/a"));
+    }
+    gtk_label_set_text(GTK_LABEL(help_tlabel[2][1]), buf);
+
+    strcpy(buf, _("n/a"));
+    if (pterrain->irrigation_result == pterrain) {
+      if (pterrain->irrigation_food_incr > 0) {
+	sprintf(buf, _("+%d Food / %d"),
+		pterrain->irrigation_food_incr,
+		pterrain->irrigation_time);
+      }
+    } else if (pterrain->irrigation_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->irrigation_result),
+	      pterrain->irrigation_time);
+    }
+    gtk_label_set_text(GTK_LABEL(help_tlabel[2][4]), buf);
+
+    strcpy(buf, _("n/a"));
+    if (pterrain->mining_result == pterrain) {
+      if (pterrain->mining_shield_incr > 0) {
+	sprintf(buf, _("+%d Res. / %d"),
+		pterrain->mining_shield_incr,
+		pterrain->mining_time);
+      }
+    } else if (pterrain->mining_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->mining_result),
+	      pterrain->mining_time);
+    }
+    gtk_label_set_text(GTK_LABEL(help_tlabel[3][1]), buf);
+
+    if (pterrain->transform_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->transform_result),
+	      pterrain->transform_time);
+    } else {
+      strcpy(buf, "n/a");
+    }
+    gtk_label_set_text(GTK_LABEL(help_tlabel[3][4]), buf);
+  }
+
+  helptext_terrain(buf, sizeof(buf), client.conn.playing, pitem->text, pterrain);
 
-  helptext_terrain(buf, i, pitem->text);
-  
   gtk_text_buffer_set_text(help_text, buf, -1);
   gtk_widget_show(help_text_sw);
 
@@ -1106,14 +1140,15 @@ static void help_update_terrain(const struct help_item *pitem,
   This is currently just a text page, with special text:
 **************************************************************************/
 static void help_update_government(const struct help_item *pitem,
-				   char *title, struct government *gov)
+				   char *title)
 {
-  char *buf = &long_buffer[0];
+  char buf[8192];
+  struct government *gov = find_government_by_translated_name(title);
 
   if (!gov) {
     strcat(buf, pitem->text);
   } else {
-    helptext_government(buf, gov-governments, pitem->text);
+    helptext_government(buf, sizeof(buf), client.conn.playing, pitem->text, gov);
   }
   create_help_page(HELP_TEXT);
   gtk_text_buffer_set_text(help_text, buf, -1);
@@ -1125,7 +1160,6 @@ static void help_update_government(const struct help_item *pitem,
 **************************************************************************/
 static void help_update_dialog(const struct help_item *pitem)
 {
-  int i;
   char *top;
 
   /* figure out what kind of item is required for pitem ingo */
@@ -1139,26 +1173,22 @@ static void help_update_dialog(const struct help_item *pitem)
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && is_wonder(i)) i = B_LAST;
-    help_update_improvement(pitem, top, i);
+    help_update_improvement(pitem, top);
     break;
   case HELP_WONDER:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && !is_wonder(i)) i = B_LAST;
-    help_update_wonder(pitem, top, i);
+    help_update_wonder(pitem, top);
     break;
   case HELP_UNIT:
-    help_update_unit_type(pitem, top, find_unit_type_by_name(top));
+    help_update_unit_type(pitem, top);
     break;
   case HELP_TECH:
-    help_update_tech(pitem, top, find_tech_by_name(top));
+    help_update_tech(pitem, top);
     break;
   case HELP_TERRAIN:
-    help_update_terrain(pitem, top, get_terrain_by_name(top));
+    help_update_terrain(pitem, top);
     break;
   case HELP_GOVERNMENT:
-    help_update_government(pitem, top, find_government_by_name(top));
+    help_update_government(pitem, top);
     break;
   case HELP_TEXT:
   default:
