@@ -18,6 +18,7 @@
 #include <assert.h>
 #include <gtk/gtk.h>
 
+/* common & utility */
 #include "combat.h"
 #include "fcintl.h"
 #include "game.h"
@@ -28,15 +29,17 @@
 
 #include "overview_common.h"
 
+/* client */
 #include "chatline.h"
 #include "citydlg.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climap.h"
-#include "clinet.h"
 #include "climisc.h"
 #include "colors.h"
 #include "control.h"
 #include "dialogs.h"
+#include "editgui.h"
+#include "editor.h"
 #include "graphics.h"
 #include "gui_main.h"
 #include "inputdlg.h"
@@ -113,7 +116,7 @@ static void popit(GdkEventButton *event, struct tile *ptile)
   static struct tmousepos mousepos;
   struct unit *punit;
 
-  if (client_tile_get_known(ptile) >= TILE_KNOWN_FOGGED) {
+  if (TILE_UNKNOWN != client_tile_get_known(ptile)) {
     p=gtk_window_new(GTK_WINDOW_POPUP);
     gtk_widget_set_app_paintable(p, TRUE);
     gtk_container_set_border_width(GTK_CONTAINER(p), 4);
@@ -158,14 +161,23 @@ void popupinfo_popdown_callback(GtkWidget *w, gpointer data)
   mapdeco_clear_gotoroutes();
 }
 
- /**************************************************************************
-...
+/**************************************************************************
+  Callback from city name dialog for new city.
 **************************************************************************/
-static void name_new_city_callback(GtkWidget * w, gpointer data)
+static void name_new_city_popup_callback(gpointer data, gint response,
+                                         const char *input)
 {
-  dsend_packet_unit_build_city(&aconnection, GPOINTER_TO_INT(data),
-			       input_dialog_get_input(w));
-  input_dialog_destroy(w);
+  int idx = GPOINTER_TO_INT(data);
+
+  switch (response) {
+  case GTK_RESPONSE_OK:
+    finish_city(index_to_tile(idx), input);
+    break;
+  case GTK_RESPONSE_CANCEL:
+  case GTK_RESPONSE_DELETE_EVENT:
+    cancel_city(index_to_tile(idx));
+    break;
+  }
 }
 
 /**************************************************************************
@@ -176,12 +188,10 @@ static void name_new_city_callback(GtkWidget * w, gpointer data)
 void popup_newcity_dialog(struct unit *punit, char *suggestname)
 {
   input_dialog_create(GTK_WINDOW(toplevel), /*"shellnewcityname" */
-		     _("Build New City"),
-		     _("What should we call our new city?"), suggestname,
-		     G_CALLBACK(name_new_city_callback),
-                     GINT_TO_POINTER(punit->id),
-		     G_CALLBACK(name_new_city_callback),
-                     GINT_TO_POINTER(0));
+                      _("Build New City"),
+                      _("What should we call our new city?"), suggestname,
+                      name_new_city_popup_callback,
+                      GINT_TO_POINTER(tile_index(unit_tile(punit))));
 }
 
 /**************************************************************************
@@ -199,6 +209,10 @@ void set_turn_done_button_state(bool state)
 **************************************************************************/
 gboolean butt_release_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
+  if (editor_is_active()) {
+    return handle_edit_mouse_button_release(ev);
+  }
+
   if (ev->button == 1 || ev->button == 3) {
     release_goto_button(ev->x, ev->y);
   }
@@ -218,13 +232,17 @@ gboolean butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
   struct city *pcity = NULL;
   struct tile *ptile = NULL;
 
+  if (editor_is_active()) {
+    return handle_edit_mouse_button_press(ev);
+  }
+
   if (!can_client_change_view()) {
     return TRUE;
   }
 
   gtk_widget_grab_focus(map_canvas);
   ptile = canvas_pos_to_tile(ev->x, ev->y);
-  pcity = ptile ? ptile->city : NULL;
+  pcity = ptile ? tile_city(ptile) : NULL;
 
   switch (ev->button) {
 
@@ -274,17 +292,31 @@ gboolean butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
 
   case 3: /* RIGHT mouse button */
 
+    /* <CONTROL> + <ALT> + RMB : insert city or tile chat link. */
+    /* <CONTROL> + <ALT> + <SHIFT> + RMB : insert unit chat link. */
+    if (ptile && (ev->state & GDK_MOD1_MASK)
+        && (ev->state & GDK_CONTROL_MASK)) {
+      inputline_make_chat_link(ptile, (ev->state & GDK_SHIFT_MASK) != 0);
+    }
+    /* <SHIFT> + <ALT> + RMB : Show/hide workers. */
+    else if ((ev->state & GDK_SHIFT_MASK) && (ev->state & GDK_MOD1_MASK)) {
+      overlay_workers_at_city();
+    }
     /* <SHIFT + CONTROL> + RMB: Paste Production. */
-    if ((ev->state & GDK_SHIFT_MASK) && (ev->state & GDK_CONTROL_MASK)
-        && pcity != NULL) {
+    else if ((ev->state & GDK_SHIFT_MASK) && (ev->state & GDK_CONTROL_MASK)
+             && pcity != NULL) {
       clipboard_paste_production(pcity);
       cancel_tile_hiliting();
     }
+    /* <SHIFT> + RMB: Copy Production. */
+    else if (ev->state & GDK_SHIFT_MASK) {
+      clipboard_copy_production(ptile);
+    }
     /* <CONTROL> + RMB : Quickselect a land unit. */
     else if (ev->state & GDK_CONTROL_MASK) {
       action_button_pressed(ev->x, ev->y, SELECT_LAND);
     }
-    /* Plain RMB click, possibly with <SHIFT>. */
+    /* Plain RMB click. */
     else {
       /*  A foolproof user will depress button on canvas,
        *  release it on another widget, and return to canvas
@@ -355,10 +387,14 @@ void update_rect_at_mouse_pos(void)
 **************************************************************************/
 gboolean move_mapcanvas(GtkWidget *w, GdkEventMotion *ev, gpointer data)
 {
-  if (!GTK_WIDGET_HAS_FOCUS(map_canvas)) {
+  if (gui_gtk2_mouse_over_map_focus && !GTK_WIDGET_HAS_FOCUS(map_canvas)) {
     gtk_widget_grab_focus(map_canvas);
   }
 
+  if (editor_is_active()) {
+    return handle_edit_mouse_move(ev);
+  }
+
   cur_x = ev->x;
   cur_y = ev->y;
   update_line(ev->x, ev->y);
@@ -381,10 +417,10 @@ gboolean leave_mapcanvas(GtkWidget *widget, GdkEventCrossing *event)
   if (gtk_notebook_get_current_page(GTK_NOTEBOOK(top_notebook))
       != gtk_notebook_page_num(GTK_NOTEBOOK(top_notebook), map_widget)) {
     /* Map is not currently topmost tab. Do not use tile specific cursors. */
-    action_state = CURSOR_ACTION_DEFAULT;
+    update_mouse_cursor(CURSOR_DEFAULT);
     return TRUE;
   }
-  
+
   /* Bizarrely, this function can be called even when we don't "leave"
    * the map canvas, for instance, it gets called any time the mouse is
    * clicked. */
@@ -394,7 +430,7 @@ gboolean leave_mapcanvas(GtkWidget *widget, GdkEventCrossing *event)
       && canvas_x < mapview.width && canvas_y < mapview.height) {
     control_mouse_cursor(canvas_pos_to_tile(canvas_x, canvas_y));
   } else {
-    action_state = CURSOR_ACTION_DEFAULT;
+    update_mouse_cursor(CURSOR_DEFAULT);
   }
 
   update_unit_info_label(get_units_in_focus());
@@ -433,7 +469,7 @@ gboolean butt_down_overviewcanvas(GtkWidget *w, GdkEventButton *ev, gpointer dat
 }
 
 /**************************************************************************
-  Draws the on the map the tiles the given city is using
+  Best effort to center the map on the currently selected unit(s)
 **************************************************************************/
 void center_on_unit(void)
 {
@@ -441,9 +477,9 @@ void center_on_unit(void)
 }
 
 /**************************************************************************
-  Draws the on the map the tiles the given city is using
+  Shows/hides overlay on the map for the city at this location
 **************************************************************************/
-void key_city_workers(void)
+void overlay_workers_at_city(void)
 {
   int x, y;
   
