@@ -195,7 +195,7 @@ struct sprite *load_gfxfile(const char *filename)
   GdkPixbuf *im;
 
   if (!(im = gdk_pixbuf_new_from_file(filename, NULL))) {
-    freelog(LOG_FATAL, "Failed reading graphics file: %s", filename);
+    freelog(LOG_FATAL, "Failed reading graphics file: \"%s\"", filename);
     exit(EXIT_FAILURE);
   }
 
