@@ -23,6 +23,9 @@
 /* common */
 #include "game.h"
 
+/* client */
+#include "client_main.h"
+
 /* gui-sdl */
 #include "citydlg.h"
 #include "cma_fe.h"
@@ -87,7 +90,7 @@ static int popup_citydlg_from_city_report_callback(struct widget *pWidget)
 static int popup_worklist_from_city_report_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    popup_worklist_editor(pWidget->data.city, &pWidget->data.city->worklist);
+    popup_worklist_editor(pWidget->data.city, NULL);
   }
   return -1;
 }
@@ -256,7 +259,7 @@ static void real_info_city_report_dialog_update(void)
   
   pLast = pBuf;
   count = 0; 
-  city_list_iterate(game.player_ptr->cities, pCity) {
+  city_list_iterate(client.conn.playing->cities, pCity) {
     
     pStr = create_str16_from_char(city_name(pCity), adj_font(12));
     pStr->style |= TTF_STYLE_BOLD;
@@ -493,19 +496,20 @@ static void real_info_city_report_dialog_update(void)
     add_to_gui_list(MAX_ID - pCity->id, pBuf);
 
     /* ----------- */
-    if(pCity->production.is_unit) {
-      struct unit_type *pUnitType = utype_by_number(pCity->production.value);
+    if(VUT_UTYPE == pCity->production.kind) {
+      struct unit_type *pUnitType = pCity->production.value.utype;
       pLogo = ResizeSurfaceBox(get_unittype_surface(pUnitType),
                                adj_size(36), adj_size(24), 1,
-                               TRUE, TRUE);      
-      togrow = unit_build_shield_cost(pUnitType);
+                               TRUE, TRUE);
+      togrow = utype_build_shield_cost(pUnitType);
       pName = utype_name_translation(pUnitType);
     } else {
-      pLogo = ResizeSurfaceBox(get_building_surface(pCity->production.value),
+      struct impr_type *pImprove = pCity->production.value.building;
+      pLogo = ResizeSurfaceBox(get_building_surface(pCity->production.value.building),
                                adj_size(36), adj_size(24), 1,
                                TRUE, TRUE);
-      togrow = impr_build_shield_cost(pCity->production.value);
-      pName = improvement_name_translation(pCity->production.value);
+      togrow = impr_build_shield_cost(pImprove);
+      pName = improvement_name_translation(pImprove);
     }
     
     if(!worklist_is_empty(&(pCity->worklist))) {
@@ -513,9 +517,7 @@ static void real_info_city_report_dialog_update(void)
       dst.y = 0;
       alphablit(pIcons->pWorklist, NULL, pLogo, &dst);
       my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)\n%s",
-      	pName, pCity->shield_stock, togrow,
-		pCity->worklist.name[0] != '\0' ?
-			      pCity->worklist.name : _("worklist"));
+      	pName, pCity->shield_stock, togrow, _("worklist"));
     } else {
       my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)%s",
       	pName, pCity->shield_stock, togrow,
@@ -526,8 +528,7 @@ static void real_info_city_report_dialog_update(void)
     pStr = create_str16_from_char(cBuf, adj_font(10));
     pStr->style |= SF_CENTER;
     
-    togrow = city_turns_to_build(pCity,
-    	pCity->production, TRUE);
+    togrow = city_production_turns_to_build(pCity, TRUE);
     if(togrow == 999)
     {
       my_snprintf(cBuf, sizeof(cBuf), "%s", _("never"));
@@ -646,8 +647,9 @@ static void real_info_city_report_dialog_update(void)
   dst.h = area.h - adj_size(2);
   SDL_FillRectAlpha(pWindow->theme, &dst, &bg_color);
   
-  putframe(pWindow->theme, dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
-    map_rgba(pWindow->theme->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putframe(pWindow->theme,
+           dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
+           get_game_colorRGB(COLOR_THEME_CITYREP_FRAME));
   
   dst.y += (pText2->h - pText3->h) / 2;
   dst.x += ((name_w + adj_size(6)) - pText3->w) / 2;
@@ -678,8 +680,9 @@ static void real_info_city_report_dialog_update(void)
   dst.h = area.h - adj_size(2);
   SDL_FillRectAlpha(pWindow->theme, &dst, get_game_colorRGB(COLOR_THEME_CITYREP_FOODSTOCK));
   
-  putframe(pWindow->theme, dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
-           map_rgba(pWindow->theme->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putframe(pWindow->theme,
+           dst.x, dst.y, dst.x + dst.w, dst.y + dst.h - 1,
+           get_game_colorRGB(COLOR_THEME_CITYREP_FRAME));
   
   dst.y = area.y + 1 + (pText2->h - pIcons->pBIG_Food->h) / 2;
   dst.x += adj_size(5);
@@ -713,8 +716,9 @@ static void real_info_city_report_dialog_update(void)
   
   SDL_FillRectAlpha(pWindow->theme, &dst, get_game_colorRGB(COLOR_THEME_CITYREP_TRADE));
   
-  putframe(pWindow->theme, dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
-           map_rgba(pWindow->theme->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putframe(pWindow->theme,
+           dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
+           get_game_colorRGB(COLOR_THEME_CITYREP_FRAME));
   
   dst.y = area.y + 1 + (hh - pIcons->pBIG_Trade->h) / 2;
   dst.x += adj_size(5);
@@ -750,8 +754,9 @@ static void real_info_city_report_dialog_update(void)
   
   SDL_FillRectAlpha(pWindow->theme, &dst, get_game_colorRGB(COLOR_THEME_CITYREP_PROD));
   
-  putframe(pWindow->theme, dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
-    map_rgba(pWindow->theme->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putframe(pWindow->theme,
+           dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
+           get_game_colorRGB(COLOR_THEME_CITYREP_FRAME));
   
   dst.y = area.y + 1 + (hh - pIcons->pBIG_Shield->h) / 2;
   dst.x += adj_size(5);
@@ -782,8 +787,9 @@ static void real_info_city_report_dialog_update(void)
     
   SDL_FillRectAlpha(pWindow->theme, &dst, get_game_colorRGB(COLOR_THEME_CITYREP_PROD));
   
-  putframe(pWindow->theme, dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
-    map_rgba(pWindow->theme->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putframe(pWindow->theme,
+           dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
+           get_game_colorRGB(COLOR_THEME_CITYREP_FRAME));
   
   dst.y = area.y + 1 + (hh - pText4->h) / 2;
   dst.x += (dst.w - pText4->w) / 2;;
@@ -1018,16 +1024,18 @@ static struct widget * real_city_report_dialog_update_city(struct widget *pWidge
   copy_chars_to_string16(pWidget->string16, cBuf);
   
   /* change production */
-  if(pCity->production.is_unit) {
-    struct unit_type *pUnitType = utype_by_number(pCity->production.value);
-    pLogo = ResizeSurface(get_unittype_surface(utype_by_number(pCity->production.value)),
+  if(VUT_UTYPE == pCity->production.kind) {
+    struct unit_type *pUnitType = pCity->production.value.utype;
+    pLogo = ResizeSurface(get_unittype_surface(pUnitType),
               adj_size(36), adj_size(24), 1);
-    togrow = unit_build_shield_cost(pUnitType);
+    togrow = utype_build_shield_cost(pUnitType);
     pName = utype_name_translation(pUnitType);
   } else {
-    pLogo = ResizeSurface(get_building_surface(pCity->production.value), adj_size(36), adj_size(24), 1);
-    togrow = impr_build_shield_cost(pCity->production.value);
-    pName = improvement_name_translation(pCity->production.value);
+    struct impr_type *pImprove = pCity->production.value.building;
+    pLogo = ResizeSurface(get_building_surface(pCity->production.value.building),
+              adj_size(36), adj_size(24), 1);
+    togrow = impr_build_shield_cost(pImprove);
+    pName = improvement_name_translation(pImprove);
   }
     
   if(!worklist_is_empty(&(pCity->worklist))) {
@@ -1035,9 +1043,7 @@ static struct widget * real_city_report_dialog_update_city(struct widget *pWidge
     dst.y = 0;
     alphablit(pIcons->pWorklist, NULL, pLogo, &dst);
     my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)\n%s",
-      	pName, pCity->shield_stock, togrow,
-		pCity->worklist.name[0] != '\0' ?
-			      pCity->worklist.name : _("worklist"));
+      	pName, pCity->shield_stock, togrow, _("worklist"));
   } else {
     my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)",
       		pName, pCity->shield_stock, togrow);
@@ -1051,8 +1057,7 @@ static struct widget * real_city_report_dialog_update_city(struct widget *pWidge
   
   /* hurry productions */
   pWidget = pWidget->prev;
-  togrow = city_turns_to_build(pCity,
-    	pCity->production, TRUE);
+  togrow = city_production_turns_to_build(pCity, TRUE);
   if(togrow == 999)
   {
     my_snprintf(cBuf, sizeof(cBuf), "%s", _("never"));
@@ -1095,7 +1100,7 @@ void city_report_dialog_update(void)
   
     /* find if the lists are identical (if not then rebuild all) */
     pWidget = pCityRep->pEndActiveWidgetList;/* name of first city */
-    city_list_iterate(game.player_ptr->cities, pCity) {
+    city_list_iterate(client.conn.playing->cities, pCity) {
       if (pCity->id == MAX_ID - pWidget->ID) {
         count = COL;
         while(count) {
@@ -1115,7 +1120,7 @@ void city_report_dialog_update(void)
 
     /* update widget city list (widget list is the same that city list) */
     pWidget = pCityRep->pEndActiveWidgetList;
-    city_list_iterate(game.player_ptr->cities, pCity) {
+    city_list_iterate(client.conn.playing->cities, pCity) {
       pWidget = real_city_report_dialog_update_city(pWidget, pCity);  
     } city_list_iterate_end;
 
