@@ -207,7 +207,7 @@ SDL_Surface *mask_surface(SDL_Surface * pSrc, SDL_Surface * pMask,
                           int mask_offset_x, int mask_offset_y)
 {
   SDL_Surface *pDest = NULL;
-  
+
   int row, col;  
   bool free_pMask = FALSE;
   Uint32 *pSrc_Pixel = NULL;
@@ -262,10 +262,17 @@ SDL_Surface *mask_surface(SDL_Surface * pSrc, SDL_Surface * pMask,
 
 SDL_Surface *blend_surface(SDL_Surface *pSrc, unsigned char alpha)
 {
+  SDL_Surface *ret;
+
   SDL_Surface *pMask = SDL_DisplayFormatAlpha(pSrc);
   SDL_Color c = {255, 255, 255, alpha}; 
   SDL_FillRect(pMask, NULL, map_rgba(pMask->format, c));
-  return mask_surface(pSrc, pMask, 0, 0);   
+
+  ret = mask_surface(pSrc, pMask, 0, 0);
+
+  FREESURFACE(pMask);
+
+  return ret;
 }
 
 /**************************************************************************
@@ -546,6 +553,7 @@ void init_sdl(int iFlags)
   Main.guis = NULL;
   Main.gui = NULL;
   Main.map = NULL;
+  Main.dummy = NULL; /* can't create yet -- hope we don't need it */
   Main.rects_count = 0;
   Main.guis_count = 0;
 
@@ -580,6 +588,7 @@ void quit_sdl(void)
   FC_FREE(Main.guis);
   gui_layer_destroy(&Main.gui);
   FREESURFACE(Main.map);
+  FREESURFACE(Main.dummy);
 }
 
 /**************************************************************************
@@ -622,6 +631,12 @@ int set_video_mode(int iWidth, int iHeight, int iFlags)
 	  					iWidth, iHeight, iDepth);
   }
 
+  /* Create a dummy surface to be used when sprites are missing, etc.
+   * Various things (such as zoomSurfaceRGBA() don't cope well with a
+   * zero-sized surface, so we use a transparent 1x1 surface. */
+  FREESURFACE(Main.dummy);
+  Main.dummy = create_surf_alpha(1, 1, SDL_SWSURFACE);
+
   FREESURFACE(Main.map);
   Main.map = SDL_DisplayFormat(Main.screen);
   
