@@ -15,38 +15,31 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include <SDL/SDL.h>
+#include "SDL.h"
 
+/* utility */
+#include "astring.h"
 #include "fcintl.h"
 
-#include "game.h"
-#include "packets.h"
-#include "nation.h"
-#include "player.h"
-#include "support.h"
+/* common */
 
-#include "chatline.h"
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
-#include "graphics.h"
-#include "gui_main.h"
-#include "gui_id.h"
-#include "gui_tilespec.h"
-#include "gui_string.h"
-#include "gui_stuff.h"
-#include "gui_zoom.h"
-#include "inteldlg.h"
-#include "spaceshipdlg.h"
+
+/* gui-sdl */
 #include "colors.h"
-#include "mapview.h"
 #include "diplodlg.h"
+#include "graphics.h"
+#include "gui_id.h"
+#include "gui_main.h"
+#include "gui_tilespec.h"
+#include "inteldlg.h"
+#include "mapview.h"
+#include "sprite.h"
+#include "themespec.h"
+#include "widget.h"
+
 #include "plrdlg.h"
 
 #ifndef M_PI
@@ -58,14 +51,16 @@
 
 static struct SMALL_DLG  *pPlayers_Dlg = NULL;
 
-static int exit_players_dlg_callback(struct GUI *pWidget)
+static int exit_players_dlg_callback(struct widget *pWidget)
 {
-  popdown_players_dialog();
-  flush_dirty();
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    popdown_players_dialog();
+    flush_dirty();
+  }
   return -1;
 }
 
-static int player_callback(struct GUI *pWidget)
+static int player_callback(struct widget *pWidget)
 {
   struct player *pPlayer = pWidget->data.player;
   switch(Main.event.button.button) {
@@ -94,91 +89,103 @@ static int player_callback(struct GUI *pWidget)
   return -1;
 }
 
-static int players_window_dlg_callback(struct GUI *pWindow)
+static int players_window_dlg_callback(struct widget *pWindow)
 {
-  if (move_window_group_dialog(pPlayers_Dlg->pBeginWidgetList, pWindow)) {
-    sellect_window_group_dialog(pPlayers_Dlg->pBeginWidgetList, pWindow);
-    update_players_dialog();
-  } else {
-    if(sellect_window_group_dialog(pPlayers_Dlg->pBeginWidgetList, pWindow)) {
-      flush_rect(pWindow->size);
-    }      
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    if (move_window_group_dialog(pPlayers_Dlg->pBeginWidgetList, pWindow)) {
+      sellect_window_group_dialog(pPlayers_Dlg->pBeginWidgetList, pWindow);
+      update_players_dialog();
+    } else {
+      if(sellect_window_group_dialog(pPlayers_Dlg->pBeginWidgetList, pWindow)) {
+        widget_flush(pWindow);
+      }      
+    }
   }
   return -1;
 }
 
-static int toggle_draw_war_status_callback(struct GUI *pWidget)
+static int toggle_draw_war_status_callback(struct widget *pWidget)
 {
-  /* exit button -> neutral -> war -> casefire -> peace -> alliance */
-  struct GUI *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
-  SDL_Client_Flags ^= CF_DRAW_PLAYERS_WAR_STATUS;
-  do{
-    pPlayer = pPlayer->prev;
-    FREESURFACE(pPlayer->gfx);
-  } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
-  update_players_dialog();
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    /* exit button -> neutral -> war -> casefire -> peace -> alliance */
+    struct widget *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
+    SDL_Client_Flags ^= CF_DRAW_PLAYERS_WAR_STATUS;
+    do{
+      pPlayer = pPlayer->prev;
+      FREESURFACE(pPlayer->gfx);
+    } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
+    update_players_dialog();
+  }
   return -1;
 }
 
-static int toggle_draw_ceasefire_status_callback(struct GUI *pWidget)
+static int toggle_draw_ceasefire_status_callback(struct widget *pWidget)
 {
-  /* exit button -> neutral -> war -> casefire -> peace -> alliance */
-  struct GUI *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
-  SDL_Client_Flags ^= CF_DRAW_PLAYERS_CEASEFIRE_STATUS;
-  do{
-    pPlayer = pPlayer->prev;
-    FREESURFACE(pPlayer->gfx);
-  } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
-  update_players_dialog();
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    /* exit button -> neutral -> war -> casefire -> peace -> alliance */
+    struct widget *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
+    SDL_Client_Flags ^= CF_DRAW_PLAYERS_CEASEFIRE_STATUS;
+    do{
+      pPlayer = pPlayer->prev;
+      FREESURFACE(pPlayer->gfx);
+    } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
+    update_players_dialog();
+  }
   return -1;
 }
 
-static int toggle_draw_pease_status_callback(struct GUI *pWidget)
+static int toggle_draw_pease_status_callback(struct widget *pWidget)
 {
-  /* exit button -> neutral -> war -> casefire -> peace -> alliance */
-  struct GUI *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
-  SDL_Client_Flags ^= CF_DRAW_PLAYERS_PEACE_STATUS;
-  do{
-    pPlayer = pPlayer->prev;
-    FREESURFACE(pPlayer->gfx);
-  } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
-  update_players_dialog();
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    /* exit button -> neutral -> war -> casefire -> peace -> alliance */
+    struct widget *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
+    SDL_Client_Flags ^= CF_DRAW_PLAYERS_PEACE_STATUS;
+    do{
+      pPlayer = pPlayer->prev;
+      FREESURFACE(pPlayer->gfx);
+    } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
+    update_players_dialog();
+  }
   return -1;
 }
 
-static int toggle_draw_alliance_status_callback(struct GUI *pWidget)
+static int toggle_draw_alliance_status_callback(struct widget *pWidget)
 {
-  /* exit button -> neutral -> war -> casefire -> peace -> alliance */
-  struct GUI *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
-  SDL_Client_Flags ^= CF_DRAW_PLAYERS_ALLIANCE_STATUS;
-  do{
-    pPlayer = pPlayer->prev;
-    FREESURFACE(pPlayer->gfx);
-  } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
-  update_players_dialog();
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    /* exit button -> neutral -> war -> casefire -> peace -> alliance */
+    struct widget *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
+    SDL_Client_Flags ^= CF_DRAW_PLAYERS_ALLIANCE_STATUS;
+    do{
+      pPlayer = pPlayer->prev;
+      FREESURFACE(pPlayer->gfx);
+    } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
+    update_players_dialog();
+  }
   return -1;
 }
 
-static int toggle_draw_neutral_status_callback(struct GUI *pWidget)
+static int toggle_draw_neutral_status_callback(struct widget *pWidget)
 {
-  /* exit button -> neutral -> war -> casefire -> peace -> alliance */
-  struct GUI *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
-  
-  SDL_Client_Flags ^= CF_DRAW_PLAYERS_NEUTRAL_STATUS;
-  do{
-    pPlayer = pPlayer->prev;
-    FREESURFACE(pPlayer->gfx);
-  } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
-  update_players_dialog();
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    /* exit button -> neutral -> war -> casefire -> peace -> alliance */
+    struct widget *pPlayer = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
+    
+    SDL_Client_Flags ^= CF_DRAW_PLAYERS_NEUTRAL_STATUS;
+    do{
+      pPlayer = pPlayer->prev;
+      FREESURFACE(pPlayer->gfx);
+    } while(pPlayer != pPlayers_Dlg->pBeginWidgetList);
+    update_players_dialog();
+  }
   return -1;
 }
 
 
 static bool have_diplomat_info_about(struct player *pPlayer)
 {
-  return (pPlayer == game.player_ptr ||
-  	(pPlayer != game.player_ptr
-	  && player_has_embassy(game.player_ptr, pPlayer)));
+  return (pPlayer == client.conn.playing
+	  || (pPlayer != client.conn.playing
+	   && player_has_embassy(client.conn.playing, pPlayer)));
 }
 
 /**************************************************************************
@@ -187,13 +194,14 @@ static bool have_diplomat_info_about(struct player *pPlayer)
 void update_players_dialog(void)
 {
   if(pPlayers_Dlg) {
-    struct GUI *pPlayer0, *pPlayer1;
+    struct widget *pPlayer0, *pPlayer1;
     struct player *pPlayer;
-    char cBuf[128], *state, *team;
-    int idle, x0, y0, x1, y1;
+    SDL_Rect dst0, dst1;
+    int i;
+    struct astring astr = ASTRING_INIT;
           
     /* redraw window */
-    redraw_widget(pPlayers_Dlg->pEndWidgetList);
+    widget_redraw(pPlayers_Dlg->pEndWidgetList);
     
     /* exit button -> neutral -> war -> casefire -> peace -> alliance */
     pPlayer0 = pPlayers_Dlg->pEndWidgetList->prev->prev->prev->prev->prev->prev;
@@ -202,86 +210,75 @@ void update_players_dialog(void)
       pPlayer1 = pPlayer0;
       pPlayer = pPlayer0->data.player;
       
-      /* the team */
-      if (pPlayer->team != TEAM_NONE) {
-        team = team_get_by_id(pPlayer->team)->name;
-      } else {
-        team = _("none");
-      }
-            
-      if(pPlayer->is_alive) {
-        if (pPlayer->ai.control) {
-	  state = _("AI");
-        } else {
-          if (pPlayer->is_connected) {
-            if (pPlayer->turn_done) {
-              state = _("done");
-            } else {
-              state = _("moving");
-            }
-          } else {
-            state = _("disconnected");
+      for (i = 0; i < num_player_dlg_columns; i++) {
+        if (player_dlg_columns[i].show) {
+          switch (player_dlg_columns[i].type) {
+            case COL_TEXT:
+            case COL_RIGHT_TEXT:
+              astr_add_line(&astr, "%s: %s", player_dlg_columns[i].title,
+                                             player_dlg_columns[i].func(pPlayer));
+              break;
+            case COL_BOOLEAN:
+              astr_add_line(&astr, "%s: %s", player_dlg_columns[i].title,
+                            player_dlg_columns[i].bool_func(pPlayer) ? 
+                              _("Yes") : _("No"));
+              break;
+            default:
+              break;
           }
         }
-      } else {
-        state = _("R.I.P");
-      }
-  
-      /* text for idleness */
-      if (pPlayer->nturns_idle > 3) {
-        idle = pPlayer->nturns_idle - 1;
-      } else {
-        idle = 0;
       }
       
-      my_snprintf(cBuf, sizeof(cBuf), _("Name : %s\nNation : %s\nTeam : %s\n"
-      					"Reputation : %s\nEmbassy :%s\n"
-    					"State : %s\nIdle : %d %s"),
-                   pPlayer->name, get_nation_name(pPlayer->nation),
-		   team, reputation_text(pPlayer->reputation),
-                   get_embassy_status(game.player_ptr, pPlayer),
-		   state, idle, PL_("turn", "turns", idle));
+      copy_chars_to_string16(pPlayer0->string16, astr.str);
       
-      copy_chars_to_string16(pPlayer0->string16, cBuf);
+      astr_free(&astr);
           
       /* now add some eye candy ... */
       if(pPlayer1 != pPlayers_Dlg->pBeginWidgetList) {
-	x0 = pPlayer0->size.x + pPlayer0->size.w / 2;
-        y0 = pPlayer0->size.y + pPlayer0->size.h / 2;
+        dst0.x = pPlayer0->size.x + pPlayer0->size.w / 2;
+        dst0.y = pPlayer0->size.y + pPlayer0->size.h / 2;
+
         do{
           pPlayer1 = pPlayer1->prev;
 	  if (have_diplomat_info_about(pPlayer) ||
 	     have_diplomat_info_about(pPlayer1->data.player)) {
-            x1 = pPlayer1->size.x + pPlayer1->size.w / 2;
-            y1 = pPlayer1->size.y + pPlayer1->size.h / 2;
+            dst1.x = pPlayer1->size.x + pPlayer1->size.w / 2;
+            dst1.y = pPlayer1->size.y + pPlayer1->size.h / 2;
+               
             switch (pplayer_get_diplstate(pPlayer, pPlayer1->data.player)->type) {
-	      case DS_NEUTRAL:
+	      case DS_ARMISTICE:
 	        if(SDL_Client_Flags & CF_DRAW_PLAYERS_NEUTRAL_STATUS) {
-	          putline(pPlayer1->dst, x0, y0, x1, y1, 0xFF000000);
+	          putline(pPlayer1->dst->surface,
+	                  dst0.x, dst0.y, dst1.x, dst1.y,
+	                  get_game_colorRGB(COLOR_THEME_PLRDLG_ARMISTICE));
 	        }
 	      break;
               case DS_WAR:
 	        if(SDL_Client_Flags & CF_DRAW_PLAYERS_WAR_STATUS) {
-	          putline(pPlayer1->dst, x0, y0, x1, y1,
-	    		get_game_color(COLOR_STD_RED, pPlayer1->dst));
+	          putline(pPlayer1->dst->surface,
+	                  dst0.x, dst0.y, dst1.x, dst1.y,
+	                  get_game_colorRGB(COLOR_THEME_PLRDLG_WAR));
 	        }
               break;
 	      case DS_CEASEFIRE:
 	        if (SDL_Client_Flags & CF_DRAW_PLAYERS_CEASEFIRE_STATUS) {
-	          putline(pPlayer1->dst, x0, y0, x1, y1,
-	    		get_game_color(COLOR_STD_YELLOW, pPlayer1->dst));
+	          putline(pPlayer1->dst->surface,
+	                  dst0.x, dst0.y, dst1.x, dst1.y,
+	                  get_game_colorRGB(COLOR_THEME_PLRDLG_CEASEFIRE));
 	        }
               break;
               case DS_PEACE:
 	        if (SDL_Client_Flags & CF_DRAW_PLAYERS_PEACE_STATUS) {
-	          putline(pPlayer1->dst, x0, y0, x1, y1,
-	    		get_game_color(COLOR_STD_GROUND, pPlayer1->dst));
+	          putline(pPlayer1->dst->surface,
+	                  dst0.x, dst0.y, dst1.x, dst1.y,
+	                  get_game_colorRGB(COLOR_THEME_PLRDLG_PEACE));
 	        }
               break;
 	      case DS_ALLIANCE:
 	        if (SDL_Client_Flags & CF_DRAW_PLAYERS_ALLIANCE_STATUS) {
-	          putline(pPlayer1->dst, x0, y0, x1, y1,
-	    		get_game_color(COLOR_STD_CITY_GOLD, pPlayer1->dst));
+	          putline(pPlayer1->dst->surface,
+	                  dst0.x, dst0.y, dst1.x, dst1.y,
+	                  get_game_colorRGB(COLOR_THEME_PLRDLG_ALLIANCE));
 	        }
               break;
               default:
@@ -299,7 +296,7 @@ void update_players_dialog(void)
     /* redraw */
     redraw_group(pPlayers_Dlg->pBeginWidgetList,
     			pPlayers_Dlg->pEndWidgetList->prev, 0);
-    sdl_dirty_rect(pPlayers_Dlg->pEndWidgetList->size);
+    widget_mark_dirty(pPlayers_Dlg->pEndWidgetList);
   
     flush_dirty();
   }
@@ -308,38 +305,38 @@ void update_players_dialog(void)
 /**************************************************************************
   Popup (or raise) the player list dialog.
 **************************************************************************/
-void popup_players_dialog(void)
+void popup_players_dialog(bool raise)
 {
-  struct GUI *pWindow = NULL, *pBuf = NULL;
+  struct widget *pWindow = NULL, *pBuf = NULL;
   SDL_Surface *pLogo = NULL, *pZoomed = NULL;
   SDL_String16 *pStr;
   SDL_Rect dst;
-  int i, n, w = 0, h;
+  int i, n, h;
   double a, b, r;
-  struct player *pPlayer;
+  SDL_Rect area;
   
   if (pPlayers_Dlg) {
     return;
   }
   
   n = 0;
-  for(i=0; i<game.nplayers; i++) {
-    if(is_barbarian(get_player(i))) {
+  players_iterate(pPlayer) {
+    if(is_barbarian(pPlayer)) {
       continue;
     }
     n++;
-  }
+  } players_iterate_end;
 
   if(n < 2) {
     return;
   }
     
-  pPlayers_Dlg = MALLOC(sizeof(struct SMALL_DLG));
+  pPlayers_Dlg = fc_calloc(1, sizeof(struct SMALL_DLG));
   
-  pStr = create_str16_from_char(_("Players"), 12);
+  pStr = create_str16_from_char(_("Players"), adj_font(12));
   pStr->style |= TTF_STYLE_BOLD;
   
-  pWindow = create_window(NULL, pStr, 10, 10, WF_DRAW_THEME_TRANSPARENT);
+  pWindow = create_window_skeleton(NULL, pStr, 0);
     
   pWindow->action = players_window_dlg_callback;
   set_wstate(pWindow, FC_WS_NORMAL);
@@ -349,7 +346,8 @@ void popup_players_dialog(void)
   /* ---------- */
   /* exit button */
   pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
-  			  			WF_DRAW_THEME_TRANSPARENT);
+  			  WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
+  pBuf->string16 = create_str16_from_char(_("Close Dialog (Esc)"), adj_font(12));
   pBuf->action = exit_players_dlg_callback;
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->key = SDLK_ESCAPE;
@@ -359,38 +357,38 @@ void popup_players_dialog(void)
   
   for(i = 0; i<DS_LAST; i++) {
     switch (i) {
-      case DS_NEUTRAL:
+      case DS_ARMISTICE:
 	pBuf = create_checkbox(pWindow->dst,
 		(SDL_Client_Flags & CF_DRAW_PLAYERS_NEUTRAL_STATUS),
-      						WF_DRAW_THEME_TRANSPARENT);
+      						WF_RESTORE_BACKGROUND);
 	pBuf->action = toggle_draw_neutral_status_callback;
 	pBuf->key = SDLK_n;
       break;
       case DS_WAR:
 	pBuf = create_checkbox(pWindow->dst,
 		(SDL_Client_Flags & CF_DRAW_PLAYERS_WAR_STATUS),
-      						WF_DRAW_THEME_TRANSPARENT);
+      						WF_RESTORE_BACKGROUND);
 	pBuf->action = toggle_draw_war_status_callback;
 	pBuf->key = SDLK_w;
       break;
       case DS_CEASEFIRE:
 	pBuf = create_checkbox(pWindow->dst,
 		(SDL_Client_Flags & CF_DRAW_PLAYERS_CEASEFIRE_STATUS),
-      						WF_DRAW_THEME_TRANSPARENT);
+      						WF_RESTORE_BACKGROUND);
 	pBuf->action = toggle_draw_ceasefire_status_callback;
 	pBuf->key = SDLK_c;
       break;
       case DS_PEACE:
 	pBuf = create_checkbox(pWindow->dst,
 		(SDL_Client_Flags & CF_DRAW_PLAYERS_PEACE_STATUS),
-      						WF_DRAW_THEME_TRANSPARENT);
+      						WF_RESTORE_BACKGROUND);
 	pBuf->action = toggle_draw_pease_status_callback;
 	pBuf->key = SDLK_p;
       break;
       case DS_ALLIANCE:
 	pBuf = create_checkbox(pWindow->dst,
 		(SDL_Client_Flags & CF_DRAW_PLAYERS_ALLIANCE_STATUS),
-      						WF_DRAW_THEME_TRANSPARENT);
+      						WF_RESTORE_BACKGROUND);
 	pBuf->action = toggle_draw_alliance_status_callback;
 	pBuf->key = SDLK_a;
       break;
@@ -404,38 +402,32 @@ void popup_players_dialog(void)
   } 
   /* ---------- */
   
-  for(i=0; i<game.nplayers; i++) {
-    pPlayer = get_player(i);
-      
+  players_iterate(pPlayer) {
     if(is_barbarian(pPlayer)) {
       continue;
     }
                 
-    pStr = create_string16(NULL, 0, 10);
+    pStr = create_string16(NULL, 0, adj_font(10));
     pStr->style |= (TTF_STYLE_BOLD|SF_CENTER);
    
-    pLogo = GET_SURF(get_nation_by_idx(pPlayer->nation)->flag_sprite);
-    pLogo = make_flag_surface_smaler(pLogo);
-    pZoomed = ZoomSurface(pLogo, 3.0 - n * 0.05, 3.0 - n * 0.05 , 1);
-    SDL_SetColorKey(pZoomed, SDL_SRCCOLORKEY|SDL_RLEACCEL,
-    			getpixel(pZoomed, pZoomed->w - 1, pZoomed->h - 1));
-    FREESURFACE(pLogo);
+    pLogo = get_nation_flag_surface(nation_of_player(pPlayer));
+    pZoomed = zoomSurface(pLogo, DEFAULT_ZOOM * (3.0 - n * 0.05), DEFAULT_ZOOM * (3.0 - n * 0.05), 1);
             
     pBuf = create_icon2(pZoomed, pWindow->dst,
-    	(WF_DRAW_THEME_TRANSPARENT|WF_WIDGET_HAS_INFO_LABEL|WF_FREE_THEME));
+    	(WF_RESTORE_BACKGROUND|WF_WIDGET_HAS_INFO_LABEL|WF_FREE_THEME));
     
     pBuf->string16 = pStr;
       
     if(!pPlayer->is_alive) {
-      pStr = create_str16_from_char("R.I.P" , 10);
+      pStr = create_str16_from_char(_("R.I.P.") , adj_font(10));
       pStr->style |= TTF_STYLE_BOLD;
-      pStr->fgcol = *(get_game_colorRGB(COLOR_STD_WHITE));
+      pStr->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_TEXT);
       pLogo = create_text_surf_from_str16(pStr);
       FREESTRING16(pStr);
 	
       dst.x = (pZoomed->w - pLogo->w) / 2;
       dst.y = (pZoomed->h - pLogo->h) / 2;
-      SDL_BlitSurface(pLogo, NULL, pZoomed, &dst);
+      alphablit(pLogo, NULL, pZoomed, &dst);
       FREESURFACE(pLogo);
     }
      
@@ -449,51 +441,50 @@ void popup_players_dialog(void)
     
     add_to_gui_list(ID_LABEL, pBuf);
     
-  }
+  } players_iterate_end;
   
   pPlayers_Dlg->pBeginWidgetList = pBuf;
 
-  w = 500;
-  h = 400;
-  r = MIN(w,h);
-  r -= ((MAX(pBuf->size.w, pBuf->size.h) * 2) + WINDOW_TILE_HIGH + FRAME_WH);
+  resize_window(pWindow, NULL, NULL, adj_size(500), adj_size(400));
+
+  area = pWindow->area;
+  
+  r = MIN(area.w, area.h);
+  r -= ((MAX(pBuf->size.w, pBuf->size.h) * 2));
   r /= 2;
   a = (2.0 * M_PI) / n;
-   
-  pWindow->size.x = (Main.screen->w - w) / 2;
-  pWindow->size.y = (Main.screen->h - h) / 2;
   
-  resize_window(pWindow, NULL, NULL, w, h);
-  
-  putframe(pWindow->theme, 0, 0, w - 1, h - 1, 0xFFFFFFFF);
+  widget_set_position(pWindow,
+                      (Main.screen->w - pWindow->size.w) / 2,
+                      (Main.screen->h - pWindow->size.h) / 2);
   
   /* exit button */
   pBuf = pWindow->prev;
   
-  pBuf->size.x = pWindow->size.x + pWindow->size.w-pBuf->size.w-FRAME_WH-1;
-  pBuf->size.y = pWindow->size.y + 1;
+  pBuf->size.x = area.x + area.w - pBuf->size.w - 1;
+  pBuf->size.y = pWindow->size.y + adj_size(2);
     
-  n = WINDOW_TILE_HIGH + 4;
-  pStr = create_string16(NULL, 0, 10);
+  n = area.y;
+  pStr = create_string16(NULL, 0, adj_font(10));
   pStr->style |= TTF_STYLE_BOLD;
-  pStr->render = 3;
-  pStr->bgcol.unused = 128;
+  pStr->bgcol = (SDL_Color) {0, 0, 0, 0};
+  
   for(i = 0; i<DS_LAST; i++) {
       switch (i) {
-	case DS_NEUTRAL:
-	  pStr->fgcol = *(get_game_colorRGB(COLOR_STD_BLACK));
+	case DS_ARMISTICE:
+	  pStr->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_ARMISTICE);
 	break;
         case DS_WAR:
-	  pStr->fgcol = *(get_game_colorRGB(COLOR_STD_RED));
+	  pStr->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_WAR);
 	break;
 	case DS_CEASEFIRE:
-	  pStr->fgcol = *(get_game_colorRGB(COLOR_STD_YELLOW));
+	  pStr->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_CEASEFIRE);
 	break;
         case DS_PEACE:
-	  pStr->fgcol = *(get_game_colorRGB(COLOR_STD_GROUND));
+	  pStr->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_PEACE);
         break;
 	case DS_ALLIANCE:
-	  pStr->fgcol = *(get_game_colorRGB(COLOR_STD_CITY_GOLD));
+	  pStr->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_ALLIANCE);
 	break;
         default:
 	   /* no contact */
@@ -503,33 +494,32 @@ void popup_players_dialog(void)
       
       copy_chars_to_string16(pStr, diplstate_text(i));
       pLogo = create_text_surf_from_str16(pStr);
-      SDL_SetAlpha(pLogo, 0x0, 0x0);
   
       pBuf = pBuf->prev;
       h = MAX(pBuf->size.h, pLogo->h);
-      pBuf->size.x = pWindow->size.x + 5;
-      pBuf->size.y = pWindow->size.y + n + (h - pBuf->size.h) / 2;
+      pBuf->size.x = area.x + adj_size(5);
+      pBuf->size.y = n + (h - pBuf->size.h) / 2;
       
-      dst.x = 5 + pBuf->size.w + 6;
+      dst.x = adj_size(5) + pBuf->size.w + adj_size(6);
       dst.y = n + (h - pLogo->h) / 2;
-      SDL_BlitSurface(pLogo, NULL, pWindow->theme, &dst);
+      alphablit(pLogo, NULL, pWindow->theme, &dst);
       n += h;
       FREESURFACE(pLogo);
   }
   FREESTRING16(pStr);
-     
+
   /* first player shield */
   pBuf = pBuf->prev;
-  pBuf->size.x = pWindow->size.x + pWindow->size.w / 2 - pBuf->size.w / 2;
-  pBuf->size.y = pWindow->size.y + pWindow->size.h / 2 - r;
+  pBuf->size.x = area.x + area.w / 2 - pBuf->size.w / 2;
+  pBuf->size.y = area.y + area.h / 2 - r - pBuf->size.h / 2;
   
   n = 1;
   if(pBuf != pPlayers_Dlg->pBeginWidgetList) {
     do{
       pBuf = pBuf->prev;
       b = M_PI_2 + n * a;
-      pBuf->size.x = pWindow->size.x + pWindow->size.w / 2 - r * cos(b) - pBuf->size.w / 2;
-      pBuf->size.y = pWindow->size.y + pWindow->size.h / 2 - r * sin(b);
+      pBuf->size.x = area.x + area.w / 2 - r * cos(b) - pBuf->size.w / 2;
+      pBuf->size.y = area.y + area.h / 2 - r * sin(b) - pBuf->size.h / 2;
       n++;
     } while(pBuf != pPlayers_Dlg->pBeginWidgetList);
   }
@@ -546,7 +536,7 @@ void popdown_players_dialog(void)
   if (pPlayers_Dlg) {
     popdown_window_group_dialog(pPlayers_Dlg->pBeginWidgetList,
 			pPlayers_Dlg->pEndWidgetList);
-    FREE(pPlayers_Dlg);
+    FC_FREE(pPlayers_Dlg);
   }
 }
 
@@ -554,19 +544,21 @@ void popdown_players_dialog(void)
 /* ============================== SHORT =============================== */
 static struct ADVANCED_DLG  *pShort_Players_Dlg = NULL;
 
-static int players_nations_window_dlg_callback(struct GUI *pWindow)
+static int players_nations_window_dlg_callback(struct widget *pWindow)
 {
   return -1;
 }
 
-static int exit_players_nations_dlg_callback(struct GUI *pWidget)
+static int exit_players_nations_dlg_callback(struct widget *pWidget)
 {
-  popdown_players_nations_dialog();
-  flush_dirty();
+  if (Main.event.button.button == SDL_BUTTON_LEFT) {
+    popdown_players_nations_dialog();
+    flush_dirty();
+  }
   return -1;
 }
 
-static int player_nation_callback(struct GUI *pWidget)
+static int player_nation_callback(struct widget *pWidget)
 {
   struct player *pPlayer = pWidget->data.player;
   popdown_players_nations_dialog();
@@ -587,7 +579,7 @@ static int player_nation_callback(struct GUI *pWidget)
       }
     break;
     default:
-      if(pPlayer->player_no != game.player_idx) {
+      if (pPlayer != client.conn.playing) {
         popup_diplomacy_dialog(pPlayer);
       }
     break;
@@ -601,38 +593,40 @@ static int player_nation_callback(struct GUI *pWidget)
 **************************************************************************/
 void popup_players_nations_dialog(void)
 {
-  struct GUI *pWindow = NULL, *pBuf = NULL;
+  struct widget *pWindow = NULL, *pBuf = NULL;
   SDL_Surface *pLogo = NULL;
   SDL_String16 *pStr;
   char cBuf[128], *state;
-  int i, n = 0, w = 0, h, units_h = 0;
-  struct player *pPlayer;
+  int n = 0, w = 0, units_h = 0;
   const struct player_diplstate *pDS;
+  SDL_Rect area;
   
   if (pShort_Players_Dlg) {
     return;
   }
      
-  h = WINDOW_TILE_HIGH + 3 + FRAME_WH;
-      
-  pShort_Players_Dlg = MALLOC(sizeof(struct ADVANCED_DLG));
+  pShort_Players_Dlg = fc_calloc(1, sizeof(struct ADVANCED_DLG));
   
-  pStr = create_str16_from_char(_("Nations") , 12);
+  /* TRANS: Nations report title */
+  pStr = create_str16_from_char(_("Nations") , adj_font(12));
   pStr->style |= TTF_STYLE_BOLD;
   
-  pWindow = create_window(NULL, pStr, 10, 10, WF_DRAW_THEME_TRANSPARENT);
+  pWindow = create_window_skeleton(NULL, pStr, 0);
     
   pWindow->action = players_nations_window_dlg_callback;
   set_wstate(pWindow, FC_WS_NORMAL);
-  w = MAX(w, pWindow->size.w);
   
   add_to_gui_list(ID_WINDOW, pWindow);
   pShort_Players_Dlg->pEndWidgetList = pWindow;
+  
+  area = pWindow->area;
+  
   /* ---------- */
   /* exit button */
   pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
-  			  			WF_DRAW_THEME_TRANSPARENT);
-  w += pBuf->size.w + 10;
+  			  WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
+  pBuf->string16 = create_str16_from_char(_("Close Dialog (Esc)"), adj_font(12));
+  area.w = MAX(area.w, pBuf->size.w + adj_size(10));
   pBuf->action = exit_players_nations_dlg_callback;
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->key = SDLK_ESCAPE;
@@ -640,21 +634,19 @@ void popup_players_nations_dialog(void)
   add_to_gui_list(ID_BUTTON, pBuf);
   /* ---------- */
   
-  for(i=0; i<game.nplayers; i++) {
-    if(i != game.player_idx) {
-      pPlayer = get_player(i);
-      
+  players_iterate(pPlayer) {
+    if (pPlayer != client.conn.playing) {
       if(!pPlayer->is_alive || is_barbarian(pPlayer)) {
         continue;
       }
       
-      pDS = pplayer_get_diplstate(game.player_ptr, pPlayer);
+      pDS = pplayer_get_diplstate(client.conn.playing, pPlayer);
             
-      if(pPlayer->ai.control) {
+      if(pPlayer->ai_data.control) {
 	state = _("AI");
       } else {
         if (pPlayer->is_connected) {
-          if (pPlayer->turn_done) {
+          if (pPlayer->phase_done) {
       	    state = _("done");
           } else {
       	    state = _("moving");
@@ -666,54 +658,58 @@ void popup_players_nations_dialog(void)
      
       if(pDS->type == DS_CEASEFIRE) {
 	my_snprintf(cBuf, sizeof(cBuf), "%s(%s) - %d %s",
-                get_nation_name(pPlayer->nation), state,
+                    nation_adjective_for_player(pPlayer),
+                    state,
 		pDS->turns_left, PL_("turn", "turns", pDS->turns_left));
       } else {
 	my_snprintf(cBuf, sizeof(cBuf), "%s(%s)",
-                           get_nation_name(pPlayer->nation), state);
+                    nation_adjective_for_player(pPlayer),
+                    state);
       }
       
-      pStr = create_str16_from_char(cBuf, 10);
+      pStr = create_str16_from_char(cBuf, adj_font(10));
       pStr->style |= TTF_STYLE_BOLD;
    
-      pLogo = GET_SURF(get_nation_by_idx(pPlayer->nation)->flag_sprite);
-      pLogo = make_flag_surface_smaler(pLogo);
+      pLogo = get_nation_flag_surface(nation_of_player(pPlayer));
       
       pBuf = create_iconlabel(pLogo, pWindow->dst, pStr, 
-    	(WF_FREE_THEME|WF_DRAW_THEME_TRANSPARENT|WF_DRAW_TEXT_LABEL_WITH_SPACE));
+    	(/*WF_FREE_THEME|*/WF_RESTORE_BACKGROUND|WF_DRAW_TEXT_LABEL_WITH_SPACE));
                       
       /* now add some eye candy ... */
       switch (pDS->type) {
+	case DS_ARMISTICE:
+	  pBuf->string16->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_ARMISTICE);
+	  set_wstate(pBuf, FC_WS_NORMAL);
+        break;
         case DS_WAR:
 	  if(can_meet_with_player(pPlayer) || can_intel_with_player(pPlayer)) {
             set_wstate(pBuf, FC_WS_NORMAL);
-	    pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_STD_RED));
+	    pBuf->string16->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_WAR);
           } else {
-	    pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_STD_RED_DISABLED));
+	    pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_THEME_PLRDLG_WAR_RESTRICTED));
 	  }
         break;
 	case DS_CEASEFIRE:
-	  pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_STD_YELLOW));
+	  pBuf->string16->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_CEASEFIRE);
 	  set_wstate(pBuf, FC_WS_NORMAL);
         break;
         case DS_PEACE:
-	  pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_STD_GROUND));
+	  pBuf->string16->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_PEACE);
 	  set_wstate(pBuf, FC_WS_NORMAL);
         break;
 	case DS_ALLIANCE:
-	  pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_STD_CITY_GOLD));
+	  pBuf->string16->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_ALLIANCE);
 	  set_wstate(pBuf, FC_WS_NORMAL);
         break;
 	case DS_NO_CONTACT:
-	  pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_STD_DISABLED));
+	  pBuf->string16->fgcol = *(get_game_colorRGB(COLOR_THEME_WIDGET_DISABLED_TEXT));
 	break;
         default:
 	  set_wstate(pBuf, FC_WS_NORMAL);
         break;
       }
       
-      pBuf->string16->render = 3;
-      pBuf->string16->bgcol.unused = 128;
+      pBuf->string16->bgcol = (SDL_Color) {0, 0, 0, 0};
     
       pBuf->data.player = pPlayer;
   
@@ -722,8 +718,8 @@ void popup_players_nations_dialog(void)
   
       add_to_gui_list(ID_LABEL, pBuf);
     
-      w = MAX(w, pBuf->size.w);
-      h += pBuf->size.h;
+      area.w = MAX(w, pBuf->size.w);
+      area.h += pBuf->size.h;
     
       if (n > 19)
       {
@@ -732,9 +728,9 @@ void popup_players_nations_dialog(void)
       
       n++;  
     }
-  }
+  } players_iterate_end;
   pShort_Players_Dlg->pBeginWidgetList = pBuf;
-  pShort_Players_Dlg->pBeginActiveWidgetList = pBuf;
+  pShort_Players_Dlg->pBeginActiveWidgetList = pShort_Players_Dlg->pBeginWidgetList;
   pShort_Players_Dlg->pEndActiveWidgetList = pWindow->prev->prev;
   pShort_Players_Dlg->pActiveWidgetList = pShort_Players_Dlg->pEndActiveWidgetList;
   
@@ -747,30 +743,33 @@ void popup_players_nations_dialog(void)
     pShort_Players_Dlg->pScroll->count = n;
     
     n = units_h;
-    w += n;
+    area.w += n;
     
-    units_h = 20 * pBuf->size.h + WINDOW_TILE_HIGH + 3 + FRAME_WH;
+    units_h = 20 * pBuf->size.h;
     
   } else {
-    units_h = h;
+    units_h = area.h;
   }
         
   /* ---------- */
   
-  w += DOUBLE_FRAME_WH;
-  
-  h = units_h;
+  area.h = units_h;
 
-  pWindow->size.x = ((Main.event.motion.x + w < pWindow->dst->w) ?
-                     (Main.event.motion.x + 10) : (pWindow->dst->w - w - 10));
-  pWindow->size.y = 
-      ((Main.event.motion.y - (WINDOW_TILE_HIGH + 2) + h < pWindow->dst->h) ?
-             (Main.event.motion.y - (WINDOW_TILE_HIGH + 2)) :
-             (pWindow->dst->h - h - 10));
-      
-  resize_window(pWindow, NULL, NULL, w, h);
+  resize_window(pWindow, NULL, NULL,
+                (pWindow->size.w - pWindow->area.w) + area.w,
+                (pWindow->size.h + pWindow->area.h) + area.h);
   
-  w -= DOUBLE_FRAME_WH;
+  area = pWindow->area;
+  
+  widget_set_position(pWindow,
+    ((Main.event.motion.x + pWindow->size.w + adj_size(10) < Main.screen->w) ?
+      (Main.event.motion.x + adj_size(10)) :
+      (Main.screen->w - pWindow->size.w - adj_size(10))),
+    ((Main.event.motion.y - adj_size(2) + pWindow->size.h < Main.screen->h) ?
+      (Main.event.motion.y - adj_size(2)) :
+      (Main.screen->h - pWindow->size.h - adj_size(10))));
+  
+  w = area.w;
   
   if (pShort_Players_Dlg->pScroll)
   {
@@ -779,27 +778,26 @@ void popup_players_nations_dialog(void)
   
   /* exit button */
   pBuf = pWindow->prev;
-  pBuf->size.x = pWindow->size.x + pWindow->size.w-pBuf->size.w-FRAME_WH-1;
-  pBuf->size.y = pWindow->size.y + 1;
+  pBuf->size.x = area.x + area.w - pBuf->size.w - 1;
+  pBuf->size.y = pWindow->size.y + adj_size(2);
   
   /* cities */
   pBuf = pBuf->prev;
   setup_vertical_widgets_position(1,
-	pWindow->size.x + FRAME_WH, pWindow->size.y + WINDOW_TILE_HIGH + 2,
+	area.x, area.y,
 	w, 0, pShort_Players_Dlg->pBeginActiveWidgetList, pBuf);
   
   if (pShort_Players_Dlg->pScroll)
   {
     setup_vertical_scrollbar_area(pShort_Players_Dlg->pScroll,
-	pWindow->size.x + pWindow->size.w - FRAME_WH,
-    	pWindow->size.y + WINDOW_TILE_HIGH + 1,
-    	pWindow->size.h - (FRAME_WH + WINDOW_TILE_HIGH + 1), TRUE);
+	area.x + area.w, area.y,
+    	area.h, TRUE);
   }
   
   /* -------------------- */
   /* redraw */
   redraw_group(pShort_Players_Dlg->pBeginWidgetList, pWindow, 0);
-  sdl_dirty_rect(pWindow->size);
+  widget_mark_dirty(pWindow);
   
   flush_dirty();
 }
@@ -812,7 +810,7 @@ void popdown_players_nations_dialog(void)
   if (pShort_Players_Dlg) {
     popdown_window_group_dialog(pShort_Players_Dlg->pBeginWidgetList,
 			pShort_Players_Dlg->pEndWidgetList);
-    FREE(pShort_Players_Dlg->pScroll);
-    FREE(pShort_Players_Dlg);
+    FC_FREE(pShort_Players_Dlg->pScroll);
+    FC_FREE(pShort_Players_Dlg);
   }
 }
