@@ -20,22 +20,29 @@
 #include <commctrl.h>
 #include <assert.h>
 
-#include "city.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
 #include "genlist.h"
-#include "map.h"
+#include "log.h"
 #include "mem.h"
 #include "packets.h"
-#include "player.h"
 #include "shared.h"
 #include "support.h"
+
+/* common */
+#include "city.h"
+#include "game.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
 #include "unitlist.h"
 
+/* client */
 #include "canvas.h"
+#include "cityrep.h"
+#include "client_main.h"
 #include "climap.h"
 #include "climisc.h"
-#include "cityrep.h"
 #include "cma_fe.h"
 #include "colors.h"
 #include "control.h"
@@ -50,13 +57,14 @@
 #include "sprite.h"
 #include "tilespec.h"
 #include "wldlg.h"   
+
 #include "gui_main.h"
 #include "citydlg.h"
 #include "text.h"
 
 #define NUM_UNITS_SHOWN  12
 #define NUM_CITIZENS_SHOWN 25   
-#define NUM_INFO_FIELDS 11      /* number of fields in city_info */
+#define NUM_INFO_FIELDS 12      /* number of fields in city_info */
 #define ID_CITYOPT_RADIO 100
 #define ID_CITYOPT_TOGGLE 200
 #define NUM_TABS 6
@@ -99,7 +107,7 @@ struct city_dialog {
   
   int support_unit_ids[NUM_UNITS_SHOWN];
   int present_unit_ids[NUM_UNITS_SHOWN];
-  struct city_production change_list_targets[MAX_NUM_PRODUCTION_TARGETS];
+  struct universal change_list_targets[MAX_NUM_PRODUCTION_TARGETS];
   int change_list_num_improvements;
   int building_values[B_LAST];
   int is_modal;    
@@ -109,12 +117,12 @@ struct city_dialog {
 };
 
 
-extern struct connection aconnection; 
 extern HFONT font_8courier;
 extern HFONT font_12courier;
 extern HFONT font_12arial;
 extern HINSTANCE freecivhinst;
 extern HWND root_window;
+
 static int city_map_width;
 static int city_map_height;
 static struct genlist *dialog_list;
@@ -198,7 +206,7 @@ void refresh_city_dialog(struct city *pcity)
     refresh_happiness_box(pdialog->happiness);
     resize_city_dialog(pdialog);
   }
-  if (city_owner(pcity) == game.player_ptr) {
+  if (city_owner(pcity) == client.conn.playing) {
     city_report_dialog_update_city(pcity);
     economy_report_dialog_update();   
     if (pdialog != NULL) {
@@ -229,30 +237,31 @@ bool city_dialog_is_open(struct city *pcity)
 void city_dialog_update_improvement_list(struct city_dialog *pdialog)
 {
   LV_COLUMN lvc;
-  int changed, total, item, targets_used;
-  struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+  int total, item, targets_used;
+  struct universal targets[MAX_NUM_PRODUCTION_TARGETS];
   struct item items[MAX_NUM_PRODUCTION_TARGETS];
   char buf[100];
+  bool changed;
 
   /* Test if the list improvements of pcity has changed */
-  changed = 0;
-  impr_type_iterate(i) {
-    if (pdialog->pcity->improvements[i] !=
-        pdialog->last_improvlist_seen[i]) {
-      changed = 1;
-      break;
+  changed = false;
+  improvement_iterate(pimprove) {
+    int index = improvement_index(pimprove);
+    bool has_building = city_has_building(pdialog->pcity, pimprove);
+    bool had_building =
+      pdialog->last_improvlist_seen[index];
+
+    if ((has_building && !had_building)
+        || (!has_building && had_building)) {
+      changed = true;
+      pdialog->last_improvlist_seen[index] = has_building;
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 
   if (!changed) {
     return;
   }
   
-  /* Update pdialog->last_improvlist_seen */
-  impr_type_iterate(i) {
-    pdialog->last_improvlist_seen[i] = pdialog->pcity->improvements[i];
-  } impr_type_iterate_end;
-  
   targets_used = collect_already_built_targets(targets, pdialog->pcity);
   name_and_sort_items(targets, targets_used, items, FALSE, pdialog->pcity);
    
@@ -263,21 +272,21 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
     char *strings[2];
     int upkeep;
     int row;
-    struct city_production target = items[item].item;
+    struct universal target = items[item].item;
 
-    assert (!target.is_unit);
+    assert(VUT_IMPROVEMENT == target.kind);
 
     strings[0] = items[item].descr;
     strings[1] = buf;
 
     /* This takes effects (like Adam Smith's) into account. */
-    upkeep = improvement_upkeep(pdialog->pcity, target.value);
+    upkeep = city_improvement_upkeep(pdialog->pcity, target.value.building);
 
     my_snprintf(buf, sizeof(buf), "%d", upkeep);
    
     row=fcwin_listview_add_row(pdialog->buildings_list,
 			   item, 2, strings);
-    pdialog->building_values[row] = target.value;
+    pdialog->building_values[row] = improvement_index(target.value.building);
     total += upkeep;
   }
   lvc.mask=LVCF_TEXT;
@@ -305,7 +314,7 @@ void city_dialog_update_present_units(HDC hdc,struct city_dialog *pdialog)
   store.wnd = NULL;
   store.tmp = NULL;
  
-  if (city_owner(pdialog->pcity) != game.player_ptr) {
+  if (city_owner(pdialog->pcity) != client.conn.playing) {
     plist = pdialog->pcity->info_units_present;
   } else {
     plist = pdialog->pcity->tile->units;
@@ -345,6 +354,7 @@ void city_dialog_update_supported_units(HDC hdc, struct city_dialog *pdialog)
   int i;
   struct unit_list *plist;
   struct canvas store;
+  int free_unhappy;
 
   store.type = CANVAS_DC;
   store.hdc = hdc;
@@ -352,39 +362,47 @@ void city_dialog_update_supported_units(HDC hdc, struct city_dialog *pdialog)
   store.wnd = NULL;
   store.tmp = NULL;
 
-  if  (city_owner(pdialog->pcity) != game.player_ptr) {
+  free_unhappy = get_city_bonus(pdialog->pcity, EFT_MAKE_CONTENT_MIL);
+
+  if (city_owner(pdialog->pcity) != client.conn.playing) {
     plist = pdialog->pcity->info_units_supported;
   } else {
     plist = pdialog->pcity->units_supported;
-  }      
-     
-  for(i = 0; i < NUM_UNITS_SHOWN; i++) {   
+  }
+
+  for(i = 0; i < NUM_UNITS_SHOWN; i++) {
     RECT rc;
-    pdialog->support_unit_ids[i]=0;      
+    pdialog->support_unit_ids[i]=0;
     rc.top=pdialog->supported_y;
-    rc.left=pdialog->pop_x+i*(tileset_small_sprite_width(tileset)+tileset_tile_width(tileset));
+    rc.left=pdialog->pop_x+i*(tileset_small_sprite_width(tileset) +
+            tileset_tile_width(tileset));
     rc.right=rc.left+tileset_tile_width(tileset);
-    rc.bottom=rc.top+tileset_small_sprite_height(tileset)+tileset_tile_height(tileset);
-    FillRect(hdc, &rc, 
-	     (HBRUSH)GetClassLong(pdialog->mainwindow,GCL_HBRBACKGROUND));
+    rc.bottom=rc.top+tileset_small_sprite_height(tileset) +
+              tileset_tile_height(tileset);
+    FillRect(hdc, &rc,
+             (HBRUSH)GetClassLong(pdialog->mainwindow,GCL_HBRBACKGROUND));
   }
 
   i = 0;
-  
+
   unit_list_iterate(plist, punit) {
+    int happy_cost = city_unit_unhappiness(punit, &free_unhappy);
+
     put_unit(punit, &store,
-	     pdialog->pop_x + i * (tileset_small_sprite_width(tileset) + tileset_tile_width(tileset)),
+	     pdialog->pop_x + i * (tileset_small_sprite_width(tileset) +
+                      tileset_tile_width(tileset)),
 	     pdialog->supported_y);
     put_unit_city_overlays(punit, &store,
-	     pdialog->pop_x + i * (tileset_small_sprite_width(tileset) + tileset_tile_width(tileset)),
-	     pdialog->supported_y);
+             pdialog->pop_x + i * (tileset_small_sprite_width(tileset) +
+                                   tileset_tile_width(tileset)),
+             pdialog->supported_y, punit->upkeep, happy_cost);
     pdialog->support_unit_ids[i] = punit->id;
     i++;
     if (i == NUM_UNITS_SHOWN) {
       break;
     }
   } unit_list_iterate_end;
-}   
+}
 
 /**************************************************************************
 ...
@@ -392,20 +410,14 @@ void city_dialog_update_supported_units(HDC hdc, struct city_dialog *pdialog)
 void city_dialog_update_building(struct city_dialog *pdialog)
 {
   char buf2[100], buf[32];
-  const char *descr;
   struct city *pcity=pdialog->pcity;    
+  const char *descr = city_production_name_translation(pcity);
   
   EnableWindow(pdialog->buy_but, city_can_buy(pcity));
   EnableWindow(pdialog->sell_but, !pcity->did_sell);
 
   get_city_dialog_production(pcity, buf, sizeof(buf));
   
-  if (pcity->production.is_unit) {
-    descr = utype_name_translation(utype_by_number(pcity->production.value));
-  } else {
-    descr = get_impr_name_ex(pcity, pcity->production.value);
-  }
-
   my_snprintf(buf2, sizeof(buf2), "%s\r\n%s", descr, buf);
   SetWindowText(pdialog->build_area, buf2);
   SetWindowText(pdialog->build_area, buf2);
@@ -424,7 +436,8 @@ static void city_dialog_update_information(HWND *info_label,
   struct city *pcity = pdialog->pcity;
   int granaryturns;
   enum { FOOD, SHIELD, TRADE, GOLD, LUXURY, SCIENCE, 
-	 GRANARY, GROWTH, CORRUPTION, WASTE, POLLUTION 
+	 GRANARY, GROWTH, CORRUPTION, WASTE, POLLUTION,
+         ILLNESS
   };
 
   /* fill the buffers with the necessary info */
@@ -460,12 +473,20 @@ static void city_dialog_update_information(HWND *info_label,
 		PL_("%d turn", "%d turns", abs(granaryturns)),
 		abs(granaryturns));
   }
-  my_snprintf(buf[CORRUPTION], sizeof(buf[CORRUPTION]), "%2d",
-	      pcity->waste[O_TRADE]);
-  my_snprintf(buf[WASTE], sizeof(buf[WASTE]), "%2d",
-          pcity->waste[O_SHIELD]);
-  my_snprintf(buf[POLLUTION], sizeof(buf[POLLUTION]), "%2d",
-	      pcity->pollution);
+  my_snprintf(buf[CORRUPTION], sizeof(buf[CORRUPTION]), "%4d",
+              pcity->waste[O_TRADE]);
+  my_snprintf(buf[WASTE], sizeof(buf[WASTE]), "%4d",
+              pcity->waste[O_SHIELD]);
+  my_snprintf(buf[POLLUTION], sizeof(buf[POLLUTION]), "%4d",
+              pcity->pollution);
+  if (!game.info.illness_on) {
+    my_snprintf(buf[ILLNESS], sizeof(buf[ILLNESS]), " -.-");
+  } else {
+    /* illness is in tenth of percent */
+    my_snprintf(buf[ILLNESS], sizeof(buf[ILLNESS]), "%4.1f",
+                (float)city_illness_calc(pcity, NULL, NULL, NULL, NULL)
+                / 10.0);
+  }
 
   /* stick 'em in the labels */
 
@@ -659,7 +680,8 @@ static struct fcwin_box *create_city_info_table(HWND owner, HWND *info_label)
   struct fcwin_box *hbox, *column1, *column2;
   HWND label;
 
-  static const char *output_label[NUM_INFO_FIELDS] = { N_("Food:"),
+  static const char *output_label[NUM_INFO_FIELDS] = {
+    N_("Food:"),
     N_("Prod:"),
     N_("Trade:"),
     N_("Gold:"),
@@ -669,7 +691,8 @@ static struct fcwin_box *create_city_info_table(HWND owner, HWND *info_label)
     N_("Change in:"),
     N_("Corruption:"),
     N_("Waste:"),
-    N_("Pollution:")
+    N_("Pollution:"),
+    N_("Plague Risk:")
   };
 
   hbox = fcwin_hbox_new(owner, TRUE);
@@ -761,7 +784,7 @@ static void CityDlgCreate(HWND hWnd,struct city_dialog *pdialog)
   cmadata->pcity = pdialog->pcity;
   cmadata->pdialog = NULL;
   user_data[PAGE_CMA] = cmadata;
-  wl_init.pwl = &pdialog->pcity->worklist;
+  wl_init.global_worklist_id = -1;
   wl_init.pcity = pdialog->pcity;
   wl_init.parent = pdialog->mainwindow;
   wl_init.user_data = pdialog;
@@ -849,9 +872,11 @@ static void CityDlgCreate(HWND hWnd,struct city_dialog *pdialog)
   lvc.pszText=_("Upkeep");
   ListView_InsertColumn(pdialog->buildings_list,1,&lvc);
   ListView_SetColumnWidth(pdialog->buildings_list,0,LVSCW_AUTOSIZE);
-  ListView_SetColumnWidth(pdialog->buildings_list,1,LVSCW_AUTOSIZE_USEHEADER);   impr_type_iterate(i) {
-    pdialog->last_improvlist_seen[i] = 0;
-  } impr_type_iterate_end;
+  ListView_SetColumnWidth(pdialog->buildings_list,1,LVSCW_AUTOSIZE_USEHEADER);
+
+  improvement_iterate(pimprove) {
+    pdialog->last_improvlist_seen[improvement_index(pimprove)] = 0;
+  } improvement_iterate_end;
   
   pdialog->mainwindow=hWnd;
   fcwin_set_box(pdialog->tab_childs[0],child_vbox);
@@ -887,21 +912,15 @@ static void buy_callback_no(HWND w, void * data)
 **************************************************************************/
 static void buy_callback(struct city_dialog *pdialog)
 {
-  int value;
-  const char *name;
-  char buf[512];    
-   if(pdialog->pcity->production.is_unit) {
-    name=utype_name_translation(utype_by_number(pdialog->pcity->production.value));
-  }
-  else {
-    name=get_impr_name_ex(pdialog->pcity, pdialog->pcity->production.value);
-  }
-  value=city_buy_cost(pdialog->pcity);
+  char buf[512];
+  struct city *pcity = pdialog->pcity;
+  const char *name = city_production_name_translation(pcity);
+  int value = city_production_buy_gold_cost(pcity);
  
-  if(game.player_ptr->economic.gold>=value) {
+  if (value <= client.conn.playing->economic.gold) {
     my_snprintf(buf, sizeof(buf),
             _("Buy %s for %d gold?\nTreasury contains %d gold."),
-            name, value, game.player_ptr->economic.gold);
+            name, value, client.conn.playing->economic.gold);
  
     popup_message_dialog(pdialog->mainwindow, /*"buydialog"*/ _("Buy It!"), buf,
                          _("_Yes"), buy_callback_yes, pdialog,
@@ -910,7 +929,7 @@ static void buy_callback(struct city_dialog *pdialog)
   else {
     my_snprintf(buf, sizeof(buf),
             _("%s costs %d gold.\nTreasury contains %d gold."),
-            name, value, game.player_ptr->economic.gold);
+            name, value, client.conn.playing->economic.gold);
  
     popup_message_dialog(NULL, /*"buynodialog"*/ _("Buy It!"), buf,
                          _("Darn"), buy_callback_no, 0, 0);
@@ -941,7 +960,7 @@ static void sell_callback_no(HWND w, void * data)
  
 /****************************************************************
 ...
-*****************************************************************/            
+*****************************************************************/
 static void sell_callback(struct city_dialog *pdialog)
 {
   char buf[100];
@@ -949,14 +968,14 @@ static void sell_callback(struct city_dialog *pdialog)
     return;
   }
  
-  if (!can_city_sell_building(pdialog->pcity, pdialog->id_selected)) {
+  if (!can_city_sell_building(pdialog->pcity, improvement_by_number(pdialog->id_selected))) {
     return;
   }
   
   pdialog->sell_id = pdialog->id_selected;
   my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"),
-	      get_impr_name_ex(pdialog->pcity, pdialog->id_selected),
-	      impr_sell_gold(pdialog->id_selected));
+	      city_improvement_name_translation(pdialog->pcity, improvement_by_number(pdialog->id_selected)),
+	      impr_sell_gold(improvement_by_number(pdialog->id_selected)));
   
   popup_message_dialog(pdialog->mainwindow, /*"selldialog" */
 		       _("Sell It!"), buf, _("_Yes"),
@@ -977,7 +996,7 @@ static LONG CALLBACK changedlg_proc(HWND hWnd,
 				    LPARAM lParam) 
 {
   int sel,i,n;
-  struct city_production target;
+  struct universal target;
   struct city_dialog *pdialog;
   pdialog=(struct city_dialog *)fcwin_get_user_data(hWnd);
   switch(message)
@@ -1006,14 +1025,14 @@ static LONG CALLBACK changedlg_proc(HWND hWnd,
 	case ID_PRODCHANGE_HELP:
 	  if (sel>=0)
 	    {
-	      if (target.is_unit) {
-		popup_help_dialog_typed(utype_name_translation(utype_by_number(target.value)),
+	      if (VUT_UTYPE == target.kind) {
+		popup_help_dialog_typed(utype_name_translation(target.value.utype),
 					HELP_UNIT);
-	      } else if(is_great_wonder(target.value)) {
-		popup_help_dialog_typed(improvement_name_translation(target.value),
+	      } else if (is_great_wonder(target.value.building)) {
+		popup_help_dialog_typed(improvement_name_translation(target.value.building),
 					HELP_WONDER);
 	      } else {
-		popup_help_dialog_typed(improvement_name_translation(target.value),
+		popup_help_dialog_typed(improvement_name_translation(target.value.building),
 					HELP_IMPROVEMENT);
 	      }                                                                     
 	    }
@@ -1064,7 +1083,7 @@ static void change_callback(struct city_dialog *pdialog)
       HWND lv;
       LV_COLUMN lvc;
       LV_ITEM lvi;
-      struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+      struct universal targets[MAX_NUM_PRODUCTION_TARGETS];
       struct item items[MAX_NUM_PRODUCTION_TARGETS];
       int targets_used, item;
 
@@ -1103,8 +1122,8 @@ static void change_callback(struct city_dialog *pdialog)
 
       n = 0;
       for (item = 0; item < targets_used; item++) {
-	if (city_can_build_impr_or_unit(pdialog->pcity, items[item].item)) {
-	  struct city_production target = items[item].item;
+	if (can_city_build_now(pdialog->pcity, items[item].item)) {
+	  struct universal target = items[item].item;
 
 	  get_city_dialog_production_row(row, sizeof(buf[0]), target,
 				       pdialog->pcity);
@@ -1129,58 +1148,13 @@ static void change_callback(struct city_dialog *pdialog)
 static void commit_city_worklist(struct worklist *pwl, void *data)
 {
   struct city_dialog *pdialog = data;
-  int k;
 
-  /* Update the worklist.  Remember, though -- the current build
-     target really isn't in the worklist; don't send it to the server
-     as part of the worklist.  Of course, we have to search through
-     the current worklist to find the first _now_available_ build
-     target (to cope with players who try mean things like adding a
-     Battleship to a city worklist when the player doesn't even yet
-     have the Map Making tech).  */
-
-  for (k = 0; k < MAX_LEN_WORKLIST; k++) {
-    int same_as_current_build;
-    struct city_production target;
-    if (!worklist_peek_ith(pwl, &target, k))
-      break;
-
-    same_as_current_build = pdialog->pcity->production.value == target.value
-	&& pdialog->pcity->production.is_unit == target.is_unit;
-
-    /* Very special case: If we are currently building a wonder we
-       allow the construction to continue, even if we the wonder is
-       finished elsewhere, ie unbuildable. */
-    if (k == 0 && !target.is_unit && is_great_wonder(target.value)
-	&& same_as_current_build) {
-      worklist_remove(pwl, k);
-      break;
-    }
-
-    /* If it can be built... */
-    if (city_can_build_impr_or_unit(pdialog->pcity, target)) {
-      /* ...but we're not yet building it, then switch. */
-      if (!same_as_current_build) {
-        /* Change the current target */
-	city_change_production(pdialog->pcity, target);
-      }
-
-      /* This item is now (and may have always been) the current
-         build target.  Drop it out of the worklist. */
-      worklist_remove(pwl, k);
-      break;
-    }
-  }
-
-  /* Send the rest of the worklist on its way. */
-  city_set_worklist(pdialog->pcity, pwl);
+  city_worklist_commit(pdialog->pcity, pwl);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-
-
 static void rename_city_callback(HWND w, void * data)
 {
   struct city_dialog *pdialog = data;
@@ -1245,7 +1219,7 @@ static void city_dialog_update_tradelist(struct city_dialog *pdialog)
 
   buf[0] = '\0';
 
-  for (i = 0; i < NUM_TRADEROUTES; i++) {
+  for (i = 0; i < NUM_TRADE_ROUTES; i++) {
     if (pdialog->pcity->trade[i]) {
       struct city *pcity;
       x = 1;
@@ -1272,32 +1246,35 @@ static void city_dialog_update_tradelist(struct city_dialog *pdialog)
 /**************************************************************************
 ...
 **************************************************************************/
+static void supported_units_activate_close_callback(HWND w, void * data)
+{
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
 
-
-static void supported_units_activate_close_callback(HWND w, void * data){
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
- 
   destroy_message_dialog(w);
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data))) {
+
+  if (NULL != punit) {
+    struct city *pcity =
+      player_find_city_by_id(client.conn.playing, punit->homecity);
+
     set_unit_focus(punit);
-    if((pcity=player_find_city_by_id(game.player_ptr, punit->homecity)))
-      if((pdialog=get_city_dialog(pcity)))
+    if (NULL != pcity) {
+      struct city_dialog *pdialog = get_city_dialog(pcity);
+
+      if (NULL != pdialog) {
         CityDlgClose(pdialog);
+      }
+    }
   }
-}   
+}
 
 /**************************************************************************
 ...
 **************************************************************************/
-
-
 static void activate_callback(struct city_dialog *pdialog)
 {
   activate_all_units(pdialog->pcity->tile);
-}      
+}
 
 /****************************************************************
 ...
@@ -1309,36 +1286,37 @@ static void show_units_callback(struct city_dialog *pdialog)
   if(unit_list_size(ptile->units))
     popup_unit_select_dialog(ptile);
 }
-     
 
 /**************************************************************************
 ...
 **************************************************************************/
-
-
 static void present_units_disband_callback(HWND w, void *data)
 {
-  struct unit *punit;
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
+
+  if (NULL != punit) {
     request_unit_disband(punit);
- 
+  }
+
   destroy_message_dialog(w);
 }
-      
+
 /****************************************************************
 ...
 *****************************************************************/
 static void present_units_homecity_callback(HWND w, void * data)
 {
-  struct unit *punit;
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
+
+  if (NULL != punit) {
     request_unit_change_homecity(punit);
- 
+  }
+
   destroy_message_dialog(w);
 }
- 
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1346,81 +1324,91 @@ static void present_units_cancel_callback(HWND w, void *data)
 {
   destroy_message_dialog(w);
 }
- 
+
 /****************************************************************
 ...
 *****************************************************************/              
-
 static void present_units_activate_callback(HWND w, void * data)
 {
-  struct unit *punit;
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
+
+  if (NULL != punit) {
     set_unit_focus(punit);
+  }
+
   destroy_message_dialog(w);
 }
- 
+
 /****************************************************************
 ...
 *****************************************************************/
 static void present_units_activate_close_callback(HWND w, void * data)
 {
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
- 
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
+
   destroy_message_dialog(w);
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data))) {
+
+  if (NULL != punit) {
+    struct city *pcity = tile_city(punit->tile);
+
     set_unit_focus(punit);
-    if((pcity=tile_get_city(punit->tile)))
-      if((pdialog=get_city_dialog(pcity)))
-       CityDlgClose(pdialog);
+    if (NULL != pcity) {
+      struct city_dialog *pdialog = get_city_dialog(pcity);
+
+      if (NULL != pdialog) {
+        CityDlgClose(pdialog);
+      }
+    }
   }
-}              
+}
 
 /**************************************************************************
 ...
 **************************************************************************/
-
-
 static void present_units_sentry_callback(HWND w, void * data)
 {
-  struct unit *punit;
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
+
+  if (NULL != punit) {
     request_unit_sentry(punit);
+  }
+
   destroy_message_dialog(w);
 }
- 
+
 /****************************************************************
 ...
 *****************************************************************/
 static void present_units_fortify_callback(HWND w, void * data)
 {
-  struct unit *punit;
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
+
+  if (NULL != punit) {
     request_unit_fortify(punit);
+  }
+
   destroy_message_dialog(w);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-
-
 static void unitupgrade_callback_yes(HWND w, void * data)
 {
-  struct unit *punit;
- 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data))) {
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)data);
+
+  if (NULL != punit) {
     request_unit_upgrade(punit);
   }
+
   destroy_message_dialog(w);
 }
- 
- 
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1428,17 +1416,17 @@ static void unitupgrade_callback_no(HWND w, void * data)
 {
   destroy_message_dialog(w);
 }
-            
+
 /****************************************************************
 ...
-*****************************************************************/       
+*****************************************************************/
 static void upgrade_callback(HWND w, void * data)
 {
-  struct unit *punit = player_find_unit_by_id(game.player_ptr,
-					      (size_t) data);
   char buf[512];
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t) data);
 
-  if (!punit) {
+  if (NULL == punit) {
     return;
   }
 
@@ -1462,18 +1450,15 @@ static void upgrade_callback(HWND w, void * data)
 /**************************************************************************
 ...
 **************************************************************************/
-
-
 static void city_dlg_click_supported(struct city_dialog *pdialog, int n)
 {
-  struct unit *punit;
-  struct city *pcity;  
-  HWND wd;
+  struct city *pcity;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, pdialog->support_unit_ids[n]);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, 
-				   pdialog->support_unit_ids[n])) &&
-     (pcity = game_find_city_by_number(punit->homecity))) {   
-    wd = popup_message_dialog(NULL,
+  if (NULL != punit
+      && (pcity = game_find_city_by_number(punit->homecity))) {
+    HWND wd = popup_message_dialog(NULL,
            /*"supportunitsdialog"*/ _("Unit Commands"),
            unit_description(punit),
            _("_Activate unit"),
@@ -1484,6 +1469,7 @@ static void city_dlg_click_supported(struct city_dialog *pdialog, int n)
              present_units_disband_callback, punit->id,
            _("_Cancel"),
              present_units_cancel_callback, 0, 0, NULL);
+
     if (unit_has_type_flag(punit, F_UNDISBANDABLE)) {
       message_dialog_button_set_sensitive(wd, 3, FALSE);
     }
@@ -1493,37 +1479,36 @@ static void city_dlg_click_supported(struct city_dialog *pdialog, int n)
 /**************************************************************************
 ...
 **************************************************************************/
-
-
 static void city_dlg_click_present(struct city_dialog *pdialog, int n)
 {
-  struct unit *punit;
   struct city *pcity;
-  HWND wd;
-  if((punit=player_find_unit_by_id(game.player_ptr, 
-				   pdialog->present_unit_ids[n])) &&
-     (pcity=tile_get_city(punit->tile)) &&
-     (pdialog=get_city_dialog(pcity))) {   
-     wd=popup_message_dialog(NULL,
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, pdialog->present_unit_ids[n]);
+
+  if (NULL != punit
+      && (pcity=tile_city(punit->tile))
+      && (pdialog=get_city_dialog(pcity))) { /* ??? */
+     HWND wd = popup_message_dialog(NULL,
                            /*"presentunitsdialog"*/_("Unit Commands"),
                            unit_description(punit),
-                           _("_Activate unit"),
+                           _("_Activate Unit"),
                              present_units_activate_callback, punit->id,
-                           _("Activate unit, _close dialog"),
+                           _("Activate Unit, _Close Dialog"),
                              present_units_activate_close_callback, punit->id,
-                           _("_Sentry unit"),
+                           _("_Sentry Unit"),
                              present_units_sentry_callback, punit->id,
-                           _("_Fortify unit"),
+                           _("_Fortify Unit"),
                              present_units_fortify_callback, punit->id,
-                           _("_Disband unit"),
+                           _("_Disband Unit"),
                              present_units_disband_callback, punit->id,
-                           _("Make new _homecity"),
+                           _("Set _Home City"),
                              present_units_homecity_callback, punit->id,
-                           _("_Upgrade unit"),
+                           _("_Upgrade Unit"),
                              upgrade_callback, punit->id,
                            _("_Cancel"),
                              present_units_cancel_callback, 0,
-                           NULL);                   
+                           NULL);
+
      if (punit->activity == ACTIVITY_SENTRY
 	 || !can_unit_do_activity(punit, ACTIVITY_SENTRY)) {
        message_dialog_button_set_sensitive(wd,2, FALSE);
@@ -1538,9 +1523,10 @@ static void city_dlg_click_present(struct city_dialog *pdialog, int n)
      if (punit->homecity == pcity->id) {
        message_dialog_button_set_sensitive(wd,5, FALSE);
      }
-     if (can_upgrade_unittype(game.player_ptr,unit_type(punit)) == NULL) {
+
+     if (NULL == can_upgrade_unittype(client.conn.playing,unit_type(punit))) {
        message_dialog_button_set_sensitive(wd,6, FALSE);
-     }        
+     }
    }
 }
 
@@ -1834,16 +1820,17 @@ static void initialize_city_dialogs(void)
 
 struct city_dialog *get_city_dialog(struct city *pcity)
 {   
-  struct genlist_link *myiter;
-  if (!city_dialogs_have_been_initialised)
+  if (!city_dialogs_have_been_initialised) {
     initialize_city_dialogs();
-  myiter = dialog_list->head_link;  
-  
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter))
-    if(((struct city_dialog *)ITERATOR_PTR(myiter))->pcity==pcity)
-      return ITERATOR_PTR(myiter);
-  
-  return 0;      
+  }
+
+  TYPED_LIST_ITERATE(struct city_dialog, dialog_list, pdialog) {
+    if (pdialog->pcity == pcity) {
+      return pdialog;
+    }
+  } LIST_ITERATE_END;
+
+  return 0;
 }
 
 /**************************************************************************
@@ -1906,17 +1893,16 @@ popdown_all_city_dialogs(void)
 **************************************************************************/
 void citydlg_tileset_change(void)
 {
-  struct genlist_link *myiter;
-  if (!city_dialogs_have_been_initialised)
+  if (!city_dialogs_have_been_initialised) {
     initialize_city_dialogs();
+  }
 
   city_map_width = get_citydlg_canvas_width();
   city_map_height = get_citydlg_canvas_height();
 
-  myiter = dialog_list->head_link;
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
+  TYPED_LIST_ITERATE(struct city_dialog, dialog_list, pdialog) {
     HDC hdc;
-    struct city_dialog *pdialog = (struct city_dialog *)ITERATOR_PTR(myiter);
+
     DeleteObject(pdialog->map_bmp);
     hdc = GetDC(pdialog->mainwindow);
     pdialog->map_bmp = CreateCompatibleBitmap(hdc, city_map_width,
@@ -1926,21 +1912,18 @@ void citydlg_tileset_change(void)
     pdialog->map_h = city_map_height;
     resize_city_dialog(pdialog);
     refresh_city_dialog(pdialog->pcity);
-  }
+  } LIST_ITERATE_END;
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-
-
-void
-refresh_unit_city_dialogs(struct unit *punit)
+void refresh_unit_city_dialogs(struct unit *punit)
 {
-  struct city *pcity_sup, *pcity_pre;
-  struct city_dialog *pdialog;      
-  pcity_sup=player_find_city_by_id(game.player_ptr, punit->homecity);
-  pcity_pre=tile_get_city(punit->tile);     
+  struct city_dialog *pdialog;
+  struct city *pcity_pre = tile_city(punit->tile);
+  struct city *pcity_sup =
+    player_find_city_by_id(client.conn.playing, punit->homecity);
   
   if(pcity_sup && (pdialog=get_city_dialog(pcity_sup)))     
     {
@@ -2032,7 +2015,7 @@ LONG APIENTRY citydlg_config_proc(HWND hWnd,
           BV_SET(new_options, CITYO_NEW_TAXMAN);
 	}
 
-	dsend_packet_city_options_req(&aconnection, pcity->id, new_options);
+	dsend_packet_city_options_req(&client.conn, pcity->id, new_options);
       }
       break;
     default: 
