@@ -961,7 +961,7 @@ static void select_help_item_string(const char *item,
 void
 popup_help_dialog_string(const char *item)
 {
-  popup_help_dialog_typed(_(item), HELP_ANY); 
+  popup_help_dialog_typed(Q_(item), HELP_ANY); 
 }
 
 /**************************************************************************
