@@ -218,7 +218,12 @@ static void get_contents_of_pollution(struct city_dialog *pdialog,
                   (float)illness / 10.0);
     }
   }
-  my_snprintf(retbuf, n, _("Pollution:    %3d"), pollution);
+
+  my_snprintf(retbuf, n, _("Corruption:   %4d\n"
+                           "Waste:        %4d\n"
+                           "Pollution:    %4d\n"
+                           "Plague Risk:  %s"),
+              corruption, waste, pollution, buf);
 }
 
 /****************************************************************
@@ -252,9 +257,9 @@ static void get_contents_of_storage(struct city_dialog *pdialog,
   }
 
   /* We used to mark cities with a granary with a "*" here. */
-  /* FIXME: actually use the "Change in:" info in buf[] */
-  my_snprintf(retbuf, n, _("Granary:  %3d/%-3d"),
-	      foodstock, foodbox);
+  my_snprintf(retbuf, n, _("Granary:   %3d/%-3d\n"
+                           "Change in: %s"),
+	      foodstock, foodbox, buf);
 }
 
 /****************************************************************
@@ -2419,7 +2424,7 @@ void cityopt_newcit_triggle_callback(Widget w, XtPointer client_data,
 					XtPointer call_data);
 
 char *newcitizen_labels[] = {
-  N_("?city:Workers"),
+  N_("Workers"),
   N_("Scientists"),
   N_("Taxmen")
 };
