@@ -171,7 +171,7 @@ callers of this function should do so themselves... --dwp
 *****************************************************************/
 void popup_help_dialog_string(const char *item)
 {
-  popup_help_dialog_typed(_(item), HELP_ANY);
+  popup_help_dialog_typed(Q_(item), HELP_ANY);
 }
 
 
