@@ -10,16 +10,15 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
@@ -32,6 +31,8 @@
 #include <X11/Xaw/SmeBSB.h>
 #include <X11/Xaw/SmeLine.h>
 
+/* common & utility */
+#include "climap.h"
 #include "fcintl.h"
 #include "game.h"
 #include "map.h"
@@ -40,10 +41,10 @@
 #include "support.h"
 #include "unit.h"
 
+/* client */
 #include "chatline.h"
 #include "citydlg.h"
-#include "civclient.h"
-#include "clinet.h"
+#include "client_main.h"
 #include "colors.h"
 #include "control.h"
 #include "dialogs.h"
@@ -54,15 +55,12 @@
 #include "inputdlg.h"
 #include "mapview.h"
 #include "menu.h"
+#include "overview_common.h"
+#include "text.h"
 #include "tilespec.h"
 
 #include "mapctrl.h"
 
-/* Update the workers for a city on the map, when the update is received */
-struct city *city_workers_display = NULL;
-/* Color to use to display the workers */
-int city_workers_color=COLOR_STD_WHITE;
-
 static void popupinfo_popdown_callback(Widget w, XtPointer client_data, XtPointer call_data);
 
 /**************************************************************************
@@ -71,13 +69,12 @@ static void popupinfo_popdown_callback(Widget w, XtPointer client_data, XtPointe
 static void name_new_city_callback(Widget w, XtPointer client_data, 
 				   XtPointer call_data)
 {
-  size_t unit_id;
-  
-  if((unit_id=(size_t)client_data)) {
-    struct packet_unit_request req;
-    req.unit_id=unit_id;
-    sz_strlcpy(req.name, input_dialog_get_input(w));
-    send_packet_unit_request(&aconnection, &req, PACKET_UNIT_BUILD_CITY);
+  int idx = XTPOINTER_TO_INT(client_data);
+
+  if (idx < 0) {
+    cancel_city(index_to_tile(-idx-1));
+  } else {
+    finish_city(index_to_tile(idx), input_dialog_get_input(w));
   }
     
   input_dialog_destroy(w);
@@ -92,104 +89,62 @@ void popup_newcity_dialog(struct unit *punit, char *suggestname)
 {
   input_dialog_create(toplevel, "shellnewcityname",
 		      _("What should we call our new city?"), suggestname,
-		      name_new_city_callback, INT_TO_XTPOINTER(punit->id),
-		      name_new_city_callback, INT_TO_XTPOINTER(0));
+		      name_new_city_callback,
+                      INT_TO_XTPOINTER(tile_index(punit->tile)),
+		      name_new_city_callback,
+                      INT_TO_XTPOINTER(-tile_index(punit->tile)-1));
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-static void popit(int xin, int yin, int xtile, int ytile)
+static void popit(int xin, int yin, struct tile *ptile)
 {
   Position x, y;
   int dw, dh;
   Dimension w, h, b;
-  static struct map_position cross_list[2+1];
-  struct map_position *cross_head = cross_list;
+  static struct tile *cross_list[2+1];
+  struct tile **cross_head = cross_list;
   int i;
-  char s[512];
-  struct city *pcity;
   struct unit *punit;
-  struct tile *ptile=map_get_tile(xtile, ytile);
-
+  char *content;
+  static bool is_orders;
   
-  if(ptile->known>=TILE_KNOWN_FOGGED) {
+  if (TILE_UNKNOWN != client_tile_get_known(ptile)) {
     Widget p=XtCreatePopupShell("popupinfo", simpleMenuWidgetClass,
 				map_canvas, NULL, 0);
-    my_snprintf(s, sizeof(s), _("Terrain: %s"),
-		map_get_tile_info_text(xtile, ytile));
-    XtCreateManagedWidget(s, smeBSBObjectClass, p, NULL, 0);
-
-    my_snprintf(s, sizeof(s), _("Food/Prod/Trade: %s"),
-		map_get_tile_fpt_text(xtile, ytile));
-    XtCreateManagedWidget(s, smeBSBObjectClass, p, NULL, 0);
-
-    if (tile_has_special(ptile, S_HUT)) {
-      XtCreateManagedWidget(_("Minor Tribe Village"), smeBSBObjectClass,
-			    p, NULL, 0);
-    }
-    
-    if((pcity=map_get_city(xtile, ytile))) {
-      my_snprintf(s, sizeof(s), _("City: %s(%s) %s"), pcity->name,
-		  get_nation_name(city_owner(pcity)->nation),
-		  city_got_citywalls(pcity) ? _("with City Walls") : "");
-      XtCreateManagedWidget(s, smeBSBObjectClass, p, NULL, 0);
-    }
-
-    if(get_tile_infrastructure_set(ptile)) {
-      sz_strlcpy(s, _("Infrastructure: "));
-      sz_strlcat(s, map_get_infrastructure_text(ptile->special));
-      XtCreateManagedWidget(s, smeBSBObjectClass, p, NULL, 0);
-    }
-
-    if((punit=find_visible_unit(ptile)) && !pcity) {
-      char cn[64];
-      struct unit_type *ptype=unit_type(punit);
-      cn[0]='\0';
-      if(punit->owner==game.player_idx) {
-	struct city *pcity;
-	pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
-	if(pcity)
-	  my_snprintf(cn, sizeof(cn), "/%s", pcity->name);
+    content = (char *) popup_info_text(ptile);
+    /* content is provided to us as a single string with multiple lines,
+       but xaw doens't support multi-line labels.  So we break it up.
+       We mangle it in the process, but who cares?  It's never going to be
+       used again anyway. */
+    while (1) {
+      char *end = strchr(content, '\n'); 
+      if (end) {
+	*end='\0';
       }
-      my_snprintf(s, sizeof(s), _("Unit: %s(%s%s)"), ptype->name,
-		  get_nation_name(unit_owner(punit)->nation), cn);
-      XtCreateManagedWidget(s, smeBSBObjectClass, p, NULL, 0);
-
-      if(punit->owner==game.player_idx)  {
-	char uc[64] = "";
-	if(unit_list_size(&ptile->units)>=2) {
-	  my_snprintf(uc, sizeof(uc), _("  (%d more)"),
-		      unit_list_size(&ptile->units) - 1);
-	}
-        my_snprintf(s, sizeof(s),
-		_("A:%d D:%d FP:%d HP:%d/%d%s%s"), ptype->attack_strength, 
-	        ptype->defense_strength, ptype->firepower, punit->hp, 
-	        ptype->hp, punit->veteran?_(" V"):"", uc);
-
-        if(punit->activity==ACTIVITY_GOTO)  {
-	  cross_head->x = punit->goto_dest_x;
-	  cross_head->y = punit->goto_dest_y;
-	  cross_head++;
-        }
+      XtCreateManagedWidget(content, smeBSBObjectClass, p, NULL, 0);
+      if (end) {
+	content = end+1;
       } else {
-        my_snprintf(s, sizeof(s),
-		    _("A:%d D:%d FP:%d HP:%d0%%"), ptype->attack_strength, 
-		    ptype->defense_strength, ptype->firepower, 
-		    (punit->hp*100/ptype->hp + 9)/10 );
+	break;
       }
-      XtCreateManagedWidget(s, smeBSBObjectClass, p, NULL, 0);
     }
 
-    cross_head->x = xtile;
-    cross_head->y = ytile;
+    punit = find_visible_unit(ptile);
+    is_orders = show_unit_orders(punit);
+    if (punit && punit->goto_tile) {
+      *cross_head = punit->goto_tile;
+      cross_head++;
+    }
+    *cross_head = ptile;
     cross_head++;
 
-    xin /= NORMAL_TILE_WIDTH;
-    xin *= NORMAL_TILE_WIDTH;
-    yin /= NORMAL_TILE_HEIGHT;
-    yin *= NORMAL_TILE_HEIGHT;
-    xin += (NORMAL_TILE_WIDTH / 2);
+    xin /= tileset_tile_width(tileset);
+    xin *= tileset_tile_width(tileset);
+    yin /= tileset_tile_height(tileset);
+    yin *= tileset_tile_height(tileset);
+    xin += (tileset_tile_width(tileset) / 2);
     XtTranslateCoords(map_canvas, xin, yin, &x, &y);
     dw = XDisplayWidth (display, screen_number);
     dh = XDisplayHeight (display, screen_number);
@@ -205,12 +160,12 @@ static void popit(int xin, int yin, int xtile, int ytile)
     if (y < 0) y = 0;
     XtVaSetValues(p, XtNx, x, XtNy, y, NULL);
 
-    cross_head->x = -1;
-    for (i = 0; cross_list[i].x >= 0; i++) {
-      put_cross_overlay_tile(cross_list[i].x,cross_list[i].y);
+    *cross_head = NULL;
+    for (i = 0; cross_list[i]; i++) {
+      put_cross_overlay_tile(cross_list[i]);
     }
     XtAddCallback(p,XtNpopdownCallback,popupinfo_popdown_callback,
-		  (XtPointer)cross_list);
+		  (XtPointer)&is_orders);
 
     XtPopupSpringLoaded(p);
   }
@@ -223,11 +178,12 @@ static void popit(int xin, int yin, int xtile, int ytile)
 void popupinfo_popdown_callback(Widget w, XtPointer client_data,
         XtPointer call_data)
 {
-  struct map_position *cross_list=(struct map_position *)client_data;
+  bool *full = client_data;
 
-  while (cross_list->x >= 0) {
-    refresh_tile_mapcanvas(cross_list->x, cross_list->y, TRUE);
-    cross_list++;
+  if (*full) {
+    update_map_canvas_visible();
+  } else {
+    dirty_all();
   }
 
   XtDestroyWidget(w);
@@ -238,18 +194,7 @@ void popupinfo_popdown_callback(Widget w, XtPointer client_data,
 **************************************************************************/
 void mapctrl_btn_wakeup(XEvent *event)
 {
-  int map_x, map_y, is_real;
-  XButtonEvent *ev=&event->xbutton;
-
-  if(get_client_state()!=CLIENT_GAME_RUNNING_STATE)
-    return;
-
-  map_x = map_view_x0 + ev->x / NORMAL_TILE_WIDTH;
-  map_y = map_view_y0 + ev->y / NORMAL_TILE_HEIGHT;
-  is_real = normalize_map_pos(&map_x, &map_y);
-  assert(is_real);
-
-  wakeup_sentried_units(map_x, map_y);
+  wakeup_button_pressed(event->xbutton.x, event->xbutton.y);
 }
 
 /**************************************************************************
@@ -257,38 +202,23 @@ void mapctrl_btn_wakeup(XEvent *event)
 **************************************************************************/
 void mapctrl_btn_mapcanvas(XEvent *event)
 {
-  int x, y;
   XButtonEvent *ev=&event->xbutton;
+  struct tile *ptile = canvas_pos_to_tile(ev->x, ev->y);
 
-  if(get_client_state()!=CLIENT_GAME_RUNNING_STATE)
+  if (!can_client_change_view()) {
     return;
-
-  get_map_xy(ev->x, ev->y, &x, &y);
-
-  if (ev->button==Button1)
-    do_map_click(x, y);
-  else if (ev->button==Button2||ev->state&ControlMask)
-    popit(ev->x, ev->y, x, y);
-  else if (ev->button == Button3) {
-    center_tile_mapcanvas(x, y);
   }
-}
 
-/**************************************************************************
-...
-**************************************************************************/
-void update_line(int window_x, int window_y)
-{
-  int x, y, old_x, old_y;
-
-  if ((hover_state == HOVER_GOTO || hover_state == HOVER_PATROL)
-      && draw_goto_line) {
-    get_map_xy(window_x, window_y, &x, &y);
-
-    get_line_dest(&old_x, &old_y);
-    if (!same_pos(old_x, old_y, x, y)) {
-      draw_line(x, y);
-    }
+  if (ev->button == Button1 && (ev->state & ControlMask)) {
+    action_button_pressed(ev->x, ev->y, SELECT_SEA);
+  } else if (ev->button == Button1) {
+    action_button_pressed(ev->x, ev->y, SELECT_POPUP);
+  } else if (ev->button == Button2 && ptile) {
+    popit(ev->x, ev->y, ptile);
+  } else if (ev->button == Button3 && (ev->state & ControlMask)) {
+    action_button_pressed(ev->x, ev->y, SELECT_LAND);
+  } else if (ev->button == Button3) {
+    recenter_button_pressed(ev->x, ev->y);
   }
 }
 
@@ -315,46 +245,12 @@ void create_line_at_mouse_pos(void)
 }
 
 /**************************************************************************
-  Adjust the position of city workers from the mapcanvas
+ The Area Selection rectangle. Called by center_tile_mapcanvas() and
+ when the mouse pointer moves.
 **************************************************************************/
-void mapctrl_btn_adjust_workers(XEvent *event)
+void update_rect_at_mouse_pos(void)
 {
-  int map_x, map_y, x, y, is_valid;
-  XButtonEvent *ev=&event->xbutton;
-  struct city *pcity;
-  struct packet_city_request packet;
-  enum city_tile_type wrk;
-
-  if(get_client_state()!=CLIENT_GAME_RUNNING_STATE)
-    return;
-
-  map_x = map_view_x0 + ev->x / NORMAL_TILE_WIDTH;
-  map_y = map_view_y0 + ev->y / NORMAL_TILE_HEIGHT;
-  is_valid = normalize_map_pos(&map_x, &map_y);
-  assert(is_valid);
-
-  if (!(pcity = find_city_near_tile(map_x, map_y)))
-    return;
-
-  is_valid = map_to_city_map(&x, &y, pcity, map_x, map_y);
-  assert(is_valid);
-
-  packet.city_id=pcity->id;
-  packet.worker_x=x;
-  packet.worker_y=y;
-
-  wrk = get_worker_city(pcity, x, y);
-  if(wrk==C_TILE_WORKER)
-    send_packet_city_request(&aconnection, &packet, 
-			     PACKET_CITY_MAKE_SPECIALIST);
-  else if(wrk==C_TILE_EMPTY)
-    send_packet_city_request(&aconnection, &packet, 
-			     PACKET_CITY_MAKE_WORKER);
-
-  /* When the city info packet is received, update the workers on the map*/
-  city_workers_display = pcity;
-
-  return;
+  /* PORTME */
 }
 
 /**************************************************************************
@@ -362,24 +258,7 @@ void mapctrl_btn_adjust_workers(XEvent *event)
 **************************************************************************/
 void mapctrl_key_city_workers(XEvent *event)
 {
-  int x,y, is_real;
-  XButtonEvent *ev=&event->xbutton;
-  struct city *pcity;
-
-  if(get_client_state()!=CLIENT_GAME_RUNNING_STATE)
-    return;
-
-  x = map_view_x0 + ev->x / NORMAL_TILE_WIDTH;
-  y = map_view_y0 + ev->y / NORMAL_TILE_HEIGHT;
-  is_real = normalize_map_pos(&x, &y);
-  assert(is_real);
-
-  pcity = find_city_near_tile(x,y);
-  if(!pcity) return;
-
-  /* Shade tiles on usage */
-  city_workers_color = (city_workers_color%3)+1;
-  put_city_workers(pcity, city_workers_color);
+  key_city_overlay(event->xbutton.x, event->xbutton.y);
 }
 
 /**************************************************************************
@@ -387,27 +266,24 @@ void mapctrl_key_city_workers(XEvent *event)
 **************************************************************************/
 void mapctrl_btn_overviewcanvas(XEvent *event)
 {
-  int xtile, ytile;
-  XButtonEvent *ev=&event->xbutton;
+  int map_x, map_y;
+  struct tile *ptile;
+  XButtonEvent *ev = &event->xbutton;
 
-  xtile=ev->x/2-(map.xsize/2-(map_view_x0+map_canvas_store_twidth/2));
-  ytile=ev->y/2;
+  if (!can_client_change_view()) {
+    return;
+  }
 
-  if(get_client_state()!=CLIENT_GAME_RUNNING_STATE)
-     return;
+  overview_to_map_pos(&map_x, &map_y, event->xbutton.x, event->xbutton.y);
+  ptile = map_pos_to_tile(map_x, map_y);
+  if (!ptile) {
+    return;
+  }
 
   if(ev->button==Button1)
-    do_unit_goto(xtile,ytile);
+    do_map_click(ptile, SELECT_POPUP);
   else if(ev->button==Button3)
-    center_tile_mapcanvas(xtile, ytile);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void focus_to_next_unit(void)
-{
-  advance_unit_focus();
+    center_tile_mapcanvas(ptile);
 }
 
 /**************************************************************************
