@@ -13,12 +13,11 @@
 #ifndef FC__CITYDLG_G_H
 #define FC__CITYDLG_G_H
 
+#include "fc_types.h"
+
 #include "citydlg_common.h"
 
-struct unit;
-struct city;
-
-void popup_city_dialog(struct city *pcity, bool make_modal);
+void popup_city_dialog(struct city *pcity);
 void popdown_city_dialog(struct city *pcity);
 void popdown_all_city_dialogs(void);
 void refresh_city_dialog(struct city *pcity);
