@@ -16,33 +16,43 @@
 #include "shared.h"		/* bool type */
 
 #include "fc_types.h"
-#include "nation.h"		/* Nation_Type_id */
+#include "featured_text.h"      /* struct text_tag_list */
+#include "nation.h"		/* Nation_type_id */
 #include "terrain.h"		/* enum tile_special_type */
+#include "unitlist.h"
 
 struct packet_nations_selected_info;
 
 void popup_notify_goto_dialog(const char *headline, const char *lines,
-			      struct tile *ptile);
+			      const struct text_tag_list *tags,
+                              struct tile *ptile);
 void popup_notify_dialog(const char *caption, const char *headline,
-			 const char *lines);
+                         const char *lines);
+void popup_connect_msg(const char *headline, const char *message);
 
-void popup_races_dialog(void);
+void popup_races_dialog(struct player *pplayer);
 void popdown_races_dialog(void);
 
 void popup_unit_select_dialog(struct tile *ptile);
 
-void races_toggles_set_sensitive(bool *nations_used);
+void races_toggles_set_sensitive(void);
 
 void popup_caravan_dialog(struct unit *punit,
 			  struct city *phomecity, struct city *pdestcity);
-bool caravan_dialog_is_open(void);
+bool caravan_dialog_is_open(int* unit_id, int* city_id);
+void caravan_dialog_update(void);
+
 void popup_diplomat_dialog(struct unit *punit, struct tile *ptile);
-bool diplomat_dialog_is_open(void);
-void popup_incite_dialog(struct city *pcity);
-void popup_bribe_dialog(struct unit *punit);
+int diplomat_handled_in_diplomat_dialog(void);
+void close_diplomat_dialog(void);
+void popup_incite_dialog(struct city *pcity, int cost);
+void popup_bribe_dialog(struct unit *punit, int cost);
 void popup_sabotage_dialog(struct city *pcity);
-void popup_pillage_dialog(struct unit *punit,
-			  enum tile_special_type may_pillage);
+void popup_pillage_dialog(struct unit *punit, bv_special may_pillage,
+                          bv_bases bases);
+void popup_upgrade_dialog(struct unit_list *punits);
+void popup_tileset_suggestion_dialog(void);
+bool popup_theme_suggestion_dialog(const char *theme_name);
 
 void popdown_all_game_dialogs(void);
 
