@@ -1233,7 +1233,8 @@ const char *option_class_name(enum client_option_class option_class)
   case COC_GRAPHICS:
     return _("Graphics");
   case COC_OVERVIEW:
-    return _("Overview");
+    /* TRANS: Options section for overview map (mini-map) */
+    return Q_("?map:Overview");
   case COC_SOUND:
     return _("Sound");
   case COC_INTERFACE:
@@ -1826,8 +1827,14 @@ static void settable_options_save(struct section_file *sf)
   RETURN_IF_FAIL(NULL != settable_options_hash);
 
   hash_iterate(settable_options_hash, iter) {
+    const char *name = (const char *) hash_iter_get_key(iter);
+    if (!mystrcasecmp(name, "gameseed") || !mystrcasecmp(name, "mapseed")) {
+      /* Do not save mapseed or gameseed. */
+      continue;
+    }
+
     secfile_insert_str(sf, (const char *) hash_iter_get_value(iter),
-                       "server.%s", (const char *) hash_iter_get_key(iter));
+                       "server.%s", name);
   } hash_iterate_end;
 }
 
