@@ -55,6 +55,7 @@
 #include <winsock.h>
 #endif
 
+/* utility */
 #include "fcintl.h"
 #include "hash.h"
 #include "log.h"
@@ -64,13 +65,15 @@
 #include "registry.h"
 #include "support.h"
 
+/* common */
 #include "capstr.h"
 #include "dataio.h"
 #include "game.h"
 #include "packets.h"
 #include "version.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "servers.h"
 
 #include "gui_main_g.h"
@@ -96,6 +99,8 @@ struct server_scan {
   } meta;
 };
 
+extern enum announce_type announce;
+
 /**************************************************************************
  The server sends a stream in a registry 'ini' type format.
  Read it using secfile functions and fill the server_list structs.
@@ -106,13 +111,12 @@ static struct server_list *parse_metaserver_data(fz_FILE *f)
   struct section_file the_file, *file = &the_file;
   int nservers, i, j;
 
-  server_list = server_list_new();
-
   /* This call closes f. */
   if (!section_file_load_from_stream(file, f)) {
-    return server_list;
+    return NULL;
   }
 
+  server_list = server_list_new();
   nservers = secfile_lookup_int_default(file, 0, "main.nservers");
 
   for (i = 0; i < nservers; i++) {
@@ -124,7 +128,7 @@ static struct server_list *parse_metaserver_data(fz_FILE *f)
     pserver->host = mystrdup(host);
 
     port = secfile_lookup_str_default(file, "", "server%d.port", i);
-    pserver->port = mystrdup(port);
+    pserver->port = atoi(port);
 
     version = secfile_lookup_str_default(file, "", "server%d.version", i);
     pserver->version = mystrdup(version);
@@ -136,8 +140,8 @@ static struct server_list *parse_metaserver_data(fz_FILE *f)
     pserver->message = mystrdup(message);
 
     nplayers = secfile_lookup_str_default(file, "0", "server%d.nplayers", i);
-    pserver->nplayers = mystrdup(nplayers);
     n = atoi(nplayers);
+    pserver->nplayers = n;
 
     if (n > 0) {
       pserver->players = fc_malloc(n * sizeof(*pserver->players));
@@ -295,7 +299,7 @@ static void meta_send_request(struct server_scan *scan)
 
   my_uname(machine_string, sizeof(machine_string));
 
-  capstr = my_url_encode(our_capability);
+  capstr = fc_url_encode(our_capability);
 
   my_snprintf(str, sizeof(str),
     "POST %s HTTP/1.1\r\n"
@@ -312,9 +316,9 @@ static void meta_send_request(struct server_scan *scan)
     (unsigned long) (strlen("client_cap=") + strlen(capstr)),
     capstr);
 
-  if (my_writesocket(scan->sock, str, strlen(str)) != strlen(str)) {
+  if (fc_writesocket(scan->sock, str, strlen(str)) != strlen(str)) {
     /* Even with non-blocking this shouldn't fail. */
-    (scan->error_func)(scan, mystrerror());
+    scan->error_func(scan, fc_strerror(fc_get_errno()));
     return;
   }
 
@@ -336,25 +340,25 @@ static void meta_read_response(struct server_scan *scan)
     GetTempPath(sizeof(filename), filename);
     cat_snprintf(filename, sizeof(filename), "fctmp%d", myrand(1000));
 
-    scan->meta.fp = fopen(filename, "w+b");
+    scan->meta.fp = fc_fopen(filename, "w+b");
 #else
     scan->meta.fp = tmpfile();
 #endif
 
     if (!scan->meta.fp) {
-      (scan->error_func)(scan, _("Could not open temp file."));
+      scan->error_func(scan, _("Could not open temp file."));
     }
   }
 
   while (1) {
-    result = my_readsocket(scan->sock, buf, sizeof(buf));
+    result = fc_readsocket(scan->sock, buf, sizeof(buf));
 
     if (result < 0) {
       if (errno == EAGAIN || errno == EINTR || errno == EWOULDBLOCK) {
 	/* Keep waiting. */
 	return;
       }
-      (scan->error_func)(scan, mystrerror());
+      scan->error_func(scan, fc_strerror(fc_get_errno()));
       return;
     } else if (result == 0) {
       fz_FILE *f;
@@ -366,7 +370,6 @@ static void meta_read_response(struct server_scan *scan)
       f = fz_from_stream(scan->meta.fp);
       assert(f != NULL);
 
-
       /* skip HTTP headers */
       /* XXX: TODO check for magic Content-Type: text/x-ini -vasc */
       while (fz_fgets(str, sizeof(str), f) && strcmp(str, "\r\n") != 0) {
@@ -378,10 +381,21 @@ static void meta_read_response(struct server_scan *scan)
       /* parse HTTP message body */
       scan->servers = parse_metaserver_data(f);
       scan->meta.state = META_DONE;
+
+      /* 'f' (hence 'meta.fp') was closed in parse_metaserver_data(). */
+      scan->meta.fp = NULL;
+
+      if (NULL == scan->servers) {
+        my_snprintf(str, sizeof(str),
+                    _("Failed to parse the metaserver data from http://%s."),
+                    scan->meta.name);
+        scan->error_func(scan, str);
+      }
+
       return;
     } else {
       if (fwrite(buf, 1, result, scan->meta.fp) != result) {
-	(scan->error_func)(scan, mystrerror());
+	scan->error_func(scan, fc_strerror(fc_get_errno()));
       }
     }
   }
@@ -397,95 +411,121 @@ static void meta_read_response(struct server_scan *scan)
 ****************************************************************************/
 static bool begin_metaserver_scan(struct server_scan *scan)
 {
-  union my_sockaddr addr;
-  int s;
+  union fc_sockaddr names[2];
+  int i, name_count;
+  int s = -1;
 
-  scan->meta.urlpath = my_lookup_httpd(scan->meta.name, &scan->meta.port,
+  scan->meta.urlpath = fc_lookup_httpd(scan->meta.name, &scan->meta.port,
 				       metaserver);
   if (!scan->meta.urlpath) {
-    (scan->error_func)(scan,
-		       _("Invalid $http_proxy or metaserver value, must "
-			 "start with 'http://'"));
+    scan->error_func(scan,
+                     _("Invalid $http_proxy or metaserver value, must "
+                       "start with 'http://'"));
     return FALSE;
   }
 
-  if (!net_lookup_service(scan->meta.name, scan->meta.port, &addr)) {
-    (scan->error_func)(scan, _("Failed looking up metaserver's host"));
+  if (!net_lookup_service(scan->meta.name, scan->meta.port,
+                          &names[0], FALSE)) {
+    scan->error_func(scan, _("Failed looking up metaserver's host"));
     return FALSE;
   }
-  
-  if ((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-    (scan->error_func)(scan, mystrerror());
-    return FALSE;
-  }
-
-  my_nonblock(s);
-  
-  if (my_connect(s, (struct sockaddr *) &addr.sockaddr, sizeof(addr)) == -1) {
-    if (errno == EINPROGRESS) {
-      /* With non-blocking sockets this is the expected result. */
-      scan->meta.state = META_CONNECTING;
-      scan->sock = s;
-    } else {
-      my_closesocket(s);
-      (scan->error_func)(scan, mystrerror());
-      return FALSE;
+  name_count = 1;
+#ifdef IPV6_SUPPORT
+  if (names[0].saddr.sa_family == AF_INET6) {
+    /* net_lookup_service() prefers IPv6 address.
+     * Check if there is also IPv4 address.
+     * TODO: This would be easier using getaddrinfo() */
+    if (net_lookup_service(scan->meta.name, scan->meta.port,
+                           &names[1], TRUE /* force IPv4 */)) {
+      name_count = 2;
     }
-  } else {
-    /* Instant connection?  Whoa. */
-    scan->sock = s;
-    scan->meta.state = META_CONNECTING;
-    meta_send_request(scan);
+  }
+#endif
+
+  /* Try all (IPv4, IPv6, ...) addresses until we have a connection. */  
+  for (i = 0; i < name_count; i++) {
+    if ((s = socket(names[i].saddr.sa_family, SOCK_STREAM, 0)) == -1) {
+      /* Probably EAFNOSUPPORT or EPROTONOSUPPORT. */
+      continue;
+    }
+
+    fc_nonblock(s);
+  
+    if (fc_connect(s, &names[i].saddr, sockaddr_size(&names[i])) == -1) {
+      if (errno == EINPROGRESS) {
+        /* With non-blocking sockets this is the expected result. */
+        scan->meta.state = META_CONNECTING;
+        scan->sock = s;
+        break;
+      } else {
+        fc_closesocket(s);
+        s = -1;
+        continue;
+      }
+    } else {
+      /* Instant connection?  Whoa. */
+      scan->sock = s;
+      scan->meta.state = META_CONNECTING;
+      meta_send_request(scan);
+      break;
+    }
+  }
+
+  if (s == -1) {
+    scan->error_func(scan, fc_strerror(fc_get_errno()));
+    return FALSE;
   }
 
   return TRUE;
 }
 
 /**************************************************************************
- Create the list of servers from the metaserver
- The result must be free'd with delete_server_list() when no
- longer used
+  Check for data received from the metaserver.
 **************************************************************************/
-static struct server_list *get_metaserver_list(struct server_scan *scan)
+static enum server_scan_status
+get_metaserver_list(struct server_scan *scan)
 {
-  struct timeval tv = {.tv_sec = 0, .tv_usec = 0};
+  struct timeval tv;
   fd_set sockset;
 
-  if (scan->sock < 0) {
-    return NULL;
+  if (!scan || scan->sock < 0) {
+    return SCAN_STATUS_ERROR;
   }
 
+  tv.tv_sec = 0;
+  tv.tv_usec = 0;
   FD_ZERO(&sockset);
   FD_SET(scan->sock, &sockset);
 
   switch (scan->meta.state) {
   case META_CONNECTING:
-    if (my_select(scan->sock + 1, NULL, &sockset, NULL, &tv) < 0) {
-      (scan->error_func)(scan, mystrerror());
+    if (fc_select(scan->sock + 1, NULL, &sockset, NULL, &tv) < 0) {
+      scan->error_func(scan, fc_strerror(fc_get_errno()));
     } else if (FD_ISSET(scan->sock, &sockset)) {
       meta_send_request(scan);
-    } else {
-      /* Keep waiting. */
     }
-    return NULL;
+    /* Keep waiting. */
+    return SCAN_STATUS_WAITING;
+    break;
   case META_WAITING:
-    if (my_select(scan->sock + 1, &sockset, NULL, NULL, &tv) < 0) {
-      (scan->error_func)(scan, mystrerror());
+    if (fc_select(scan->sock + 1, &sockset, NULL, NULL, &tv) < 0) {
+      scan->error_func(scan, fc_strerror(fc_get_errno()));
     } else if (FD_ISSET(scan->sock, &sockset)) {
       meta_read_response(scan);
-      return scan->servers;
-    } else {
-      /* Keep waiting. */
+      return SCAN_STATUS_PARTIAL;
     }
-    return NULL;
+    /* Keep waiting. */
+    return SCAN_STATUS_WAITING;
+    break;
   case META_DONE:
-    /* We already have a server list but we don't return it since it hasn't
-     * changed. */
-    return NULL;
+    return SCAN_STATUS_DONE;
+    break;
+  default:
+    break;
   }
 
   assert(0);
-  return NULL;
+  return SCAN_STATUS_ERROR;
 }
 
 /**************************************************************************
@@ -495,12 +535,15 @@ static struct server_list *get_metaserver_list(struct server_scan *scan)
 **************************************************************************/
 static void delete_server_list(struct server_list *server_list)
 {
+  if (!server_list) {
+    return;
+  }
+
   server_list_iterate(server_list, ptmp) {
     int i;
-    int n = atoi(ptmp->nplayers);
+    int n = ptmp->nplayers;
 
     free(ptmp->host);
-    free(ptmp->port);
     free(ptmp->version);
     free(ptmp->state);
     free(ptmp->message);
@@ -514,12 +557,10 @@ static void delete_server_list(struct server_list *server_list)
       }
       free(ptmp->players);
     }
-    free(ptmp->nplayers);
 
     free(ptmp);
   } server_list_iterate_end;
 
-  server_list_unlink_all(server_list);
   server_list_free(server_list);
 }
 
@@ -530,7 +571,7 @@ static void delete_server_list(struct server_list *server_list)
 **************************************************************************/
 static bool begin_lanserver_scan(struct server_scan *scan)
 {
-  union my_sockaddr addr;
+  union fc_sockaddr addr;
   struct data_out dout;
   int sock, opt = 1;
 #ifndef HAVE_WINSOCK
@@ -538,30 +579,66 @@ static bool begin_lanserver_scan(struct server_scan *scan)
 #else  /* HAVE_WINSOCK */
   char buffer[MAX_LEN_PACKET];
 #endif /* HAVE_WINSOCK */
-  struct ip_mreq mreq;
+  struct ip_mreq mreq4;
   const char *group;
   size_t size;
+  int family;
+
+#ifdef IPV6_SUPPORT
+  struct ipv6_mreq mreq6;
+#endif
+
 #ifndef HAVE_WINSOCK
   unsigned char ttl;
 #endif
 
+  if (announce == ANNOUNCE_NONE) {
+    /* Succeeded in doing nothing */
+    return TRUE;
+  }
+
+#ifdef IPV6_SUPPORT
+  if (announce == ANNOUNCE_IPV6) {
+    family = AF_INET6;
+  } else
+#endif
+  {
+    family = AF_INET;
+  }
+
   /* Create a socket for broadcasting to servers. */
-  if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-    freelog(LOG_ERROR, "socket failed: %s", mystrerror());
+  if ((sock = socket(family, SOCK_DGRAM, 0)) < 0) {
+    freelog(LOG_ERROR, "socket failed: %s", fc_strerror(fc_get_errno()));
     return FALSE;
   }
 
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
                  (char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
   }
 
   /* Set the UDP Multicast group IP address. */
-  group = get_multicast_group();
+  group = get_multicast_group(announce == ANNOUNCE_IPV6);
   memset(&addr, 0, sizeof(addr));
-  addr.sockaddr_in.sin_family = AF_INET;
-  addr.sockaddr_in.sin_addr.s_addr = inet_addr(get_multicast_group());
-  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT);
+
+#ifndef IPV6_SUPPORT
+  {
+#ifdef HAVE_INET_ATON
+    inet_aton(group, &addr.saddr_in4.sin_addr);
+#else  /* HAVE_INET_ATON */
+    addr.saddr_in4.sin_addr.s_addr = inet_addr(group);
+#endif /* HAVE_INET_ATON */
+#else  /* IPv6 support */
+  if (family == AF_INET6) {
+    addr.saddr.sa_family = AF_INET6;
+    inet_pton(AF_INET6, group, &addr.saddr_in6.sin6_addr);
+    addr.saddr_in6.sin6_port = htons(SERVER_LAN_PORT);
+  } else {
+    inet_pton(AF_INET, group, &addr.saddr_in4.sin_addr);
+#endif /* IPv6 support */
+    addr.saddr.sa_family = AF_INET;
+    addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT);
+  }
 
 /* this setsockopt call fails on Windows 98, so we stick with the default
  * value of 1 on Windows, which should be fine in most cases */
@@ -570,14 +647,14 @@ static bool begin_lanserver_scan(struct server_scan *scan)
   ttl = SERVER_LAN_TTL;
   if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, (const char*)&ttl, 
                  sizeof(ttl))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return FALSE;
   }
-#endif
+#endif /* HAVE_WINSOCK */
 
   if (setsockopt(sock, SOL_SOCKET, SO_BROADCAST, (const char*)&opt, 
                  sizeof(opt))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return FALSE;
   }
 
@@ -586,47 +663,78 @@ static bool begin_lanserver_scan(struct server_scan *scan)
   size = dio_output_used(&dout);
  
 
-  if (sendto(sock, buffer, size, 0, &addr.sockaddr,
-      sizeof(addr)) < 0) {
+  if (sendto(sock, buffer, size, 0, &addr.saddr,
+             sockaddr_size(&addr)) < 0) {
     /* This can happen when there's no network connection - it should
      * give an in-game message. */
-    freelog(LOG_ERROR, "sendto failed: %s", mystrerror());
+    freelog(LOG_ERROR, "lanserver scan sendto failed: %s",
+	    fc_strerror(fc_get_errno()));
     return FALSE;
   } else {
     freelog(LOG_DEBUG, ("Sending request for server announcement on LAN."));
   }
 
-  my_closesocket(sock);
+  fc_closesocket(sock);
 
   /* Create a socket for listening for server packets. */
-  if ((scan->sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-    (scan->error_func)(scan, mystrerror());
+  if ((scan->sock = socket(family, SOCK_DGRAM, 0)) < 0) {
+    scan->error_func(scan, fc_strerror(fc_get_errno()));
     return FALSE;
   }
 
-  my_nonblock(scan->sock);
+  fc_nonblock(scan->sock);
 
   if (setsockopt(scan->sock, SOL_SOCKET, SO_REUSEADDR,
                  (char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
   }
                                                                                
   memset(&addr, 0, sizeof(addr));
-  addr.sockaddr_in.sin_family = AF_INET;
-  addr.sockaddr_in.sin_addr.s_addr = htonl(INADDR_ANY); 
-  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT + 1);
 
-  if (bind(scan->sock, &addr.sockaddr, sizeof(addr)) < 0) {
-    (scan->error_func)(scan, mystrerror());
+#ifdef IPV6_SUPPORT
+  if (family == AF_INET6) {
+    addr.saddr.sa_family = AF_INET6;
+    addr.saddr_in6.sin6_port = htons(SERVER_LAN_PORT + 1);
+    addr.saddr_in6.sin6_addr = in6addr_any;
+  } else
+#endif /* IPv6 support */
+  {
+    addr.saddr.sa_family = AF_INET;
+    addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT + 1);
+    addr.saddr_in4.sin_addr.s_addr = htonl(INADDR_ANY);
+  }
+
+  if (bind(scan->sock, &addr.saddr, sockaddr_size(&addr)) < 0) {
+    scan->error_func(scan, fc_strerror(fc_get_errno()));
     return FALSE;
   }
 
-  mreq.imr_multiaddr.s_addr = inet_addr(group);
-  mreq.imr_interface.s_addr = htonl(INADDR_ANY);
-  if (setsockopt(scan->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
-                 (const char*)&mreq, sizeof(mreq)) < 0) {
-    (scan->error_func)(scan, mystrerror());
-    return FALSE;
+#ifndef IPV6_SUPPORT
+  {
+#ifdef HAVE_INET_ATON
+    inet_aton(group, &mreq4.imr_multiaddr);
+#else  /* HAVE_INET_ATON */
+    mreq4.imr_multiaddr.s_addr = inet_addr(group);
+#endif /* HAVE_INET_ATON */
+#else  /* IPv6 support */
+  if (family == AF_INET6) {
+    inet_pton(AF_INET6, group, &mreq6.ipv6mr_multiaddr.s6_addr);
+    mreq6.ipv6mr_interface = 0; /* TODO: Interface selection */
+
+    if (setsockopt(scan->sock, IPPROTO_IPV6, FC_IPV6_ADD_MEMBERSHIP,
+                   (const char*)&mreq6, sizeof(mreq6)) < 0) {
+      scan->error_func(scan, fc_strerror(fc_get_errno()));
+    }
+  } else {
+    inet_pton(AF_INET, group, &mreq4.imr_multiaddr.s_addr);
+#endif /* IPv6 support */
+    mreq4.imr_interface.s_addr = htonl(INADDR_ANY);
+
+    if (setsockopt(scan->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                   (const char*)&mreq4, sizeof(mreq4)) < 0) {
+      scan->error_func(scan, fc_strerror(fc_get_errno()));
+      return FALSE;
+    }
   }
 
   scan->servers = server_list_new();
@@ -638,16 +746,17 @@ static bool begin_lanserver_scan(struct server_scan *scan)
   Listens for UDP packets broadcasted from a server that responded
   to the request-packet sent from the client. 
 **************************************************************************/
-static struct server_list *get_lan_server_list(struct server_scan *scan)
+static enum server_scan_status
+get_lan_server_list(struct server_scan *scan)
 {
   socklen_t fromlen;
-  union my_sockaddr fromend;
-  struct hostent *from;
+  union fc_sockaddr fromend;
   char msgbuf[128];
   int type;
   struct data_in din;
   char servername[512];
-  char port[256];
+  char portstr[256];
+  int port;
   char version[256];
   char status[256];
   char players[256];
@@ -664,7 +773,7 @@ static struct server_list *get_lan_server_list(struct server_scan *scan)
     /* Try to receive a packet from a server.  No select loop is needed;
      * we just keep on reading until recvfrom returns -1. */
     if (recvfrom(scan->sock, msgbuf, sizeof(msgbuf), 0,
-		 &fromend.sockaddr, &fromlen) < 0) {
+		 &fromend.saddr, &fromlen) < 0) {
       break;
     }
 
@@ -673,22 +782,54 @@ static struct server_list *get_lan_server_list(struct server_scan *scan)
       continue;
     }
     dio_get_string(&din, servername, sizeof(servername));
-    dio_get_string(&din, port, sizeof(port));
+    dio_get_string(&din, portstr, sizeof(portstr));
+    port = atoi(portstr);
     dio_get_string(&din, version, sizeof(version));
     dio_get_string(&din, status, sizeof(status));
     dio_get_string(&din, players, sizeof(players));
     dio_get_string(&din, message, sizeof(message));
 
     if (!mystrcasecmp("none", servername)) {
-      from = gethostbyaddr((char *) &fromend.sockaddr_in.sin_addr,
-			   sizeof(fromend.sockaddr_in.sin_addr), AF_INET);
-      sz_strlcpy(servername, inet_ntoa(fromend.sockaddr_in.sin_addr));
+      bool nameinfo = FALSE;
+#ifdef IPV6_SUPPORT
+      char dst[INET6_ADDRSTRLEN];
+      char host[NI_MAXHOST], service[NI_MAXSERV];
+
+      if (!getnameinfo(&fromend.saddr, fromlen, host, NI_MAXHOST,
+                       service, NI_MAXSERV, NI_NUMERICSERV)) {
+        nameinfo = TRUE;
+      }
+      if (!nameinfo) {
+        if (fromend.saddr.sa_family == AF_INET6) {
+          inet_ntop(AF_INET6, &fromend.saddr_in6.sin6_addr,
+                    dst, sizeof(dst));
+        } else {
+          inet_ntop(AF_INET, &fromend.saddr_in4.sin_addr, dst, sizeof(dst));;
+        }
+      }
+#else  /* IPv6 support */
+      const char *dst = NULL;
+      struct hostent *from;
+      const char *host = NULL;
+
+      from = gethostbyaddr((char *) &fromend.saddr_in4.sin_addr,
+			   sizeof(fromend.saddr_in4.sin_addr), AF_INET);
+      if (from) {
+        host = from->h_name;
+        nameinfo = TRUE;
+      }
+      if (!nameinfo) {
+        dst = inet_ntoa(fromend.saddr_in4.sin_addr);
+      }
+#endif /* IPv6 support */
+
+      sz_strlcpy(servername, nameinfo ? host : dst);
     }
 
     /* UDP can send duplicate or delayed packets. */
     server_list_iterate(scan->servers, aserver) {
-      if (!mystrcasecmp(aserver->host, servername) 
-          && !mystrcasecmp(aserver->port, port)) {
+      if (0 == mystrcasecmp(aserver->host, servername)
+          && aserver->port == port) {
 	duplicate = TRUE;
 	break;
       } 
@@ -702,10 +843,10 @@ static struct server_list *get_lan_server_list(struct server_scan *scan)
     
     pserver = fc_malloc(sizeof(*pserver));
     pserver->host = mystrdup(servername);
-    pserver->port = mystrdup(port);
+    pserver->port = port;
     pserver->version = mystrdup(version);
     pserver->state = mystrdup(status);
-    pserver->nplayers = mystrdup(players);
+    pserver->nplayers = atoi(players);
     pserver->message = mystrdup(message);
     pserver->players = NULL;
     found_new = TRUE;
@@ -713,47 +854,52 @@ static struct server_list *get_lan_server_list(struct server_scan *scan)
     server_list_prepend(scan->servers, pserver);
   }
 
-  return found_new ? scan->servers : NULL;
+  if (found_new) {
+    return SCAN_STATUS_PARTIAL;
+  }
+  return SCAN_STATUS_WAITING;
 }
 
 /****************************************************************************
-  Creates a new server scan, and starts scanning.
+  Creates a new server scan and returns it, or NULL if impossible.
 
   Depending on 'type' the scan will look for either local or internet
   games.
 
   error_func provides a callback to be used in case of error; this
   callback probably should call server_scan_finish.
+
+  NB: You must call server_scan_finish() when you are done with the
+  scan to free the memory and resources allocated by it.
 ****************************************************************************/
 struct server_scan *server_scan_begin(enum server_scan_type type,
 				      ServerScanErrorFunc error_func)
 {
-  struct server_scan *scan = fc_calloc(sizeof(*scan), 1);
+  struct server_scan *scan;
+  bool ok = FALSE;
 
+  scan = fc_calloc(1, sizeof(*scan));
   scan->type = type;
   scan->error_func = error_func;
-
   scan->sock = -1;
 
   switch (type) {
   case SERVER_SCAN_GLOBAL:
-    if (begin_metaserver_scan(scan)) {
-      return scan;
-    } else {
-      return NULL;
-    }
+    ok = begin_metaserver_scan(scan);
+    break;
   case SERVER_SCAN_LOCAL:
-    if (begin_lanserver_scan(scan)) {
-      return scan;
-    } else {
-      return NULL;
-    }
-  case SERVER_SCAN_LAST:
+    ok = begin_lanserver_scan(scan);
+    break;
+  default:
     break;
   }
 
-  assert(0);
-  return NULL;
+  if (!ok) {
+    server_scan_finish(scan);
+    scan = NULL;
+  }
+
+  return scan;
 }
 
 /****************************************************************************
@@ -762,45 +908,70 @@ struct server_scan *server_scan_begin(enum server_scan_type type,
 ****************************************************************************/
 enum server_scan_type server_scan_get_type(const struct server_scan *scan)
 {
+  if (!scan) {
+    return SERVER_SCAN_LAST;
+  }
   return scan->type;
 }
 
 /****************************************************************************
-  A function to query servers of the server scan.  This will check any
-  pending network data and update the server list.  It then returns
-  a server_list if any new servers are found, or NULL if the list has not
-  changed.
+  A function to query servers of the server scan. This will check any
+  pending network data and update the server list.
 
-  Note that unless the list has changed NULL will be returned.  Since
-  polling is likely to be used with the server scans, callers should poll
-  this function often (every 100 ms) but only need to take further action
-  when a non-NULL value is returned.
+  The return value indicates the status of the server scan:
+    SCAN_STATUS_ERROR   - The scan failed and should be aborted.
+    SCAN_STATUS_WAITING - The scan is in progress (continue polling).
+    SCAN_STATUS_PARTIAL - The scan received some data, with more expected.
+                          Get the servers with server_scan_get_list(), and
+                          continue polling.
+    SCAN_STATUS_DONE    - The scan received all data it expected to receive.
+                          Get the servers with server_scan_get_list(), and
+                          stop calling this function.
 ****************************************************************************/
-struct server_list *server_scan_get_servers(struct server_scan *scan)
+enum server_scan_status server_scan_poll(struct server_scan *scan)
 {
+  if (!scan) {
+    return SCAN_STATUS_ERROR;
+  }
+
   switch (scan->type) {
   case SERVER_SCAN_GLOBAL:
     return get_metaserver_list(scan);
+    break;
   case SERVER_SCAN_LOCAL:
-   return get_lan_server_list(scan);
-  case SERVER_SCAN_LAST:
+    return get_lan_server_list(scan);
+    break;
+  default:
     break;
   }
 
-  assert(0);
-  return NULL;
+  return SCAN_STATUS_ERROR;
 }
 
 /**************************************************************************
-  Closes the socket listening on the scan and frees the list of servers.
+  Returns the server_list currently held by the scan (may be NULL).
+**************************************************************************/
+const struct server_list *
+server_scan_get_list(const struct server_scan *scan)
+{
+  if (!scan) {
+    return NULL;
+  }
+  return scan->servers;
+}
+
+/**************************************************************************
+  Closes the socket listening on the scan, frees the list of servers, and
+  frees the memory allocated for 'scan' by server_scan_begin().
 **************************************************************************/
 void server_scan_finish(struct server_scan *scan)
 {
   if (!scan) {
     return;
   }
+
   if (scan->sock >= 0) {
-    my_closesocket(scan->sock);
+    fc_closesocket(scan->sock);
     scan->sock = -1;
   }
 
@@ -809,5 +980,10 @@ void server_scan_finish(struct server_scan *scan)
     scan->servers = NULL;
   }
 
-  /* FIXME: do we need to close scan->meta.fp or free scan->meta.urlpath? */
+  if (scan->meta.fp) {
+    fclose(scan->meta.fp);
+    scan->meta.fp = NULL;
+  }
+
+  free(scan);
 }
