@@ -1286,17 +1286,17 @@ const char *get_report_title(const char *report_name)
   if (pplayer != NULL) {
     /* TRANS: <nation adjective> <government name>.
      * E.g. "Polish Republic". */
-    astr_add_line(&str, _("?nationgovernment:%s %s"),
+    astr_add_line(&str, Q_("?nationgovernment:%s %s"),
                   nation_adjective_for_player(pplayer),
                   government_name_for_player(pplayer));
 
-    astr_add_line(&str, "%s %s: %s",
+    astr_add_line(&str, "%s %s - %s",
                   ruler_title_translation(pplayer),
                   player_name(pplayer),
                   textyear(game.info.year));
   } else {
     /* TRANS: "Observer: 1985" */
-    astr_add_line(&str, _("Observer: %s"),
+    astr_add_line(&str, _("Observer - %s"),
 		  textyear(game.info.year));
   }
   return str.str;
