@@ -15,6 +15,7 @@
 #define FC__TEXT_H
 
 #include "fc_types.h"
+#include "unitlist.h"
 
 struct player_spaceship;
 
@@ -29,14 +30,14 @@ const char *unit_description(struct unit *punit);
 const char *science_dialog_text(void);
 const char *get_science_target_text(double *percent);
 const char *get_science_goal_text(Tech_type_id goal);
-const char *get_info_label_text(void);
+const char *get_info_label_text(bool moreinfo);
 const char *get_info_label_text_popup(void);
 const char *get_bulb_tooltip(void);
 const char *get_global_warming_tooltip(void);
 const char *get_nuclear_winter_tooltip(void);
 const char *get_government_tooltip(void);
 const char *get_unit_info_label_text1(struct unit_list *punits);
-const char *get_unit_info_label_text2(struct unit_list *punits);
+const char *get_unit_info_label_text2(struct unit_list *punits, int linebreaks);
 bool get_units_upgrade_info(char *buf, size_t bufsz,
 			    struct unit_list *punits);
 const char *get_spaceship_descr(struct player_spaceship *pship);
