@@ -113,13 +113,16 @@ struct connection {
   int sock;
   bool used;
   bool established;		/* have negotiated initial packets */
-  struct player *player;	/* NULL for connections not yet associated
-				   with a specific player */
-  /* 
-   * connection is "observer", not controller; may be observing
+
+  /* connection is "observer", not controller; may be observing
    * specific player, or all (implementation incomplete).
    */
   bool observer;
+
+  /* NULL for connections not yet associated with a specific player.
+   */
+  struct player *playing;
+
   struct socket_packet_buffer *buffer;
   struct socket_packet_buffer *send_buffer;
   struct timer *last_write;
@@ -149,10 +152,8 @@ struct connection {
   bool delayed_disconnect;
 
   void (*notify_of_writable_data) (struct connection * pc,
-				   bool data_available_and_socket_full);
+                                   bool data_available_and_socket_full);
 
-  /* Determines whether client or server behavior should be used. */
-  bool is_server;
   struct {
     /* 
      * Increases for every packet send to the server.
@@ -281,6 +282,9 @@ enum cmdlevel_id conn_get_access(const struct connection *pconn);
 struct player;
 struct player *conn_get_player(const struct connection *pconn);
 
+bool can_conn_edit(const struct connection *pconn);
+bool can_conn_enable_editing(const struct connection *pconn);
+
 int get_next_request_id(int old_request_id);
 
 extern const char blank_addr_str[];
