@@ -64,18 +64,18 @@ enum impr_genus_id find_genus_by_rule_name(const char *s)
 }
 
 /****************************************************************************
-  Inialize building structures.
+  Initialize building structures.
 ****************************************************************************/
 void improvements_init(void)
 {
   int i;
 
-  /* Can't use impr_type_iterate or improvement_by_number here because
-   * num_impr_types isn't known yet. */
+  /* Can't use improvement_iterate() or improvement_by_number() here
+   * because num_impr_types isn't known yet. */
   for (i = 0; i < ARRAY_SIZE(improvement_types); i++) {
     struct impr_type *p = &improvement_types[i];
 
-    p->index = i;
+    p->item_number = i;
     requirement_vector_init(&p->reqs);
   }
 }
@@ -83,10 +83,8 @@ void improvements_init(void)
 /**************************************************************************
   Frees the memory associated with this improvement.
 **************************************************************************/
-static void improvement_free(Impr_type_id id)
+static void improvement_free(struct impr_type *p)
 {
-  struct impr_type *p = improvement_by_number(id);
-
   free(p->helptext);
   p->helptext = NULL;
 
@@ -98,34 +96,60 @@ static void improvement_free(Impr_type_id id)
 ***************************************************************/
 void improvements_free()
 {
-  impr_type_iterate(impr) {
-    improvement_free(impr);
-  } impr_type_iterate_end;
+  improvement_iterate(p) {
+    improvement_free(p);
+  } improvement_iterate_end;
 }
 
 /**************************************************************************
-Returns 1 if the improvement_type "exists" in this game, 0 otherwise.
-An improvement_type doesn't exist if one of:
-- id is out of range;
-- the improvement_type has been flagged as removed by setting its
-  tech_req to A_LAST;
-- it is a space part, and the spacerace is not enabled.
-Arguably this should be called improvement_type_exists, but that's too long.
+  Return the first item of improvements.
 **************************************************************************/
-bool improvement_exists(Impr_type_id id)
+struct impr_type *improvement_array_first(void)
 {
-  if (id < 0 || id >= B_LAST || id>= game.control.num_impr_types)
-    return FALSE;
-
-  if (!game.info.spacerace
-      && (building_has_effect(id, EFT_SS_STRUCTURAL)
-	  || building_has_effect(id, EFT_SS_COMPONENT)
-	  || building_has_effect(id, EFT_SS_MODULE))) {
-    /* This assumes that space parts don't have any other effects. */
-    return FALSE;
+  if (game.control.num_impr_types > 0) {
+    return improvement_types;
   }
+  return NULL;
+}
 
-  return TRUE;
+/**************************************************************************
+  Return the last item of improvements.
+**************************************************************************/
+const struct impr_type *improvement_array_last(void)
+{
+  if (game.control.num_impr_types > 0) {
+    return &improvement_types[game.control.num_impr_types - 1];
+  }
+  return NULL;
+}
+
+/**************************************************************************
+  Return the number of improvements.
+**************************************************************************/
+Impr_type_id improvement_count(void)
+{
+  return game.control.num_impr_types;
+}
+
+/**************************************************************************
+  Return the improvement index.
+
+  Currently same as improvement_number(), paired with improvement_count()
+  indicates use as an array index.
+**************************************************************************/
+Impr_type_id improvement_index(const struct impr_type *pimprove)
+{
+  assert(pimprove);
+  return pimprove - improvement_types;
+}
+
+/**************************************************************************
+  Return the improvement index.
+**************************************************************************/
+Impr_type_id improvement_number(const struct impr_type *pimprove)
+{
+  assert(pimprove);
+  return pimprove->item_number;
 }
 
 /**************************************************************************
@@ -134,44 +158,79 @@ bool improvement_exists(Impr_type_id id)
 **************************************************************************/
 struct impr_type *improvement_by_number(const Impr_type_id id)
 {
-  if (id < 0 || id >= game.control.num_impr_types) {
+  if (id < 0 || id >= improvement_count()) {
     return NULL;
   }
   return &improvement_types[id];
 }
 
+/**************************************************************************
+  Returns pointer when the improvement_type "exists" in this game,
+  returns NULL otherwise.
+
+  An improvement_type doesn't exist for any of:
+   - the improvement_type has been flagged as removed by setting its
+     tech_req to A_LAST; [was not in current 2007-07-27]
+   - it is a space part, and the spacerace is not enabled.
+**************************************************************************/
+struct impr_type *valid_improvement(struct impr_type *pimprove)
+{
+  if (NULL == pimprove) {
+    return NULL;
+  }
+
+  if (!game.info.spacerace
+      && (building_has_effect(pimprove, EFT_SS_STRUCTURAL)
+	  || building_has_effect(pimprove, EFT_SS_COMPONENT)
+	  || building_has_effect(pimprove, EFT_SS_MODULE))) {
+    /* This assumes that space parts don't have any other effects. */
+    return NULL;
+  }
+
+  return pimprove;
+}
+
+/**************************************************************************
+  Returns pointer when the improvement_type "exists" in this game,
+  returns NULL otherwise.
+
+  In addition to valid_improvement(), tests for id is out of range.
+**************************************************************************/
+struct impr_type *valid_improvement_by_number(const Impr_type_id id)
+{
+  return valid_improvement(improvement_by_number(id));
+}
+
 /**************************************************************************
   Return the (translated) name of the given improvement. 
   You don't have to free the return pointer.
 **************************************************************************/
-const char *improvement_name_translation(Impr_type_id id)
+const char *improvement_name_translation(struct impr_type *pimprove)
 {
-  struct impr_type *itp = improvement_by_number(id);
-
-  if (NULL == itp->name.translated) {
+  if (NULL == pimprove->name.translated) {
     /* delayed (unified) translation */
-    itp->name.translated = ('\0' == itp->name.vernacular[0])
-			   ? itp->name.vernacular
-			   : Q_(itp->name.vernacular);
+    pimprove->name.translated = ('\0' == pimprove->name.vernacular[0])
+				? pimprove->name.vernacular
+				: Q_(pimprove->name.vernacular);
   }
-  return itp->name.translated;
+  return pimprove->name.translated;
 }
 
 /****************************************************************************
   Return the (untranslated) rule name of the improvement.
   You don't have to free the return pointer.
 ****************************************************************************/
-const char *improvement_rule_name(Impr_type_id id)
+const char *improvement_rule_name(const struct impr_type *pimprove)
 {
-  return Qn_(improvement_by_number(id)->name.vernacular); 
+  return Qn_(pimprove->name.vernacular); 
 }
 
 /****************************************************************************
   Returns the number of shields it takes to build this improvement.
 ****************************************************************************/
-int impr_build_shield_cost(Impr_type_id id)
+int impr_build_shield_cost(const struct impr_type *pimprove)
 {
-  int base = improvement_by_number(id)->build_cost;
+  int base = pimprove->build_cost;
 
   return MAX(base * game.info.shieldbox / 100, 1);
 }
@@ -179,12 +238,12 @@ int impr_build_shield_cost(Impr_type_id id)
 /****************************************************************************
   Returns the amount of gold it takes to rush this improvement.
 ****************************************************************************/
-int impr_buy_gold_cost(Impr_type_id id, int shields_in_stock)
+int impr_buy_gold_cost(const struct impr_type *pimprove, int shields_in_stock)
 {
   int cost = 0;
-  const int missing = impr_build_shield_cost(id) - shields_in_stock;
+  const int missing = impr_build_shield_cost(pimprove) - shields_in_stock;
 
-  if (improvement_has_flag(id, IF_GOLD)) {
+  if (improvement_has_flag(pimprove, IF_GOLD)) {
     /* Can't buy capitalization. */
     return 0;
   }
@@ -193,7 +252,7 @@ int impr_buy_gold_cost(Impr_type_id id, int shields_in_stock)
     cost = 2 * missing;
   }
 
-  if (is_wonder(id)) {
+  if (is_wonder(pimprove)) {
     cost *= 2;
   }
   if (shields_in_stock == 0) {
@@ -205,58 +264,59 @@ int impr_buy_gold_cost(Impr_type_id id, int shields_in_stock)
 /****************************************************************************
   Returns the amount of gold received when this improvement is sold.
 ****************************************************************************/
-int impr_sell_gold(Impr_type_id id)
+int impr_sell_gold(const struct impr_type *pimprove)
 {
-  return impr_build_shield_cost(id);
+  return impr_build_shield_cost(pimprove);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-bool is_wonder(Impr_type_id id)
+bool is_wonder(const struct impr_type *pimprove)
 {
-  return (is_great_wonder(id) || is_small_wonder(id));
+  return (is_great_wonder(pimprove) || is_small_wonder(pimprove));
 }
 
 /**************************************************************************
   Does a linear search of improvement_types[].name.translated
-  Returns B_LAST if none match.
+  Returns NULL when none match.
 **************************************************************************/
-Impr_type_id find_improvement_by_translated_name(const char *s)
+struct impr_type *find_improvement_by_translated_name(const char *name)
 {
-  impr_type_iterate(i) {
-    if (0 == strcmp(improvement_name_translation(i), s)) {
-      return i;
+  improvement_iterate(pimprove) {
+    if (0 == strcmp(improvement_name_translation(pimprove), name)) {
+      return pimprove;
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 
-  return B_LAST;
+  return NULL;
 }
 
 /****************************************************************************
   Does a linear search of improvement_types[].name.vernacular
-  Returns B_LAST if none match.
+  Returns NULL when none match.
 ****************************************************************************/
-Impr_type_id find_improvement_by_rule_name(const char *s)
+struct impr_type *find_improvement_by_rule_name(const char *name)
 {
-  const char *qs = Qn_(s);
+  const char *qname = Qn_(name);
 
-  impr_type_iterate(i) {
-    if (0 == mystrcasecmp(improvement_rule_name(i), qs)) {
-      return i;
+  improvement_iterate(pimprove) {
+    if (0 == mystrcasecmp(improvement_rule_name(pimprove), qname)) {
+      return pimprove;
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 
-  return B_LAST;
+  return NULL;
 }
 
 /**************************************************************************
  Return TRUE if the impr has this flag otherwise FALSE
 **************************************************************************/
-bool improvement_has_flag(Impr_type_id id, enum impr_flag_id flag)
+bool improvement_has_flag(const struct impr_type *pimprove,
+			  enum impr_flag_id flag)
 {
   assert(flag >= 0 && flag < IF_LAST);
-  return TEST_BIT(improvement_by_number(id)->flags, flag);
+  return TEST_BIT(pimprove->flags, flag);
 }
 
 /**************************************************************************
@@ -280,29 +340,51 @@ enum impr_flag_id find_improvement_flag_by_rule_name(const char *s)
 /**************************************************************************
  Return TRUE if the improvement should be visible to others without spying
 **************************************************************************/
-bool is_improvement_visible(Impr_type_id id)
+bool is_improvement_visible(const struct impr_type *pimprove)
 {
-  return (is_wonder(id) || improvement_has_flag(id, IF_VISIBLE_BY_OTHERS));
+  return (is_wonder(pimprove)
+          || improvement_has_flag(pimprove, IF_VISIBLE_BY_OTHERS));
 }
 
 /**************************************************************************
  Returns 1 if the improvement is obsolete, now also works for wonders
 **************************************************************************/
-bool improvement_obsolete(const struct player *pplayer, Impr_type_id id) 
+bool improvement_obsolete(const struct player *pplayer,
+			  const struct impr_type *pimprove)
 {
-  struct impr_type *impr = improvement_by_number(id);
-
-  if (!tech_exists(impr->obsolete_by)) {
+  if (!valid_advance(pimprove->obsolete_by)) {
     return FALSE;
   }
 
-  if (is_great_wonder(id)) {
+  if (is_great_wonder(pimprove)) {
     /* a great wonder is obsolete, as soon as *any* player researched the
        obsolete tech */
-   return game.info.global_advances[impr->obsolete_by];
+   return game.info.global_advances[advance_index(pimprove->obsolete_by)];
   }
 
-  return (get_invention(pplayer, impr->obsolete_by) == TECH_KNOWN);
+  return (player_invention_state(pplayer, advance_number(pimprove->obsolete_by)) == TECH_KNOWN);
+}
+
+/**************************************************************************
+  Returns TRUE iff improvement provides units buildable by player
+**************************************************************************/
+bool impr_provides_buildable_units(const struct player *pplayer,
+                                   const struct impr_type *pimprove)
+{
+  /* Fast check */
+  if (! pimprove->allows_units) {
+    return FALSE;
+  }
+
+  unit_type_iterate(ut) {
+    if (ut->need_improvement == pimprove) {
+      if (can_player_build_unit_now(pplayer, ut)) {
+        return TRUE;
+      }
+    }
+  } unit_type_iterate_end;
+
+  return FALSE;
 }
 
 /**************************************************************************
@@ -310,19 +392,15 @@ bool improvement_obsolete(const struct player *pplayer, Impr_type_id id)
    is obsolete.
 **************************************************************************/
 bool can_player_build_improvement_direct(const struct player *p,
-					 Impr_type_id id)
+					 struct impr_type *pimprove)
 {
-  struct impr_type *impr;
   bool space_part = FALSE;
 
-  /* This also checks if tech req is Never */
-  if (!improvement_exists(id)) {
+  if (!valid_improvement(pimprove)) {
     return FALSE;
   }
 
-  impr = improvement_by_number(id);
-
-  requirement_vector_iterate(&impr->reqs, preq) {
+  requirement_vector_iterate(&pimprove->reqs, preq) {
     if (preq->range >= REQ_RANGE_PLAYER
         && !is_req_active(p, NULL, NULL, NULL, NULL, NULL, NULL, preq,
                           RPT_CERTAIN)) {
@@ -332,19 +410,19 @@ bool can_player_build_improvement_direct(const struct player *p,
 
   /* Check for space part construction.  This assumes that space parts have
    * no other effects. */
-  if (building_has_effect(id, EFT_SS_STRUCTURAL)) {
+  if (building_has_effect(pimprove, EFT_SS_STRUCTURAL)) {
     space_part = TRUE;
     if (p->spaceship.structurals >= NUM_SS_STRUCTURALS) {
       return FALSE;
     }
   }
-  if (building_has_effect(id, EFT_SS_COMPONENT)) {
+  if (building_has_effect(pimprove, EFT_SS_COMPONENT)) {
     space_part = TRUE;
     if (p->spaceship.components >= NUM_SS_COMPONENTS) {
       return FALSE;
     }
   }
-  if (building_has_effect(id, EFT_SS_MODULE)) {
+  if (building_has_effect(pimprove, EFT_SS_MODULE)) {
     space_part = TRUE;
     if (p->spaceship.modules >= NUM_SS_MODULES) {
       return FALSE;
@@ -356,9 +434,9 @@ bool can_player_build_improvement_direct(const struct player *p,
     return FALSE;
   }
 
-  if (is_great_wonder(id)) {
+  if (is_great_wonder(pimprove)) {
     /* Can't build wonder if already built */
-    if (great_wonder_was_built(id)) {
+    if (!great_wonder_is_available(pimprove)) {
       return FALSE;
     }
   }
@@ -367,16 +445,16 @@ bool can_player_build_improvement_direct(const struct player *p,
 }
 
 /**************************************************************************
-  Whether player can _eventually_ build given building somewhere -- i.e.,
-  returns TRUE if building is available with current tech OR will be
-  available with future tech.  Returns FALSE if building is obsolete.
+  Whether player can build given building somewhere immediately.
+  Returns FALSE if building is obsolete.
 **************************************************************************/
-bool can_player_build_improvement(const struct player *p, Impr_type_id id)
+bool can_player_build_improvement_now(const struct player *p,
+				      struct impr_type *pimprove)
 {
-  if (!can_player_build_improvement_direct(p, id)) {
+  if (!can_player_build_improvement_direct(p, pimprove)) {
     return FALSE;
   }
-  if (improvement_obsolete(p, id)) {
+  if (improvement_obsolete(p, pimprove)) {
     return FALSE;
   }
   return TRUE;
@@ -387,22 +465,19 @@ bool can_player_build_improvement(const struct player *p, Impr_type_id id)
   returns TRUE if building is available with current tech OR will be
   available with future tech.  Returns FALSE if building is obsolete.
 **************************************************************************/
-bool can_player_eventually_build_improvement(const struct player *p,
-					     Impr_type_id id)
+bool can_player_build_improvement_later(const struct player *p,
+					struct impr_type *pimprove)
 {
-  struct impr_type *building;
-
-  if (!improvement_exists(id)) {
+  if (!valid_improvement(pimprove)) {
     return FALSE;
   }
-  if (improvement_obsolete(p, id)) {
+  if (improvement_obsolete(p, pimprove)) {
     return FALSE;
   }
 
   /* Check for requirements that aren't met and that are unchanging (so
    * they can never be met). */
-  building = improvement_by_number(id);
-  requirement_vector_iterate(&building->reqs, preq) {
+  requirement_vector_iterate(&pimprove->reqs, preq) {
     if (preq->range >= REQ_RANGE_PLAYER
 	&& is_req_unchanging(preq)
         && !is_req_active(p, NULL, NULL, NULL, NULL, NULL, NULL, preq,
@@ -419,68 +494,256 @@ bool can_player_eventually_build_improvement(const struct player *p,
 /**************************************************************************
   Is this building a great wonder?
 **************************************************************************/
-bool is_great_wonder(Impr_type_id id)
+bool is_great_wonder(const struct impr_type *pimprove)
 {
-  return (improvement_by_number(id)->genus == IG_GREAT_WONDER);
+  return (pimprove->genus == IG_GREAT_WONDER);
 }
 
 /**************************************************************************
   Is this building a small wonder?
 **************************************************************************/
-bool is_small_wonder(Impr_type_id id)
+bool is_small_wonder(const struct impr_type *pimprove)
 {
-  return (improvement_by_number(id)->genus == IG_SMALL_WONDER);
+  return (pimprove->genus == IG_SMALL_WONDER);
 }
 
 /**************************************************************************
   Is this building a regular improvement?
 **************************************************************************/
-bool is_improvement(Impr_type_id id)
+bool is_improvement(const struct impr_type *pimprove)
 {
-  return (improvement_by_number(id)->genus == IG_IMPROVEMENT);
+  return (pimprove->genus == IG_IMPROVEMENT);
 }
 
 /**************************************************************************
-  Get the world city with this great wonder.
+  Returns TRUE if this is a "special" improvement. For example, spaceship
+  parts and coinage in the default ruleset are considered special.
 **************************************************************************/
-struct city *find_city_from_great_wonder(Impr_type_id id)
+bool is_special_improvement(const struct impr_type *pimprove)
 {
-  return (game_find_city_by_number(game.info.great_wonders[id]));
+  return pimprove->genus == IG_SPECIAL;
+}
+
+/**************************************************************************
+  Build a wonder in the city.
+**************************************************************************/
+void wonder_built(const struct city *pcity, const struct impr_type *pimprove)
+{
+  struct player *pplayer;
+  int index = improvement_number(pimprove);
+
+  RETURN_IF_FAIL(NULL != pcity);
+  RETURN_IF_FAIL(is_wonder(pimprove));
+
+  pplayer = city_owner(pcity);
+  pplayer->wonders[index] = pcity->id;
+
+  if (is_great_wonder(pimprove)) {
+    game.info.great_wonder_owners[index] = player_number(pplayer);
+  }
+}
+
+/**************************************************************************
+  Remove a wonder from a city and destroy it if it's a great wonder.  To
+  transfer a great wonder, use great_wonder_transfer.
+**************************************************************************/
+void wonder_destroyed(const struct city *pcity,
+                      const struct impr_type *pimprove)
+{
+  struct player *pplayer;
+  int index = improvement_number(pimprove);
+
+  RETURN_IF_FAIL(NULL != pcity);
+  RETURN_IF_FAIL(is_wonder(pimprove));
+
+  pplayer = city_owner(pcity);
+  RETURN_IF_FAIL(pplayer->wonders[index] == pcity->id);
+  pplayer->wonders[index] = WONDER_NOT_BUILT;
+
+  if (is_great_wonder(pimprove)) {
+    RETURN_IF_FAIL(game.info.great_wonder_owners[index]
+                   == player_number(pplayer));
+    game.info.great_wonder_owners[index] = WONDER_DESTROYED;
+  }
+}
+
+/**************************************************************************
+  Returns whether the player has built this wonder (small or great).
+**************************************************************************/
+bool wonder_is_built(const struct player *pplayer,
+                     const struct impr_type *pimprove)
+{
+  RETURN_VAL_IF_FAIL(NULL != pplayer, NULL);
+  RETURN_VAL_IF_FAIL(is_wonder(pimprove), NULL);
+
+  return WONDER_BUILT(pplayer->wonders[improvement_index(pimprove)]);
+}
+
+/**************************************************************************
+  Get the world city with this wonder (small or great).  This doesn't
+  always success on the client side.
+**************************************************************************/
+struct city *find_city_from_wonder(const struct player *pplayer,
+                                   const struct impr_type *pimprove)
+{
+  int city_id = pplayer->wonders[improvement_index(pimprove)];
+
+  RETURN_VAL_IF_FAIL(NULL != pplayer, NULL);
+  RETURN_VAL_IF_FAIL(is_wonder(pimprove), NULL);
+
+  if (!WONDER_BUILT(city_id)) {
+    return NULL;
+  }
+
+#ifdef DEBUG
+  if (is_server()) {
+    /* On client side, this info is not always known. */
+    struct city *pcity = player_find_city_by_id(pplayer, city_id);
+
+    if (NULL == pcity) {
+      freelog(LOG_ERROR, "Player %s (nb %d) has outdated wonder info for "
+              "%s (nb %d), it points to city nb %d.",
+              player_name(pplayer), player_number(pplayer),
+              improvement_rule_name(pimprove), improvement_number(pimprove),
+              city_id);
+    } else if (!city_has_building(pcity, pimprove)) {
+      freelog(LOG_ERROR, "Player %s (nb %d) has outdated wonder info for "
+              "%s (nb %d), the city %s (nb %d) doesn't have this wonder.",
+              player_name(pplayer), player_number(pplayer),
+              improvement_rule_name(pimprove), improvement_number(pimprove),
+              city_name(pcity), pcity->id);
+      return NULL;
+    }
+
+    return pcity;
+  }
+#endif /* DEBUG */
+
+  return player_find_city_by_id(pplayer, city_id);
+}
+
+/**************************************************************************
+  Returns whether this wonder is currently built.
+**************************************************************************/
+bool great_wonder_is_built(const struct impr_type *pimprove)
+{
+  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), FALSE);
+
+  return WONDER_OWNED(game.info.great_wonder_owners
+                      [improvement_index(pimprove)]);
+}
+
+/**************************************************************************
+  Returns whether this wonder has been destroyed.
+**************************************************************************/
+bool great_wonder_is_destroyed(const struct impr_type *pimprove)
+{
+  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), FALSE);
+
+  return (WONDER_DESTROYED
+          == game.info.great_wonder_owners[improvement_index(pimprove)]);
+}
+
+/**************************************************************************
+  Returns whether this wonder can be currently built.
+**************************************************************************/
+bool great_wonder_is_available(const struct impr_type *pimprove)
+{
+  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), FALSE);
+
+  return (WONDER_NOT_OWNED
+          == game.info.great_wonder_owners[improvement_index(pimprove)]);
+}
+
+/**************************************************************************
+  Get the world city with this great wonder.  This doesn't always success
+  on the client side.
+**************************************************************************/
+struct city *find_city_from_great_wonder(const struct impr_type *pimprove)
+{
+  int player_id = game.info.great_wonder_owners[improvement_index(pimprove)];
+
+  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), NULL);
+
+  if (WONDER_OWNED(player_id)) {
+#ifdef DEBUG
+    const struct player *pplayer = player_by_number(player_id);
+    struct city *pcity = find_city_from_wonder(pplayer, pimprove);
+
+    if (is_server() && NULL == pcity) {
+      freelog(LOG_ERROR, "Game has outdated wonder info for %s (nb %d), "
+              "the player %s (nb %d) doesn't have this wonder.",
+              improvement_rule_name(pimprove), improvement_number(pimprove),
+              player_name(pplayer), player_number(pplayer));
+    }
+
+    return pcity;
+#else
+    return find_city_from_wonder(player_by_number(player_id), pimprove);
+#endif /* DEBUG */
+  } else {
+    return NULL;
+  }
+}
+
+/**************************************************************************
+  Get the player owning this small wonder.  This doesn't always success on
+  the client side.
+**************************************************************************/
+struct player *great_wonder_owner(const struct impr_type *pimprove)
+{
+  int player_id = game.info.great_wonder_owners[improvement_index(pimprove)];
+
+  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), NULL);
+
+  if (WONDER_OWNED(player_id)) {
+    return player_by_number(player_id);
+  } else {
+    return NULL;
+  }
+}
+
+/**************************************************************************
+  Returns whether the player has built this small wonder.
+**************************************************************************/
+bool small_wonder_is_built(const struct player *pplayer,
+                           const struct impr_type *pimprove)
+{
+  RETURN_VAL_IF_FAIL(is_small_wonder(pimprove), FALSE);
+
+  return (NULL != pplayer
+          && wonder_is_built(pplayer, pimprove));
 }
 
 /**************************************************************************
   Get the player city with this small wonder.
 **************************************************************************/
 struct city *find_city_from_small_wonder(const struct player *pplayer,
-					 Impr_type_id id)
+                                         const struct impr_type *pimprove)
 {
-  if (!pplayer) {
-    return NULL; /* Used in some places in the client. */
-  }
-  return (player_find_city_by_id(pplayer, pplayer->small_wonders[id]));
-}
+  RETURN_VAL_IF_FAIL(is_small_wonder(pimprove), NULL);
 
-/**************************************************************************
-  Was this great wonder built?
-**************************************************************************/
-bool great_wonder_was_built(Impr_type_id id)
-{
-  return (game.info.great_wonders[id] != 0);
+  if (NULL == pplayer) {
+    return NULL; /* Used in some places in the client. */
+  } else {
+    return find_city_from_wonder(pplayer, pimprove);
+  }
 }
 
 /**************************************************************************
   Return TRUE iff the improvement can be sold.
 **************************************************************************/
-bool can_sell_building(Impr_type_id id)
+bool can_sell_building(struct impr_type *pimprove)
 {
-  return is_improvement(id);
+  return (valid_improvement(pimprove) && is_improvement(pimprove));
 }
 
 /****************************************************************************
   Return TRUE iff the city can sell the given improvement.
 ****************************************************************************/
-bool can_city_sell_building(struct city *pcity, Impr_type_id id)
+bool can_city_sell_building(const struct city *pcity,
+			    struct impr_type *pimprove)
 {
-  return (city_got_building(pcity, id) ? can_sell_building(id) : FALSE);
+  return (city_has_building(pcity, pimprove) && is_improvement(pimprove));
 }
 
