@@ -14,7 +14,7 @@
 #define FC__MAP_H
 
 #include <assert.h>
-#include <math.h>
+#include <math.h> /* sqrt */
 
 #include "fc_types.h"
 
@@ -33,39 +33,51 @@ miscellaneous terrain information
 *****************************************************************/
 #define terrain_misc packet_ruleset_terrain_control
 
+struct hash_table;
+
 struct civ_map {
   int topology_id;
   enum direction8 valid_dirs[8], cardinal_dirs[8];
   int num_valid_dirs, num_cardinal_dirs;
   struct iter_index *iterate_outwards_indices;
   int num_iterate_outwards_indices;
-  int size; /* used to calculate [xy]size */
   int xsize, ysize; /* native dimensions */
-  int seed;
-  int riches;
-  int huts;
-  int landpercent;
-  int generator;
-  int startpos;
-  bool tinyisles;
-  bool separatepoles;
-  bool alltemperate;
-  int temperature;
-  int wetness;
-  int steepness;
-  int num_start_positions;
-  bool have_resources;
-  bool have_huts;
-  bool have_rivers_overlay;	/* only applies if !have_resources */
   int num_continents;
   int num_oceans;               /* not updated at the client */
   struct tile *tiles;
+  struct hash_table *startpos_table;
 
-  /* Only used by server. */
-  struct start_position {
-    struct tile *tile;
-    struct nation_type *nation; /* May be NO_NATION_SELECTED. */
-  } *start_positions;	/* allocated at runtime */
+  union {
+    struct {
+      /* Nothing yet. */
+    } client;
+
+    struct {
+      int size; /* used to calculate [xy]size */
+      int seed;
+      int riches;
+      int huts;
+      int landpercent;
+      int generator;
+      int startpos;
+      bool tinyisles;
+      bool separatepoles;
+      bool alltemperate;
+      int temperature;
+      int wetness;
+      int steepness;
+      int num_start_positions;
+      bool have_resources;
+      bool ocean_resources;         /* Resources in the middle of the ocean */
+      bool have_huts;
+      bool have_rivers_overlay;	/* only applies if !have_resources */
+
+      struct start_position {
+        struct tile *tile;
+        struct nation_type *nation; /* May be NO_NATION_SELECTED. */
+      } *start_positions;	/* allocated at runtime */
+    } server;
+  };
 };
 
 enum topo_flag {
@@ -108,6 +120,14 @@ bool base_get_direction_for_step(const struct tile *src_tile,
 int get_direction_for_step(const struct tile *src_tile,
 			   const struct tile *dst_tile);
 
+bool map_startpositions_set(void);
+void map_set_startpos(const struct tile *ptile,
+                      const struct nation_type *pnation);
+bool map_has_startpos(const struct tile *ptile);
+const struct nation_type *map_get_startpos(const struct tile *ptile);
+void map_clear_startpos(const struct tile *ptile);
+
+
 /* Number of index coordinates (for sanity checks and allocations) */
 #define MAP_INDEX_SIZE (map.xsize * map.ysize)
 
@@ -246,12 +266,14 @@ bool is_water_adjacent_to_tile(const struct tile *ptile);
 bool is_tiles_adjacent(const struct tile *ptile0, const struct tile *ptile1);
 bool is_move_cardinal(const struct tile *src_tile,
 		      const struct tile *dst_tile);
-int map_move_cost(struct unit *punit, const struct tile *ptile);
+int map_move_cost_unit(struct unit *punit, const struct tile *ptile);
 int map_move_cost_ai(const struct tile *tile0, const struct tile *tile1);
+int map_move_cost(const struct tile *src_tile, const struct tile *dst_tile);
 bool is_safe_ocean(const struct tile *ptile);
 bool is_cardinally_adj_to_ocean(const struct tile *ptile);
 bv_special get_tile_infrastructure_set(const struct tile *ptile,
 					  int *count);
+bv_bases get_tile_pillageable_base_set(const struct tile *ptile, int *pcount);
 
 bool can_channel_land(const struct tile *ptile);
 bool can_reclaim_ocean(const struct tile *ptile);
@@ -260,44 +282,37 @@ extern struct civ_map map;
 
 extern struct terrain_misc terrain_control;
 
-#define vision_layer_iterate(vision)					    \
-  {									    \
-    enum vision_layer vision;						    \
-									    \
-  for (vision = 0; vision < V_COUNT; vision++) {
-
-#define vision_layer_iterate_end					    \
-  }									    \
-}
-
 /* This iterates outwards from the starting point. Every tile within max_dist
  * will show up exactly once, in an outward (based on real map distance)
  * order.  The returned values are always real and are normalized.  The
  * starting position must be normal.
  *
  * See also iterate_outward() */
-#define iterate_outward_dxy(start_tile, max_dist, tile_itr, dx_itr, dy_itr) \
+#define iterate_outward_dxy(start_tile, max_dist, _tile, _x, _y)	    \
 {									    \
-  const struct tile *_start_tile = (start_tile);			    \
-  struct tile *tile_itr;						    \
-  int _max_dist = (max_dist), _x_itr, _y_itr, dx_itr, dy_itr, _index;	    \
-  bool _is_border = is_border_tile(_start_tile, _max_dist);		    \
-									    \
-  for (_index = 0; _index < map.num_iterate_outwards_indices; _index++) {   \
-    if (map.iterate_outwards_indices[_index].dist > _max_dist) {	    \
+  int _x, _y, _tile##_x, _tile##_y;					    \
+  struct tile *_tile;							    \
+  const struct tile *_tile##_start = (start_tile);			    \
+  int _tile##_max = (max_dist);						    \
+  bool _tile##_is_border = is_border_tile(_tile##_start, _tile##_max);	    \
+  int _tile##_index = 0;						    \
+  for (;								    \
+       _tile##_index < map.num_iterate_outwards_indices;		    \
+       _tile##_index++) { 						    \
+    if (map.iterate_outwards_indices[_tile##_index].dist > _tile##_max) {   \
       break;								    \
     }									    \
-    dx_itr = map.iterate_outwards_indices[_index].dx;			    \
-    dy_itr = map.iterate_outwards_indices[_index].dy;			    \
-    _x_itr = dx_itr + _start_tile->x;					    \
-    _y_itr = dy_itr + _start_tile->y;					    \
-    if (_is_border && !normalize_map_pos(&_x_itr, &_y_itr)) {		    \
+    _x = map.iterate_outwards_indices[_tile##_index].dx;		    \
+    _y = map.iterate_outwards_indices[_tile##_index].dy;		    \
+    _tile##_x = _x + _tile##_start->x;					    \
+    _tile##_y = _y + _tile##_start->y;					    \
+    if (_tile##_is_border && !normalize_map_pos(&_tile##_x, &_tile##_y)) {  \
       continue;								    \
     }									    \
-    tile_itr = map.tiles + map_pos_to_index(_x_itr, _y_itr);
+    _tile = map.tiles + map_pos_to_index(_tile##_x, _tile##_y);
 
 #define iterate_outward_dxy_end						    \
-  }                                                                         \
+  }									    \
 }
 
 /* See iterate_outward_dxy() */
@@ -343,15 +358,15 @@ extern struct terrain_misc terrain_control;
 /* dx, dy, dr are distance from center to tile in x, y and square distance;
  * do not rely on x, y distance, since they do not work for hex topologies */
 #define circle_dxyr_iterate(center_tile, sq_radius,			    \
-			    tile_itr, dx, dy, dr)			    \
+			    _tile, dx, dy, dr)				    \
 {									    \
-  const int _sq_radius = (sq_radius);					    \
-  const int _cr_radius = (int)sqrt((double)MAX(_sq_radius, 0));		    \
+  const int _tile##_sq_radius = (sq_radius);				    \
+  const int _tile##_cr_radius = (int)sqrt((double)MAX(_tile##_sq_radius, 0)); \
 									    \
-  square_dxy_iterate(center_tile, _cr_radius, tile_itr, dx, dy) {	    \
+  square_dxy_iterate(center_tile, _tile##_cr_radius, _tile, dx, dy) {	    \
     const int dr = map_vector_to_sq_distance(dx, dy);			    \
 									    \
-    if (dr <= _sq_radius) {
+    if (dr <= _tile##_sq_radius) {
 
 #define circle_dxyr_iterate_end						    \
     }									    \
@@ -389,44 +404,49 @@ extern struct terrain_misc terrain_control;
 #define cardinal_adjc_dir_iterate_end adjc_dirlist_iterate_end
 
 /* Iterate through all tiles adjacent to a tile using the given list of
- * directions.  dir_itr is the directional value, (center_x, center_y) is
- * the center tile (which must be normalized), and (x_itr, y_itr) is the
- * position corresponding to dir_itr.
+ * directions.  _dir is the directional value, (center_x, center_y) is
+ * the center tile (which must be normalized).
  *
  * This macro should not be used directly.  Instead, use adjc_dir_iterate
  * or cartesian_adjacent_iterate. */
-#define adjc_dirlist_iterate(center_tile, itr_tile, dir_itr,		    \
+#define adjc_dirlist_iterate(center_tile, _tile, _dir,			    \
 			     dirlist, dircount)				    \
 {									    \
-  const struct tile *_center_tile = (center_tile);			    \
-  struct tile *itr_tile;						    \
-  int _dir_index, _x_itr, _y_itr;					    \
-  enum direction8 dir_itr;						    \
-  bool _is_border = is_border_tile(_center_tile, 1);			    \
-									    \
-  for (_dir_index = 0; _dir_index < (dircount); _dir_index++) {		    \
-    dir_itr = dirlist[_dir_index];					    \
-    DIRSTEP(_x_itr, _y_itr, dir_itr);					    \
-    _x_itr += _center_tile->x;						    \
-    _y_itr += _center_tile->y;						    \
-    if (_is_border && !normalize_map_pos(&_x_itr, &_y_itr)) {		    \
+  enum direction8 _dir;							    \
+  int _tile##_x, _tile##_y;						    \
+  struct tile *_tile;							    \
+  const struct tile *_tile##_center = (center_tile);			    \
+  bool _tile##_is_border = is_border_tile(_tile##_center, 1);		    \
+  int _tile##_index = 0;						    \
+  for (;								    \
+       _tile##_index < (dircount);					    \
+       _tile##_index++) {						    \
+    _dir = dirlist[_tile##_index];					    \
+    DIRSTEP(_tile##_x, _tile##_y, _dir);				    \
+    _tile##_x += _tile##_center->x;					    \
+    _tile##_y += _tile##_center->y;					    \
+    if (_tile##_is_border && !normalize_map_pos(&_tile##_x, &_tile##_y)) {  \
       continue;								    \
     }									    \
-    itr_tile = map.tiles + map_pos_to_index(_x_itr, _y_itr);
+    _tile = map.tiles + map_pos_to_index(_tile##_x, _tile##_y);
 
 #define adjc_dirlist_iterate_end					    \
     }									    \
 }
 
-/* Iterate over all positions on the globe. */
-#define whole_map_iterate(ptile)					    \
-{                                                                           \
-  int _index; /* We use index positions for cache efficiency. */	    \
-  for (_index = 0; _index < MAP_INDEX_SIZE; _index++) {			    \
-    struct tile *ptile = map.tiles + _index;				    \
+/* Iterate over all positions on the globe.
+ * Use index positions for cache efficiency. */
+#define whole_map_iterate(_tile)					    \
+{									    \
+  struct tile *_tile;							    \
+  int _tile##_index = 0;						    \
+  for (;								    \
+       _tile##_index < MAP_INDEX_SIZE;					    \
+       _tile##_index++) {						    \
+    _tile = map.tiles + _tile##_index;
 
-#define whole_map_iterate_end                                               \
-  }                                                                         \
+#define whole_map_iterate_end						    \
+  }									    \
 }
 
 BV_DEFINE(dir_vector, 8);
@@ -450,10 +470,12 @@ extern const int DIR_DY[8];
 #define MAP_MIN_HUTS             0
 #define MAP_MAX_HUTS             500
 
-/* Size of the map in thousands of tiles */
+/* Size of the map in thousands of tiles.  Setting the maximal size over
+ * than 30 is dangerous, because some parts of the code (e.g. path finding)
+ * assume the tile index is of type (signed short int). */
 #define MAP_DEFAULT_SIZE         4
 #define MAP_MIN_SIZE             1
-#define MAP_MAX_SIZE             29
+#define MAP_MAX_SIZE             30
 
 /* This defines the maximum linear size in _map_ coordinates.
  * This must be smaller than 255 because of the way coordinates are sent
