@@ -21,29 +21,20 @@
 #include <assert.h>
 #include <limits.h>
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-#ifdef HAVE_WINSOCK
-#include <winsock.h>
-#endif
-
+/* utility */
 #include "capability.h"
-#include "dataio.h"
-#include "events.h"
 #include "fcintl.h"
 #include "hash.h"
 #include "log.h"
-#include "map.h"
 #include "mem.h"
 #include "support.h"
 
+/* commmon */
+#include "dataio.h"
+#include "game.h"
+#include "events.h"
+#include "map.h"
+
 #include "packets.h"
 
 #ifdef USE_COMPRESSION
@@ -64,7 +55,7 @@
 #define JUMBO_BORDER 		(64*1024-COMPRESSION_BORDER-1)
 #endif
 
-#define BASIC_PACKET_LOG_LEVEL	LOG_DEBUG
+#define BASIC_PACKET_LOG_LEVEL	LOG_VERBOSE
 #define COMPRESS_LOG_LEVEL	LOG_DEBUG
 #define COMPRESS2_LOG_LEVEL	LOG_DEBUG
 
@@ -93,22 +84,22 @@
 ***********************************************************************/
 
 /**************************************************************************
-It returns the request id of the outgoing packet or 0 if the packet
-was no request (i.e. server sends packet).
+  It returns the request id of the outgoing packet (or 0 if is_server()).
 **************************************************************************/
 int send_packet_data(struct connection *pc, unsigned char *data, int len)
 {
   /* default for the server */
   int result = 0;
 
-  freelog(BASIC_PACKET_LOG_LEVEL, "sending packet type=%s(%d) len=%d",
-	  get_packet_name(data[2]), data[2], len);
+  freelog(BASIC_PACKET_LOG_LEVEL, "sending packet type=%s(%d) len=%d to %s",
+          get_packet_name(data[2]), data[2], len,
+          is_server() ? pc->username : "server");
 
-  if (!is_server) {
+  if (!is_server()) {
     pc->client.last_request_id_used =
-	get_next_request_id(pc->client.last_request_id_used);
+        get_next_request_id(pc->client.last_request_id_used);
     result = pc->client.last_request_id_used;
-    freelog(LOG_DEBUG, "sending request %d", result);
+    freelog(BASIC_PACKET_LOG_LEVEL, "sending request %d", result);
   }
 
   if (pc->outgoing_packet_notify) {
@@ -133,6 +124,12 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
       compression_level_initialized = TRUE;
     }
 
+    /* TODO: PACKET_FREEZE_HINT and PACKET_THAW_HINT are meaningful
+     * only internally. They should not be sent to connection at all.
+     * Freezing could also be handled via separate functions, and
+     * not by special packets.
+     * Only problem is backward compatibility, so this cannot be
+     * changed in stable branch. */
     if (packet_type == PACKET_PROCESSING_STARTED
 	|| packet_type == PACKET_FREEZE_HINT) {
       if (pc->compression.frozen_level == 0) {
@@ -159,9 +156,9 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
 	PACKET_PROCESSING_FINISHED || packet_type == PACKET_THAW_HINT) {
       pc->compression.frozen_level--;
       if (pc->compression.frozen_level == 0) {
-	long int compressed_size = 12 + pc->compression.queue.size * 1.001;
+	uLongf compressed_size = 12 + pc->compression.queue.size * 1.001;
 	int error;
-	char compressed[compressed_size];
+	Bytef compressed[compressed_size];
 
 	error =
 	    compress2(compressed, &compressed_size,
@@ -281,11 +278,11 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
 		packets_stats[i].size / packets_stats[i].counter,
 		get_packet_name(i),i);
       }
-      freelog(LOG_NORMAL,
+      freelog(LOG_TEST,
 	      "turn=%d; transmitted %d bytes in %d packets;average size "
 	      "per packet %d bytes", game.turn, sum, packet_counter,
 	      sum / packet_counter);
-      freelog(LOG_NORMAL, "turn=%d; transmitted %d bytes", game.turn,
+      freelog(LOG_TEST, "turn=%d; transmitted %d bytes", game.turn,
 	      pc->statistics.bytes_send);
     }    
     if (clear) {
@@ -362,18 +359,18 @@ void *get_packet_from_connection(struct connection *pc,
   }
 #endif
 
-  if (whole_packet_len > pc->buffer->ndata) {
+  if ((unsigned)whole_packet_len > pc->buffer->ndata) {
     return NULL;		/* not all data has been read */
   }
 
 #ifdef USE_COMPRESSION
   if (compressed_packet) {
-    int compressed_size = whole_packet_len - header_size;
+    uLong compressed_size = whole_packet_len - header_size;
     /* 
      * We don't know the decompressed size. We assume a bad case
      * here: an expansion by an factor of 100. 
      */
-    long int decompressed_size = 100 * compressed_size;
+    unsigned long int decompressed_size = 100 * compressed_size;
     void *decompressed = fc_malloc(decompressed_size);
     int error;
     struct socket_packet_buffer *buffer = pc->buffer;
@@ -420,7 +417,7 @@ void *get_packet_from_connection(struct connection *pc,
 
     buffer->ndata += decompressed_size;
     
-    freelog(COMPRESS_LOG_LEVEL, "COMPRESS: decompressed %d into %ld",
+    freelog(COMPRESS_LOG_LEVEL, "COMPRESS: decompressed %ld into %ld",
 	    compressed_size, decompressed_size);
 
     return get_packet_from_connection(pc, ptype, presult);
@@ -444,8 +441,9 @@ void *get_packet_from_connection(struct connection *pc,
 
   dio_get_uint8(&din, &utype.itype);
 
-  freelog(BASIC_PACKET_LOG_LEVEL, "got packet type=(%s)%d len=%d",
-	  get_packet_name(utype.type), utype.itype, whole_packet_len);
+  freelog(BASIC_PACKET_LOG_LEVEL, "got packet type=(%s)%d len=%d from %s",
+          get_packet_name(utype.type), utype.itype, whole_packet_len,
+          is_server() ? pc->username : "server");
 
   *ptype = utype.type;
   *presult = TRUE;
@@ -478,23 +476,22 @@ void *get_packet_from_connection(struct connection *pc,
     packets_stats[packet_type].size += size;
 
     packet_counter++;
-    if ((is_server && (packet_counter % 10 == 0))
-	|| (!is_server && (packet_counter % 1000 == 0))) {
+    if (packet_counter % 100 == 0) {
       int i, sum = 0;
 
-      freelog(LOG_NORMAL, "Received packets:");
+      freelog(LOG_TEST, "Received packets:");
       for (i = 0; i < PACKET_LAST; i++) {
 	if (packets_stats[i].counter == 0)
 	  continue;
 	sum += packets_stats[i].size;
-	freelog(LOG_NORMAL,
+	freelog(LOG_TEST,
 		"  [%-25.25s %3d]: %6d packets; %8d bytes total; "
 		"%5d bytes/packet average",
 		get_packet_name(i), i, packets_stats[i].counter,
 		packets_stats[i].size,
 		packets_stats[i].size / packets_stats[i].counter);
       }
-      freelog(LOG_NORMAL,
+      freelog(LOG_TEST,
 	      "received %d bytes in %d packets;average size "
 	      "per packet %d bytes",
 	      sum, packet_counter, sum / packet_counter);
@@ -573,12 +570,17 @@ void generic_handle_player_attribute_chunk(struct player *pplayer,
 					   packet_player_attribute_chunk
 					   *chunk)
 {
-  freelog(LOG_DEBUG, "received attribute chunk %d/%d %d", chunk->offset,
-	  chunk->total_length, chunk->chunk_length);
+  freelog(BASIC_PACKET_LOG_LEVEL, "received attribute chunk %u/%u %u",
+	  (unsigned int) chunk->offset,
+	  (unsigned int) chunk->total_length,
+	  (unsigned int) chunk->chunk_length);
 
   if (chunk->total_length < 0
+      || chunk->chunk_length < 0
       || chunk->total_length >= MAX_ATTRIBUTE_BLOCK
       || chunk->offset < 0
+      || chunk->offset > chunk->total_length /* necessary check on 32 bit systems */
+      || chunk->chunk_length > chunk->total_length
       || chunk->offset + chunk->chunk_length > chunk->total_length
       || (chunk->offset != 0
           && chunk->total_length != pplayer->attribute_block_buffer.length)) {
@@ -650,6 +652,13 @@ void send_attribute_block(const struct player *pplayer,
 	   packet.chunk_length);
     bytes_left -= packet.chunk_length;
 
+    if (packet.chunk_length < ATTRIBUTE_CHUNK_SIZE) {
+      /* Last chunk is not full. Make sure that delta does
+       * not use random data. */
+      memset(packet.data + packet.chunk_length, 0,
+             ATTRIBUTE_CHUNK_SIZE - packet.chunk_length);
+    }
+
     send_packet_player_attribute_chunk(pconn, &packet);
   }
 
@@ -695,6 +704,9 @@ void post_receive_packet_chat_msg(struct connection *pc,
   }
 }
 
+/**************************************************************************
+  Test and log for sending player attribute_block
+**************************************************************************/
 void pre_send_packet_player_attribute_chunk(struct connection *pc,
 					    struct packet_player_attribute_chunk
 					    *packet)
@@ -707,18 +719,25 @@ void pre_send_packet_player_attribute_chunk(struct connection *pc,
   assert(packet->chunk_length <= packet->total_length);
   assert(packet->offset >= 0 && packet->offset < packet->total_length);
 
-  freelog(LOG_DEBUG, "sending attribute chunk %d/%d %d", packet->offset,
-	  packet->total_length, packet->chunk_length);
+  freelog(BASIC_PACKET_LOG_LEVEL, "sending attribute chunk %d/%d %d",
+	  packet->offset, packet->total_length, packet->chunk_length);
 
 }
 
-void post_receive_packet_game_state(struct connection *pc,
-				    struct packet_game_state *packet)
+/**************************************************************************
+  ...
+**************************************************************************/
+void post_receive_packet_ruleset_control(struct connection *pc,
+                                         struct packet_ruleset_control *packet)
 {
   conn_clear_packet_cache(pc);
 }
-void post_send_packet_game_state(struct connection *pc,
-				 const struct packet_game_state *packet)
+
+/**************************************************************************
+  ...
+**************************************************************************/
+void post_send_packet_ruleset_control(struct connection *pc,
+                                      const struct packet_ruleset_control *packet)
 {
   conn_clear_packet_cache(pc);
 }
