@@ -17,12 +17,12 @@
 
 #include "city.h"
 #include "connection.h"
-#include "improvement.h"	/* Impr_Status */
 #include "nation.h"
 #include "shared.h"
 #include "spaceship.h"
 #include "tech.h"
-#include "unit.h"
+#include "unitlist.h"
+#include "vision.h"
 
 #define PLAYER_DEFAULT_TAX_RATE 0
 #define PLAYER_DEFAULT_SCIENCE_RATE 100
@@ -31,37 +31,29 @@
 #define ANON_PLAYER_NAME "noname"
 #define ANON_USER_NAME "Unassigned"
 
-/*
- * pplayer->ai.barbarian_type uses this enum. Note that the values
- * have to stay since they are used in savegames.
- */
-enum barbarian_type {
-  NOT_A_BARBARIAN = 0,
-  LAND_BARBARIAN = 1,
-  SEA_BARBARIAN = 2
-};
-
 enum handicap_type {
-  H_NONE = 0,         /* No handicaps */
-  H_DIPLOMAT = 1,     /* Can't build offensive diplomats */
-  H_AWAY = 2,         /* Away mode */
-  H_LIMITEDHUTS = 4,  /* Can get only 25 gold and barbs from huts */
-  H_DEFENSIVE = 8,    /* Build defensive buildings without calculating need */
-  H_EXPERIMENTAL = 16,/* Enable experimental AI features (for testing) */
-  H_RATES = 32,       /* Can't set its rates beyond government limits */
-  H_TARGETS = 64,     /* Can't target anything it doesn't know exists */
-  H_HUTS = 128,       /* Doesn't know which unseen tiles have huts on them */
-  H_FOG = 256,        /* Can't see through fog of war */
-  H_NOPLANES = 512,   /* Doesn't build air units */
-  H_MAP = 1024,       /* Only knows map_is_known tiles */
-  H_DIPLOMACY = 2048, /* Not very good at diplomacy */
-  H_REVOLUTION = 4096, /* Cannot skip anarchy */
-  H_EXPANSION = 8192, /* Don't like being much larger than human */
-  H_DANGER = 16384    /* Always thinks its city is in danger */
+  H_DIPLOMAT = 0,     /* Can't build offensive diplomats */
+  H_AWAY,             /* Away mode */
+  H_LIMITEDHUTS,      /* Can get only 25 gold and barbs from huts */
+  H_DEFENSIVE,        /* Build defensive buildings without calculating need */
+  H_EXPERIMENTAL,     /* Enable experimental AI features (for testing) */
+  H_RATES,            /* Can't set its rates beyond government limits */
+  H_TARGETS,          /* Can't target anything it doesn't know exists */
+  H_HUTS,             /* Doesn't know which unseen tiles have huts on them */
+  H_FOG,              /* Can't see through fog of war */
+  H_NOPLANES,         /* Doesn't build air units */
+  H_MAP,              /* Only knows map_is_known tiles */
+  H_DIPLOMACY,        /* Not very good at diplomacy */
+  H_REVOLUTION,       /* Cannot skip anarchy */
+  H_EXPANSION,        /* Don't like being much larger than human */
+  H_DANGER,           /* Always thinks its city is in danger */
+  H_LAST
 };
 
 BV_DEFINE(bv_player, MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS);
 
+BV_DEFINE(bv_handicap, H_LAST);
+
 struct player_economic {
   int gold;
   int tax;
@@ -104,8 +96,8 @@ struct player_ai {
   int est_upkeep; /* estimated upkeep of buildings in cities */
   /* The units of tech_want seem to be shields */
   int tech_want[A_LAST+1];
-  int handicap;			/* sum of enum handicap_type */
-  int skill_level;		/* 0-10 value for save/load/display */
+  bv_handicap handicaps;        /* sum of enum handicap_type */
+  enum ai_level skill_level;   	/* 0-10 value for save/load/display */
   int fuzzy;			/* chance in 1000 to mis-decide */
   int expand;			/* percentage factor to value new cities */
   int science_cost;             /* Cost in bulbs to get new tech, relative
@@ -163,8 +155,10 @@ struct attribute_block_s {
 #define MAX_ATTRIBUTE_BLOCK     (256*1024)	/* largest attribute block */
 };
 
+struct ai_type;
+
 struct player {
-  int player_no;
+  bool used;
   char name[MAX_LEN_NAME];
   char username[MAX_LEN_NAME];
   char ranked_username[MAX_LEN_NAME]; /* the user who will be ranked */
@@ -179,42 +173,48 @@ struct player {
   int nturns_idle;
   bool is_alive;
   bool is_dying; /* set once the player is in the process of dying */
+  bool is_winner; /* This player is winner in scenario game */
   bool surrendered; /* has indicated willingness to surrender */
 
   /* Turn in which the player's revolution is over; see update_revolution. */
   int revolution_finishes;
 
   bool capital; /* used to give player init_buildings in first city. */
-  bv_player embassy;
+  bv_player real_embassy;
   struct player_diplstate diplstates[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
   int city_style;
-  struct unit_list *units;
   struct city_list *cities;
+  struct unit_list *units;
   struct player_score score;
   struct player_economic economic;
 
   int bulbs_last_turn;    /* # bulbs researched last turn only */
   struct player_spaceship spaceship;
-  struct player_ai ai;
+  struct player_ai ai_data;
+  struct ai_type *ai;
   bool was_created;                    /* if the player was /created */
   bool is_connected;
   struct connection *current_conn;     /* non-null while handling packet */
   struct conn_list *connections;       /* will replace conn */
-  struct worklist worklists[MAX_NUM_WORKLISTS];
   struct player_tile *private_map;
   unsigned int gives_shared_vision; /* bitvector those that give you shared vision */
   unsigned int really_gives_vision; /* takes into account that p3 may see what p1 has via p2 */
-  int small_wonders[B_LAST];              /* contains city id's */
-         /* wonders[] may also be (-1), or the id of a city
-	    which no longer exists, if the wonder has been destroyed */
-
+  int wonders[B_LAST];              /* contains city id's or WONDER_NOT_BUILT */
   struct attribute_block_s attribute_block;
   struct attribute_block_s attribute_block_buffer;
   bv_debug debug;
 };
 
+/* A slot is a possibly uninitialized player. */
+int player_slot_count(void);
+bool player_slot_is_used(const struct player *pslot);
+void player_slot_set_used(struct player *pslot, bool used);
+struct player *player_slot_by_number(int player_id);
+
 /* General player accessor functions. */
 int player_count(void);
+void set_player_count(int count);
+
 int player_index(const struct player *pplayer);
 int player_number(const struct player *pplayer);
 
@@ -229,8 +229,13 @@ struct player *find_player_by_user(const char *name);
 
 bool player_set_nation(struct player *pplayer, struct nation_type *pnation);
 void player_set_unit_focus_status(struct player *pplayer);
+
 bool player_has_embassy(const struct player *pplayer,
-			const struct player *pplayer2);
+                        const struct player *pplayer2);
+bool player_has_real_embassy(const struct player *pplayer,
+                             const struct player *pplayer2);
+bool player_has_embassy_from_effect(const struct player *pplayer,
+                                    const struct player *pplayer2);
 
 bool can_player_see_unit(const struct player *pplayer,
 			 const struct unit *punit);
@@ -303,19 +308,35 @@ bool gives_shared_vision(const struct player *me, const struct player *them);
 
 struct player_research *get_player_research(const struct player *p1);
 
+void player_set_winner(struct player *plr);
+
 /* Initialization and iteration */
 void player_init(struct player *plr);
 
-#define players_iterate(PI_player)                                            \
-{                                                                             \
-  struct player *PI_player;                                                   \
-  int PI_p_itr;                                                               \
-  for (PI_p_itr = 0; PI_p_itr < game.info.nplayers; PI_p_itr++) {            \
-    PI_player = get_player(PI_p_itr);
+#define player_slots_iterate(NAME_pslot)\
+do {\
+  struct player *NAME_pslot;\
+  int MY_i;\
+  for (MY_i = 0; MY_i < player_slot_count(); MY_i++) {\
+    NAME_pslot = player_slot_by_number(MY_i);\
+    if (NAME_pslot != NULL) {\
 
-#define players_iterate_end                                                   \
-  }                                                                           \
-}
+#define player_slots_iterate_end\
+    }\
+  }\
+} while (0)
+
+/* NB!!! This will only iterate over players for
+ * which player_slot_is_used() returns TRUE. */
+#define players_iterate(NAME_pplayer)\
+do {\
+  player_slots_iterate(NAME_pplayer) {\
+    if (player_slot_is_used(NAME_pplayer)) {
+
+#define players_iterate_end\
+    }\
+  } player_slots_iterate_end;\
+} while (0)
 
 /* ai love values should be in range [-MAX_AI_LOVE..MAX_AI_LOVE] */
 #define MAX_AI_LOVE 1000
@@ -324,5 +345,10 @@ void player_init(struct player *plr);
 /* User functions. */
 bool is_valid_username(const char *name);
 
+enum ai_level find_ai_level_by_name(const char *name);
+const char *ai_level_name(enum ai_level level);
+const char *ai_level_cmd(enum ai_level level);
+bool is_settable_ai_level(enum ai_level level);
+int number_of_ai_levels(void);
 
 #endif  /* FC__PLAYER_H */
