@@ -23,22 +23,26 @@
   #include <mysql/mysql.h>
 #endif
 
-#include "auth.h"
-#include "connection.h"
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "md5.h"
-#include "packets.h"
 #include "registry.h"
 #include "shared.h"
 #include "support.h"
 
-#include "auth.h"
+/* common */
+#include "connection.h"
+#include "packets.h"
+
+/* server */
 #include "connecthand.h"
-#include "plrhand.h"
+#include "notify.h"
 #include "sernet.h"
 #include "srv_main.h"
 
+#include "auth.h"
+
 /* where our mysql database is located and how to get to it */
 #define DEFAULT_AUTH_HOST     "localhost"
 #define DEFAULT_AUTH_PORT     "3306"
@@ -199,7 +203,7 @@ static void print_auth_option(int loglevel, enum show_source_type show_source,
 
   if (buffer[0] != '\0') {
     /* There is line to print */
-    freelog(loglevel, buffer);
+    freelog(loglevel, "%s", buffer);
   }
 }
 
@@ -258,11 +262,8 @@ static void load_auth_option(struct section_file *file,
                              struct auth_option *target)
 {
   const char *value;
-  char option[512];
 
-  my_snprintf(option, sizeof(option), "auth.%s", target->name);
-
-  value = secfile_lookup_str_default(file, "", option);
+  value = secfile_lookup_str_default(file, "", "auth.%s", target->name);
   if (value[0] != '\0') {
     /* We really loaded something from file */
     set_auth_option(target, value, AOS_FILE);
@@ -280,11 +281,6 @@ static bool load_auth_config(const char *filename)
 
   assert(filename != NULL);
 
-  if (!is_safe_filename(filename)) {
-    freelog(LOG_ERROR, _("Auth config filename \"%s\" not allowed!"), filename);
-    return FALSE;
-  }
-
   if (!section_file_load_nodup(&file, filename)) {
     freelog(LOG_ERROR, _("Cannot load auth config file \"%s\"!"), filename);
     return FALSE;
@@ -379,7 +375,7 @@ void auth_free(void)
 }
 
 /**************************************************************************
-  handle authentication of a user; called only by server_join_request()
+  handle authentication of a user; called by handle_login_request()
   if authentication is enabled.
 
   if the connection is rejected right away, return FALSE, otherwise return TRUE
@@ -396,16 +392,15 @@ bool authenticate_user(struct connection *pconn, char *username)
       get_unique_guest_name(username);
 
       if (strncmp(tmpname, username, MAX_LEN_NAME) != 0) {
-        notify_conn(pconn->self, NULL, E_CONNECTION,
+        notify_conn(pconn->self, NULL, E_CONNECTION, ftc_warning,
                     _("Warning: the guest name '%s' has been "
                       "taken, renaming to user '%s'."), tmpname, username);
       }
       sz_strlcpy(pconn->username, username);
       establish_new_connection(pconn);
     } else {
-      reject_new_connection(pconn,
-                            N_("Guests are not allowed on this server."
-                              " Sorry."));
+      reject_new_connection(_("Guests are not allowed on this server. "
+                              "Sorry."), pconn);
       freelog(LOG_NORMAL, _("%s was rejected: Guests not allowed."), username);
       return FALSE;
     }
@@ -424,15 +419,15 @@ bool authenticate_user(struct connection *pconn, char *username)
         sz_strlcpy(pconn->username, tmpname);
 
         freelog(LOG_ERROR, "Error reading database; connection -> guest");
-        notify_conn(pconn->self, NULL, E_CONNECTION,
+        notify_conn(pconn->self, NULL, E_CONNECTION, ftc_warning,
                     _("There was an error reading the user "
                       "database, logging in as guest connection '%s'."), 
                     pconn->username);
         establish_new_connection(pconn);
       } else {
-        reject_new_connection(pconn,
-                              N_("There was an error reading the user database"
-                                 " and guest logins are not allowed. Sorry"));
+        reject_new_connection(_("There was an error reading the user database "
+                                "and guest logins are not allowed. Sorry"), 
+                              pconn);
         freelog(LOG_NORMAL, 
                 _("%s was rejected: Database error and guests not allowed."),
                 pconn->username);
@@ -455,11 +450,10 @@ bool authenticate_user(struct connection *pconn, char *username)
         pconn->server.auth_settime = time(NULL);
         pconn->server.status = AS_REQUESTING_NEW_PASS;
       } else {
-        reject_new_connection(pconn,
-                              N_("This server allows only preregistered users."
-                                 " Sorry."));
+        reject_new_connection(_("This server allows only preregistered "
+                                "users. Sorry."), pconn);
         freelog(LOG_NORMAL,
-                _("%s was rejected: Only preregister users allowed."),
+                _("%s was rejected: Only preregistered users allowed."),
                 pconn->username);
 
         return FALSE;
@@ -487,8 +481,7 @@ bool handle_authentication_reply(struct connection *pconn, char *password)
     /* check if the new password is acceptable */
     if (!is_good_password(password, msg)) {
       if (pconn->server.auth_tries++ >= MAX_AUTH_TRIES) {
-        reject_new_connection(pconn,
-                              N_("Sorry, too many wrong tries..."));
+        reject_new_connection(_("Sorry, too many wrong tries..."), pconn);
         freelog(LOG_NORMAL, _("%s was rejected: Too many wrong password "
                 "verifies for new user."), pconn->username);
 
@@ -501,10 +494,11 @@ bool handle_authentication_reply(struct connection *pconn, char *password)
 
     /* the new password is good, create a database entry for
      * this user; we establish the connection in handle_db_lookup */
-    sz_strlcpy(pconn->server.password, password);
+    create_md5sum((unsigned char *)password, strlen(password),
+                  pconn->server.password);
 
     if (!auth_db_save(pconn)) {
-      notify_conn(pconn->self, NULL, E_CONNECTION,
+      notify_conn(pconn->self, NULL, E_CONNECTION, ftc_warning,
 		  _("Warning: There was an error in saving to the database. "
                     "Continuing, but your stats will not be saved."));
       freelog(LOG_ERROR, "Error writing to database for: %s", pconn->username);
@@ -546,8 +540,7 @@ void process_authentication_status(struct connection *pconn)
 
       if (pconn->server.auth_tries >= MAX_AUTH_TRIES) {
         pconn->server.status = AS_NOT_ESTABLISHED;
-        reject_new_connection(pconn,
-                              N_("Sorry, too many wrong tries..."));
+        reject_new_connection(_("Sorry, too many wrong tries..."), pconn);
         freelog(LOG_NORMAL,
                 _("%s was rejected: Too many wrong password tries."),
                 pconn->username);
@@ -568,8 +561,7 @@ void process_authentication_status(struct connection *pconn)
     /* waiting on the client to send us a password... don't wait too long */
     if (time(NULL) >= pconn->server.auth_settime + MAX_WAIT_TIME) {
       pconn->server.status = AS_NOT_ESTABLISHED;
-      reject_new_connection(pconn,
-                            N_("Sorry, your connection timed out..."));
+      reject_new_connection(_("Sorry, your connection timed out..."), pconn);
       freelog(LOG_NORMAL,
               _("%s was rejected: Connection timeout waiting for password."),
               pconn->username);
@@ -675,7 +667,7 @@ static bool is_good_password(const char *password, char *msg)
 
 
 /**************************************************************************
-  Check if the password with length len matches that given in 
+  Check if the password with length len matches the hashed one in
   pconn->server.password.
 ***************************************************************************/
 static bool authdb_check_password(struct connection *pconn, 
@@ -685,12 +677,12 @@ static bool authdb_check_password(struct connection *pconn,
   bool ok = FALSE;
   char buffer[512] = "";
   const int bufsize = sizeof(buffer);
-  char checksum[DIGEST_HEX_BYTES];
+  char checksum[MD5_HEX_BYTES + 1];
   MYSQL *sock, mysql;
 
   /* do the password checking right here */
-  create_md5sum(password, len, checksum);
-  ok = (strncmp(checksum, pconn->server.password, DIGEST_HEX_BYTES) == 0)
+  create_md5sum((const unsigned char *)password, len, checksum);
+  ok = (strncmp(checksum, pconn->server.password, MD5_HEX_BYTES) == 0)
                                                               ? TRUE : FALSE;
 
   /* we don't really need the stuff below here to 
@@ -890,7 +882,7 @@ static bool auth_db_save(struct connection *pconn)
    * also in seconds from 1970, the users address (twice) and the logincount */
   str_result = my_snprintf(buffer, bufsize,
                            "insert into %s values "
-                           "(NULL, '%s', md5('%s'), NULL, "
+                           "(NULL, '%s', '%s', NULL, "
                            "unix_timestamp(), unix_timestamp(),"
                            "'%s', '%s', 0)",
                            auth_config.table.value, name_buffer, pw_buffer,
