@@ -20,23 +20,30 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "city.h"
-#include "events.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "idex.h"
 #include "log.h"
-#include "map.h"
-#include "player.h"
 #include "rand.h"
 #include "support.h"
+
+/* common */
+#include "city.h"
+#include "events.h"
+#include "game.h"
+#include "idex.h"
+#include "map.h"
+#include "player.h"
+#include "specialist.h"
 #include "unit.h"
 #include "worklist.h"
 
+/* server */
 #include "citytools.h"
 #include "cityturn.h"
+#include "notify.h"
 #include "plrhand.h"
 #include "sanitycheck.h"
+#include "unithand.h"
 
 #include "cityhand.h"
 
@@ -45,19 +52,47 @@
   suggested name and with same id which was passed in (either unit id
   for city builder or existing city id for rename, we don't care here).
 **************************************************************************/
-void handle_city_name_suggestion_req(struct player *pplayer, int value)
+void handle_city_name_suggestion_req(struct player *pplayer, int unit_id)
 {
-  struct unit *punit = player_find_unit_by_id(pplayer, value);
-  
-  if (!punit) {
+  struct unit *punit = player_find_unit_by_id(pplayer, unit_id);
+  enum add_build_city_result res;
+
+  if (NULL == punit) {
+    /* Probably died or bribed. */
+    freelog(LOG_VERBOSE, "handle_city_name_suggestion_req() invalid unit %d",
+            unit_id);
     return;
   }
 
-  freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos=(%d,%d))",
-	  punit->tile->x, punit->tile->y);
+  res = test_unit_add_or_build_city(punit);
 
-  dlsend_packet_city_name_suggestion_info(pplayer->connections, value, 
-      city_name_suggestion(pplayer, punit->tile));
+  switch (res) {
+  case AB_BUILD_OK:
+    freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos (%d, %d))",
+            TILE_XY(unit_tile(punit)));
+    dlsend_packet_city_name_suggestion_info(pplayer->connections, unit_id,
+        city_name_suggestion(pplayer, unit_tile(punit)));
+    break;
+
+  case AB_NOT_BUILD_LOC:
+  case AB_NOT_BUILD_UNIT:
+  case AB_NO_MOVES_BUILD:
+    freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos (%d, %d)): "
+            "cannot build there.", TILE_XY(unit_tile(punit)));
+    city_add_or_build_error(pplayer, punit, res);       /* Message. */
+    break;
+
+  case AB_ADD_OK:
+  case AB_NOT_ADDABLE_UNIT:
+  case AB_NO_MOVES_ADD:
+  case AB_NOT_OWNER:
+  case AB_TOO_BIG:
+  case AB_NO_SPACE:
+    freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos (%d, %d)): "
+            "there is already a city there.", TILE_XY(unit_tile(punit)));
+    /* Ignoring. */
+    break;
+  }
 }
 
 /**************************************************************************
@@ -73,8 +108,8 @@ void handle_city_change_specialist(struct player *pplayer, int city_id,
     return;
   }
 
-  if (to < 0 || to >= SP_COUNT
-      || from < 0 || from >= SP_COUNT
+  if (to < 0 || to >= specialist_count()
+      || from < 0 || from >= specialist_count()
       || !city_can_use_specialist(pcity, to)
       || pcity->specialists[from] == 0) {
     /* This could easily just be due to clicking faster on the specialist
@@ -97,25 +132,52 @@ void handle_city_change_specialist(struct player *pplayer, int city_id,
 void handle_city_make_specialist(struct player *pplayer, int city_id,
 				 int worker_x, int worker_y)
 {
+  struct tile *ptile;
+  struct tile *pcenter;
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
 
-  if (!pcity) {
+  if (NULL == pcity) {
+    /* Probably lost. */
+    freelog(LOG_VERBOSE, "handle_city_make_specialist() bad city number %d.",
+            city_id);
     return;
   }
-  if (is_free_worked_tile(worker_x, worker_y)) {
+  
+  if (!is_valid_city_coords(worker_x, worker_y)) {
+    freelog(LOG_ERROR,
+            "handle_city_make_specialist() invalid city map {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
+    return;
+  }
+  pcenter = city_tile(pcity);
+
+  if (NULL == (ptile = city_map_to_tile(pcenter, worker_x, worker_y))) {
+    freelog(LOG_VERBOSE,
+            "handle_city_make_specialist() unavailable city map {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
+    return;
+  }
+
+  if (is_free_worked(pcity, ptile)) {
     auto_arrange_workers(pcity);
     sync_cities();
     return;
   }
-  if (is_worker_here(pcity, worker_x, worker_y)) {
-    server_remove_worker_city(pcity, worker_x, worker_y);
+
+  if (tile_worked(ptile) == pcity) {
+    city_map_update_empty(pcity, ptile, worker_x, worker_y);
     pcity->specialists[DEFAULT_SPECIALIST]++;
     city_refresh(pcity);
     sync_cities();
   } else {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
-		     _("You don't have a worker here.")); 
+    freelog(LOG_VERBOSE,
+            "handle_city_make_specialist() not working {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
   }
+
   sanity_check_city(pcity);
 }
 
@@ -125,38 +187,72 @@ void handle_city_make_specialist(struct player *pplayer, int city_id,
 void handle_city_make_worker(struct player *pplayer, int city_id,
 			     int worker_x, int worker_y)
 {
+  struct tile *ptile;
+  struct tile *pcenter;
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
-  int i;
+
+  if (NULL == pcity) {
+    /* Probably lost. */
+    freelog(LOG_VERBOSE, "handle_city_make_worker() bad city number %d.",
+            city_id);
+    return;
+  }
 
   if (!is_valid_city_coords(worker_x, worker_y)) {
-    freelog(LOG_ERROR, "invalid city coords %d,%d in package",
-	    worker_x, worker_y);
+    freelog(LOG_ERROR,
+            "handle_city_make_worker() invalid city map {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
     return;
   }
-  
-  if (!pcity) {
+  pcenter = city_tile(pcity);
+
+  if (NULL == (ptile = city_map_to_tile(pcenter, worker_x, worker_y))) {
+    freelog(LOG_VERBOSE,
+            "handle_city_make_worker() unavailable city map {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
     return;
   }
 
-  if (is_free_worked_tile(worker_x, worker_y)) {
+  if (is_free_worked(pcity, ptile)) {
     auto_arrange_workers(pcity);
     sync_cities();
     return;
   }
 
-  if (city_specialists(pcity) == 0
-      || get_worker_city(pcity, worker_x, worker_y) != C_TILE_EMPTY)
+  if (tile_worked(ptile) == pcity) {
+    freelog(LOG_VERBOSE,
+            "handle_city_make_worker() already working {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
     return;
+  }
 
-  server_set_worker_city(pcity, worker_x, worker_y);
+  if (0 == city_specialists(pcity)) {
+    freelog(LOG_VERBOSE,
+            "handle_city_make_worker() no specialists {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
+    return;
+  }
 
-  for (i = 0; i < SP_COUNT; i++) {
+  if (!city_can_work_tile(pcity, ptile)) {
+    freelog(LOG_VERBOSE,
+            "handle_city_make_worker() cannot work here {%d,%d} \"%s\".",
+            worker_x, worker_y,
+            city_name(pcity));
+    return;
+  }
+
+  city_map_update_worker(pcity, ptile, worker_x, worker_y);
+
+  specialist_type_iterate(i) {
     if (pcity->specialists[i] > 0) {
       pcity->specialists[i]--;
       break;
     }
-  }
-  assert(i < SP_COUNT);
+  } specialist_type_iterate_end;
 
   sanity_check_city(pcity);
   city_refresh(pcity);
@@ -167,24 +263,24 @@ void handle_city_make_worker(struct player *pplayer, int city_id,
 ...
 **************************************************************************/
 void really_handle_city_sell(struct player *pplayer, struct city *pcity,
-			     Impr_type_id id)
-{  
+			     struct impr_type *pimprove)
+{
   if (pcity->did_sell) {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, 
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
 		  _("You have already sold something here this turn."));
     return;
   }
 
-  if (!can_city_sell_building(pcity, id))
+  if (!can_city_sell_building(pcity, pimprove))
     return;
 
   pcity->did_sell=TRUE;
-  notify_player(pplayer, pcity->tile, E_IMP_SOLD,
-		   _("You sell %s in %s for %d gold."), 
-		   improvement_name_translation(id),
-		   city_name(pcity),
-		   impr_sell_gold(id));
-  do_sell_building(pplayer, pcity, id);
+  notify_player(pplayer, pcity->tile, E_IMP_SOLD, ftc_server,
+                _("You sell %s in %s for %d gold."), 
+                improvement_name_translation(pimprove),
+                city_link(pcity),
+                impr_sell_gold(pimprove));
+  do_sell_building(pplayer, pcity, pimprove);
 
   city_refresh(pcity);
 
@@ -199,11 +295,12 @@ void really_handle_city_sell(struct player *pplayer, struct city *pcity,
 void handle_city_sell(struct player *pplayer, int city_id, int build_id)
 {
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct impr_type *pimprove = improvement_by_number(build_id);
 
-  if (!pcity || build_id >= game.control.num_impr_types) {
+  if (!pcity || !pimprove) {
     return;
   }
-  really_handle_city_sell(pplayer, pcity, build_id);
+  really_handle_city_sell(pplayer, pcity, pimprove);
 }
 
 /**************************************************************************
@@ -211,7 +308,6 @@ void handle_city_sell(struct player *pplayer, int city_id, int build_id)
 **************************************************************************/
 void really_handle_city_buy(struct player *pplayer, struct city *pcity)
 {
-  /*const char *name;*/
   int cost, total;
 
   /* This function corresponds to city_can_buy() in the client. */
@@ -219,48 +315,41 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
   assert(pcity && player_owns_city(pplayer, pcity));
  
   if (pcity->turn_founded == game.info.turn) {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
-		  _("Cannot buy in city created this turn."));
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
+                  _("Cannot buy in city created this turn."));
     return;
   }
 
   if (pcity->did_buy) {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
 		  _("You have already bought this turn."));
     return;
   }
 
-  if (!pcity->production.is_unit
-      && improvement_has_flag(pcity->production.value, IF_GOLD)) {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
-                     _("You don't buy %s!"),
-		     improvement_name_translation(pcity->production.value));
+  if (city_production_has_flag(pcity, IF_GOLD)) {
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
+                  _("You don't buy %s!"),
+                  improvement_name_translation(pcity->production.value.building));
     return;
   }
 
-  if (pcity->production.is_unit && pcity->anarchy != 0) {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, 
-		     _("Can't buy units when city is in disorder."));
+  if (VUT_UTYPE == pcity->production.kind && pcity->anarchy != 0) {
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
+                  _("Can't buy units when city is in disorder."));
     return;
   }
 
-  if (pcity->production.is_unit) {
-    /*name = utype_name_translation(utype_by_number(pcity->production.value));*/
-    total = unit_build_shield_cost(utype_by_number(pcity->production.value));
-  } else {
-    /*name = improvement_name_translation(pcity->production.value);*/
-    total = impr_build_shield_cost(pcity->production.value);
-  }
-  cost = city_buy_cost(pcity);
+  total = city_production_build_shield_cost(pcity);
+  cost = city_production_buy_gold_cost(pcity);
   if (cost <= 0) {
     return; /* sanity */
   }
   if (cost > pplayer->economic.gold) {
     /* In case something changed while player tried to buy, or player 
      * tried to cheat! */
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
-		     _("%d gold required.  You only have %d gold."), cost,
-                     pplayer->economic.gold);
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
+                  _("%d gold required.  You only have %d gold."), cost,
+                  pplayer->economic.gold);
     return;
   }
 
@@ -284,7 +373,7 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
 ...
 **************************************************************************/
 void handle_city_worklist(struct player *pplayer, int city_id,
-			  struct worklist *worklist)
+                          const struct worklist *worklist)
 {
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
 
@@ -326,41 +415,52 @@ void handle_city_refresh(struct player *pplayer, int city_id)
     city_refresh(pcity);
     send_city_info(pplayer, pcity);
   } else {
-    global_city_refresh(pplayer);
+    city_refresh_for_player(pplayer);
   }
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-void handle_city_change(struct player *pplayer, int city_id, int build_id,
-			bool is_build_id_unit_id)
+void handle_city_change(struct player *pplayer, int city_id,
+			int production_kind, int production_value)
 {
+  struct universal prod;
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
-  struct city_production prod = {
-    .is_unit = is_build_id_unit_id,
-    .value = build_id
-  };
+
+  if (production_kind < VUT_NONE || production_kind >= VUT_LAST) {
+    freelog(LOG_ERROR, "handle_city_change()"
+            " bad production_kind %d.",
+            production_kind);
+    prod.kind = VUT_NONE;
+    return;
+  } else {
+    prod = universal_by_number(production_kind, production_value);
+    if (prod.kind < VUT_NONE || prod.kind >= VUT_LAST) {
+      freelog(LOG_ERROR, "handle_city_change()"
+              " production_kind %d with bad production_value %d.",
+              production_kind,
+              production_value);
+      prod.kind = VUT_NONE;
+      return;
+    }
+  }
 
   if (!pcity) {
     return;
   }
 
-  if (pcity->production.is_unit == is_build_id_unit_id
-      && pcity->production.value == build_id) {
+  if (are_universals_equal(&pcity->production, &prod)) {
     /* The client probably shouldn't send such a packet. */
     return;
   }
 
-  if (is_build_id_unit_id
-      && !can_build_unit(pcity, utype_by_number(build_id))) {
+  if (!can_city_build_now(pcity, prod)) {
     return;
   }
-   if (!is_build_id_unit_id && !can_build_improvement(pcity, build_id))
-     return;
   if (!city_can_change_build(pcity)) {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
-		     _("You have bought this turn, can't change."));
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
+                  _("You have bought this turn, can't change."));
     return;
   }
 
@@ -384,8 +484,8 @@ void handle_city_rename(struct player *pplayer, int city_id, char *name)
   }
 
   if (!is_allowed_city_name(pplayer, name, message, sizeof(message))) {
-    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, "%s",
-		  message);
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+                  ftc_server, "%s", message);
     return;
   }
 
