@@ -14,8 +14,11 @@
 #include <config.h>
 #endif
 
+#include <math.h> /* sqrt, HUGE_VAL */
+
 #include "log.h"
 #include "fcintl.h"
+#include "movement.h"
 
 #include "map.h"
 
@@ -41,31 +44,34 @@ static int *islands_index;
 ****************************************************************************/
 static int get_tile_value(struct tile *ptile)
 {
-  Terrain_type_id old_terrain;
-  enum tile_special_type old_special;
+  struct terrain *old_terrain;
+  bv_special old_special;
   int value, irrig_bonus, mine_bonus;
 
   /* Give one point for each food / shield / trade produced. */
-  value = (get_food_tile(ptile)
-	   + get_shields_tile(ptile)
-	   + get_trade_tile(ptile));
+  value = 0;
+  output_type_iterate(o) {
+    value += city_tile_output(NULL, ptile, FALSE, o);
+  } output_type_iterate_end;
 
   old_terrain = ptile->terrain;
   old_special = ptile->special;
 
-  map_set_special(ptile, S_ROAD);
-  map_irrigate_tile(ptile);
-  irrig_bonus = (get_food_tile(ptile)
-		 + get_shields_tile(ptile)
-		 + get_trade_tile(ptile)) - value;
+  tile_set_special(ptile, S_ROAD);
+  tile_apply_activity(ptile, ACTIVITY_IRRIGATE);
+  irrig_bonus = -value;
+  output_type_iterate(o) {
+    irrig_bonus += city_tile_output(NULL, ptile, FALSE, o);
+  } output_type_iterate_end;
 
   ptile->terrain = old_terrain;
   ptile->special = old_special;
-  map_set_special(ptile, S_ROAD);
-  map_mine_tile(ptile);
-  mine_bonus = (get_food_tile(ptile)
-		+ get_shields_tile(ptile)
-		+ get_trade_tile(ptile)) - value;
+  tile_set_special(ptile, S_ROAD);
+  tile_apply_activity(ptile, ACTIVITY_MINE);
+  mine_bonus = -value;
+  output_type_iterate(o) {
+    mine_bonus += city_tile_output(NULL, ptile, FALSE, o);
+  } output_type_iterate_end;
 
   ptile->terrain = old_terrain;
   ptile->special = old_special;
@@ -78,6 +84,7 @@ static int get_tile_value(struct tile *ptile)
 struct start_filter_data {
   int count;			/* Number of existing start positions. */
   int min_value;
+  struct unit_type *initial_unit;
   int *value;
 };
 
@@ -91,17 +98,17 @@ struct start_filter_data {
   - Too close to another starter on the same continent:
     'dist' is too close (real_map_distance)
     'nr' is the number of other start positions in
-    map.start_positions to check for too closeness.
+    map.server.start_positions to check for too closeness.
 **************************************************************************/
 static bool is_valid_start_pos(const struct tile *ptile, const void *dataptr)
 {
   const struct start_filter_data *pdata = dataptr;
   int i;
   struct islands_data_type *island;
-  int cont_size, cont = map_get_continent(ptile);
+  int cont_size, cont = tile_continent(ptile);
 
   /* Only start on certain terrain types. */  
-  if (pdata->value[ptile->index] < pdata->min_value) {
+  if (pdata->value[tile_index(ptile)] < pdata->min_value) {
       return FALSE;
   } 
 
@@ -111,7 +118,12 @@ static bool is_valid_start_pos(const struct tile *ptile, const void *dataptr)
   }
 
   /* Don't start on a hut. */
-  if (map_has_special(ptile, S_HUT)) {
+  if (tile_has_special(ptile, S_HUT)) {
+    return FALSE;
+  }
+
+  /* Has to be native tile for initial unit */
+  if (!is_native_tile(pdata->initial_unit, ptile)) {
     return FALSE;
   }
 
@@ -127,9 +139,9 @@ static bool is_valid_start_pos(const struct tile *ptile, const void *dataptr)
   cont_size = get_continent_size(cont);
   island = islands + islands_index[cont];
   for (i = 0; i < pdata->count; i++) {
-    struct tile *tile1 = map.start_positions[i].tile;
+    struct tile *tile1 = map.server.start_positions[i].tile;
 
-    if ((map_get_continent(ptile) == map_get_continent(tile1)
+    if ((tile_continent(ptile) == tile_continent(tile1)
 	 && (real_map_distance(ptile, tile1) * 1000 / pdata->min_value
 	     <= (sqrt(cont_size / island->total))))
 	|| (real_map_distance(ptile, tile1) * 1000 / pdata->min_value < 5)) {
@@ -175,7 +187,7 @@ static void initialize_isle_data(void)
 ****************************************************************************/
 static bool filter_starters(const struct tile *ptile, const void *data)
 {
-  return terrain_has_flag(map_get_terrain(ptile), TER_STARTER);
+  return terrain_has_flag(tile_terrain(ptile), TER_STARTER);
 }
 
 /**************************************************************************
@@ -187,18 +199,20 @@ static bool filter_starters(const struct tile *ptile, const void *data)
   MT_ALL: all players in asingle isle
   MT_VARIABLE: at least 2 player per isle
   
+  Assumes assign_continent_numbers() has already been done!
   Returns true on success
 **************************************************************************/
-bool create_start_positions(enum start_mode mode)
+bool create_start_positions(enum start_mode mode,
+			    struct unit_type *initial_unit)
 {
   struct tile *ptile;
   int k, sum;
   struct start_filter_data data;
-  int tile_value_aux[MAX_MAP_INDEX], tile_value[MAX_MAP_INDEX];
-  int min_goodies_per_player = 2000;
+  int tile_value_aux[MAP_INDEX_SIZE], tile_value[MAP_INDEX_SIZE];
+  int min_goodies_per_player = 1500;
   int total_goodies = 0;
   /* this is factor is used to maximize land used in extreme little maps */
-  float efactor =  game.nplayers / map.size / 4; 
+  float efactor =  player_count() / map.server.size / 4; 
   bool failure = FALSE;
   bool is_tmap = temperature_is_initialized();
 
@@ -211,11 +225,6 @@ bool create_start_positions(enum start_mode mode)
     create_tmap(FALSE);
   }
 
-  /* Unsafe terrains separate continents, otherwise small areas of green
-   * near the poles could be populated by a civilization if that pole
-   * was part of a larger continent. */
-  assign_continent_numbers(TRUE);
-
   /* If the default is given, just use MT_VARIABLE. */
   if (mode == MT_DEFAULT) {
     mode = MT_VARIABLE;
@@ -223,25 +232,26 @@ bool create_start_positions(enum start_mode mode)
 
   /* get the tile value */
   whole_map_iterate(ptile) {
-    tile_value_aux[ptile->index] = get_tile_value(ptile);
+    tile_value_aux[tile_index(ptile)] = get_tile_value(ptile);
   } whole_map_iterate_end;
 
   /* select the best tiles */
   whole_map_iterate(ptile) {
-    int this_tile_value = tile_value_aux[ptile->index];
+    int this_tile_value = tile_value_aux[tile_index(ptile)];
     int lcount = 0, bcount = 0;
 
-    map_city_radius_iterate(ptile, ptile1) {
-      if (this_tile_value > tile_value_aux[ptile1->index]) {
+    city_tile_iterate(ptile, ptile1) {
+      if (this_tile_value > tile_value_aux[tile_index(ptile1)]) {
 	lcount++;
-      } else if (this_tile_value < tile_value_aux[ptile1->index]) {
+      } else if (this_tile_value < tile_value_aux[tile_index(ptile1)]) {
 	bcount++;
       }
-    } map_city_radius_iterate_end;
+    } city_tile_iterate_end;
+
     if (lcount <= bcount) {
       this_tile_value = 0;
     }
-    tile_value[ptile->index] = 100 * this_tile_value;
+    tile_value[tile_index(ptile)] = 100 * this_tile_value;
   } whole_map_iterate_end;
   /* get an average value */
   smooth_int_map(tile_value, TRUE);
@@ -251,10 +261,10 @@ bool create_start_positions(enum start_mode mode)
   /* oceans are not good for starters; discard them */
   whole_map_iterate(ptile) {
     if (!filter_starters(ptile, NULL)) {
-      tile_value[ptile->index] = 0;
+      tile_value[tile_index(ptile)] = 0;
     } else {
-      islands[map_get_continent(ptile)].goodies += tile_value[ptile->index];
-      total_goodies += tile_value[ptile->index];
+      islands[tile_continent(ptile)].goodies += tile_value[tile_index(ptile)];
+      total_goodies += tile_value[tile_index(ptile)];
     }
   } whole_map_iterate_end;
 
@@ -268,16 +278,16 @@ bool create_start_positions(enum start_mode mode)
 
   /* If we can't place starters according to the first choice, change the
    * choice. */
-  if (mode == MT_SINGLE && map.num_continents < game.nplayers + 3) {
+  if (mode == MT_SINGLE && map.num_continents < player_count() + 3) {
     mode = MT_2or3;
   }
 
-  if (mode == MT_2or3 && map.num_continents < game.nplayers / 2 + 4) {
+  if (mode == MT_2or3 && map.num_continents < player_count() / 2 + 4) {
     mode = MT_VARIABLE;
   }
 
   if (mode == MT_ALL 
-      && (islands[1].goodies < game.nplayers * min_goodies_per_player
+      && (islands[1].goodies < player_count() * min_goodies_per_player
 	  || islands[1].goodies < total_goodies * (0.5 + 0.8 * efactor)
 	  / (1 + efactor))) {
     mode = MT_VARIABLE;
@@ -286,11 +296,11 @@ bool create_start_positions(enum start_mode mode)
   /* the variable way is the last posibility */
   if (mode == MT_VARIABLE) {
     min_goodies_per_player = total_goodies * (0.65 + 0.8 * efactor) 
-      / (1 + efactor)  / game.nplayers;
+      / (1 + efactor)  / player_count();
   }
 
   { 
-    int nr, to_place = game.nplayers, first = 1;
+    int nr, to_place = player_count(), first = 1;
 
     /* inizialize islands_index */
     for (nr = 1; nr <= map.num_continents; nr++) {
@@ -301,7 +311,7 @@ bool create_start_positions(enum start_mode mode)
     if ((mode == MT_SINGLE) || (mode == MT_2or3)) {
       float var_goodies, best = HUGE_VAL;
       int num_islands
-	= (mode == MT_SINGLE) ? game.nplayers : (game.nplayers / 2);
+	= (mode == MT_SINGLE) ? player_count() : (player_count() / 2);
 
       for (nr = 1; nr <= 1 + map.num_continents - num_islands; nr++) {
 	if (islands[nr + num_islands - 1].goodies < min_goodies_per_player) {
@@ -331,7 +341,7 @@ bool create_start_positions(enum start_mode mode)
 	to_place--;
       }
       if (mode == MT_2or3 && to_place > 0 && nr >= first) {
-	islands[nr].starters = 2 + (nr == 1 ? (game.nplayers % 2) : 0);
+	islands[nr].starters = 2 + (nr == 1 ? (player_count() % 2) : 0);
 	to_place -= islands[nr].total = islands[nr].starters;
       }
 
@@ -346,6 +356,7 @@ bool create_start_positions(enum start_mode mode)
   data.count = 0;
   data.value = tile_value;
   data.min_value = 900;
+  data.initial_unit = initial_unit;
   sum = 0;
   for (k = 1; k <= map.num_continents; k++) {
     sum += islands[islands_index[k]].starters;
@@ -353,37 +364,40 @@ bool create_start_positions(enum start_mode mode)
       freelog(LOG_VERBOSE, "starters on isle %i", k);
     }
   }
-  assert(game.nplayers <= data.count + sum);
+  assert(player_count() <= data.count + sum);
 
   /* now search for the best place and set start_positions */
-  map.start_positions = fc_realloc(map.start_positions,
-				   game.nplayers
-				   * sizeof(*map.start_positions));
-  while (data.count < game.nplayers) {
+  map.server.start_positions =
+    fc_realloc(map.server.start_positions, player_count()
+               * sizeof(*map.server.start_positions));
+  while (data.count < player_count()) {
     if ((ptile = rand_map_pos_filtered(&data, is_valid_start_pos))) {
-      islands[islands_index[(int) map_get_continent(ptile)]].starters--;
-      map.start_positions[data.count].tile = ptile;
-      map.start_positions[data.count].nation = NO_NATION_SELECTED;
+      islands[islands_index[(int) tile_continent(ptile)]].starters--;
+      map.server.start_positions[data.count].tile = ptile;
+      map.server.start_positions[data.count].nation = NO_NATION_SELECTED;
       freelog(LOG_DEBUG,
 	      "Adding %d,%d as starting position %d, %d goodies on islands.",
 	      TILE_XY(ptile), data.count,
-	      islands[islands_index[(int) map_get_continent(ptile)]].goodies);
+	      islands[islands_index[(int) tile_continent(ptile)]].goodies);
       data.count++;
 
     } else {
-      data.min_value *= 0.9;
+      data.min_value *= 0.95;
       if (data.min_value <= 10) {
 	freelog(LOG_ERROR,
 	        _("The server appears to have gotten into an infinite loop "
 	          "in the allocation of starting positions.\n"
-	          "Maybe the numbers of players/ia is too much for this map.\n"
-	          "Please report this bug at %s."), WEBSITE_URL);
+	          "Maybe the number of players is too high for this map."));
+	freelog(LOG_ERROR,
+		/* TRANS: No full stop after the URL, could cause confusion. */
+		_("Please report this message at %s"),
+		BUG_URL);
 	failure = TRUE;
 	break;
       }
     }
   }
-  map.num_start_positions = game.nplayers;
+  map.server.num_start_positions = player_count();
 
   free(islands);
   free(islands_index);
