@@ -21,7 +21,7 @@ bool server_handle_packet(enum packet_type type, void *packet,
 {
   switch(type) {
   case PACKET_NATION_SELECT_REQ:
-    handle_nation_select_req(pplayer,
+    handle_nation_select_req(pconn,
       ((struct packet_nation_select_req *)packet)->player_no,
       ((struct packet_nation_select_req *)packet)->nation_no,
       ((struct packet_nation_select_req *)packet)->is_male,
@@ -54,8 +54,8 @@ bool server_handle_packet(enum packet_type type, void *packet,
   case PACKET_CITY_CHANGE:
     handle_city_change(pplayer,
       ((struct packet_city_change *)packet)->city_id,
-      ((struct packet_city_change *)packet)->build_id,
-      ((struct packet_city_change *)packet)->is_build_id_unit_id);
+      ((struct packet_city_change *)packet)->production_kind,
+      ((struct packet_city_change *)packet)->production_value);
     return TRUE;
 
   case PACKET_CITY_WORKLIST:
@@ -251,7 +251,8 @@ bool server_handle_packet(enum packet_type type, void *packet,
     handle_unit_change_activity(pplayer,
       ((struct packet_unit_change_activity *)packet)->unit_id,
       ((struct packet_unit_change_activity *)packet)->activity,
-      ((struct packet_unit_change_activity *)packet)->activity_target);
+      ((struct packet_unit_change_activity *)packet)->activity_target,
+      ((struct packet_unit_change_activity *)packet)->activity_base);
     return TRUE;
 
   case PACKET_DIPLOMACY_INIT_MEETING_REQ:
@@ -300,6 +301,11 @@ bool server_handle_packet(enum packet_type type, void *packet,
     handle_conn_pong(pconn);
     return TRUE;
 
+  case PACKET_CLIENT_INFO:
+    handle_client_info(pconn,
+      ((struct packet_client_info *)packet)->gui);
+    return TRUE;
+
   case PACKET_SPACESHIP_LAUNCH:
     handle_spaceship_launch(pplayer);
     return TRUE;
@@ -314,12 +320,151 @@ bool server_handle_packet(enum packet_type type, void *packet,
     handle_single_want_hack_req(pconn, packet);
     return TRUE;
 
+  case PACKET_SCENARIO_INFO:
+    handle_scenario_info(pconn, packet);
+    return TRUE;
+
+  case PACKET_SAVE_SCENARIO:
+    handle_save_scenario(pconn,
+      ((struct packet_save_scenario *)packet)->name);
+    return TRUE;
+
   case PACKET_VOTE_SUBMIT:
     handle_vote_submit(pconn,
       ((struct packet_vote_submit *)packet)->vote_no,
       ((struct packet_vote_submit *)packet)->value);
     return TRUE;
 
+  case PACKET_EDIT_MODE:
+    handle_edit_mode(pconn,
+      ((struct packet_edit_mode *)packet)->state);
+    return TRUE;
+
+  case PACKET_EDIT_RECALCULATE_BORDERS:
+    handle_edit_recalculate_borders(pconn);
+    return TRUE;
+
+  case PACKET_EDIT_CHECK_TILES:
+    handle_edit_check_tiles(pconn);
+    return TRUE;
+
+  case PACKET_EDIT_TOGGLE_FOGOFWAR:
+    handle_edit_toggle_fogofwar(pconn,
+      ((struct packet_edit_toggle_fogofwar *)packet)->player);
+    return TRUE;
+
+  case PACKET_EDIT_TILE_TERRAIN:
+    handle_edit_tile_terrain(pconn,
+      ((struct packet_edit_tile_terrain *)packet)->x,
+      ((struct packet_edit_tile_terrain *)packet)->y,
+      ((struct packet_edit_tile_terrain *)packet)->terrain,
+      ((struct packet_edit_tile_terrain *)packet)->size);
+    return TRUE;
+
+  case PACKET_EDIT_TILE_RESOURCE:
+    handle_edit_tile_resource(pconn,
+      ((struct packet_edit_tile_resource *)packet)->x,
+      ((struct packet_edit_tile_resource *)packet)->y,
+      ((struct packet_edit_tile_resource *)packet)->resource,
+      ((struct packet_edit_tile_resource *)packet)->size);
+    return TRUE;
+
+  case PACKET_EDIT_TILE_SPECIAL:
+    handle_edit_tile_special(pconn,
+      ((struct packet_edit_tile_special *)packet)->x,
+      ((struct packet_edit_tile_special *)packet)->y,
+      ((struct packet_edit_tile_special *)packet)->special,
+      ((struct packet_edit_tile_special *)packet)->remove,
+      ((struct packet_edit_tile_special *)packet)->size);
+    return TRUE;
+
+  case PACKET_EDIT_TILE_BASE:
+    handle_edit_tile_base(pconn,
+      ((struct packet_edit_tile_base *)packet)->x,
+      ((struct packet_edit_tile_base *)packet)->y,
+      ((struct packet_edit_tile_base *)packet)->base_type_id,
+      ((struct packet_edit_tile_base *)packet)->remove,
+      ((struct packet_edit_tile_base *)packet)->size);
+    return TRUE;
+
+  case PACKET_EDIT_STARTPOS:
+    handle_edit_startpos(pconn,
+      ((struct packet_edit_startpos *)packet)->x,
+      ((struct packet_edit_startpos *)packet)->y,
+      ((struct packet_edit_startpos *)packet)->nation);
+    return TRUE;
+
+  case PACKET_EDIT_TILE:
+    handle_edit_tile(pconn, packet);
+    return TRUE;
+
+  case PACKET_EDIT_UNIT_CREATE:
+    handle_edit_unit_create(pconn, packet);
+    return TRUE;
+
+  case PACKET_EDIT_UNIT_REMOVE:
+    handle_edit_unit_remove(pconn,
+      ((struct packet_edit_unit_remove *)packet)->owner,
+      ((struct packet_edit_unit_remove *)packet)->x,
+      ((struct packet_edit_unit_remove *)packet)->y,
+      ((struct packet_edit_unit_remove *)packet)->type,
+      ((struct packet_edit_unit_remove *)packet)->count);
+    return TRUE;
+
+  case PACKET_EDIT_UNIT_REMOVE_BY_ID:
+    handle_edit_unit_remove_by_id(pconn,
+      ((struct packet_edit_unit_remove_by_id *)packet)->id);
+    return TRUE;
+
+  case PACKET_EDIT_UNIT:
+    handle_edit_unit(pconn, packet);
+    return TRUE;
+
+  case PACKET_EDIT_CITY_CREATE:
+    handle_edit_city_create(pconn,
+      ((struct packet_edit_city_create *)packet)->owner,
+      ((struct packet_edit_city_create *)packet)->x,
+      ((struct packet_edit_city_create *)packet)->y,
+      ((struct packet_edit_city_create *)packet)->size,
+      ((struct packet_edit_city_create *)packet)->tag);
+    return TRUE;
+
+  case PACKET_EDIT_CITY_REMOVE:
+    handle_edit_city_remove(pconn,
+      ((struct packet_edit_city_remove *)packet)->id);
+    return TRUE;
+
+  case PACKET_EDIT_CITY:
+    handle_edit_city(pconn, packet);
+    return TRUE;
+
+  case PACKET_EDIT_PLAYER_CREATE:
+    handle_edit_player_create(pconn,
+      ((struct packet_edit_player_create *)packet)->tag);
+    return TRUE;
+
+  case PACKET_EDIT_PLAYER_REMOVE:
+    handle_edit_player_remove(pconn,
+      ((struct packet_edit_player_remove *)packet)->id);
+    return TRUE;
+
+  case PACKET_EDIT_PLAYER:
+    handle_edit_player(pconn, packet);
+    return TRUE;
+
+  case PACKET_EDIT_PLAYER_VISION:
+    handle_edit_player_vision(pconn,
+      ((struct packet_edit_player_vision *)packet)->player,
+      ((struct packet_edit_player_vision *)packet)->x,
+      ((struct packet_edit_player_vision *)packet)->y,
+      ((struct packet_edit_player_vision *)packet)->known,
+      ((struct packet_edit_player_vision *)packet)->size);
+    return TRUE;
+
+  case PACKET_EDIT_GAME:
+    handle_edit_game(pconn, packet);
+    return TRUE;
+
   default:
     return FALSE;
   }
