@@ -13,16 +13,6 @@
 #ifndef FC__PLRHAND_H
 #define FC__PLRHAND_H
 
-#include <stdarg.h>
-
-#include "shared.h"		/* fc__attribute */
-
-#include "events.h"
-#include "fc_types.h"
-#include "packets.h"
-
-#include "hand_gen.h"
-
 struct section_file;
 struct connection;
 struct conn_list;
@@ -31,13 +21,15 @@ enum plr_info_level { INFO_MINIMUM, INFO_MEETING, INFO_EMBASSY, INFO_FULL };
 
 void server_player_init(struct player *pplayer,
 			bool initmap, bool needs_team);
+struct player *server_create_player(void);
 void server_remove_player(struct player *pplayer);
 void kill_player(struct player *pplayer);
 void update_revolution(struct player *pplayer);
 
 struct nation_type *pick_a_nation(struct nation_type **choices,
                                   bool ignore_conflicts,
-				  bool only_available);
+                                  bool only_available,
+                                  enum barbarian_type barb_type);
 
 void check_player_max_rates(struct player *pplayer);
 void make_contact(struct player *pplayer1, struct player *pplayer2,
@@ -46,56 +38,39 @@ void maybe_make_contact(struct tile *ptile, struct player *pplayer);
 
 void send_player_info(struct player *src, struct player *dest);
 void send_player_info_c(struct player *src, struct conn_list *dest);
-
-void notify_conn(struct conn_list *dest, struct tile *ptile,
-		 enum event_type event, const char *format, ...)
-                 fc__attribute((__format__ (__printf__, 4, 5)));
-void vnotify_conn(struct conn_list *dest, struct tile *ptile,
-		  enum event_type event, const char *format,
-		  va_list vargs);
-void notify_player(const struct player *pplayer, struct tile *ptile,
-		   enum event_type event, const char *format, ...)
-                   fc__attribute((__format__ (__printf__, 4, 5)));
-void notify_embassies(struct player *pplayer, struct player *exclude,
-		      struct tile *ptile, enum event_type event,
-		      const char *format, ...)
-		      fc__attribute((__format__ (__printf__, 5, 6)));
-void notify_team(const struct player* pplayer, struct tile *ptile,
-                 enum event_type event, const char *format, ...)
-                 fc__attribute((__format__ (__printf__, 4, 5)));
-void notify_research(struct player *pplayer,
-		     enum event_type event, const char *format, ...)
-                     fc__attribute((__format__ (__printf__, 3, 4)));
+void send_player_slot_info_c(struct player *src, struct conn_list *dest);
 
 struct conn_list *player_reply_dest(struct player *pplayer);
 
-void send_player_turn_notifications(struct conn_list *dest);
-
 void shuffle_players(void);
 void set_shuffled_players(int *shuffled_players);
 struct player *shuffled_player(int i);
 void reset_all_start_commands(void);
 
-#define shuffled_players_iterate(pplayer)                                   \
-{                                                                           \
-  struct player *pplayer;                                                   \
-  int i;                                                                    \
-  for (i = 0; i < game.info.nplayers; i++) {                               \
-    pplayer = shuffled_player(i);                                           \
-    {
+#define shuffled_players_iterate(NAME_pplayer)\
+do {\
+  int MY_i;\
+  struct player *NAME_pplayer;\
+  freelog(LOG_DEBUG, "shuffled_players_iterate @ %s line %d",\
+          __FILE__, __LINE__);\
+  for (MY_i = 0; MY_i < player_slot_count(); MY_i++) {\
+    NAME_pplayer = shuffled_player(MY_i);\
+    if (NAME_pplayer != NULL) {\
 
-#define shuffled_players_iterate_end                                        \
-    }                                                                       \
-  }                                                                         \
-}
+#define shuffled_players_iterate_end\
+    }\
+  }\
+} while (0)
 
-#define phase_players_iterate(pplayer) \
-  shuffled_players_iterate(pplayer) { \
+#define phase_players_iterate(pplayer)\
+do {\
+  shuffled_players_iterate(pplayer) {\
     if (is_player_phase(pplayer, game.info.phase)) {
 
-#define phase_players_iterate_end		\
-    }						\
-  } shuffled_players_iterate_end
+#define phase_players_iterate_end\
+    }\
+  } shuffled_players_iterate_end;\
+} while (0)
 
 bool civil_war_triggered(struct player *pplayer);
 void civil_war(struct player *pplayer);
