@@ -17,22 +17,26 @@
 
 #include <math.h>
 
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "support.h"
 
+/* common */
 #include "capability.h"
 #include "connection.h"
 #include "packets.h"
 #include "player.h"
 
+/* server */
 #include "commands.h"
 #include "console.h"
-#include "plrhand.h"
+#include "hand_gen.h"
+#include "notify.h"
 #include "settings.h"
 #include "stdinhand.h"
-#include "voting.h"
 
+#include "voting.h"
 
 struct vote_list *vote_list = NULL;
 int vote_number_sequence = 0;
@@ -86,8 +90,7 @@ static void lsend_vote_new(struct conn_list *dest, struct vote *pvote)
   }
 
   conn_list_iterate(dest, conn) {
-    if (!has_capability("voteinfo", conn->capability)
-        || !conn_can_vote(conn, pvote)) {
+    if (!conn_can_see_vote(conn, pvote)) {
       continue;
     }
     send_packet_vote_new(conn, &packet);
@@ -126,8 +129,7 @@ static void lsend_vote_update(struct conn_list *dest, struct vote *pvote,
   }
 
   conn_list_iterate(dest, aconn) {
-    if (!has_capability("voteinfo", aconn->capability)
-        || !conn_can_vote(aconn, pvote)) {
+    if (!conn_can_see_vote(aconn, pvote)) {
       continue;
     }
     send_packet_vote_update(aconn, &packet);
@@ -152,9 +154,6 @@ static void lsend_vote_remove(struct conn_list *dest, struct vote *pvote)
   }
 
   conn_list_iterate(dest, pconn) {
-    if (!has_capability("voteinfo", pconn->capability)) {
-      continue;
-    }
     send_packet_vote_remove(pconn, &packet);
   } conn_list_iterate_end;
 }
@@ -179,8 +178,7 @@ static void lsend_vote_resolve(struct conn_list *dest,
   }
 
   conn_list_iterate(dest, pconn) {
-    if (!has_capability("voteinfo", pconn->capability)
-        || !conn_can_vote(pconn, pvote)) {
+    if (!conn_can_see_vote(pconn, pvote)) {
       continue;
     }
     send_packet_vote_resolve(pconn, &packet);
@@ -230,7 +228,7 @@ void clear_all_votes(void)
     lsend_vote_remove(NULL, pvote);
     free_vote(pvote);
   } vote_list_iterate_end;
-  vote_list_unlink_all(vote_list);
+  vote_list_clear(vote_list);
 }
 
 /***************************************************************************
@@ -357,9 +355,9 @@ struct vote *vote_new(struct connection *caller,
   pvote = fc_malloc(sizeof(struct vote));
   pvote->caller_id = caller->id;
   pvote->command_id = command_id;
-  pcmd = &commands[command_id];
+  pcmd = command_by_number(command_id);
 
-  sz_strlcpy(pvote->cmdline, pcmd->name);
+  sz_strlcpy(pvote->cmdline, command_name(pcmd));
   if (allargs != NULL && allargs[0] != '\0') {
     sz_strlcat(pvote->cmdline, " ");
     sz_strlcat(pvote->cmdline, allargs);
@@ -371,8 +369,8 @@ struct vote *vote_new(struct connection *caller,
 
   vote_list_append(vote_list, pvote);
 
-  pvote->flags = pcmd->vote_flags;
-  pvote->need_pc = (double) pcmd->vote_percent / 100.0;
+  pvote->flags = command_vote_flags(pcmd);
+  pvote->need_pc = (double) command_vote_percent(pcmd) / 100.0;
 
   if (pvote->flags & VCF_NOPASSALONE) {
     int num_voters = count_voters(pvote);
@@ -509,22 +507,26 @@ static void check_vote(struct vote *pvote)
   if (vote_is_team_only(pvote)) {
     const struct connection *caller;
 
+    /* TRANS: "Vote" as a process. Used as part of a sentence. */
     title = _("Teamvote");
     caller = vote_get_caller(pvote);
     callplr = conn_get_player(caller);
   } else {
+    /* TRANS: "Vote" as a process. Used as part of a sentence. */
     title = _("Vote");
     callplr = NULL;
   }
 
   if (passed) {
-    notify_team(callplr, NULL, E_CHAT_MSG,
+    notify_team(callplr, NULL, E_VOTE_RESOLVED, ftc_vote_passed,
+                /* TRANS: "[Vote|Teamvote] 3 \"proposed change\" is ..." */
                 _("%s %d \"%s\" is passed %d to %d with "
                   "%d abstentions and %d who did not vote."),
                 title, pvote->vote_no, pvote->cmdline, pvote->yes,
                 pvote->no, pvote->abstain, num_voters - num_cast);
   } else {
-    notify_team(callplr, NULL, E_CHAT_MSG,
+    notify_team(callplr, NULL, E_VOTE_RESOLVED, ftc_vote_failed,
+                /* TRANS: "[Vote|Teamvote] 3 \"proposed change\" failed ..." */
                 _("%s %d \"%s\" failed with %d against, %d for, "
                   "%d abstentions and %d who did not vote."),
                 title, pvote->vote_no, pvote->cmdline, pvote->no,
@@ -544,17 +546,17 @@ static void check_vote(struct vote *pvote)
 
     switch (pvc->vote_cast) {
     case VOTE_YES:
-      notify_team(callplr, NULL, E_CHAT_MSG,
+      notify_team(callplr, NULL, E_VOTE_RESOLVED, ftc_vote_yes,
                   _("%s %d: %s voted yes."),
                   title, pvote->vote_no, pconn->username);
       break;
     case VOTE_NO:
-      notify_team(callplr, NULL, E_CHAT_MSG,
+      notify_team(callplr, NULL, E_VOTE_RESOLVED, ftc_vote_no,
                   _("%s %d: %s voted no."),
                   title, pvote->vote_no, pconn->username);
       break;
     case VOTE_ABSTAIN:
-      notify_team(callplr, NULL, E_CHAT_MSG,
+      notify_team(callplr, NULL, E_VOTE_RESOLVED, ftc_vote_abstain,
                   _("%s %d: %s chose to abstain."),
                   title, pvote->vote_no, pconn->username);
       break;
@@ -750,7 +752,9 @@ void handle_vote_submit(struct connection *pconn, int vote_no, int value)
 
   pvote = get_vote_by_no(vote_no);
   if (pvote == NULL) {
-    freelog(LOG_ERROR, "Submit request for unknown vote_no %d "
+    /* The client is out of synchronization: this vote is probably just
+     * resolved or cancelled. Not an error, let's just ignore the packet. */
+    freelog(LOG_VERBOSE, "Submit request for unknown vote_no %d "
             "from %s ignored.",
             vote_no, conn_description(pconn));
     return;
@@ -771,35 +775,64 @@ void handle_vote_submit(struct connection *pconn, int vote_no, int value)
   connection_vote(pconn, pvote, type);
 }
 
-/**************************************************************************
-  Sends a packet_vote_new to pconn (if voteinfo capable) for every
-  currently running vote.
-**************************************************************************/
-void send_running_votes(struct connection *pconn)
+/****************************************************************************
+  Sends a packet_vote_new to pconn for every currently running votes.
+****************************************************************************/
+void send_running_votes(struct connection *pconn, bool only_team_votes)
 {
-  if (!pconn || !has_capability("voteinfo", pconn->capability)) {
+  if (NULL == vote_list
+      || vote_list_size(vote_list) < 1
+      || NULL == pconn
+      || (only_team_votes && NULL == conn_get_player(pconn))) {
     return;
   }
 
-  if (!vote_list || vote_list_size(vote_list) < 1) {
+  freelog(LOG_DEBUG, "Sending %s running votes to %s.",
+          only_team_votes ? "team" : "all", conn_description(pconn));
+
+  connection_do_buffer(pconn);
+  vote_list_iterate(vote_list, pvote) {
+    if (vote_is_team_only(pvote)) {
+      if (conn_can_see_vote(pconn, pvote)) {
+        lsend_vote_new(pconn->self, pvote);
+        lsend_vote_update(pconn->self, pvote, count_voters(pvote));
+      }
+    } else if (!only_team_votes) {
+      lsend_vote_new(pconn->self, pvote);
+      lsend_vote_update(pconn->self, pvote, count_voters(pvote));
+    }
+  } vote_list_iterate_end;
+  connection_do_unbuffer(pconn);
+}
+
+/****************************************************************************
+  Sends a packet_vote_remove to pconn for every currently running team vote
+  'pconn' can see.
+****************************************************************************/
+void send_remove_team_votes(struct connection *pconn)
+{
+  if (NULL == vote_list
+      || vote_list_size(vote_list) < 1
+      || NULL == pconn
+      || NULL == conn_get_player(pconn)) {
     return;
   }
 
-  freelog(LOG_DEBUG, "Sending running votes to %s.",
+  freelog(LOG_DEBUG, "Sending remove info of the team votes to %s.",
           conn_description(pconn));
 
   connection_do_buffer(pconn);
   vote_list_iterate(vote_list, pvote) {
-    if (conn_can_vote(pconn, pvote)) {
-      lsend_vote_new(pconn->self, pvote);
+    if (vote_is_team_only(pvote) && conn_can_see_vote(pconn, pvote)) {
+      lsend_vote_remove(pconn->self, pvote);
     }
   } vote_list_iterate_end;
   connection_do_unbuffer(pconn);
 }
 
 /**************************************************************************
-  Sends a packet_vote_update to every voteinfo-capabale conn in dest. If
-  dest is NULL, then sends to all established connections.
+  Sends a packet_vote_update to every conn in dest. If dest is NULL, then
+  sends to all established connections.
 **************************************************************************/
 void send_updated_vote_totals(struct conn_list *dest)
 {
