@@ -51,13 +51,15 @@
 #undef ECONNRESET
 #undef ECONNREFUSED
 #undef EADDRNOTAVAIL
+#undef ETIMEDOUT
 #define EINTR         WSAEINTR
 #define EINPROGRESS   WSAEWOULDBLOCK
 #define EWOULDBLOCK   WSAEWOULDBLOCK
 #define ECONNRESET    WSAECONNRESET
 #define ECONNREFUSED  WSAECONNREFUSED
 #define EADDRNOTAVAIL WSAEADDRNOTAVAIL
-#endif   
+#define ETIMEDOUT     WSAETIMEDOUT
+#endif /* HAVE_WINSOCK */
 
 #ifdef FD_ZERO
 #define MY_FD_ZERO FD_ZERO
