@@ -44,6 +44,7 @@
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
+#include <math.h> /* ceil() */
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -71,6 +72,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>		/* usleep, fcntl, gethostname */
 #endif
+#ifdef HAVE_LIBZ
+#include <zlib.h>
+#endif
 #ifdef WIN32_NATIVE
 #include <process.h>
 #include <windows.h>
@@ -82,7 +86,10 @@
 #  include <strings.h>
 #endif
 
+/* utility */
+#include "fciconv.h"
 #include "fcintl.h"
+#include "log.h"
 #include "mem.h"
 #include "netintf.h"
 
@@ -93,6 +100,12 @@
 ***************************************************************/
 int mystrcasecmp(const char *str0, const char *str1)
 {
+  if (str0 == NULL) {
+    return -1;
+  }
+  if (str1 == NULL) {
+    return 1;
+  }
 #ifdef HAVE_STRCASECMP
   return strcasecmp (str0, str1);
 #else
@@ -113,6 +126,12 @@ int mystrcasecmp(const char *str0, const char *str1)
 ***************************************************************/
 int mystrncasecmp(const char *str0, const char *str1, size_t n)
 {
+  if (str0 == NULL) {
+    return -1;
+  }
+  if (str1 == NULL) {
+    return 1;
+  }
 #ifdef HAVE_STRNCASECMP
   return strncasecmp (str0, str1, n);
 #else
@@ -134,51 +153,252 @@ int mystrncasecmp(const char *str0, const char *str1, size_t n)
 }
 
 /***************************************************************
-  Compare strings like strncasecmp() but ignoring quotes in
-  either string. Quotes still count towards n.
+  Count length of string without possible surrounding quotes.
+***************************************************************/
+size_t effectivestrlenquote(const char *str)
+{
+  int len;
+  if (!str) {
+    return 0;
+  }
+
+  len = strlen(str);
+
+  if (str[0] == '"' && str[len-1] == '"') {
+    return len - 2;
+  }
+
+  return len;
+}
+
+/***************************************************************
+  Compare strings like strncasecmp() but ignoring surrounding
+  quotes in either string.
 ***************************************************************/
 int mystrncasequotecmp(const char *str0, const char *str1, size_t n)
 {
   size_t i;
-  char c;
+  size_t len0;
+  size_t len1;
+  size_t cmplen;
 
-  for (i = 0; i < n && *str0 != '\0'; i++) {
-    if (my_tolower(*str0) != my_tolower(*str1)
-        && *str0 != '"' && *str1 != '"') {
+  if (str0 == NULL) {
+    return -1;
+  }
+  if (str1 == NULL) {
+    return 1;
+  }
+
+  len0 = strlen(str0); /* TODO: We iterate string once already here, */
+  len1 = strlen(str1); /*       could iterate only once */
+
+  if (str0[0] == '"') {
+    if (str0[len0 - 1] == '"') {
+      /* Surrounded with quotes */
+      str0++;
+      len0 -= 2;
+    }
+  }
+
+  if (str1[0] == '"') {
+    if (str1[len1 - 1] == '"') {
+      /* Surrounded with quotes */
+      str1++;
+      len1 -= 2;
+    }
+  }
+
+  if (len0 < n || len1 < n) {
+    /* One of the strings is shorter than what should be compared... */
+    if (len0 != len1) {
+      /* ...and another is longer than it. */
+      return len0 - len1;
+    }
+
+    cmplen = len0; /* This avoids comparing ending quote */
+  } else {
+    cmplen = n;
+  }
+
+  for (i = 0; i < cmplen ; i++, str0++, str1++) {
+    if (my_tolower(*str0) != my_tolower(*str1)) {
       return ((int) (unsigned char) my_tolower(*str0))
              - ((int) (unsigned char) my_tolower(*str1));
     }
-    c = *str0;
-    str0 += (*str1 != '"');
-    str1 += (c != '"');
   }
+
+  /* All characters compared and all matched */
   return 0;
 }
 
 /***************************************************************
-  Return a string which describes a given error (errno-style.)
+  Return the needle in the haystack (or NULL).
+  Naive implementation.
 ***************************************************************/
-const char *mystrerror(void)
+char *mystrcasestr(const char *haystack, const char *needle)
+{
+#ifdef HAVE_STRCASESTR
+  return strcasestr(haystack, needle);
+#else
+  size_t haystacks;
+  size_t needles;
+  const char *p;
+
+  if (NULL == needle || '\0' == *needle) {
+    return (char *)haystack;
+  }
+  if (NULL == haystack || '\0' == *haystack) {
+    return NULL;
+  }
+  haystacks = strlen(haystack);
+  needles = strlen(needle);
+  if (haystacks < needles) {
+    return NULL;
+  }
+
+  for (p = haystack; p <= &haystack[haystacks - needles]; p++) {
+    if (0 == mystrncasecmp(p, needle, needles)) {
+      return (char *)p;
+    }
+  }
+  return NULL;
+#endif
+}
+
+/****************************************************************
+  Wrapper function for fopen() with filename conversion to local
+  encoding on Windows.
+****************************************************************/
+FILE *fc_fopen(const char *filename, const char *opentype)
+{
+#ifdef WIN32_NATIVE
+	FILE *result;
+	char *filename_in_local_encoding =
+	     internal_to_local_string_malloc(filename);
+	result = fopen(filename_in_local_encoding, opentype);
+	free(filename_in_local_encoding);
+	return result;
+#else
+	return fopen(filename, opentype);
+#endif
+}
+
+/*****************************************************************
+  Wrapper function for gzopen() with filename conversion to local
+  encoding on Windows.
+*****************************************************************/
+#ifdef HAVE_LIBZ
+FILE *fc_gzopen(const char *filename, const char *opentype)
+{
+#ifdef WIN32_NATIVE
+	FILE *result;
+	char *filename_in_local_encoding =
+	     internal_to_local_string_malloc(filename);
+	result = gzopen(filename_in_local_encoding, opentype);
+	free(filename_in_local_encoding);
+	return result;
+#else
+	return gzopen(filename, opentype);
+#endif
+}
+#endif
+
+/******************************************************************
+  Wrapper function for opendir() with filename conversion to local
+  encoding on Windows.
+******************************************************************/
+DIR *fc_opendir(const char *dirname)
+{
+#ifdef WIN32_NATIVE
+	DIR *result;
+	char *dirname_in_local_encoding =
+	     internal_to_local_string_malloc(dirname);
+	result = opendir(dirname_in_local_encoding);
+	free(dirname_in_local_encoding);
+	return result;
+#else
+	return opendir(dirname);
+#endif
+}
+
+/*****************************************************************
+  Wrapper function for remove() with filename conversion to local
+  encoding on Windows.
+*****************************************************************/
+int fc_remove(const char *filename)
+{
+#ifdef WIN32_NATIVE
+	int result;
+	char *filename_in_local_encoding =
+	     internal_to_local_string_malloc(filename);
+	result = remove(filename_in_local_encoding);
+	free(filename_in_local_encoding);
+	return result;
+#else
+	return remove(filename);
+#endif
+}
+
+/*****************************************************************
+  Wrapper function for stat() with filename conversion to local
+  encoding on Windows.
+*****************************************************************/
+int fc_stat(const char *filename, struct stat *buf)
+{
+#ifdef WIN32_NATIVE
+	int result;
+	char *filename_in_local_encoding =
+	     internal_to_local_string_malloc(filename);
+	result = stat(filename_in_local_encoding, buf);
+	free(filename_in_local_encoding);
+	return result;
+#else
+	return stat(filename, buf);
+#endif
+}
+
+/***************************************************************
+  Returns last error code.
+***************************************************************/
+fc_errno fc_get_errno(void)
+{
+#ifdef WIN32_NATIVE
+  return GetLastError();
+#else
+  return errno;
+#endif
+}
+
+/***************************************************************
+  Return a string which describes a given error (errno-style.)
+  The string is converted as necessary from the local_encoding
+  to internal_encoding, for inclusion in translations.  May be
+  subsequently converted back to local_encoding for display.
+
+  Note that this is not the reentrant form.
+***************************************************************/
+const char *fc_strerror(fc_errno err)
 {
 #ifdef WIN32_NATIVE
   static char buf[256];
-  long int error;
 
-  error = GetLastError();
   if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
-		     NULL, error, 0, buf, sizeof(buf), NULL)) {
+		     NULL, err, 0, buf, sizeof(buf), NULL)) {
     my_snprintf(buf, sizeof(buf),
-		_("error %ld (failed FormatMessage)"), error);
+		_("error %ld (failed FormatMessage)"), err);
   }
   return buf;
 #else
 #ifdef HAVE_STRERROR
-  return strerror(errno);
+  static char buf[256];
+
+  return local_to_internal_string_buffer(strerror(err),
+                                         buf, sizeof(buf));
 #else
   static char buf[64];
 
   my_snprintf(buf, sizeof(buf),
-	      _("error %d (compiled without strerror)"), errno);
+	      _("error %d (compiled without strerror)"), err);
   return buf;
 #endif
 #endif
@@ -211,13 +431,85 @@ void myusleep(unsigned long usec)
   tv.tv_usec=usec;
   /* FIXME: an interrupt can cause an EINTR return here.  In that case we
    * need to have another select call. */
-  my_select(0, NULL, NULL, NULL, &tv);
+  fc_select(0, NULL, NULL, NULL, &tv);
 #endif
 #endif
 #endif
 #endif
 }
 
+/**************************************************************************
+  Replace 'search' by 'replace' within 'str'. If needed 'str' is resized
+  using realloc() to fit the modified string. The new pointer to the string
+  is returned.
+**************************************************************************/
+char *fc_strrep_resize(char *str, size_t *len, const char *search,
+                       const char *replace)
+{
+  size_t len_max;
+
+  RETURN_VAL_IF_FAIL(str != NULL, NULL);
+  RETURN_VAL_IF_FAIL(len != NULL, NULL);
+  if (search == NULL || replace == NULL) {
+    return str;
+  }
+
+  len_max = ceil((double)strlen(str) * strlen(replace) / strlen(search)) + 1;
+  if ((*len) < len_max) {
+    /* replace string is longer than search string; allocated enough memory
+     * for the worst case */
+    (*len) = len_max;
+    str = fc_realloc(str, len_max);
+  }
+
+  /* should never happen */
+  RETURN_VAL_IF_FAIL_MSG(fc_strrep(str, (*len), search, replace), NULL,
+                         "Can't replace '%s' by '%s' in '%s'. To small "
+                         "size after reallocation: %lu.", search, replace,
+                         str, (long unsigned int)*len);
+
+  return str;
+}
+
+/**************************************************************************
+  Replace 'search' by 'replace' within 'str'. sizeof(str) should be large
+  enough for the modified value of 'str'. Returns TRUE if the replacement
+  was successful.
+**************************************************************************/
+bool fc_strrep(char *str, size_t len, const char *search,
+               const char *replace)
+{
+  size_t len_search, len_replace;
+  char *s, *p;
+
+  RETURN_VAL_IF_FAIL(str != NULL, FALSE);
+  if (search == NULL || replace == NULL) {
+    return TRUE;
+  }
+
+  len_search = strlen(search);
+  len_replace = strlen(replace);
+
+  s = str;
+  while (s != NULL) {
+    p = strstr(s, search);
+    if (p == NULL) {
+      /* nothing found */
+      break;
+    }
+
+    if (len < (strlen(str) + len_replace - len_search + 1)) {
+      /* sizeof(str) not large enough to do the replacement */
+      return FALSE;
+    }
+
+    memmove(p + len_replace, p + len_search, strlen(p + len_search) + 1);
+    memcpy(p, replace, len_replace);
+    s = p + len_replace;
+  }
+
+  return TRUE;
+}
 
 /**********************************************************************
  mystrlcpy() and mystrlcat() provide (non-standard) functions
@@ -481,7 +773,7 @@ void my_init_console(void)
   if (!initialized) {
     initialized = 1;
 #ifdef HAVE_FILENO
-    my_nonblock(fileno(stdin));
+    fc_nonblock(fileno(stdin));
 #endif
   }
 #endif
@@ -507,7 +799,7 @@ char *my_read_console(void)
   if (!feof(stdin)) {    /* input from server operator */
     static char *bufptr = mybuf;
     /* fetch chars until \n, or run out of space in buffer */
-    /* blocks if my_nonblock() in my_init_console() failed */
+    /* blocks if fc_nonblock() in my_init_console() failed */
     while ((*bufptr = fgetc(stdin)) != EOF) {
       if (*bufptr == '\n') *bufptr = '\0';
       if (*bufptr == '\0') {
@@ -531,80 +823,109 @@ bool is_reg_file_for_access(const char *name, bool write_access)
 {
   struct stat tmp;
 
-  if (stat(name, &tmp) == 0) {
+  if (fc_stat(name, &tmp) == 0) {
     return S_ISREG(tmp.st_mode);
   } else {
     return write_access && errno == ENOENT;
   }
 }
 
-/**********************************************************************
+/****************************************************************************
   Character function wrappers
 
-  Some OS's (win32, Solaris) require a non-negative value as input
-  for these functions.
-***********************************************************************/
+  These functions are wrappers for the libc character class functions,
+  without any locale-dependent behavior. The character functions work as
+  documented for ASCII. Bytes outside of the ASCII set will not be reported
+  to belong to any character class, and will be left unchanged by
+  transformations. This behavior is safe but not strictly correct
+  forsingle-byte 8-bit- or UTF-8 encoded text; in UTF-8, any byte that is
+  part of a multibyte sequence is non-ASCII.
+****************************************************************************/
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 bool my_isalnum(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return FALSE;
+  }
   return isalnum((int)((unsigned char)c)) != 0;
 }
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 bool my_isalpha(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return FALSE;
+  }
   return isalpha((int)((unsigned char)c)) != 0;
 }
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 bool my_isdigit(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return FALSE;
+  }
   return isdigit((int)((unsigned char)c)) != 0;
 }
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 bool my_isprint(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return FALSE;
+  }
   return isprint((int)((unsigned char)c)) != 0;
 }
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 bool my_isspace(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return FALSE;
+  }
   return isspace((int)((unsigned char)c)) != 0;
 }
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 bool my_isupper(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return FALSE;
+  }
   return isupper((int)((unsigned char)c)) != 0;
 }
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 char my_toupper(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return c;
+  }
   return (char) toupper((int)((unsigned char)c));
 }
 
-/**********************************************************************
-  Wrapper function to work around broken libc implementations.
-***********************************************************************/
+/****************************************************************************
+  Wrapper function to work around broken libc implementations. See above.
+****************************************************************************/
 char my_tolower(char c)
 {
+  if (128 <= (unsigned char) c) {
+    return c;
+  }
   return (char) tolower((int)((unsigned char)c));
 }
